import random from "../../util/random.mjs";
function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t = arguments[e];
      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
function overallLayout(params, api, distance, displayRoot, d3) {
  var context = params.context;
  d3.pack().size([api.getWidth() - 2, api.getHeight() - 2]).padding(distance)(displayRoot);
  context.nodes = {};
  displayRoot.descendants().forEach(function(node) {
    context.nodes[node.id] = node;
  });
}
function setChartPosition(params) {
  var iChartOption = params.iChartOption, height = params.height, width = params.width;
  var radius = params.radius, widthDis = params.widthDis, heightDis = params.heightDis;
  if (!iChartOption.chartPosition) {
    iChartOption.chartPosition = {
      center: ["50%", "50%"],
      radius: "80%"
    };
  } else {
    if (!iChartOption.chartPosition.center) {
      iChartOption.chartPosition.center = ["50%", "50%"];
    } else {
      if (!iChartOption.chartPosition.radius) {
        iChartOption.chartPosition.radius = "80%";
      }
    }
  }
  radius = iChartOption.chartPosition.radius || "80%";
  if (typeof radius !== "number") {
    if (radius.indexOf("%") !== -1) {
      radius = (radius.substring(0, radius.indexOf("%")) - 0) / 100;
    } else {
      radius = parseFloat(radius) / Math.min(height, width);
    }
  } else {
    radius = radius / Math.min(height, width);
  }
  var leftValue = iChartOption.chartPosition.center[0] || "50%";
  if (typeof leftValue !== "number") {
    if (leftValue.indexOf("%") !== -1) {
      leftValue = (leftValue.substring(0, leftValue.indexOf("%")) - 0) / 100;
      widthDis = width * (leftValue - radius / 2);
    } else {
      widthDis = parseFloat(leftValue);
    }
  } else {
    widthDis = leftValue;
  }
  var topValue = iChartOption.chartPosition.center[1] || "50%";
  if (typeof topValue !== "number") {
    if (topValue.indexOf("%") !== -1) {
      topValue = (topValue.substring(0, topValue.indexOf("%")) - 0) / 100;
      heightDis = height * (topValue - radius / 2);
    } else {
      heightDis = parseFloat(topValue);
    }
  } else {
    heightDis = topValue;
  }
  return {
    radius,
    widthDis,
    heightDis
  };
}
function returnValue(params, iChartOption) {
  var node = params.node, radius = params.radius, widthDis = params.widthDis, heightDis = params.heightDis, nodeName = params.nodeName, type = params.type, z2 = params.z2;
  var textStyle = iChartOption.textStyle || {};
  var value = {
    type: "circle",
    // 定义球的坐标及半径
    shape: {
      cx: node.x * radius + widthDis,
      cy: node.y * radius + heightDis,
      // 若移动页面使得球的半径小于0时则取反
      r: node.r * radius > 0 ? node.r * radius : -node.r * radius
    },
    transition: ["shape"],
    z2,
    // 定义球的文本信息等
    textContent: {
      type: "text",
      style: _extends({
        text: node.depth !== 0 && node.data.showLabel ? textStyle.formatter && textStyle.formatter(node) || nodeName : "",
        fontFamily: "Arial",
        width: node.r,
        height: node.r,
        borderRadius: node.r,
        // 文本溢出显示
        overflow: "visible",
        fontSize: Math.max(node.r / 3, 12),
        fill: "#ffffff"
      }, textStyle),
      // 在非文本溢出显示前提下，鼠标划入时展示省略号的内容
      emphasis: {
        style: {
          overflow: null,
          fontSize: Math.max(node.r / 3, 12)
        }
      }
    },
    // 设置文本显示的位置
    textConfig: {
      position: "inside"
    },
    style: {
      // 设置球的边框色
      stroke: node.depth >= 1 && type === "nested" ? node.data.borderColor : null,
      // 设置球的背景色
      fill: node.data.color
    },
    // 设置球的跳动范围
    keyframeAnimation: {
      duration: 3e3,
      loop: true,
      delay: random() * 2e3,
      keyframes: [{
        y: -3,
        percent: 0.5,
        easing: "cubicOut"
      }, {
        y: 0,
        percent: 1,
        easing: "bounceOut"
      }, {
        x: -3,
        percent: 0.5,
        easing: "cubicOut"
      }, {
        x: 0,
        percent: 1,
        easing: "bounceOut"
      }]
    }
  };
  return value;
}
function handleRootData(params) {
  var d3 = params.d3, baseOption = params.baseOption, chartInstance = params.chartInstance, iChartOption = params.iChartOption;
  var type = iChartOption.type, distance = iChartOption.distance;
  distance = distance !== void 0 ? distance : type === "non-nested" ? 50 : 5;
  function stratify() {
    return d3.stratify().parentId(function(d) {
      return d.id.substring(0, d.id.lastIndexOf("."));
    })(baseOption.dataset[0].source).sum(function(d) {
      return d.value || 0;
    }).sort(function(a, b) {
      return b.value - a.value;
    });
  }
  var displayRoot = stratify();
  function renderItem(params2, api) {
    var context = params2.context;
    if (!context.layout) {
      context.layout = true;
      overallLayout(params2, api, distance, displayRoot, d3);
    }
    var nodePath = api.value("id");
    var node = context.nodes[nodePath];
    if (!node) {
      return;
    }
    var isLeaf = !node.children || !node.children.length;
    var nodeName = isLeaf ? node.data.label : "";
    var z2 = api.value("depth") * 2;
    var width = chartInstance.getWidth();
    var height = chartInstance.getHeight();
    var widthDis = "";
    var heightDis = "";
    var radius = "";
    radius = setChartPosition({
      iChartOption,
      height,
      width,
      widthDis,
      heightDis,
      radius
    }).radius;
    widthDis = setChartPosition({
      iChartOption,
      height,
      width,
      widthDis,
      heightDis,
      radius
    }).widthDis;
    heightDis = setChartPosition({
      iChartOption,
      height,
      width,
      widthDis,
      heightDis,
      radius
    }).heightDis;
    return returnValue({
      node,
      radius,
      widthDis,
      heightDis,
      nodeName,
      type,
      z2
    }, iChartOption);
  }
  baseOption.series[baseOption.legend.data.length].renderItem = renderItem;
}
export {
  handleRootData
};
