import LineChart from "../LineChart/index.mjs";
import mini from "../../feature/mini/miniBarChart.mjs";
import init from "../../option/init/index.mjs";
import cloneDeep from "../../util/cloneDeep.mjs";
import base from "../../option/base/index.mjs";
import updateWidth from "./barChartOption.mjs";
import { mergeSeries, mergeVisualMap } from "../../util/merge.mjs";
import { setStack, setDoubleSides, setDirection } from "./handleOptipn.mjs";
import RectCoordSys from "../../option/RectSys/index.mjs";
import { setSeries, setWaterFall, setRange, setMarkLine, setLimitFormatter } from "./handleSeries.mjs";
import { ADAPTIVE_THEME, CHART_TYPE } from "../../util/constants.mjs";
import xkey from "../../option/config/xAxis/xkey.mjs";
import xdata from "../../option/config/xAxis/xdata.mjs";
import ldata from "../../option/config/legend/ldata.mjs";
import ydata from "../../option/config/yAxis/ydata.mjs";
function _defineProperty(e, r, t) {
  return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: true, configurable: true, writable: true }) : e[r] = t, e;
}
function _toPropertyKey(t) {
  var i = _toPrimitive(t, "string");
  return "symbol" == typeof i ? i : i + "";
}
function _toPrimitive(t, r) {
  if ("object" != typeof t || !t) return t;
  var e = t[Symbol.toPrimitive];
  if (void 0 !== e) {
    var i = e.call(t, r);
    if ("object" != typeof i) return i;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return ("string" === r ? String : Number)(t);
}
var BarChart = /* @__PURE__ */ function() {
  function BarChart2(iChartOption, chartInstance) {
    this.baseOption = {};
    this.baseOption = cloneDeep(base);
    this.chartInstance = chartInstance;
    this.iChartOption = init(iChartOption);
    this.updateOption();
  }
  var _proto = BarChart2.prototype;
  _proto.updateOption = function updateOption() {
    var iChartOption = this.iChartOption;
    RectCoordSys(this.baseOption, this.iChartOption, CHART_TYPE.BAR);
    var xAxisKey = xkey(iChartOption);
    var xAxisData = xdata(iChartOption.data, xAxisKey);
    var legendData = ldata(iChartOption.data, xAxisKey);
    var seriesData = ydata(iChartOption.data, legendData);
    this.baseOption.legend.data = iChartOption.legend.data || legendData;
    this.baseOption.xAxis.forEach(function(item) {
      item.data = xAxisData;
    });
    this.baseOption.series = setSeries(seriesData, legendData, iChartOption);
    setStack(this.baseOption, iChartOption, legendData, seriesData);
    setDoubleSides(this.baseOption, iChartOption);
    setWaterFall(this.baseOption, iChartOption);
    setRange(this.baseOption, iChartOption);
    setMarkLine(this.baseOption, iChartOption);
    setDirection(this.baseOption, iChartOption.direction);
    setLimitFormatter(this.baseOption, iChartOption, seriesData);
    if (iChartOption.silent) {
      this.baseOption.tooltip = {};
    }
    mergeSeries(iChartOption, this.baseOption);
    mergeVisualMap(iChartOption, this.baseOption);
    mini(iChartOption, this.baseOption);
  };
  _proto.updateOptionAgain = function updateOptionAgain() {
    var baseOption = this.baseOption;
    var iChartOption = this.iChartOption;
    var lineDataName = iChartOption.lineDataName;
    if (lineDataName && lineDataName.length > 0) {
      var lineChartBaseOpt = new LineChart(iChartOption, {}, this.chartInstance);
      var lineBaseOption = lineChartBaseOpt.getOption();
      var lineSeries = lineBaseOption.series;
      var barSeries = baseOption.series;
      lineDataName.forEach(function(lineName) {
        var bar = null;
        var line = null;
        for (var i = 0; i < lineSeries.length; i++) {
          if (lineSeries[i].name === lineName) {
            line = lineSeries[i];
            break;
          }
        }
        for (var _i = 0; _i < barSeries.length; _i++) {
          if (barSeries[_i].name === lineName) {
            bar = barSeries[_i];
            break;
          }
        }
        for (var key in line) {
          if (Object.hasOwnProperty.call(line, key)) {
            bar[key] = line[key];
          }
        }
      });
    }
    if (ADAPTIVE_THEME.includes(this.iChartOption.theme)) {
      updateWidth(baseOption, this.chartInstance, this.iChartOption);
    }
  };
  _proto.getOption = function getOption() {
    return this.baseOption;
  };
  _proto.setOption = function setOption() {
  };
  _proto.resize = function resize(callback) {
    if (ADAPTIVE_THEME.includes(this.iChartOption.theme)) {
      updateWidth(this.baseOption, this.chartInstance, this.iChartOption);
      callback && callback(this.baseOption);
    }
  };
  return BarChart2;
}();
_defineProperty(BarChart, "name", CHART_TYPE.BAR);
export {
  BarChart as default
};
