import chartToken from "./chartToken.mjs";
import { getColor, codeToRGB } from "../../util/color.mjs";
import merge from "../../util/merge.mjs";
function getDatasetInit() {
  return [{
    source: []
  }, {
    transform: {
      type: "boxplot"
    }
  }, {
    fromDatasetIndex: 1,
    fromTransformResult: 1
  }];
}
function getBoxplotUnitSeries() {
  return {
    name: "boxplot",
    type: "boxplot",
    datasetIndex: 1,
    itemStyle: {
      color: void 0,
      borderColor: void 0,
      borderWidth: chartToken.boxplotItemStyleBorderWidth,
      shadowColor: "transparent"
    },
    emphasis: {
      itemStyle: {
        color: void 0,
        borderWidth: chartToken.boxplotEmphasisItemStyleBorderWidth,
        shadowColor: "transparent"
      }
    }
  };
}
function getScatterUnitSeries() {
  return {
    name: "outlier",
    type: "scatter",
    datasetIndex: 2,
    symbolSize: chartToken.scatterSymbolSize,
    itemStyle: {
      color: void 0,
      // 常规场景不显示边框
      borderWidth: chartToken.scatterItemBorderWidth,
      // borderWidth常规为0，此处设置borderColor为透明色+是为了避免hover移出symbol瞬间白色闪烁的问题
      borderColor: chartToken.scatterItemBorderColor
    },
    emphasis: {
      itemStyle: {
        color: chartToken.scatterEmphasisItemColor,
        borderWidth: chartToken.scatterEmphasisItemBorderWidth
        // borderColor颜色在各系列数据中单独设置
      }
    }
  };
}
function setDataset(data) {
  var dataset = getDatasetInit();
  dataset[0].source = data;
  return dataset;
}
function getSeriesColor(color, index) {
  if (index === void 0) {
    index = 0;
  }
  var seriesColor = getColor(color, index);
  var itemColor = codeToRGB(seriesColor, 0.15);
  var emphasisColor = codeToRGB(seriesColor, 0.3);
  return {
    seriesColor,
    itemColor,
    emphasisColor
  };
}
function getBoxplotSeries(iChartOpt, index) {
  if (index === void 0) {
    index = 0;
  }
  var color = iChartOpt.color;
  var unitSeries = getBoxplotUnitSeries();
  var _getSeriesColor = getSeriesColor(color, index), seriesColor = _getSeriesColor.seriesColor, itemColor = _getSeriesColor.itemColor, emphasisColor = _getSeriesColor.emphasisColor;
  unitSeries.itemStyle.color = itemColor;
  unitSeries.itemStyle.borderColor = seriesColor;
  unitSeries.emphasis.itemStyle.color = emphasisColor;
  return unitSeries;
}
function getScatterSeries(iChartOpt) {
  var color = iChartOpt.color;
  var unitSeries = getScatterUnitSeries();
  var _getSeriesColor2 = getSeriesColor(color), seriesColor = _getSeriesColor2.seriesColor;
  unitSeries.itemStyle.color = seriesColor;
  unitSeries.emphasis.itemStyle.borderColor = seriesColor;
  return unitSeries;
}
function setDefaultSeries(baseOpt, iChartOpt) {
  var boxplotSeries = getBoxplotSeries(iChartOpt);
  var scatterSeries = getScatterSeries(iChartOpt);
  var series = [boxplotSeries, scatterSeries];
  baseOpt.series = series;
}
function setSeries(baseOpt, iChartOpt) {
  if (iChartOpt.data && !iChartOpt.dataset) {
    baseOpt.dataset = setDataset(iChartOpt.data);
    setDefaultSeries(baseOpt, iChartOpt);
  }
  if (iChartOpt.dataset && iChartOpt.series) {
    baseOpt.dataset = iChartOpt.dataset;
    var series = iChartOpt.series.map(function(item, index) {
      var boxplotUnitSeries = getBoxplotSeries(iChartOpt, index);
      merge(boxplotUnitSeries, item);
      return boxplotUnitSeries;
    });
    baseOpt.series = series;
  }
}
export {
  setSeries as default,
  setDataset,
  setDefaultSeries
};
