import { getColor, codeToRGB } from "../../util/color.mjs";
import cloneDeep from "../../util/cloneDeep.mjs";
import { getMarkLineDefault } from "../../option/config/mark/index.mjs";
import chartToken from "./chartToken.mjs";
import merge from "../../util/merge.mjs";
var seriesInit = {
  // 数据
  data: [],
  // 气泡图
  type: "scatter",
  // 鼠标hover时显示label，并且其他legend变成灰色
  emphasis: {
    label: {
      show: true,
      color: "#ffffff",
      fontSize: 14,
      formatter: function formatter(param) {
        return param.data[3];
      },
      position: "top"
    }
  },
  // 气泡失去焦点时，配置样式
  blur: {},
  // 气泡样式
  itemStyle: {}
};
function setSeries(_ref) {
  var legendData = _ref.legendData, data = _ref.data, markLine = _ref.markLine, color = _ref.color, iChartOption = _ref.iChartOption;
  seriesInit.emphasis.label.color = chartToken.emphasisLabelColor;
  var series = [];
  legendData.forEach(function(legend, index) {
    var seriesUnit = cloneDeep(seriesInit);
    var itemBorderColor = getColor(color, index);
    var itemColor = codeToRGB(itemBorderColor, iChartOption.symbolOpacity || 0.2);
    if (markLine) {
      seriesUnit.markLine = getMarkLineDefault(true);
      if (markLine.y) {
        seriesUnit.markLine.data.push({
          yAxis: markLine.y
        });
      }
      if (markLine.x) {
        seriesUnit.markLine.data.push({
          xAxis: markLine.x
        });
      }
    }
    seriesUnit.name = legend;
    seriesUnit.data = data[legend];
    seriesUnit.itemStyle = {
      color: itemColor,
      borderColor: itemBorderColor,
      borderWidth: 1
    };
    series.push(seriesUnit);
  });
  return series;
}
function handleSeriesExtra(baseOpt, iChartOption) {
  var symbol = iChartOption.symbol, symbolRotate = iChartOption.symbolRotate, symbolOffset = iChartOption.symbolOffset, cursor = iChartOption.cursor, label = iChartOption.label, itemStyle = iChartOption.itemStyle, blur = iChartOption.blur, emphasis = iChartOption.emphasis;
  baseOpt.series.forEach(function(item) {
    item.symbol = symbol;
    item.symbolRotate = symbolRotate;
    item.symbolOffset = symbolOffset;
    item.cursor = cursor;
    item.label = label;
    merge(item.itemStyle, itemStyle);
    merge(item.blur, blur);
    merge(item.emphasis, emphasis);
  });
}
export {
  handleSeriesExtra,
  seriesInit,
  setSeries
};
