import LineRound from "./LineRound.mjs";
import LineBezier from "./LineBezier.mjs";
import LineDirect from "./LineDirect.mjs";
import { createTransLine, transLine } from "./LineMode.mjs";
import { NODE_ID_PREFIX } from "./NodeManager.mjs";
import { attr, createSvgByTag, addClass } from "./util.mjs";
var _LineFactory;
var LINE_ID_PREFIX = "fc-line-";
var TYPE_ROUND = "Round";
var TYPE_BEZIER = "Bezier";
var TYPE_Direct = "Direct";
var TYPE_STRAIGHT = "Straight";
var defaultStyle = {
  width: 1,
  color: "#c2c2c2",
  type: TYPE_ROUND
};
var LineFactory = (_LineFactory = {}, _LineFactory[TYPE_ROUND] = LineRound, _LineFactory[TYPE_Direct] = LineDirect, _LineFactory[TYPE_BEZIER] = LineBezier, _LineFactory[TYPE_STRAIGHT] = LineRound, _LineFactory);
var transLineType = ["Direct", "Round", "Straight"];
var LineManager = /* @__PURE__ */ function() {
  function LineManager2(data, container, tagContainer, containerPosn, option) {
    this.style = void 0;
    this.lineCreator = void 0;
    this.direction = void 0;
    this.data = data;
    this.option = option;
    this.tagContainer = tagContainer;
    this.direction = option.direction;
    this.initColor(option);
    this.style = Object.assign({}, defaultStyle, option.lineStyle);
    this.createLineCreator(container);
    this.initArrow();
    this.container = container;
    this.containerPosn = containerPosn;
    this.getLineAlignStyle(data.edges);
    this.createLines(data.edges);
    this.connectVirtualLine(data.edges);
    this.createTags(data.edges);
  }
  var _proto = LineManager2.prototype;
  _proto.initColor = function initColor(option) {
    if (option.theme && option.theme === "dark") {
      defaultStyle.color = "#ffffff";
    }
  };
  _proto.initArrow = function initArrow() {
    var arrow = document.getElementById("markerArrow");
    var color = this.style.color || "#c2c2c2";
    attr(arrow, "fill", color);
  };
  _proto.createLineCreator = function createLineCreator(container) {
    if (this.style.type === TYPE_STRAIGHT) {
      this.style.radius = 0;
    }
    var CreatorClass = LineFactory[this.style.type];
    this.lineCreator = new CreatorClass(this.style, {
      gContainer: container,
      tagContainer: this.tagContainer
    });
  };
  _proto.getLineAlignStyle = function getLineAlignStyle(edges) {
    edges.forEach(function(edge) {
      if (!edge.alignType) {
        var sameSource = edges.filter(function(item) {
          return item.source === edge.source && item !== edge;
        });
        if (sameSource.length > 0) {
          edge.alignType = "source";
          sameSource.forEach(function(item) {
            item.alignType = "source";
          });
        }
      }
    });
    edges.forEach(function(edge) {
      if (!edge.alignType) {
        edge.alignType = "target";
      }
    });
  };
  _proto.setData = function setData(edge) {
    var _edge$lineStyle;
    if (edge != null && (_edge$lineStyle = edge.lineStyle) != null && _edge$lineStyle.type) {
      var _edge$lineStyle2;
      var CreatorClass = LineFactory[edge == null ? void 0 : (_edge$lineStyle2 = edge.lineStyle) == null ? void 0 : _edge$lineStyle2.type];
      var lineCreator = new CreatorClass(edge == null ? void 0 : edge.lineStyle, {
        gContainer: this.container,
        tagContainer: this.tagContainer
      });
      return lineCreator.setData(edge, this.data, this.direction);
    } else {
      this.lineCreator.setData(edge, this.data, this.direction);
    }
    if (edge.offset) {
      edge.offset(edge);
    }
  };
  _proto.setPath = function setPath(edge) {
    var _edge$lineStyle3;
    if (edge != null && (_edge$lineStyle3 = edge.lineStyle) != null && _edge$lineStyle3.type) {
      var _edge$lineStyle4;
      var CreatorClass = LineFactory[edge == null ? void 0 : (_edge$lineStyle4 = edge.lineStyle) == null ? void 0 : _edge$lineStyle4.type];
      var lineCreator = new CreatorClass(edge == null ? void 0 : edge.lineStyle, {
        gContainer: this.container,
        tagContainer: this.tagContainer
      });
      return lineCreator.setPath(edge);
    } else {
      return this.lineCreator.setPath(edge);
    }
  };
  _proto.createLines = function createLines(edges) {
    var _this = this;
    edges.forEach(function(edge) {
      _this.setData(edge);
      var path = _this.createLine(edge);
      _this.container.appendChild(path);
    });
  };
  _proto.createLine = function createLine(edge) {
    var _edge$lineStyle5;
    var path = createSvgByTag("path");
    var id = LINE_ID_PREFIX + edge.source + "-to-" + edge.target;
    var d = this.setPath(edge);
    edge.id = id;
    attr(path, "d", d);
    attr(path, "id", id);
    if (this.style.color) {
      attr(path, "stroke", this.style.color);
    }
    if (this.style.dash || this.style.mode === "dash") {
      attr(path, "stroke-dasharray", 5);
    }
    if (edge.lineStyle && edge.lineStyle.color) {
      attr(path, "stroke", edge.lineStyle.color);
    }
    if (edge.lineStyle && (edge.lineStyle.dash || edge.lineStyle.mode === "dash")) {
      attr(path, "stroke-dasharray", 5);
    }
    if (edge.lineStyle && edge.lineStyle.dash === false) {
      attr(path, "stroke-dasharray", 0);
    }
    if (this.style.width) {
      attr(path, "stroke-width", this.style.width + "px");
    }
    if (edge.lineStyle && edge.lineStyle.width) {
      attr(path, "stroke-width", edge.lineStyle.width + "px");
    }
    addClass(path, "fc-line");
    var noDirect = (edge == null ? void 0 : (_edge$lineStyle5 = edge.lineStyle) == null ? void 0 : _edge$lineStyle5.type) === TYPE_Direct || this.style.type === TYPE_Direct;
    addClass(path, noDirect ? "fc-noArrow" : "");
    if (this.isTransLine(edge)) {
      var _edge$lineStyle6;
      path = createTransLine(d, transLine(edge, id, ((_edge$lineStyle6 = edge.lineStyle) == null ? void 0 : _edge$lineStyle6.transPattern) || this.style.transPattern));
    } else {
      var childNode = document.getElementById(edge.id);
      childNode && document.getElementById("fc-trans-container").removeChild(childNode);
    }
    return path;
  };
  _proto.isTransLine = function isTransLine(edge) {
    var isTrans = false;
    if (transLineType.includes(this.style.type)) {
      if (this.style.mode === "trans") {
        isTrans = true;
        if ("lineStyle" in edge && "mode" in edge.lineStyle) {
          if (edge.lineStyle.mode !== "trans") {
            isTrans = false;
          }
        }
      } else {
        if ("lineStyle" in edge && "mode" in edge.lineStyle && edge.lineStyle.mode === "trans") {
          isTrans = true;
        }
      }
    }
    return isTrans;
  };
  _proto.connectVirtualLine = function connectVirtualLine(edges) {
    var _this2 = this;
    var targetVirtualEdges = edges.filter(function(edge) {
      return edge.target.includes("virtual");
    });
    var sourceVirtualEdges = edges.filter(function(edge) {
      return edge.source.includes("virtual");
    });
    targetVirtualEdges.forEach(function(targetEdge) {
      var sourceEdge = sourceVirtualEdges.filter(function(item) {
        return item.source === targetEdge.target;
      })[0];
      var targetEdgeDom = _this2.container.querySelector("#" + targetEdge.id);
      var sourceEdgeDom = _this2.container.querySelector("#" + sourceEdge.id);
      sourceEdgeDom.remove();
      var newPath = targetEdgeDom.getAttribute("d") + " L" + sourceEdgeDom.getAttribute("d").slice(1);
      var newId = LINE_ID_PREFIX + targetEdge.source + "-to-" + sourceEdge.target;
      targetEdgeDom.setAttribute("d", newPath);
      targetEdgeDom.setAttribute("id", newId);
    });
  };
  _proto.createTags = function createTags(edges) {
    var _this3 = this;
    edges.forEach(function(edge) {
      _this3.option.renderTag && _this3.option.renderTag(_this3.tagContainer, edge);
    });
  };
  _proto.updateLine = function updateLine(domId) {
    var _this4 = this;
    var nodeId = domId == null ? void 0 : domId.replace(NODE_ID_PREFIX, "");
    this.data.edges.forEach(function(edge) {
      if (edge.source === nodeId || edge.target === nodeId) {
        var _document$getElementB;
        _this4.setData(edge);
        var id = LINE_ID_PREFIX + edge.source + "-to-" + edge.target;
        if (_this4.isTransLine(edge)) {
          var _edge$lineStyle7;
          var parentNode = document.getElementById("fc-trans-container");
          var updateChild = document.getElementById(id);
          updateChild && parentNode.removeChild(updateChild);
          createTransLine(_this4.setPath(edge), transLine(edge, id, ((_edge$lineStyle7 = edge.lineStyle) == null ? void 0 : _edge$lineStyle7.transPattern) || _this4.style.transPattern));
        }
        (_document$getElementB = document.getElementById(id)) == null ? void 0 : _document$getElementB.setAttribute("d", _this4.setPath(edge));
      }
    });
  };
  _proto.updateAllLine = function updateAllLine() {
    var _this5 = this;
    this.data.nodes.forEach(function(node) {
      var _node$dom;
      var domId = ((_node$dom = node.dom) == null ? void 0 : _node$dom.id) || node.id;
      _this5.updateLine(domId);
    });
  };
  return LineManager2;
}();
export {
  LineManager as default
};
