import Layout from "./Layout.mjs";
import { isFunction } from "../../util/type.mjs";
var NODE_ID_PREFIX = "fc-node-";
var NodeManager = /* @__PURE__ */ function() {
  function NodeManager2(data, option, container) {
    var _this = this;
    this.data = void 0;
    this.option = option;
    this.render = option.render;
    this.createNodes(data, container);
    setTimeout(function() {
      _this.createBorder(option.renderBorder, container, data);
    }, 100);
  }
  var _proto = NodeManager2.prototype;
  _proto.layoutNodes = function layoutNodes(data, containerPosn) {
    this.data = data;
    this.getNodeSize(this.data.nodes, containerPosn);
    new Layout(this.data, this.option, containerPosn);
    var _this$correctNode = this.correctNode(this.data.nodes), minX = _this$correctNode.minX, minY = _this$correctNode.minY;
    this.moveNode(this.data.nodes, {
      minX,
      minY
    });
  };
  _proto.correctNode = function correctNode(nodes) {
    var minX = 0;
    var minY = 0;
    nodes.forEach(function(node) {
      if (node.x < minX) {
        minX = node.x;
      }
      if (node.y < minY) {
        minY = node.y;
      }
    });
    this.data.minX = minX;
    this.data.minY = minY;
    return {
      minX,
      minY
    };
  };
  _proto.moveNode = function moveNode(nodes, _ref) {
    var minX = _ref.minX, minY = _ref.minY;
    nodes.forEach(function(node) {
      if (node.dom) {
        node.dom.style.top = node.y - minY + "px";
        node.dom.style.left = node.x - minX + "px";
      }
    });
  };
  _proto.getNodeSize = function getNodeSize(nodes, containerPosn) {
    nodes.forEach(function(node) {
      var size = node.dom && node.dom.getBoundingClientRect();
      var innerSize = node.dom && node.dom.firstChild && node.dom.firstChild.getBoundingClientRect();
      node.width = size && size.width / containerPosn.scaleX;
      node.height = size && size.height / containerPosn.scaleY;
      node.innerWidth = innerSize && innerSize.width / containerPosn.scaleX;
      node.innerHeight = innerSize && innerSize.height / containerPosn.scaleY;
    });
  };
  _proto.createNode = function createNode(ndata) {
    var id = ndata.id, render = ndata.render;
    var nodeDom = document.createElement("div");
    nodeDom.classList.add("fc-node");
    nodeDom.id = NODE_ID_PREFIX + id;
    var renderFun = render || this.render;
    if (renderFun) {
      var dom = renderFun(nodeDom, ndata);
      dom && nodeDom.appendChild(dom);
    }
    return nodeDom;
  };
  _proto.createNodes = function createNodes(data, container) {
    var _this2 = this;
    data.nodes.forEach(function(node) {
      var nodeDom = _this2.createNode(node);
      node.dom = nodeDom;
      container.appendChild(nodeDom);
    });
  };
  _proto.createBorder = function createBorder(renderBorder, container, data) {
    isFunction(renderBorder) && renderBorder(container, data.nodes);
  };
  _proto.updateNode = function updateNode(domId, x, y) {
    var id = domId.replace(NODE_ID_PREFIX, "");
    this.data.nodesObj[id].x = x;
    this.data.nodesObj[id].y = y;
  };
  return NodeManager2;
}();
export {
  NODE_ID_PREFIX,
  NodeManager as default
};
