import { createEl, addClass, appendText, appendNode, appendEL, getTitle, setStyle, handleSelectTimeLineLabel, setTranslateX, handleTimeLineLabel, insetArrow } from "./util.mjs";
import CoordinateSystem from "./CoordinateSystem.mjs";
import { SECONDS } from "./constant.mjs";
var TimeLine = /* @__PURE__ */ function() {
  function TimeLine2(wrapper, option, barContainerWidth, barContainerHeight) {
    var _this = this;
    this.wrapper = null;
    this.startX = 0;
    this.distance = 0;
    this.startX2 = 0;
    this.distance2 = 0;
    this.handleLDis = 0;
    this.handleRDis = 0;
    this.mask = null;
    this.handleL = null;
    this.handleR = null;
    this.innerChartWidth = 0;
    this.option = {};
    this.innerChartHeight = 0;
    this.selectTimeLine = null;
    this.innerCtx = null;
    this.coordinateSystem = void 0;
    this.mouseMoveLeft = function(e) {
      var pageX = e.clientX;
      var moveX = pageX - _this.startX + _this.distance;
      var rightLimitValue = _this.handleRDis - 15;
      if (moveX >= rightLimitValue) {
        _this.handleLDis = rightLimitValue;
        moveX = rightLimitValue;
      } else if (moveX <= 0) {
        _this.handleLDis = 0;
        moveX = 0;
      } else {
        _this.handleLDis = moveX;
      }
      if (moveX <= rightLimitValue && moveX >= 0) {
        _this.resetTimeLine(_this.handleL, moveX, "left");
        _this.resetBody();
      }
    };
    this.mouseUpLeft = function(e) {
      var pageX = e.clientX;
      _this.distance += pageX - _this.startX;
      var rightLimitValue = _this.handleRDis - 15;
      if (_this.distance >= rightLimitValue) {
        _this.handleLDis = rightLimitValue;
        _this.distance = rightLimitValue;
      } else if (_this.distance <= 0) {
        _this.handleLDis = 0;
        _this.distance = 0;
      } else {
        _this.handleLDis = _this.distance;
      }
      document.removeEventListener("mousemove", _this.mouseMoveLeft);
      document.removeEventListener("mouseup", _this.mouseUpLeft);
    };
    this.mouseDownLeft = function(e) {
      var pageX = e.clientX;
      _this.startX = pageX;
      document.addEventListener("mousemove", _this.mouseMoveLeft);
      document.addEventListener("mouseup", _this.mouseUpLeft);
      return false;
    };
    this.mouseMoveRight = function(e) {
      var pageX = e.clientX;
      var moveX = pageX - _this.startX2 + _this.distance2;
      var leftLimitValue = _this.handleLDis + 15;
      if (moveX >= 0) {
        _this.handleRDis = _this.innerChartWidth;
        moveX = 0;
      } else if (moveX <= -(_this.innerChartWidth - leftLimitValue)) {
        _this.handleRDis = leftLimitValue;
        moveX = -(_this.innerChartWidth - leftLimitValue);
      } else {
        _this.handleRDis = _this.innerChartWidth + moveX;
      }
      if (moveX >= -(_this.innerChartWidth - leftLimitValue) && moveX <= 0) {
        _this.resetTimeLine(_this.handleR, moveX, "right");
        _this.resetBody();
      }
    };
    this.mouseUpRight = function(e) {
      var pageX = e.clientX;
      _this.distance2 += pageX - _this.startX2;
      var leftLimitValue = _this.handleLDis + 15;
      if (_this.distance2 <= -(_this.innerChartWidth - leftLimitValue)) {
        _this.handleRDis = leftLimitValue;
        _this.distance2 = -(_this.innerChartWidth - leftLimitValue);
      } else if (_this.distance2 >= 0) {
        _this.handleRDis = _this.innerChartWidth;
        _this.distance2 = 0;
      } else {
        _this.handleRDis = _this.innerChartWidth + _this.distance2;
      }
      document.removeEventListener("mousemove", _this.mouseMoveRight);
      document.removeEventListener("mouseup", _this.mouseUpRight);
    };
    this.mouseDownRight = function(e) {
      var pageX = e.clientX;
      _this.startX2 = pageX;
      document.addEventListener("mousemove", _this.mouseMoveRight);
      document.addEventListener("mouseup", _this.mouseUpRight);
      return false;
    };
    this.wrapper = wrapper;
    this.handleRDis = barContainerWidth;
    this.innerChartWidth = barContainerWidth;
    this.option = option;
    this.innerChartHeight = barContainerHeight;
  }
  var _proto = TimeLine2.prototype;
  _proto.createDom = function createDom(className) {
    var node = createEl("div");
    addClass(node, className);
    return node;
  };
  _proto.addClassName = function addClassName(node, className) {
    addClass(node, className);
  };
  _proto.addText = function addText(node, text) {
    appendText(node, text);
  };
  _proto.clearDom = function clearDom(node) {
    appendNode(node, "");
  };
  _proto.addDom = function addDom(container, node) {
    appendEL(container, node);
  };
  _proto.renderTitle = function renderTitle(node) {
    var headerTitle = this.createDom("ev_GanttChart_headerTitle");
    var title = getTitle(this.option.currentTime);
    var titleTop = this.createDom("ev_GanttChart_titleTop");
    this.addText(titleTop, title.yearLabel);
    this.addDom(headerTitle, titleTop);
    var titleBtn = this.createDom("ev_GanttChart_titleBtn");
    this.addText(titleBtn, title.dateLabel);
    this.addDom(headerTitle, titleBtn);
    this.addDom(node, headerTitle);
  };
  _proto.setMaskWidth = function setMaskWidth() {
    var width = this.handleRDis - this.handleLDis - 12 + "px";
    var left = this.handleLDis + 6 + "px";
    setStyle(this.mask, "width", width);
    setStyle(this.mask, "left", left);
  };
  _proto.getTimeScope = function getTimeScope() {
    var leftTime = parseInt(this.handleLDis / this.innerChartWidth * SECONDS.weekSeconds);
    var rightTime = parseInt(this.handleRDis / this.innerChartWidth * SECONDS.weekSeconds);
    var timeScope = {
      leftTime,
      rightTime
    };
    return timeScope;
  };
  _proto.renderSelectTimeLine = function renderSelectTimeLine(timeScope) {
    if (this.selectTimeLine) {
      this.clearDom(this.selectTimeLine);
    }
    var label = handleSelectTimeLineLabel(this.option.currentTime, timeScope);
    var selectLeft = this.createDom("ev_GanttChart_selectLeft");
    this.addText(selectLeft, label[0]);
    this.addDom(this.selectTimeLine, selectLeft);
    var verticalLineL = this.createDom("ev_GanttChart_verticalLineL");
    this.addDom(this.selectTimeLine, verticalLineL);
    var horizontalLine = this.createDom("ev_GanttChart_horizontalLine");
    this.addDom(this.selectTimeLine, horizontalLine);
    var verticalLineR = this.createDom("ev_GanttChart_verticalLineR");
    this.addDom(this.selectTimeLine, verticalLineR);
    var selectRight = this.createDom("ev_GanttChart_selectRight");
    this.addText(selectRight, label[1]);
    this.addDom(this.selectTimeLine, selectRight);
  };
  _proto.setSelectTimeLine = function setSelectTimeLine() {
    var timeScope = this.getTimeScope();
    this.renderSelectTimeLine(timeScope);
  };
  _proto.resetTimeLine = function resetTimeLine(node, distance) {
    setTranslateX(node, distance);
    this.setMaskWidth();
    this.setSelectTimeLine();
  };
  _proto.renderHeaderTime = function renderHeaderTime(node) {
    var label = handleTimeLineLabel(this.option.currentTime);
    for (var i = 0; i <= 28; i++) {
      var scale = this.createDom("ev_GanttChart_scale");
      this.addDom(node, scale);
      if (i % 4 === 0) {
        var nubScale = this.createDom("ev_GanttChart_nubScale");
        this.addClassName(scale, "nub");
        this.addText(nubScale, label[i / 4]);
        this.addDom(scale, nubScale);
      }
      if (i % 2 === 0 && i / 2 % 2 !== 0) {
        this.addClassName(scale, "high");
      }
      if (i < 28) {
        var scaleBlank = this.createDom("ev_GanttChart_scaleBlank");
        this.addDom(node, scaleBlank);
      }
      if (i === 28) {
        var handleWrapper = this.createDom("ev_GanttChart_handleWrapper");
        this.addDom(node, handleWrapper);
        var handleLeft = this.createDom("ev_GanttChart_handleLeft");
        this.handleL = handleLeft;
        var leftIcon = this.createDom("ev_GanttChart_Icon");
        insetArrow(leftIcon);
        var maskMid = this.createDom("ev_GanttChart_mask");
        this.mask = maskMid;
        this.addDom(handleWrapper, maskMid);
        var handleRight = this.createDom("ev_GanttChart_handleRight");
        this.handleR = handleRight;
        var rightIcon = this.createDom(["ev_GanttChart_Icon", "right"]);
        insetArrow(rightIcon);
        this.addDom(handleLeft, leftIcon);
        this.addDom(handleWrapper, handleLeft);
        this.addDom(handleRight, rightIcon);
        this.addDom(handleWrapper, handleRight);
        this.handleL.onmousedown = this.mouseDownLeft;
        this.handleR.onmousedown = this.mouseDownRight;
      }
    }
  };
  _proto.getCurrentTotalTime = function getCurrentTotalTime() {
    var diffDistance = this.handleRDis - this.handleLDis;
    var distancePercent = diffDistance / this.innerChartWidth;
    var time = parseInt(distancePercent * SECONDS.weekSeconds);
    return time;
  };
  _proto.resetBody = function resetBody() {
    var currentTotalTime = this.getCurrentTotalTime();
    var timeScope = this.getTimeScope();
    this.coordinateSystem.resetCanvas(currentTotalTime, timeScope);
  };
  _proto.initHandlePositon = function initHandlePositon() {
    this.handleLDis = this.distance = parseInt(this.innerChartWidth * 6 / 7);
    setTranslateX(this.handleL, this.distance);
    this.setMaskWidth("left");
    this.resetBody();
  };
  _proto.init = function init() {
    var header = this.createDom("ev_GanttChart_header");
    this.addDom(this.wrapper, header);
    var headerTimeLine = this.createDom("ev_GanttChart_headerTimeLine");
    this.addDom(header, headerTimeLine);
    this.renderTitle(headerTimeLine);
    var time = this.createDom("ev_GanttChart_headerTime");
    this.addDom(headerTimeLine, time);
    var timeTopContainer = this.createDom("ev_GanttChart_headerTimeTop_container");
    this.addDom(time, timeTopContainer);
    var timeTop = this.createDom("ev_GanttChart_headerTimeTop");
    this.addDom(timeTopContainer, timeTop);
    this.renderHeaderTime(timeTop);
    var timeBtmContainer = this.createDom("ev_GanttChart_headerTimeBtm_container");
    this.addDom(time, timeBtmContainer);
    var timeBtm = this.createDom("ev_GanttChart_headerTimeBtm");
    this.addDom(timeBtmContainer, timeBtm);
    this.selectTimeLine = timeBtm;
    this.renderSelectTimeLine();
    this.coordinateSystem = new CoordinateSystem(this.wrapper, this.option, this.innerChartWidth, this.innerChartHeight);
    this.coordinateSystem.init();
    this.initHandlePositon();
  };
  return TimeLine2;
}();
export {
  TimeLine as default
};
