import { changeRgbaOpacity } from "../../util/color.mjs";
import { SYMBOLCOLOR, CHARTTYPE } from "./BaseOption.mjs";
import xAxis from "../../option/config/xAxis/index.mjs";
import yAxis from "../../option/config/yAxis/index.mjs";
import grid from "../../option/config/grid/index.mjs";
import tooltip from "../../option/config/tooltip/index.mjs";
import _defendXSS from "../../util/defendXSS.mjs";
import { CHART_TYPE } from "../../util/constants.mjs";
function setHeatMapDeaultIchartOption(iChartOpt) {
  if (!iChartOpt.color) {
    iChartOpt.color = SYMBOLCOLOR[iChartOpt.type];
  }
  var padding = iChartOpt.chartPadding || iChartOpt.padding;
  if (!padding && iChartOpt.type === CHARTTYPE[1]) {
    iChartOpt.padding = iChartOpt.handle ? [50, 120, 20, 20] : [50, 30, 20, 20];
  }
}
function rectangularFormatter(params) {
  var color = params.color;
  var data = params.data;
  var x = data[0], y = data[1], z = data[2], name = data[3];
  var htmlString = '<div style="margin-bottom:4px;">\n                                矩形热力图\n                            </div>';
  htmlString += '<div style="margin-bottom:4px;">\n                            <span style="display:inline-block;width:10px;height:10px;\n                            margin-right:8px;border-style: solid;border-width:1px;\n                            border-color:' + _defendXSS(changeRgbaOpacity(color, 1)) + ";background-color:" + _defendXSS(color) + ';"></span>\n                            <span>' + _defendXSS(name) + "</span>\n                        </div>";
  htmlString += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;min-width:60px;">x维度</span> \n                                <span>' + _defendXSS(x) + "</span>\n                            </div>";
  htmlString += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;min-width:60px;">y维度</span> \n                                <span>' + _defendXSS(y) + "</span>\n                            </div>";
  htmlString += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;\n                                min-width:60px;">透明度维度</span> \n                                <span>' + _defendXSS(z) + "</span>\n                            </div>";
  return htmlString;
}
function calendarFormatter(params) {
  var color = params.color;
  var data = params.data;
  var name = params.name;
  var z = data[2];
  var htmlDom = '<div style="margin-bottom:4px;">\n                                日历热力图\n                            </div>';
  htmlDom += '<div style="margin-bottom:4px;">\n                            <span style="display:inline-block;width:10px;\n                            height:10px;margin-right:8px;border-style: solid;\n                            border-width:1px;border-color:' + _defendXSS(changeRgbaOpacity(color, 1)) + ";background-color:" + _defendXSS(color) + ';"></span>\n                            <span>' + _defendXSS(name) + "</span>\n                        </div>";
  htmlDom += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;\n                                min-width:60px;">Value</span> \n                                <span>' + _defendXSS(z) + "</span>\n                            </div>";
  return htmlDom;
}
function hexagonFormatter(params) {
  var color = params.color;
  var data = params.data;
  var x = data[0], y = data[1], z = data[2], name = data[3];
  var html = '<div style="margin-bottom:4px;">\n                            蜂窝热力图\n                            </div>';
  html += '<div style="margin-bottom:4px;">\n                            <span style="display:inline-block;width:10px;\n                            height:10px;margin-right:8px;border-style: solid;\n                            border-width:1px;border-color:' + _defendXSS(changeRgbaOpacity(color, 1)) + ";background-color:" + _defendXSS(color) + ';"></span>\n                            <span>' + _defendXSS(name) + "</span>\n                        </div>";
  html += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;min-width:60px;">x维度</span> \n                                <span>' + _defendXSS(x) + "</span>\n                            </div>";
  html += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;min-width:60px;">y维度</span> \n                                <span>' + _defendXSS(y) + "</span>\n                            </div>";
  html += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;min-width:60px;">颜色维度</span> \n                                <span>' + _defendXSS(z) + "</span>\n                            </div>";
  return html;
}
var heatMapTooltipFormatter = {
  RectangularHeatMapChart: rectangularFormatter,
  CalendarHeatMapChart: calendarFormatter,
  HexagonHeatMapChart: hexagonFormatter
};
function setTooltip(baseOpt, iChartOpt, type) {
  function setChartTooltip(opt) {
    var _iChartOpt$tooltip;
    if (!iChartOpt.tipHtml && !(iChartOpt != null && (_iChartOpt$tooltip = iChartOpt.tooltip) != null && _iChartOpt$tooltip.formatter)) {
      opt.formatter = heatMapTooltipFormatter[type];
    }
  }
  baseOpt.tooltip = tooltip(iChartOpt, CHART_TYPE.HEAT_MAP, setChartTooltip);
}
function handleCalendarYaxis(yAxis2, data) {
  yAxis2.type = "category";
  yAxis2.data = data[1];
  yAxis2.splitLine.show = false;
  yAxis2.axisLine.show = true;
}
function handleHexagonYaxis(yAxis2, data) {
  yAxis2.splitLine.show = false;
  yAxis2.axisLabel.show = false;
  yAxis2.min = 0;
  yAxis2.max = data[2];
}
var yAxisHandler = {
  CalendarHeatMapChart: handleCalendarYaxis,
  HexagonHeatMapChart: handleHexagonYaxis
};
function setYaxis(baseOpt, iChartOpt, chartType, data) {
  function setChartYaxis(opt, index) {
    if (chartType !== CHARTTYPE[0] && index === 0) {
      yAxisHandler[chartType](opt, data);
    }
  }
  baseOpt.yAxis = yAxis(baseOpt, iChartOpt, CHART_TYPE.HEAT_MAP, setChartYaxis);
}
function handleRectangularXaxis(xAxis2) {
  xAxis2.type = "value";
  xAxis2.axisLine.show = false;
}
function handleCalendarXaxis(xAxis2, data) {
  xAxis2.data = data[0];
  xAxis2.axisTick.show = false;
  xAxis2.axisLabel.padding = [0, 0, 0, 10 * data[0][0].length / 2];
}
function handleHexagonXaxis(xAxis2, data) {
  xAxis2.type = "value";
  xAxis2.axisLine.show = false;
  xAxis2.axisTick.show = false;
  xAxis2.axisLabel.show = false;
  xAxis2.min = 0;
  xAxis2.max = data[1];
}
var xAxisHandler = {
  RectangularHeatMapChart: handleRectangularXaxis,
  CalendarHeatMapChart: handleCalendarXaxis,
  HexagonHeatMapChart: handleHexagonXaxis
};
function setXaxis(baseOpt, iChartOpt, chartType, data) {
  function setChartXaxis(opt, index) {
    if (index === 0) xAxisHandler[chartType](opt, data);
  }
  baseOpt.xAxis = xAxis(iChartOpt, setChartXaxis);
}
function setGrid(baseOpt, iChartOpt) {
  var basicGrid = grid(iChartOpt);
  baseOpt.grid = basicGrid;
}
function initRectSys(baseOpt, iChartOpt, chartType, data) {
  baseOpt.color = iChartOpt.color;
  setGrid(baseOpt, iChartOpt);
  setXaxis(baseOpt, iChartOpt, chartType, data);
  setYaxis(baseOpt, iChartOpt, chartType, data);
  setTooltip(baseOpt, iChartOpt, chartType);
}
export {
  initRectSys,
  setHeatMapDeaultIchartOption
};
