import { SERIESUNIT, CHARTTYPE } from "./BaseOption.mjs";
import cloneDeep from "../../util/cloneDeep.mjs";
import chartToken from "./chartToken.mjs";
function handleRenderItem(params, api) {
  var center = api.coord([api.value(0), api.value(1)]);
  var points = [];
  var viewRadius = api.size([api.value(3), api.value(3)]);
  var angle = Math.PI / 6;
  for (var i = 0; i < 6; i++, angle += Math.PI / 3) {
    points.push([center[0] + viewRadius[0] * Math.cos(angle), center[1] + viewRadius[0] * Math.sin(angle)]);
  }
  return {
    type: "group",
    children: [{
      type: "polygon",
      shape: {
        points
      },
      style: {
        stroke: chartToken.hexagonStrokeColor,
        fill: api.visual("color"),
        lineWidth: 1
      }
    }]
  };
}
function handleRectangularSeries(seriesUnit, iChartOpt, data) {
  seriesUnit.symbolSize = iChartOpt.rectangleSize || 8;
  seriesUnit.data = data;
}
function handleCalendarSeries(seriesUnit, iChartOpt, data) {
  var _iChartOpt$showLabel;
  seriesUnit.data = data[2];
  seriesUnit.itemStyle.borderColor = iChartOpt.borderColor || iChartOpt.color;
  seriesUnit.label.show = (_iChartOpt$showLabel = iChartOpt.showLabel) != null ? _iChartOpt$showLabel : true;
  seriesUnit.label.color = chartToken.labelColor;
  if (iChartOpt.changeProperty && iChartOpt.changeProperty === "color") {
    seriesUnit.itemStyle.borderWidth = 0;
  }
}
function handleHexagonSeries(seriesUnit, data) {
  seriesUnit.data = data[0];
  seriesUnit.renderItem = handleRenderItem;
}
var seriesHandler = {
  RectangularHeatMapChart: handleRectangularSeries,
  CalendarHeatMapChart: handleCalendarSeries,
  HexagonHeatMapChart: handleHexagonSeries
};
function handleSeries(baseOpt, iChartOpt, data, type) {
  var series = [];
  var seriesItem = cloneDeep(SERIESUNIT[type]);
  if (type === CHARTTYPE[2]) {
    seriesHandler[type](seriesItem, data);
  } else {
    seriesHandler[type](seriesItem, iChartOpt, data);
  }
  series.push(seriesItem);
  baseOpt.series = series;
}
export {
  handleSeries
};
