import Theme from "../../feature/token/index.mjs";
import { codeToRGB, codeToHex } from "../../util/color.mjs";
import merge from "../../util/merge.mjs";
import chartToken from "./chartToken.mjs";
function handleData(iChartOption, baseOpt) {
  var dataName = [];
  iChartOption.data.forEach(function(item) {
    dataName.push(item.name);
  });
  baseOpt.xAxis.data = dataName;
  baseOpt.series[0].data = iChartOption.data;
}
function handleColor(iChartOption, baseOpt) {
  baseOpt.series[0].itemStyle = {
    // opacity: iChartOption.opacity,
    normal: {
      color: function color(params) {
        return codeToRGB(codeToHex(iChartOption.color[params.dataIndex % iChartOption.color.length]), iChartOption.opacity || 0.8);
      }
    }
  };
}
function handleText(iChartOption, baseOpt) {
  baseOpt.series[0].label = {
    normal: {
      show: iChartOption.text && iChartOption.text.show === true ? iChartOption.text && iChartOption.text.show : false,
      position: "top",
      textStyle: {
        fontSize: iChartOption.text && iChartOption.text.fontSize ? iChartOption.text && iChartOption.text.fontSize : 12,
        color: iChartOption.text && iChartOption.text.fontColor ? iChartOption.text.fontColor : chartToken.labelColor
      }
    }
  };
}
function handlePadding(iChartOption, baseOpt) {
  baseOpt["grid"] = {
    top: iChartOption.padding[0],
    right: iChartOption.padding[1],
    bottom: iChartOption.padding[2],
    left: iChartOption.padding[3] + 25
  };
}
function handleCoincide(iChartOption, baseOpt) {
  baseOpt.series[0]["barCategoryGap"] = iChartOption.coincide ? iChartOption.coincide : "-100%";
}
function setGradientColor(baseOpt, color, gradientColor, opacity, iChartOption) {
  if (gradientColor) {
    baseOpt.series[0].itemStyle = {
      normal: {
        color: function color2(params) {
          return {
            type: "linear",
            x: 0,
            y: 0,
            x2: 0,
            y2: 1,
            colorStops: [{
              offset: 0,
              color: codeToRGB(codeToHex(gradientColor[params.dataIndex % gradientColor.length]), opacity || 0.8)
            }, {
              offset: 1,
              color: codeToRGB(codeToHex(iChartOption.color[params.dataIndex % iChartOption.color.length]), opacity || 0.8)
            }],
            globalCoord: false
          };
        }
      }
    };
  }
}
var colorWhole = function colorWhole2(baseOpt, item, markLine, colorError, index) {
  baseOpt.series[0].data[index] = {
    value: item,
    itemStyle: {
      color: markLine.color || colorError
    }
  };
};
var colorExcess = function colorExcess2(baseOpt, iChartOpt, item, index, i) {
  var markLine = iChartOpt.markLine, color = iChartOpt.color, opacity = iChartOpt.opacity;
  var colorError = Theme.config.colorState.colorError;
  baseOpt.series[0].data[index] = {
    value: item,
    itemStyle: {
      color: {
        x: 0,
        y: 0,
        x2: 0,
        y2: 1,
        type: "linear",
        colorStops: [{
          offset: 0,
          color: markLine.color || colorError
        }, {
          offset: (item - markLine[i]) / item,
          color: markLine.color || colorError
        }, {
          offset: (item - markLine[i]) / item,
          color: codeToRGB(codeToHex(color[index % color.length]), opacity || 0.8)
        }, {
          offset: 1,
          color: codeToRGB(codeToHex(color[index % color.length]), opacity || 0.8)
        }]
      }
    }
  };
};
var reviseMarkLineData = function reviseMarkLineData2(baseOpt, iChartOpt, i) {
  var markLine = iChartOpt.markLine;
  var colorError = Theme.config.colorState.colorError;
  baseOpt.series[0].data.forEach(function(item, index) {
    if (item > markLine[i] && i === "top") {
      if (markLine.topWholeColor) {
        colorWhole(baseOpt, item, markLine, colorError, index);
      } else {
        colorExcess(baseOpt, iChartOpt, item, index, i);
      }
    } else if (item < markLine[i] && i === "bottom") {
      colorWhole(baseOpt, item, markLine, colorError, index);
    }
  });
};
function handleMarkLine(baseOpt, iChartOpt) {
  var markLine = iChartOpt.markLine;
  if (markLine) {
    for (var i in markLine) {
      if (i === "top" || i === "bottom") {
        baseOpt.series[0].markLine.data.push({
          yAxis: markLine[i]
        });
        reviseMarkLineData(baseOpt, iChartOpt, i);
      }
    }
    merge(baseOpt.series[0].markLine, markLine);
  }
}
export {
  handleCoincide,
  handleColor,
  handleData,
  handleMarkLine,
  handlePadding,
  handleText,
  setGradientColor
};
