import Theme from "../../feature/token/index.mjs";
import cloneDeep from "../../util/cloneDeep.mjs";
import _defendXSS from "../../util/defendXSS.mjs";
import chartToken from "./chartToken.mjs";
function setDashedLineVisualMap(seriesIndex, lineColor, predictIndex) {
  var vm = {
    type: "piecewise",
    seriesIndex,
    dimension: 0,
    show: false,
    pieces: [{
      gte: 0,
      lte: predictIndex,
      color: Theme.config.visualMapPiecesColor
    }, {
      gt: predictIndex,
      color: lineColor
    }]
  };
  return vm;
}
function setToolTip(dataLength, fontColor, selfFormatter) {
  if (selfFormatter) {
    return selfFormatter;
  }
  var tipHtml = function tipHtml2(params) {
    var htmlString = "";
    params.forEach(function(item, index) {
      if (index < dataLength) {
        if (index === 0) {
          htmlString += _defendXSS(item.name) + "<br/>";
        }
        var color_ = item.color.colorStops;
        htmlString += '\n                    <div>\n                        <span style="display:inline-block;width:10px;\n                        height:10px;border-radius:5px;background-color:' + _defendXSS(color_ ? color_[0].color : item.color) + ';">\n                        </span>\n                        <span style="margin-left:5px;color:' + _defendXSS(fontColor) + '">\n                            <span style="display:inline-block;width:80px;">' + _defendXSS(item.seriesName) + '</span> \n                            <span style="font-weight:bold">' + _defendXSS(item.value) + "</span>\n                        </span>\n                    </div>\n                ";
      }
    });
    return htmlString;
  };
  return tipHtml;
}
function handlePredict(option, predict, tipHtml, lineStyle) {
  if (!predict) return;
  var dashColor = Theme.config.visualMapDashColor;
  if (lineStyle && lineStyle.dashColor) {
    dashColor = lineStyle.dashColor;
  }
  var data = option.series;
  var dataLength = data.length;
  option.xAxis[0].data.length;
  var predictIndex = option.xAxis[0].data.indexOf(predict);
  if (predictIndex > -1) {
    for (var index = 0; index < dataLength; index++) {
      var temp = cloneDeep(data[index]);
      temp.lineStyle = {
        // 为了视觉上看不见盖住粗细+1
        width: chartToken.lineWidth + 1,
        type: [5, 8]
      };
      temp.itemStyle = {
        opacity: 0
      };
      temp.silent = true;
      temp.showSymbol = false;
      temp.showAllSymbol = false;
      option.series.push(temp);
      option.visualMap.push(setDashedLineVisualMap(dataLength + index, dashColor, predictIndex));
    }
  }
  option.tooltip.formatter = setToolTip(dataLength, Theme.config.tooltipTextColor, tipHtml);
}
export {
  handlePredict
};
