import cloneDeep from "../../util/cloneDeep.mjs";
import { isObject, isString } from "../../util/type.mjs";
import { getMarkLineDefault, getMarkPointDefault, setThresholdMarkLineLabel } from "../../option/config/mark/index.mjs";
import chartToken from "./chartToken.mjs";
import Theme from "../../feature/token/index.mjs";
function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t = arguments[e];
      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
var seriesInit = function seriesInit2() {
  return {
    label: {
      show: false
    },
    // 连线上的小圆点样式
    symbol: "emptyCircle",
    symbolSize: chartToken.symbolSize,
    showSymbol: false,
    // 数据
    data: [],
    // 线形
    type: "line",
    // 线条样式
    lineStyle: {
      width: chartToken.lineWidth
    },
    massive: false,
    // 折线阶梯
    step: false,
    // 折线平滑
    smooth: false,
    // 阈值线
    markLine: null,
    // 峰值标志
    markPoint: null,
    // 折线点的每个样式配置项
    itemStyle: {}
  };
};
function handleItemStyleWithTheme(seriesUnit) {
  var theme = Theme.themeName;
  if (theme.includes("dark")) {
    seriesUnit.symbol = "circle";
    seriesUnit.itemStyle = {
      shadowBlur: 2,
      borderWidth: chartToken.border,
      borderColor: chartToken.borderColor,
      shadowColor: "rgba(0, 0, 0, .2)"
    };
  }
}
function handleItemStyle(seriesUnit, itemStyle) {
  if (!itemStyle) return;
  if (itemStyle.symbolSize) {
    seriesUnit.symbolSize = itemStyle.symbolSize;
  }
  for (var key in itemStyle) {
    if (Object.hasOwnProperty.call(itemStyle, key)) {
      seriesUnit.itemStyle[key] = itemStyle[key];
    }
  }
}
function isTopOrBottom(markLine, seriesUnit, flag) {
  var position = "";
  var markLinePosition = "";
  var markLineLabel = "";
  var markLineColor = "";
  var markLinelLineStyle;
  var markLineData = {};
  if (flag === "top") {
    position = markLine.top;
    markLinePosition = markLine.topPosition;
    markLineLabel = markLine.topLabel;
    markLineColor = markLine.topColor;
    markLinelLineStyle = markLine.topLine;
  } else {
    position = markLine.bottom;
    markLinePosition = markLine.bottomPosition;
    markLineLabel = markLine.bottomLabel;
    markLineColor = markLine.bottomColor;
    markLinelLineStyle = markLine.bottomLine;
  }
  if (isString(position)) {
    markLineData = {
      type: position
    };
  } else {
    markLineData = {
      yAxis: position
    };
  }
  markLineData.label = {
    show: false,
    position: "insideEndTop"
  };
  markLineData.lineStyle = {};
  if (!markLineColor) {
    setThresholdMarkLineLabel(markLineData);
  }
  markLinePosition && (markLineData.label.position = markLinePosition);
  if (markLineLabel) {
    markLineData.label.show = true;
    markLineData.label.color = "inherit";
    markLineData.label.formatter = markLineLabel;
  }
  if (markLineColor === "auto") {
    markLineColor = void 0;
  }
  if (markLineColor) {
    markLineData.label.color = markLineColor;
    markLineData.lineStyle.color = markLineColor;
  }
  if (markLinelLineStyle === false) {
    markLineData.lineStyle.color = chartToken.color;
  }
  seriesUnit.markLine.data.push(markLineData);
}
function handleMarkLine(markLine, seriesUnit, seriesName) {
  seriesUnit.markLine = getMarkLineDefault();
  if (markLine.top && !(markLine.topUse && markLine.topUse.indexOf(seriesName) === -1)) {
    isTopOrBottom(markLine, seriesUnit, "top");
  }
  if (markLine.bottom && !(markLine.bottomUse && markLine.bottomUse.indexOf(seriesName) === -1)) {
    isTopOrBottom(markLine, seriesUnit, "bottom");
  }
}
function handleMarkPoint(markPoint, seriesUnit, seriesName) {
  seriesUnit.markPoint = getMarkPointDefault();
  var colorError = Theme.config.colorState.colorError;
  if (markPoint.max && !(markPoint.maxUse && markPoint.maxUse.indexOf(seriesName) === -1)) {
    var max = {
      type: "max",
      symbolOffset: [0, -11],
      itemStyle: {
        color: markPoint.maxColor || colorError
      }
    };
    if (markPoint.maxColor == "auto") delete max.itemStyle;
    seriesUnit.markPoint.data.push(max);
  }
  if (markPoint.min && !(markPoint.minUse && markPoint.minUse.indexOf(seriesName) === -1)) {
    var min = {
      type: "min",
      symbolOffset: [0, 11],
      symbolRotate: 180,
      itemStyle: {
        color: markPoint.minColor || colorError
      }
    };
    if (markPoint.minColor == "auto") delete min.itemStyle;
    seriesUnit.markPoint.data.push(min);
  }
}
function setStack(stack, seriesUnit) {
  for (var name in stack) {
    if (Object.hasOwnProperty.call(stack, name)) {
      var stackArray = stack[name];
      var seriesName = seriesUnit.name;
      if (stackArray.indexOf(seriesName) !== -1) {
        seriesUnit.stack = name;
      }
      break;
    }
  }
}
function handleStack(stack, seriesUnit) {
  if (stack === true) {
    seriesUnit.stack = "Total";
    return;
  }
  if (isObject(stack)) {
    setStack(stack, seriesUnit);
  }
}
function handleFocus(focus, seriesUnit) {
  if (focus) {
    seriesUnit.emphasis = {
      focus: "series",
      blurScope: "global"
    };
  }
}
function handleSeries(params) {
  var stack = params.stack, focus = params.focus, massive = params.massive, isStep = params.isStep, series = params.series, isSmooth = params.isSmooth, markLine = params.markLine, markPoint = params.markPoint, legendData = params.legendData, seriesData = params.seriesData, localSeriesUnit = params.localSeriesUnit;
  legendData.forEach(function(legend, index) {
    var seriesUnit = cloneDeep(localSeriesUnit);
    seriesUnit.smooth = isSmooth || false;
    seriesUnit.large = massive || false;
    seriesUnit.sampling = massive ? "lttb" : false;
    seriesUnit.step = isStep && "middle" || false;
    markLine && handleMarkLine(markLine, seriesUnit, legend);
    markPoint && handleMarkPoint(markPoint, seriesUnit, legend);
    focus && handleFocus(focus, seriesUnit);
    seriesUnit.name = legend;
    seriesUnit.data = seriesData[legend];
    stack && handleStack(stack, seriesUnit);
    series.push(seriesUnit);
  });
}
function handleYaxis(series, yAxis) {
  if (Array.isArray(yAxis)) {
    yAxis.forEach(function(y, index) {
      series.forEach(function(s, indexs) {
        if (y.dataName && y.dataName.includes(s.name)) {
          series[indexs].yAxisIndex = index;
        }
      });
    });
  }
}
function setSeries(params) {
  var yAxis = params.yAxis, itemStyle = params.itemStyle;
  var localSeriesUnit = cloneDeep(seriesInit());
  handleItemStyleWithTheme(localSeriesUnit);
  handleItemStyle(localSeriesUnit, itemStyle);
  var series = [];
  handleSeries(_extends({}, params, {
    series,
    localSeriesUnit
  }));
  handleYaxis(series, yAxis);
  return series;
}
export {
  seriesInit,
  setSeries
};
