import min from "../../util/sort/min.mjs";
import max from "../../util/sort/max.mjs";
import { isNumber } from "../../util/type.mjs";
import { getColor } from "../../util/color.mjs";
import Theme from "../../feature/token/index.mjs";
function handleVisualMapItem(_ref) {
  var index = _ref.index, topColor = _ref.topColor, top = _ref.top, bottom = _ref.bottom, colors = _ref.colors, bottomColor = _ref.bottomColor, vmColor = _ref.vmColor;
  var visualMapItem = {
    show: false,
    type: "piecewise",
    dimension: 1,
    seriesIndex: index,
    pieces: [{
      gte: top,
      // 大于 top 的
      color: topColor
    }, {
      gt: bottom,
      lt: top,
      // 小于 top, 大于 bottom 的，为正常颜色
      color: getColor(colors, index)
    }, {
      lte: bottom,
      // 小于 bottom
      color: bottomColor
    }],
    outOfRange: {
      color: vmColor
    }
  };
  return visualMapItem;
}
function setVisualMap(legendData, seriesData, markLine, colors) {
  var visualMap = [];
  if (markLine) {
    var topValue = markLine.top;
    var bottomValue = markLine.bottom;
    var vmColor = Theme.config.colorState.colorError;
    var topColor = markLine.topColor || vmColor;
    var bottomColor = markLine.bottomColor || vmColor;
    if (!isNumber(topValue)) {
      topValue = void 0;
    }
    if (!isNumber(bottomValue)) {
      bottomValue = void 0;
    }
    if (topValue === void 0 && bottomValue === void 0) {
      return visualMap;
    }
    if (topValue !== void 0 && bottomValue !== void 0 && bottomValue >= topValue) {
      throw new Error("阈值线bottom的值必须小于阈值线top的值");
    }
    legendData.forEach(function(legendName, index) {
      var data = seriesData[legendName];
      var minData = min(data);
      var maxData = max(data);
      var bottom = bottomValue;
      var top = topValue;
      if (markLine.topUse && markLine.topUse.indexOf(legendName) === -1) {
        top = void 0;
      }
      if (markLine.bottomUse && markLine.bottomUse.indexOf(legendName) === -1) {
        bottom = void 0;
      }
      if (top === void 0 && bottom === void 0) {
        return;
      }
      if (bottom === void 0) {
        bottom = Math.min(top - 0.01, minData - 0.01);
      }
      if (top === void 0) {
        top = Math.max(bottom + 0.01, maxData + 0.01);
      }
      var visualMapItem = handleVisualMapItem({
        index,
        topColor,
        top,
        bottom,
        colors,
        bottomColor,
        vmColor
      });
      visualMap.push(visualMapItem);
    });
  }
  return visualMap;
}
export {
  setVisualMap
};
