import init from "../../option/init/index.mjs";
import mini from "../../feature/mini/miniLineChart.mjs";
import { setSeries } from "./handleSeries.mjs";
import cloneDeep from "../../util/cloneDeep.mjs";
import base from "../../option/base/index.mjs";
import { setVisualMap } from "./handleVisualMap.mjs";
import { handlePredict } from "./handlePredict.mjs";
import topArea from "./AreaChart/topArea.mjs";
import bottomArea from "./AreaChart/bottomArea.mjs";
import { mergeSeries, mergeVisualMap } from "../../util/merge.mjs";
import { handleData, onlyOnePoint, discrete } from "./handleOptipn.mjs";
import RectCoordSys from "../../option/RectSys/index.mjs";
import { lttb } from "../../feature/performance/lttb.mjs";
import { CHART_TYPE } from "../../util/constants.mjs";
import xkey from "../../option/config/xAxis/xkey.mjs";
import xdata from "../../option/config/xAxis/xdata.mjs";
import ldata from "../../option/config/legend/ldata.mjs";
import ydata from "../../option/config/yAxis/ydata.mjs";
function _defineProperty(e, r, t) {
  return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: true, configurable: true, writable: true }) : e[r] = t, e;
}
function _toPropertyKey(t) {
  var i = _toPrimitive(t, "string");
  return "symbol" == typeof i ? i : i + "";
}
function _toPrimitive(t, r) {
  if ("object" != typeof t || !t) return t;
  var e = t[Symbol.toPrimitive];
  if (void 0 !== e) {
    var i = e.call(t, r);
    if ("object" != typeof i) return i;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return ("string" === r ? String : Number)(t);
}
var LineChart = /* @__PURE__ */ function() {
  function LineChart2(iChartOption, chartInstance) {
    this.baseOption = {};
    this.baseOption = cloneDeep(base);
    this.iChartOption = {};
    this.iChartOption = init(iChartOption);
    this.updateOption(chartInstance);
  }
  var _proto = LineChart2.prototype;
  _proto.updateOption = function updateOption(chartInstance) {
    var iChartOption = this.iChartOption;
    RectCoordSys(this.baseOption, this.iChartOption, CHART_TYPE.LINE);
    var xAxisKey = xkey(iChartOption);
    var data = iChartOption.massive ? lttb(iChartOption.data) : iChartOption.data;
    var xAxisData = xdata(data, xAxisKey);
    var legendData = ldata(data, xAxisKey);
    var seriesData = ydata(data, legendData);
    handleData(this.baseOption, legendData, xAxisData);
    this.baseOption.series = setSeries({
      seriesData,
      legendData,
      yAxis: iChartOption.yAxis,
      focus: iChartOption.focus,
      stack: iChartOption.stack,
      isStep: iChartOption.step,
      isArea: iChartOption.area,
      colors: this.baseOption.color,
      isSmooth: iChartOption.smooth,
      markLine: iChartOption.markLine,
      markPoint: iChartOption.markPoint,
      splitLine: iChartOption.splitLine,
      labelHtml: iChartOption.labelHtml,
      itemStyle: iChartOption.itemStyle,
      massive: iChartOption.massive
    });
    this.baseOption.visualMap = setVisualMap(legendData, seriesData, iChartOption.markLine, iChartOption.color);
    handlePredict(this.baseOption, iChartOption.predict, iChartOption.tipHtml, iChartOption.lineStyle);
    if (iChartOption.silent) {
      this.baseOption.tooltip = {};
    }
    onlyOnePoint(this.baseOption);
    discrete(iChartOption, this.baseOption);
    mergeSeries(iChartOption, this.baseOption);
    mergeVisualMap(iChartOption, this.baseOption);
    mini(iChartOption, this.baseOption);
  };
  _proto.updateOptionAgain = function updateOptionAgain(echartsIns) {
    var YAxiMax = this.getYAxisMaxValue(echartsIns, 0);
    var YAxiMin = this.getYAxisMinValue(echartsIns, 0);
    topArea(this.baseOption, this.iChartOption, YAxiMin);
    bottomArea(this.baseOption, this.iChartOption, YAxiMax);
    mergeSeries(this.iChartOption, this.baseOption);
  };
  _proto.getOption = function getOption() {
    return this.baseOption;
  };
  _proto.getYAxisMaxValue = function getYAxisMaxValue(echartsIns, index) {
    var _echartsIns$getModel$;
    return (_echartsIns$getModel$ = echartsIns.getModel().getComponent("yAxis", index)) == null ? void 0 : _echartsIns$getModel$.axis.scale._extent[1];
  };
  _proto.getYAxisMinValue = function getYAxisMinValue(echartsIns, index) {
    var _echartsIns$getModel$2;
    return (_echartsIns$getModel$2 = echartsIns.getModel().getComponent("yAxis", index)) == null ? void 0 : _echartsIns$getModel$2.axis.scale._extent[0];
  };
  return LineChart2;
}();
_defineProperty(LineChart, "name", CHART_TYPE.LINE);
export {
  LineChart as default
};
