import { codeToRGB, getColor, changeRgbaOpacity } from "../../util/color.mjs";
import cloneDeep from "../../util/cloneDeep.mjs";
function handleResObj(item, colors, index) {
  var resObj = cloneDeep(item);
  resObj.color = {
    rgba: codeToRGB(getColor(colors, index), 1),
    from: 1,
    to: 0.2
  };
  return resObj;
}
function handleTempLegend(tempLegend, legendOffset, index) {
  if (tempLegend.orient === "horizontal") {
    var offset = legendOffset || 30;
    var bottom = parseFloat(tempLegend.bottom) - index * parseFloat(offset);
    if (tempLegend.bottom.toString().indexOf("%") !== -1) {
      tempLegend.bottom = bottom + "%";
    } else {
      tempLegend.bottom = bottom;
    }
  } else {
    var _offset = legendOffset || 120;
    var left = parseFloat(tempLegend.left) + index * parseFloat(_offset);
    if (tempLegend.left.toString().indexOf("%") !== -1) {
      tempLegend.left = left + "%";
    } else {
      tempLegend.left = left;
    }
  }
}
function handleTempSeriesObj(item) {
  var tempSeriesObj = cloneDeep(item);
  tempSeriesObj.itemStyle = {
    color: item.color.rgba
  };
  return tempSeriesObj;
}
function installInnerData(data, innerData, innerIndex) {
  data.children && data.children.forEach(function(citem, cindex) {
    if (!innerData[innerIndex]) {
      innerData[innerIndex] = [];
    }
    var colorFrom = data.color.from - (data.color.from - data.color.to) / (data.children.length + 1) * (cindex + 1);
    var colorTo = data.color.from - (data.color.from - data.color.to) / (data.children.length + 1) * (cindex + 2);
    citem.color = {
      rgba: changeRgbaOpacity(data.color.rgba, colorFrom),
      from: colorFrom,
      to: colorTo
    };
    innerData[innerIndex].push(citem);
    if (citem.children) {
      installInnerData(citem, innerData, innerIndex + 1);
    }
  });
}
function handleMulti(type, baseOption, legend, data) {
  if (type === "multi-circle") {
    var colors = baseOption.color;
    var outer = data.map(function(item, index) {
      var resObj = handleResObj(item, colors, index);
      return resObj;
    });
    var inner = [];
    var innerIndex = 0;
    outer.forEach(function(data2) {
      installInnerData(data2, inner, innerIndex);
    });
    inner.forEach(function(innerData, innerIndex2) {
      var tempSeries = cloneDeep(baseOption.series[0]);
      tempSeries.data = innerData.map(function(item) {
        var tempSeriesObj = handleTempSeriesObj(item);
        return tempSeriesObj;
      });
      var innerDiff = parseFloat(tempSeries.radius[1]) - parseFloat(tempSeries.radius[0]);
      tempSeries.radius = tempSeries.radius.map(function(item) {
        return parseFloat(item) + innerDiff * (innerIndex2 + 1) + "%";
      });
      baseOption.series.push(tempSeries);
    });
    baseOption.series.forEach(function(i) {
      i.label = {
        show: false
      };
      i.labelLine = {
        show: false
      };
    });
    var dataArray = [outer].concat(inner);
    var originLegend = baseOption.legend;
    var legendOffset = legend.offset;
    baseOption.legend = [];
    dataArray.forEach(function(array, index) {
      var tempLegend = cloneDeep(originLegend);
      var tempLegendData = array.map(function(item) {
        return item.name;
      });
      tempLegend.data = tempLegendData;
      handleTempLegend(tempLegend, legendOffset, index);
      baseOption.legend.push(tempLegend);
    });
  }
}
export {
  handleMulti as default
};
