import merge from "../../util/merge.mjs";
import setLabel from "./handleLabel.mjs";
import chartToken from "./chartToken.mjs";
import cloneDeep from "../../util/cloneDeep.mjs";
import { percentToDecimal } from "../../util/math.mjs";
import { isArray, isNumber, isString } from "../../util/type.mjs";
var seriesInit = function seriesInit2() {
  return {
    type: "pie",
    roundCap: true,
    radius: ["0%", "50%"],
    center: ["50%", "45%"],
    avoidLabelOverlap: true,
    itemStyle: {
      borderWidth: chartToken.borderWidth,
      borderColor: chartToken.borderColor,
      borderRadius: chartToken.borderRadius
    },
    selectedMode: false,
    roseType: false,
    label: {},
    labelLine: {},
    data: []
  };
};
function setPieRadius(pieType, radius) {
  if (radius) {
    return radius;
  } else {
    var _radius = [];
    switch (pieType) {
      case "pie":
        _radius = ["0%", "50%"];
        break;
      case "circle":
        _radius = ["44%", "50%"];
        break;
      case "multi-circle":
        _radius = ["44%", "50%"];
        break;
      default:
        _radius = ["0%", "50%"];
        break;
    }
    return _radius;
  }
}
function handleEmptyData(data, series, center, radius, stillShowZeroSum, legend, colorArr) {
  var total = data.reduce(function(pre, cur) {
    pre = pre + cur.value;
    return pre;
  }, 0);
  if (total === 0) {
    if (stillShowZeroSum === false) {
      series.forEach(function(item) {
        item.label = false;
        item.itemStyle.borderWidth = chartToken.borderWidthShowZero;
      });
      series.push({
        type: "pie",
        radius,
        center,
        emptyCircleStyle: {
          color: chartToken.colorShowZero
        },
        silent: true,
        animation: false
      });
    } else {
      legend.data !== void 0 && legend.data.forEach(function(item, index) {
        item.itemStyle = {
          color: colorArr[index]
        };
      });
      series.forEach(function(item) {
        item.animation = false;
        item.color = chartToken.colorShowZero;
      });
    }
  }
}
function getNewRadius(radius, chartInstance) {
  if (isNumber(radius)) {
    return radius;
  } else if (isString(radius)) {
    return radius.endsWith("%") ? percentToDecimal(radius) * (Math.min(chartInstance.getWidth(), chartInstance.getHeight()) / 2) : parseFloat(radius);
  } else if (isArray(radius)) {
    var decimalRadiusArr = radius.map(function(r) {
      return isNumber(r) ? r / Math.min(chartInstance.getWidth(), chartInstance.getHeight()) : r.endsWith("%") ? percentToDecimal(r) : parseFloat(r);
    });
    var decimalRadius = decimalRadiusArr[0] === 0 ? decimalRadiusArr[1] : decimalRadiusArr[0];
    return decimalRadius * (Math.min(chartInstance.getWidth(), chartInstance.getHeight()) / 2);
  }
}
function minAngle(radius, chartInstance) {
  var newRadius = getNewRadius(radius, chartInstance);
  var cosA = (Math.pow(newRadius, 2) + Math.pow(newRadius, 2) - Math.pow(3, 2)) / (2 * newRadius * newRadius);
  var angleAInRadians = Math.acos(cosA);
  var minAngle2 = angleAInRadians * (180 / Math.PI);
  return minAngle2;
}
function mergeDefaultSeries(seriesUnit) {
  for (var key in seriesInit()) {
    if (Object.hasOwnProperty.call(seriesInit(), key)) {
      if (key === "itemStyle") {
        var series = cloneDeep(seriesInit());
        seriesUnit[key] = merge(series.itemStyle, seriesUnit.itemStyle);
      }
      if (seriesUnit[key] === void 0) {
        seriesUnit[key] = seriesInit()[key];
      }
    }
  }
}
function setColor(iChartOption) {
  var data = iChartOption.data;
  var initColorGroup = iChartOption.initColor.concat();
  var colorData = iChartOption.dataRules && iChartOption.dataRules.color;
  if (colorData && isArray(initColorGroup)) {
    var _loop = function _loop2(key2) {
      data.forEach(function(item, index) {
        if (item.name === key2 && initColorGroup[index] && colorData[key2]) {
          initColorGroup[index] = colorData[key2];
        }
      });
    };
    for (var key in colorData) {
      _loop(key);
    }
  }
  iChartOption.color = initColorGroup;
}
var config = ["label", "labelLine", "itemStyle", "radius", "center", "silent", "emphasis", "stillShowZeroSum", "selectedMode", "roseType", "minAngle"];
function handleSeries(pieType, iChartOption, chartInstance, position, legend) {
  var _position, _position2;
  var data = iChartOption.data, stillShowZeroSum = iChartOption.stillShowZeroSum;
  position = position || {};
  iChartOption.center = (_position = position) == null ? void 0 : _position.center;
  iChartOption.radius = (_position2 = position) == null ? void 0 : _position2.radius;
  var series = [];
  var selfSeries = iChartOption.series;
  if (selfSeries === void 0) {
    selfSeries = [{}];
  }
  iChartOption.initColor = iChartOption.color;
  setColor(iChartOption);
  selfSeries.forEach(function(seriesItem) {
    var seriesUnit = seriesItem;
    var temp = cloneDeep(iChartOption);
    config.forEach(function(name) {
      var existValue = merge(temp[name], seriesUnit[name]);
      if (existValue !== void 0) {
        seriesUnit[name] = existValue;
      }
    });
    seriesUnit.data = seriesUnit.data || iChartOption.data;
    seriesUnit.radius = setPieRadius(pieType, seriesUnit.radius);
    seriesUnit.minAngle = seriesUnit.minAngle !== void 0 ? seriesUnit.minAngle : minAngle(seriesUnit.radius, chartInstance);
    setLabel(seriesUnit, seriesUnit.label, seriesUnit.data);
    mergeDefaultSeries(seriesUnit);
  });
  handleEmptyData(data, selfSeries, selfSeries[0].center, selfSeries[0].radius, stillShowZeroSum, legend, iChartOption.color);
  series = selfSeries;
  return series;
}
export {
  handleSeries as default,
  seriesInit
};
