import { CHARTTYPENAME } from "./BaseOption.mjs";
import max from "../../util/sort/max.mjs";
function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t = arguments[e];
      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
function handleLimitVal(data, iChartOpt, demarcatedValue) {
  var dataItem = _extends({}, data, {
    _initValue: void 0
  });
  var minWidth = iChartOpt.minWidth;
  if (minWidth) {
    var limitValue = demarcatedValue * (parseInt(minWidth) / 100);
    if (data.value && data.value < limitValue) {
      dataItem.value = limitValue;
      dataItem._initValue = data.value;
    }
  }
  return dataItem;
}
function handleDoubleSideData(dataSet, data, iChartOpt, demarcatedValue) {
  data.forEach(function(item) {
    dataSet.seriesName.push(item.name);
    var childBarName = [];
    var childBarData = [];
    if (item.children && item.children.length !== 0) {
      item.children.forEach(function(el) {
        childBarName.push(el.name);
        var innerDataItem = handleLimitVal(el, iChartOpt, demarcatedValue);
        childBarData.push(innerDataItem);
      });
      dataSet.barName.push(childBarName);
      dataSet.barData.push(childBarData);
    }
  });
}
function handleBasicProcessData(iChartOpt, dataSet, data, doubleSide, dataMax) {
  var demarcatedValue = dataMax || 100;
  dataSet.maxValue = demarcatedValue;
  if (doubleSide) {
    handleDoubleSideData(dataSet, data, iChartOpt, demarcatedValue);
    return;
  }
  data.forEach(function(dataItem) {
    dataSet.barName.push(dataItem.name);
    var innerDataItem = handleLimitVal(dataItem, iChartOpt, demarcatedValue);
    dataSet.barData.push(innerDataItem);
  });
}
function getSerirsDataMax(dataSet, data, dataMax) {
  var sumValueMax;
  var sumValue = [];
  data.forEach(function(el) {
    var _sumValue = 0;
    el.children.forEach(function(child) {
      if (child.value) {
        _sumValue += child.value;
      }
    });
    sumValue.push(_sumValue);
  });
  sumValueMax = max(sumValue);
  if (dataMax && dataMax > sumValueMax) {
    sumValueMax = dataMax;
  }
  dataSet.maxValue = sumValueMax;
  dataSet.sumValue = sumValue;
}
function setMaxValueAgain(dataSet, dataMax) {
  var sum = [];
  var len = dataSet.seriesName.length;
  var _loop = function _loop2(i2) {
    var sumValue = 0;
    dataSet.barData.forEach(function(item) {
      if (item[i2].value) {
        sumValue += item[i2].value;
      }
    });
    sum.push(sumValue);
  };
  for (var i = 0; i < len; i++) {
    _loop(i);
  }
  var maxValue = max(sum);
  if (dataMax && dataMax > maxValue) {
    maxValue = dataMax;
  }
  dataSet.maxValue = maxValue;
}
function handleStackProcessData(iChartOpt, dataSet, data, dataMax) {
  data.forEach(function(element) {
    dataSet.seriesName.push(element.name);
  });
  data[0].children.forEach(function(el) {
    dataSet.barName.push(el.type);
  });
  getSerirsDataMax(dataSet, data, dataMax);
  dataSet.barName.forEach(function(type) {
    var seriesData = [];
    data.forEach(function(child) {
      var childItem = child.children.find(function(el) {
        return el.type === type;
      });
      var innerChidItem = handleLimitVal(childItem, iChartOpt, dataSet.maxValue);
      seriesData.push(innerChidItem);
    });
    dataSet.barData.push(seriesData);
  });
  if (iChartOpt.minWidth) {
    setMaxValueAgain(dataSet, dataMax);
  }
}
function handleData(iChartOpt, doubleSide) {
  var data = iChartOpt.data, name = iChartOpt.name;
  if (!data) return null;
  if (data && data.length > 0) {
    var dataSet = {
      // 图表子项数据
      barData: [],
      // 图表的标题数据
      barName: [],
      // 图表的最大值
      maxValue: void 0,
      // 图表的系列名称
      seriesName: []
    };
    var dataMax = iChartOpt.calibrationValue || iChartOpt.max;
    if (name === CHARTTYPENAME.ProcessBarChart) {
      handleBasicProcessData(iChartOpt, dataSet, data, doubleSide, dataMax);
    } else {
      handleStackProcessData(iChartOpt, dataSet, data, dataMax);
    }
    return dataSet;
  }
}
export {
  handleData as default
};
