import { renderSvgDom, getSmoothPathD, getPathD } from "./util.mjs";
var Belt = /* @__PURE__ */ function() {
  function Belt2(node, config, isLeft) {
    this.name = void 0;
    this.start = void 0;
    this.end = void 0;
    this.isLeft = void 0;
    this.颜色 = void 0;
    this.color = void 0;
    this.hasNodes = void 0;
    this.config = void 0;
    this.belongNode = void 0;
    this.belongNode = node;
    this.config = config;
    this.isLeft = isLeft;
    this.hasNodes = !!(node.initalNode.nodes && node.initalNode.nodes.length > 0);
  }
  var _proto = Belt2.prototype;
  _proto.init = function init(name, start, end) {
    this.setName(name);
    this.setStart(start);
    this.setEnd(end);
  };
  _proto.setName = function setName(name) {
    this.name = name;
  };
  _proto.setStart = function setStart(start) {
    this.start = start;
  };
  _proto.setEnd = function setEnd(end) {
    this.end = end;
  };
  _proto.setCommonColor = function setCommonColor() {
    var _this$config = this.config, statusColor = _this$config.statusColor, background = _this$config.background;
    var color = background;
    if (this.hasNodes) {
      var len = this.belongNode.initalNode.nodes.length;
      var itemNode = this.isLeft ? this.belongNode.initalNode.nodes[0] : this.belongNode.initalNode.nodes[len - 1];
      color = statusColor[itemNode.status];
    }
    this.color = color;
  };
  _proto.setGradientColor = function setGradientColor(parentNode) {
    var _this$config2 = this.config, statusColor = _this$config2.statusColor, background = _this$config2.background;
    var colorList = [];
    var initalNode = this.belongNode.initalNode;
    var startColor = background, endColor = background;
    if (initalNode.nodes && initalNode.nodes.length !== 0) {
      var itemNode = initalNode.nodes[0];
      endColor = statusColor[itemNode.status];
    }
    var parentInitalNode = parentNode.initalNode;
    if (parentInitalNode.nodes && parentInitalNode.nodes.length !== 0) {
      var len = parentInitalNode.nodes.length;
      var _itemNode = parentInitalNode.nodes[len - 1];
      startColor = statusColor[_itemNode.status];
    }
    var start = {
      percent: "0%",
      color: startColor
    };
    var end = {
      percent: "100%",
      color: endColor
    };
    colorList.push(start);
    colorList.push(end);
    this.color = colorList;
  };
  _proto.getBeltPoint = function getBeltPoint(startLen) {
    var endLen = this.end.value;
    var point;
    if (this.config.smoothingCurves) {
      point = {
        // 连线的起点
        x1: this.start.x,
        y1: this.start.y + startLen / 2,
        // 连线的终点
        x3: this.end.x,
        y3: this.end.y + endLen / 2
      };
    } else {
      point = {
        // 前置节点起点坐标
        x1: this.start.x,
        y1: this.start.y,
        // 前置节点终点坐标
        x2: this.start.x,
        y2: this.start.y + startLen,
        // 后置节点终点坐标
        x3: this.end.x,
        y3: this.end.y,
        // 后置节点终点坐标
        x4: this.end.x,
        y4: this.end.y + endLen
      };
    }
    return point;
  };
  _proto.getLinearGradientfillColor = function getLinearGradientfillColor(svgBeltGradientDefs) {
    var lineGradientAttributes = {
      id: "linearGradient_" + this.name + "_node",
      x2: "100%",
      y2: "0%",
      x1: "0%",
      y1: "0%"
    };
    var lineGradientDom = renderSvgDom("linearGradient", lineGradientAttributes);
    this.color.forEach(function(el) {
      var stopAttributes = {
        offset: el.percent,
        "stop-opacity": 0.2,
        "stop-color": el.color
      };
      var stopDom = renderSvgDom("stop", stopAttributes);
      lineGradientDom.appendChild(stopDom);
    });
    svgBeltGradientDefs.appendChild(lineGradientDom);
    return "url(#linearGradient_" + this.name + "_node)";
  };
  _proto.render = function render(svgBeltPathGroup, svgBeltGradientDefs) {
    var startLen = this.start.value;
    var smoothingCurves = this.config.smoothingCurves;
    var point = this.getBeltPoint(startLen);
    var path = smoothingCurves ? getSmoothPathD(point, startLen) : getPathD(point);
    var isNotgarident = typeof this.color === "string";
    var fillColor = this.color;
    if (!isNotgarident) {
      fillColor = this.getLinearGradientfillColor(svgBeltGradientDefs);
    }
    var pathAttributes = {
      id: "link_" + this.name + "_" + (this.isLeft ? "left" : "right"),
      fill: fillColor,
      d: path,
      opacity: isNotgarident ? 0.2 : 1
    };
    if (smoothingCurves && !(point.y1 === point.y3)) {
      pathAttributes["fill"] = "none";
      pathAttributes["stroke"] = fillColor;
      pathAttributes["stroke-width"] = startLen;
    }
    var pathDom = renderSvgDom("path", pathAttributes);
    svgBeltPathGroup.appendChild(pathDom);
  };
  return Belt2;
}();
export {
  Belt as default
};
