import Node from "./Node.mjs";
import { renderSvgDom } from "./util.mjs";
var Manager = /* @__PURE__ */ function() {
  function Manager2(svgContainer, nodeContainer, config, initalData) {
    this.svgContainer = void 0;
    this.nodeContainer = void 0;
    this.config = void 0;
    this.initalData = void 0;
    this.nodes = void 0;
    this.belts = void 0;
    this.legal = true;
    this.svgNodeGroup = void 0;
    this.svgNodeGradientDefs = void 0;
    this.svgBeltPathGroup = void 0;
    this.svgBeltGradientDefs = void 0;
    this.svgContainer = svgContainer;
    this.nodeContainer = nodeContainer;
    this.config = config;
    this.initalData = initalData;
    this.nodes = [];
    this.belts = [];
    this.checkInitalData();
    this.initSvgGroupContainer();
  }
  var _proto = Manager2.prototype;
  _proto.initSvgGroupContainer = function initSvgGroupContainer() {
    if (!this.legal) return;
    var nodeGroup = renderSvgDom("g", {
      id: "rc_nodeGroup"
    });
    var pathGroup = renderSvgDom("g", {
      id: "rc_pathGroup"
    });
    var nodeGradientGroup = renderSvgDom("g", {
      id: "rc_nodeGradientGroup"
    });
    var linkGradientGroup = renderSvgDom("g", {
      id: "rc_linkGradientGroup"
    });
    var nodeGradientDefs = renderSvgDom("defs");
    var linkGradientDefs = renderSvgDom("defs");
    nodeGradientGroup.appendChild(nodeGradientDefs);
    linkGradientGroup.appendChild(linkGradientDefs);
    this.svgContainer.appendChild(nodeGroup);
    this.svgContainer.appendChild(pathGroup);
    this.svgContainer.appendChild(nodeGradientGroup);
    this.svgContainer.appendChild(linkGradientGroup);
    this.svgNodeGroup = nodeGroup;
    this.svgNodeGradientDefs = nodeGradientDefs;
    this.svgBeltPathGroup = pathGroup;
    this.svgBeltGradientDefs = linkGradientDefs;
  };
  _proto.checkInitalData = function checkInitalData() {
    if (!this.initalData) return this.legal = false;
    var _this$initalData = this.initalData, nodes = _this$initalData.nodes, links = _this$initalData.links;
    if (!nodes || !links) return this.legal = false;
    if (nodes && nodes.length === 0 || links && links.length === 0) this.legal = false;
  };
  _proto.createNode = function createNode(initialNode) {
    return new Node(initialNode, this.config, this.initalData, this.belts);
  };
  _proto.addNode = function addNode(node) {
    this.nodes.push(node);
  };
  _proto.findInitalNode = function findInitalNode(name) {
    return this.initalData.nodes.find(function(item) {
      return item.name === name;
    });
  };
  _proto.findNode = function findNode(name) {
    return this.nodes.find(function(item) {
      return item.name === name;
    });
  };
  _proto.checkNodeExist = function checkNodeExist(name, initalNode) {
    var targetNode = this.findNode(name);
    if (!targetNode) {
      var initNode = initalNode ? initalNode : this.findInitalNode(name);
      var newNode = this.createNode(initNode);
      this.addNode(newNode);
      targetNode = newNode;
    }
    return targetNode;
  };
  _proto.addNodeDependency = function addNodeDependency(dependency, node, isParent) {
    var _this = this;
    if (isParent === void 0) {
      isParent = true;
    }
    if (dependency && dependency.length > 0) {
      dependency.forEach(function(name) {
        var targetNode = _this.checkNodeExist(name);
        isParent ? node.setParent(targetNode) : node.setChildren(targetNode);
      });
    }
  };
  _proto.addNodeParent = function addNodeParent(parentName, node) {
    this.addNodeDependency(parentName, node);
  };
  _proto.addNodeChildren = function addNodeChildren(childrenName, node) {
    this.addNodeDependency(childrenName, node, false);
  };
  _proto.init = function init() {
    if (!this.legal) return;
    this.initNodes();
    this.initBelts();
  };
  _proto.initNodes = function initNodes() {
    var _this2 = this;
    this.initalData.nodes.forEach(function(initialNode) {
      var node = _this2.checkNodeExist(initialNode.name, initialNode);
      var parentName = [];
      var childrenName = [];
      _this2.initalData.links.forEach(function(link) {
        if (link.target === initialNode.name) {
          parentName.push(link.source);
        }
        if (link.source === initialNode.name) {
          childrenName.push(link.target);
        }
      });
      _this2.addNodeParent(parentName, node);
      _this2.addNodeChildren(childrenName, node);
    });
  };
  _proto.initBelts = function initBelts() {
    this.nodes.forEach(function(node) {
      node.initBelts();
    });
  };
  _proto.renderNodes = function renderNodes() {
    var _this3 = this;
    this.nodes.forEach(function(node) {
      return node.render(_this3.nodeContainer, _this3.svgNodeGroup, _this3.svgNodeGradientDefs);
    });
  };
  _proto.renderBelts = function renderBelts() {
    var _this4 = this;
    this.belts.forEach(function(belt) {
      return belt.render(_this4.svgBeltPathGroup, _this4.svgBeltGradientDefs);
    });
  };
  _proto.render = function render() {
    this.renderNodes();
    this.renderBelts();
  };
  return Manager2;
}();
export {
  Manager as default
};
