import { moveToCenter, insertDom, getStyle, setStyle, pxToNumber, judgeDisabled, createDom, boundaryJudge } from "./util.mjs";
import { defaultToolTip } from "./defaultOption.mjs";
function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t = arguments[e];
      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
var NodeManager = /* @__PURE__ */ function() {
  function NodeManager2(container, data, deg, isCenter, branchInstance) {
    var _this = this;
    this.dom = void 0;
    this.wrapper = void 0;
    this.nodeClick = function(e) {
      if (_this.drag.draging || _this.data.disabled) {
        return;
      }
      _this.snowInstance.removeNodeActive();
      !_this.isCenter && _this.drillDown(true, true, true);
      _this.findSameDom();
      if (_this.data.click) {
        _this.data.click(e, _extends({}, _this.data, {
          dom: _this.dom
        }));
      }
      !_this.isCenter && _this.insertContent();
    };
    this.containerClick = function(e) {
      if (_this.drag.draging || _this.container.parentNode.classList.contains("noLeafChild")) {
        return;
      }
      if (e.target === _this.container) {
        _this.drillDown(true, false, true);
        if (_this.data.containerClick) {
          _this.data.containerClick(e, _this.data);
        }
      }
    };
    var drag = branchInstance.drag, snowInstance = branchInstance.snowInstance, chartContainer = branchInstance.chartContainer, option = branchInstance.option, equalDeg = branchInstance.equalDeg, branchWrapper = branchInstance.wrapper;
    this.container = container;
    this.data = data;
    this.option = option;
    this.deg = deg;
    this.isCenter = isCenter;
    this.chartContainer = chartContainer;
    this.drag = drag;
    this.snowInstance = snowInstance;
    this.equalDeg = equalDeg;
    this.branchWrapper = branchWrapper;
    this.branchInstance = branchInstance;
    this.mount();
    moveToCenter(branchInstance, this, true);
  }
  var _proto = NodeManager2.prototype;
  _proto.mount = function mount() {
    var domClass = this.isCenter ? "sfc-node-center" : "sfc-node-item";
    if (this.data.faultNumber) {
      domClass += " sfc-node-error";
    }
    this.dom = insertDom(this.container, {
      "class": domClass
    });
    this.wrapper = insertDom(this.dom);
    this.addEvent();
    this.isCenter && this.insertContent();
    this.addToolTip();
  };
  _proto.returnWrapper = function returnWrapper() {
    return this.wrapper;
  };
  _proto.addEvent = function addEvent() {
    this.dom.addEventListener("click", this.nodeClick);
    this.container.addEventListener("click", this.containerClick);
  };
  _proto.findSameDom = function findSameDom() {
    var _this2 = this;
    this.snowInstance.branchs.forEach(function(item) {
      item.leafsArr.forEach(function(leaf) {
        if (leaf.data.mac === _this2.data.mac) {
          _this2.dom = leaf.node.dom;
          _this2.wrapper = leaf.node.wrapper;
        }
      });
    });
    this.dom.classList.add("nodeActive");
  };
  _proto.setAngle = function setAngle() {
    var containerWidth = getStyle(this.container, "width");
    var nodeItemWidth = getStyle(this.wrapper, "width");
    setStyle(this.dom, {
      transform: " translate(-50%,-50%) rotate(" + this.deg + "deg)",
      transformOrigin: -1 * (pxToNumber(containerWidth) / 2 - pxToNumber(nodeItemWidth) / 2) + "px center"
    });
    setStyle(this.wrapper, {
      transform: "rotate(" + (360 - this.deg) + "deg)"
    });
  };
  _proto.insertContent = function insertContent() {
    judgeDisabled(this.option, [this.dom], false, this.data, true);
    var renderFun = this.data.render || this.option.render;
    if (renderFun) {
      renderFun({
        nodeContainer: this.wrapper,
        tagContainer: createDom()
      }, this.data, this.option, this.drag.scale);
    }
  };
  _proto.setSubRootWidth = function setSubRootWidth() {
    if (this.option.overAll) {
      setStyle(this.branchWrapper, {
        width: pxToNumber(getStyle(this.wrapper, "width")) + 50 * 2 + "px",
        height: pxToNumber(getStyle(this.wrapper, "height")) + 50 * 2 + "px"
      });
    }
  };
  _proto.addToolTip = function addToolTip() {
    var _this3 = this;
    var toolTip = document.querySelector(".sfc-overAllTip");
    if (!toolTip) {
      toolTip = document.createElement("div");
      toolTip.className = "sfc-overAllTip";
      document.documentElement.appendChild(toolTip);
    }
    if (this.option.overAll) {
      var toolTipFun = this.data.toolTip || this.option.toolTip || defaultToolTip;
      if (this.isCenter) {
        this.wrapper.addEventListener("mousemove", function(e) {
          e.stopPropagation();
          toolTip.style.display = "none";
        });
      } else {
        this.wrapper.addEventListener("mousemove", function(e) {
          if (_this3.drag.startPos) {
            toolTip.style.display = "none";
            return;
          }
          e.stopPropagation();
          toolTip.style.display = "block";
          var _boundaryJudge = boundaryJudge(e, _this3.drag.container, toolTip), x = _boundaryJudge.x, y = _boundaryJudge.y;
          toolTip.style.left = x + "px";
          toolTip.style.top = y + "px";
          toolTip.innerHTML = toolTipFun(_this3.data, true);
        });
        this.wrapper.addEventListener("mouseleave", function(e) {
          toolTip.style.display = "none";
        });
      }
      this.container.addEventListener("mousemove", function(e) {
        if (_this3.drag.startPos || _this3.container.parentNode.classList.contains("noLeafChild")) {
          toolTip.style.display = "none";
          return;
        }
        toolTip.style.display = "block";
        var _boundaryJudge2 = boundaryJudge(e, _this3.drag.container, toolTip), x = _boundaryJudge2.x, y = _boundaryJudge2.y;
        toolTip.style.left = x + "px";
        toolTip.style.top = y + "px";
        toolTip.innerHTML = toolTipFun(_this3.data, false);
        _this3.container.style.backgroundImage = "radial-gradient(rgba(89,144,253,0) 0%,rgba(89,144,253,.1) 100%)";
      });
      this.container.addEventListener("mouseleave", function(e) {
        toolTip.style.display = "none";
        _this3.container.style.backgroundImage = "unset";
      });
    } else {
      toolTip.style.display = "none";
    }
  };
  _proto.refreshData = function refreshData(data) {
    this.data = data !== void 0 ? data : this.data;
    this.insertContent();
    this.upDateNodeClass();
  };
  _proto.upDateNodeClass = function upDateNodeClass() {
    var faultNumber = this.data.faultNumber;
    var classList = this.dom.classList;
    if (faultNumber) {
      classList.add("sfc-node-error");
    } else {
      classList.remove("sfc-node-error");
    }
  };
  _proto.drillDown = function drillDown(drill, leafTrigger, clickTrigger, mac) {
    if (drill === void 0) {
      drill = true;
    }
    if (drill) {
      if (this.option.overAll === false && clickTrigger) {
        return;
      }
      this.option.overAll = false;
      this.chartContainer.classList.remove("overAll");
    } else {
      this.option.overAll = true;
      this.chartContainer.classList.add("overAll");
    }
    this.snowInstance.branchs[0].reRender(mac || this.data.mac, leafTrigger);
    if (drill && !clickTrigger) {
      this.snowInstance.findFitScale();
    } else {
      this.drag.reset();
    }
  };
  return NodeManager2;
}();
export {
  NodeManager as default
};
