import { insertDom, setDefault, setAttribute, onImageLoad, isDataDisabled, arrFlat, pxToNumber, getStyle, getDistance, compareRefresh, getAreaScope, getWaistLength, setStyle } from "./util.mjs";
import BranchManager from "./BranchManager.mjs";
import BaseChart from "../BaseChart/index.mjs";
import defaultOption from "./defaultOption.mjs";
import image5R from "./assets/image5R.svg.mjs";
import image5L from "./assets/image5L.svg.mjs";
import image2R from "./assets/image2R.svg.mjs";
import image2L from "./assets/image2L.svg.mjs";
import imageUp from "./assets/up.svg.mjs";
import imageDown from "./assets/down.svg.mjs";
import wirelessR from "./assets/wirelessR.svg.mjs";
import wirelessL from "./assets/wirelessL.svg.mjs";
import { CHART_TYPE } from "../../util/constants.mjs";
function _inheritsLoose(t, o) {
  t.prototype = Object.create(o.prototype), t.prototype.constructor = t, _setPrototypeOf(t, o);
}
function _setPrototypeOf(t, e) {
  return _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function(t2, e2) {
    return t2.__proto__ = e2, t2;
  }, _setPrototypeOf(t, e);
}
function _defineProperty(e, r, t) {
  return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: true, configurable: true, writable: true }) : e[r] = t, e;
}
function _toPropertyKey(t) {
  var i = _toPrimitive(t, "string");
  return "symbol" == typeof i ? i : i + "";
}
function _toPrimitive(t, r) {
  if ("object" != typeof t || !t) return t;
  var e = t[Symbol.toPrimitive];
  if (void 0 !== e) {
    var i = e.call(t, r);
    if ("object" != typeof i) return i;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return ("string" === r ? String : Number)(t);
}
var SnowFlakeChart = /* @__PURE__ */ function(_BaseChart) {
  function SnowFlakeChart2() {
    var _this;
    _this = _BaseChart.call(this) || this;
    _this.dom;
    _this.drag;
    _this.option;
    _this.data;
    _this.branchs = [];
    _this.renderCallBack = null;
    _this.firstRecordStatus = true;
    return _this;
  }
  _inheritsLoose(SnowFlakeChart2, _BaseChart);
  var _proto = SnowFlakeChart2.prototype;
  _proto.init = function init(dom) {
    this.dom = dom;
    this.dom.innerHTML = "";
    this.container = insertDom(this.dom);
  };
  _proto.setSimpleOption = function setSimpleOption(chartName, option, _ref) {
    var drag = _ref.drag;
    setDefault(option, defaultOption);
    this.data = option.data;
    this.option = option;
    this.drag = drag;
    setAttribute(this.container, {
      "class": "sfc-container " + (option.overAll ? "overAll" : "")
    });
  };
  _proto.setResize = function setResize() {
  };
  _proto.render = function render(assets, callback) {
    var _this2 = this;
    this.initImage(assets);
    onImageLoad(this.imageList, function() {
      _this2.renderBranch();
      callback && callback();
    });
    window.addEventListener("dragScaleChange", this.dragScaleChange.bind(this));
  };
  _proto.initImage = function initImage(assets) {
    if (assets === void 0) {
      assets = {};
    }
    this.imageList = {
      "image5R": {
        src: assets.image5R || image5R
      },
      "image5L": {
        src: assets.image5L || image5L
      },
      "image2R": {
        src: assets.image2R || image2R
      },
      "image2L": {
        src: assets.image2L || image2L
      },
      "imageUp": {
        src: assets.imageUp || imageUp
      },
      "imageDown": {
        src: assets.imageDown || imageDown
      },
      "wirelessR": {
        src: assets.wirelessR || wirelessR
      },
      "wirelessL": {
        src: assets.wirelessL || wirelessL
      }
    };
    for (var item in this.imageList) {
      if (Object.hasOwnProperty.call(this.imageList, item)) {
        var img = new Image();
        img.src = this.imageList[item]["src"];
        this.imageList[item]["image"] = img;
      }
    }
  };
  _proto.renderBranch = function renderBranch() {
    var _this3 = this;
    if (this.option.theme.indexOf("dark") === -1) {
      this.container.classList.add("snowLight");
    }
    if (this.data && this.data.length > 0) {
      this.data.forEach(function(dataItem) {
        _this3.branchs.push(new BranchManager(dataItem, _this3.container, _this3));
      });
      isDataDisabled(arrFlat(this.data), this.container, this.option);
    }
    if (!this.option.overAll) {
      this.setPositionLeft();
      this.setTransformOrigin();
      this.setTagPosition();
    }
    this.findFitScale();
    setTimeout(function() {
      _this3.renderCallBack && _this3.renderCallBack(_this3);
    }, 0);
  };
  _proto.setPositionLeft = function setPositionLeft() {
    var centerDomArr = [].concat(this.branchs[0].centerDomArr);
    centerDomArr.splice(0, 1);
    centerDomArr.forEach(function(item, index) {
      var equalDeg = item.equalDeg, level = item.data.level;
      var itemBranch = item.container.parentNode.parentNode;
      var itemBranchWidth = pxToNumber(getStyle(itemBranch, "width"));
      var newLeftVal = (itemBranchWidth / 2 + 40) / Math.sin(equalDeg / 2 * Math.PI / 180) - pxToNumber(getStyle(itemBranch.parentNode, "width")) / 2;
      var oldLeftVal = pxToNumber(getStyle(itemBranch, "left")) - pxToNumber(getStyle(itemBranch.parentNode, "width"));
      if (newLeftVal > oldLeftVal) {
        if (centerDomArr.length >= 3) {
          var sameLevelArr = centerDomArr.filter(function(node) {
            return node.data.level === level;
          });
          var getIndex = sameLevelArr.findIndex(function(same) {
            return same.data.mac === item.data.mac;
          });
          var lastBranch = sameLevelArr[getIndex - 1 === -1 ? sameLevelArr.length - 1 : getIndex - 1].container.parentNode.parentNode;
          var nextBranch = sameLevelArr[getIndex + 1 === sameLevelArr.length ? 0 : getIndex + 1].container.parentNode.parentNode;
          var lastBranchWidth = pxToNumber(getStyle(lastBranch, "width"));
          var nextBranchWidth = pxToNumber(getStyle(nextBranch, "width"));
          if (itemBranchWidth > lastBranchWidth && itemBranchWidth > nextBranchWidth) {
            var minEqual = itemBranchWidth / (Math.max(lastBranchWidth, nextBranchWidth) + itemBranchWidth + 40) * equalDeg;
            newLeftVal = (itemBranchWidth / 2 + 40) / Math.sin(minEqual * Math.PI / 180) - pxToNumber(getStyle(itemBranch.parentNode, "width")) / 2;
          }
        }
        itemBranch.style.left = "calc(100% + " + newLeftVal + "px)";
      }
    });
  };
  _proto.setTransformOrigin = function setTransformOrigin() {
    var _this4 = this;
    var centerDomArr = this.branchs[0].centerDomArr;
    centerDomArr.forEach(function(item, index) {
      if (index === 0) return;
      var branchContainer = item.container;
      while (!branchContainer.classList.contains("sfc-branch")) {
        branchContainer = branchContainer.parentNode;
      }
      var centerNode = branchContainer.querySelector(".sfc-node-center");
      var parentCenterNode = branchContainer.parentNode.querySelector(".sfc-node-center");
      var scale = getStyle(_this4.drag.dom, "transform").replace("matrix(", "").split(",")[0];
      var distance = getDistance(centerNode, parentCenterNode) / scale;
      branchContainer.style.transformOrigin = "calc(50% - " + distance + "px) center";
      var centerLine = _this4.branchs[0].centerLineArr.find(function(line) {
        return line.data.mac === branchContainer.getAttribute("data-mac");
      });
      centerLine.distance = distance - pxToNumber(getStyle(branchContainer.parentNode, "width")) / 2;
      centerLine.destroyCanvas();
      centerLine.createCanvas();
      centerLine.draw();
    });
  };
  _proto.setTagPosition = function setTagPosition() {
    var centerTagArr = this.branchs[0].centerTagArr;
    centerTagArr.forEach(function(tag) {
      tag.setAngle();
    });
  };
  _proto.switchView = function switchView(drill, mac, callback) {
    if (drill === void 0) {
      drill = true;
    }
    if (!drill) this.drag.reset();
    this.branchs.forEach(function(item) {
      item.centerDomArr[0].drillDown(drill, true, false, mac);
    });
    callback && callback();
  };
  _proto.switchCenterByDrill = function switchCenterByDrill(mac) {
    var _this5 = this;
    this.branchs.forEach(function(item) {
      item.centerDomArr.forEach(function(center) {
        if (center.data.mac === mac) {
          _this5.drag.moveTargetToCenter(center.dom);
        }
      });
      item.leafsArr.forEach(function(leaf) {
        if (leaf.node.data.mac === mac) {
          _this5.drag.moveTargetToCenter(leaf.node.dom);
        }
      });
    });
  };
  _proto.onRenderReady = function onRenderReady(callback) {
    this.renderCallBack = callback;
  };
  _proto.uninstall = function uninstall() {
    this.dom.innerHTML = "";
    window.removeEventListener("dragScaleChange", this.dragScaleChange.bind(this));
  };
  _proto.removeNodeActive = function removeNodeActive() {
    this.branchs.forEach(function(item) {
      item.centerDomArr.forEach(function(center) {
        if (center.dom.classList.contains("nodeActive")) {
          center.dom.classList.remove("nodeActive");
        }
      });
      item.leafsArr.forEach(function(leaf) {
        if (leaf.node.dom.classList.contains("nodeActive")) {
          leaf.node.dom.classList.remove("nodeActive");
          leaf.node.insertContent();
        }
      });
    });
  };
  _proto.refreshLine = function refreshLine(data) {
    this.data = data;
    compareRefresh(data, this.branchs, {
      centerArr: "centerLineArr",
      target: "line"
    }, this);
  };
  _proto.refreshTag = function refreshTag(data) {
    this.data = data;
    compareRefresh(data, this.branchs, {
      centerArr: "centerTagArr",
      target: "tag"
    }, this);
  };
  _proto.refreshNode = function refreshNode(data, callback) {
    this.data = data;
    compareRefresh(data, this.branchs, {
      centerArr: "centerDomArr",
      target: "node"
    }, this);
    callback && callback();
  };
  _proto.refresh = function refresh(option, callback) {
    this.branchs = [];
    this.container.innerHTML = null;
    this.firstRecordStatus = true;
    this.setSimpleOption(null, option, {
      drag: this.drag
    });
    this.renderBranch();
    callback && callback();
  };
  _proto.refreshData = function refreshData(data, callback) {
    this.option.data = data;
    this.refresh(this.option, callback);
  };
  _proto.findFitScale = function findFitScale(callback) {
    if (!this.option.scaleAdaptive || this.option.overAll) return;
    var dragScale = this.drag.scale;
    var branchArr = Array.from(this.dom.querySelectorAll(".sfc-branch"));
    var leafArr = Array.from(this.dom.querySelectorAll(".sfc-branch .sfc-node-item"));
    var centerNodeArr = Array.from(this.dom.querySelectorAll(".sfc-branch .sfc-node-center"));
    var _getAreaScope = getAreaScope(leafArr.length ? leafArr : branchArr), snowWidth = _getAreaScope.width, snowHeight = _getAreaScope.height;
    var viewContainer = this.drag.container;
    var _viewContainer$getBou = viewContainer.getBoundingClientRect(), viewWidth = _viewContainer$getBou.width, viewHeight = _viewContainer$getBou.height, x = _viewContainer$getBou.x, y = _viewContainer$getBou.y;
    var fitScale;
    if (viewWidth >= snowWidth * 0.75 / dragScale && viewHeight >= snowHeight * 0.75 / dragScale) {
      fitScale = Math.min(viewWidth / (snowWidth / dragScale), viewHeight / (snowHeight / dragScale));
      fitScale = Math.min(this.drag.scaleLimit.max, Math.floor(fitScale * 10) / 10);
      fitScale = Math.max(0.75, Math.floor(fitScale * 10) / 10);
    } else {
      this.lowScaleView();
      var _getAreaScope2 = getAreaScope(centerNodeArr), _snowWidth = _getAreaScope2.width, _snowHeight = _getAreaScope2.height;
      fitScale = Math.min(viewWidth / (_snowWidth / dragScale), viewHeight / (_snowHeight / dragScale));
      fitScale = Math.min(0.75, Math.floor(fitScale * 10) / 10);
      fitScale = Math.max(this.drag.scaleLimit.min, Math.floor(fitScale * 10) / 10);
    }
    this.drag.scale = fitScale;
    this.drag.scaleDom(x + viewWidth / 2, y + viewHeight / 2, fitScale - dragScale, false);
    var dragCurrentVal = this.drag.container.querySelector(".drag-currentVal");
    dragCurrentVal.innerHTML = (fitScale * 100).toFixed(0) + "%";
    this.refreshNode(this.data);
    this.refreshLine(this.data);
    this.refreshTag(this.data);
    callback && callback();
  };
  _proto.dragScaleChange = function dragScaleChange(e) {
    var _this6 = this;
    if (this.option.overAll || !this.option.scaleAdaptive) return;
    var scale = e.detail.scaleVal;
    if (scale >= 0.5 && scale <= 0.75) {
      this.lowScaleView();
      setTimeout(function() {
        _this6.drag.moveTargetToCenter(_this6.container);
      }, this.drag.throldHold);
    } else {
      this.highScaleView();
    }
    this.refreshNode(this.data);
    this.refreshLine(this.data);
    this.refreshTag(this.data);
  };
  _proto.lowScaleView = function lowScaleView() {
    var _this7 = this;
    var branchArr = Array.from(this.dom.querySelectorAll(".sfc-branch"));
    if (this.firstRecordStatus) {
      setAttribute(branchArr[0], {
        "data-width": branchArr[0].style.width,
        "data-height": branchArr[0].style.height
      });
    }
    branchArr[0].style.width = "150px";
    branchArr[0].style.height = "150px";
    branchArr.shift();
    branchArr.forEach(function(branch, index) {
      var equalDeg = _this7.branchs[0].centerDomArr[index + 1].equalDeg;
      var centerDomWidth = pxToNumber(getStyle(branch.querySelector(".sfc-node-center"), "width"));
      var distance = Math.max(getWaistLength(equalDeg, centerDomWidth + 20), _this7.option.distance.minDistanceDrill);
      var parentBranchWidth = pxToNumber(getStyle(branch.parentNode, "width"));
      var centerLine = _this7.branchs[0].centerLineArr.find(function(item) {
        return item.data.mac === branch.getAttribute("data-mac");
      });
      if (_this7.firstRecordStatus) {
        setAttribute(branch, {
          // 需要将scle为1的left和transformOrigin记录下来；也需要将其width和height切换掉。不然中心节点无法点击，会被容器盖住
          "data-left": branch.style.left,
          "data-transformOrigin": branch.style.transformOrigin,
          "data-distance": centerLine.distance,
          "data-width": branch.style.width,
          "data-height": branch.style.height
        });
      }
      branch.style.transformOrigin = "calc(50% - " + distance + "px) center";
      branch.style.left = "calc(100% + " + distance + "px - " + parentBranchWidth / 2 + "px)";
      branch.style.width = "150px";
      branch.style.height = "150px";
      centerLine.distance = distance - parentBranchWidth / 2;
      centerLine.destroyCanvas();
      centerLine.createCanvas();
      centerLine.draw();
    });
    var coreContainerArr = this.dom.querySelectorAll(".sfc-core");
    coreContainerArr.forEach(function(item) {
      setStyle(item, {
        backgroundSize: 0
      });
    });
    this.firstRecordStatus = false;
  };
  _proto.highScaleView = function highScaleView() {
    var _this8 = this;
    if (this.firstRecordStatus) this.lowScaleView();
    var branchArr = Array.from(this.dom.querySelectorAll(".sfc-branch"));
    branchArr[0].style.width = branchArr[0].getAttribute("data-width");
    branchArr[0].style.height = branchArr[0].getAttribute("data-height");
    branchArr.shift();
    branchArr.forEach(function(item) {
      item.style.left = item.getAttribute("data-left");
      item.style.transformOrigin = item.getAttribute("data-transformOrigin");
      item.style.width = item.getAttribute("data-width");
      item.style.height = item.getAttribute("data-height");
      var centerLine = _this8.branchs[0].centerLineArr.find(function(line) {
        return line.data.mac === item.getAttribute("data-mac");
      });
      centerLine.distance = item.getAttribute("data-distance");
      centerLine.destroyCanvas();
      centerLine.createCanvas();
      centerLine.draw();
    });
    var coreContainerArr = this.dom.querySelectorAll(".sfc-core");
    coreContainerArr.forEach(function(item) {
      setStyle(item, {
        backgroundSize: "100% 100%"
      });
    });
  };
  return SnowFlakeChart2;
}(BaseChart);
_defineProperty(SnowFlakeChart, "name", CHART_TYPE.SNOWFLAKE);
export {
  SnowFlakeChart as default
};
