import { ictLight, ictDark } from "./theme/ict/index.mjs";
import { bpitLight, bpitDark } from "./theme/bpit/index.mjs";
import { cloudLight, cloudDark } from "./theme/cloud/index.mjs";
import { hdesignLight, hdesignDark } from "./theme/hdesign/index.mjs";
import merge from "../../util/merge.mjs";
import HashMap from "../../util/hashMap.mjs";
import cloneDeep from "../../util/cloneDeep.mjs";
import tips from "../../util/tips.mjs";
import { DEFAULT_THEME_NAME, THEMES, CURRENT_THEME, THEME_ERROR_TIP_MESSAGE } from "../../util/constants.mjs";
import mergeToken from "./factory/mergeToken.mjs";
var _HashMap;
function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t = arguments[e];
      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
var themeToken = new HashMap((_HashMap = {}, _HashMap[THEMES.LIGHT] = cloneDeep(ictLight), _HashMap[THEMES.DARK] = cloneDeep(ictDark), _HashMap[THEMES.BPIT_LIGHT] = cloneDeep(bpitLight), _HashMap[THEMES.BPIT_DARK] = cloneDeep(bpitDark), _HashMap[THEMES.CLOUD_LIGHT] = cloneDeep(cloudLight), _HashMap[THEMES.CLOUD_DARK] = cloneDeep(cloudDark), _HashMap[THEMES.HDESIGN_LIGHT] = cloneDeep(hdesignLight), _HashMap[THEMES.HDESIGN_DARK] = cloneDeep(hdesignDark), _HashMap[CURRENT_THEME] = cloneDeep(ictLight), _HashMap));
var Theme = /* @__PURE__ */ function() {
  function Theme2() {
  }
  Theme2.set = function set(name, config) {
    var defaultConfig = cloneDeep(this.getConfig(DEFAULT_THEME_NAME));
    merge(defaultConfig, config);
    themeToken.set(name, defaultConfig);
  };
  Theme2.setConfig = function setConfig(name, config) {
    var themeKeys = this.getThemeKeys();
    var validate = themeKeys.includes(name);
    if (validate) {
      var newConfig = mergeToken(name, config);
      var oldConfig = this.getConfig(name);
      merge(oldConfig, newConfig);
    } else {
      var _newConfig = mergeToken(DEFAULT_THEME_NAME, config);
      themeToken.set(name, _newConfig);
    }
  };
  Theme2.getThemeKeys = function getThemeKeys() {
    return themeToken.keys();
  };
  Theme2.resetThemeCongfig = function resetThemeCongfig() {
    themeToken.set(THEMES.LIGHT, cloneDeep(ictLight));
    themeToken.set(THEMES.DARK, cloneDeep(ictDark));
    themeToken.set(THEMES.BPIT_LIGHT, cloneDeep(bpitLight));
    themeToken.set(THEMES.BPIT_DARK, cloneDeep(bpitDark));
    themeToken.set(THEMES.CLOUD_LIGHT, cloneDeep(cloudLight));
    themeToken.set(THEMES.CLOUD_DARK, cloneDeep(cloudDark));
    themeToken.set(THEMES.HDESIGN_LIGHT, cloneDeep(hdesignLight));
    themeToken.set(THEMES.HDESIGN_DARK, cloneDeep(hdesignDark));
    themeToken.set(CURRENT_THEME, cloneDeep(ictLight));
  };
  Theme2.getConfig = function getConfig(name) {
    return themeToken.get(name);
  };
  Theme2.setDefaultTheme = function setDefaultTheme(name) {
    var tempTheme = this.validate(name);
    if (this.themeName !== tempTheme) {
      this.themeName = tempTheme;
      themeToken.set(CURRENT_THEME, themeToken.get(tempTheme));
      this.config = this.getConfig(CURRENT_THEME);
    }
  };
  Theme2.validate = function validate(name) {
    var tempTheme = name || this.themeName || DEFAULT_THEME_NAME;
    if (tempTheme.toLowerCase().includes("cloud-light")) {
      tempTheme = THEMES.CLOUD_LIGHT;
    }
    if (tempTheme.toLowerCase().includes("cloud-dark")) {
      tempTheme = THEMES.CLOUD_DARK;
    }
    var themeKeys = themeToken.keys();
    var validate2 = themeKeys.includes(tempTheme);
    if (!validate2) {
      tips.error(THEME_ERROR_TIP_MESSAGE);
      tempTheme = THEMES.LIGHT;
    }
    return tempTheme;
  };
  Theme2.getTokenByName = function getTokenByName(chartName) {
    var _this = this;
    var handler = function handler2(_, prop) {
      var token = _extends({}, _this.config[chartName]);
      return token[prop];
    };
    return new Proxy({}, {
      get: handler
    });
  };
  Theme2.getChartTokenByName = function getChartTokenByName(chartName) {
    return this.config[chartName];
  };
  Theme2.getToken = function getToken() {
    return this.config;
  };
  return Theme2;
}();
Theme.themeName = void 0;
Theme.config = cloneDeep(ictLight);
export {
  THEMES,
  Theme as default
};
