import { __spreadValues } from "../chunk-G2ADBYYC.mjs";
import { isSame } from "./type.mjs";
const indexOf = (arr, data, predicate = isSame) => {
  if (Array.isArray(arr) && typeof predicate === "function") {
    for (let i = 0, len = arr.length; i < len; i++) {
      if (predicate(arr[i], data)) {
        return i;
      }
    }
  }
  return -1;
};
const find = (arr, predicate) => {
  const index = indexOf(arr, void 0, predicate);
  return index !== -1 ? arr[index] : void 0;
};
const extend = (to, _from) => {
  Object.keys(_from).forEach((key) => to[key] = _from[key]);
  return to;
};
const toObject = (arr) => {
  const res = {};
  for (let i = 0; i < arr.length; i++) {
    if (arr[i]) {
      extend(res, arr[i]);
    }
  }
  return res;
};
const transformPidToChildren = (data, pidName = "pId", childrenName = "children", idName = "id") => {
  const result = [];
  Array.isArray(data) && data.forEach((item) => {
    if (item[pidName] == "0") {
      result.push(item);
    } else {
      const parent = find(data, (i) => i[idName] == item[pidName]);
      if (!parent) {
        return;
      }
      if (!parent[childrenName]) {
        parent[childrenName] = [];
      }
      parent[childrenName].push(item);
    }
    delete item[pidName];
  });
  return result;
};
const transformTreeData = (data, key = "id", parentKey = "pId") => {
  if (!Array.isArray(data)) {
    data = [data];
  }
  data = data.map((item) => __spreadValues({}, item));
  const treeData = transformPidToChildren(data, parentKey, "children", key);
  return treeData;
};
export {
  find,
  indexOf,
  toObject,
  transformPidToChildren,
  transformTreeData
};
