import { mounted, handleClear, cancelSearch, closeModal, selectDropdownConfirm, selectedBoxConfirm, deleteSelected, toggleSelectedBox, handleQueryChange } from "./index.mjs";
import vue_popper_default from "../common/deps/vue-popper.mjs";
import popup_manager_default from "../common/deps/popup-manager.mjs";
import debounce_default from "../common/deps/debounce.mjs";
import { isBrowser } from "../common/browser.mjs";
const api = [
  "state",
  "doDestroy",
  "closeModal",
  "debouncedQueryChange",
  "toggleSelectedBox",
  "deleteSelected",
  "selectedBoxConfirm",
  "selectDropdownConfirm",
  "cancelSearch",
  "handleClear"
];
const initState = ({ reactive, computed, popper, selectVm }) => {
  const { showPopper, currentPlacement, popperElm, referenceElm } = popper;
  const state = reactive({
    showPopper,
    currentPlacement,
    popperElm,
    referenceElm,
    minWidth: "",
    query: "",
    showSelectedBox: false,
    debounce: 300,
    selectedArr: [],
    deletedArr: [],
    originValue: [],
    showClose: computed(() => selectVm.clearable && !selectVm.multiple && selectVm.modelValue !== ""),
    filterable: computed(() => selectVm.filterable),
    multiple: computed(() => selectVm.multiple),
    popperClass: computed(() => selectVm.popperClass),
    selected: computed(
      () => selectVm.renderType === "tree" && !selectVm.treeOp.checkStrictly ? selectVm.state.selected.filter((item) => !item.children) : selectVm.state.selected
    ),
    device: computed(() => selectVm.state.device),
    visible: false,
    windowScrollTop: 0,
    zIndex: 2e3,
    valueField: computed(() => selectVm.renderType === "tree" ? "id" : "value")
  });
  return state;
};
const initApi = ({ api: api2, popper, state, selectEmitter, constants, selectVm, parent, nextTick, props }) => {
  const { destroyPopper, doDestroy, updatePopper } = popper;
  Object.assign(api2, {
    state,
    doDestroy,
    handleQueryChange: handleQueryChange({ selectVm }),
    debouncedQueryChange: debounce_default(state.debounce, (value) => {
      api2.handleQueryChange(value);
    }),
    toggleSelectedBox: toggleSelectedBox({ state }),
    deleteSelected: deleteSelected({ state }),
    selectedBoxConfirm: selectedBoxConfirm({ state, selectVm }),
    selectDropdownConfirm: selectDropdownConfirm({ selectVm }),
    closeModal: closeModal({ state, selectVm, props }),
    cancelSearch: cancelSearch({ api: api2, state }),
    handleClear: handleClear({ selectVm }),
    mounted: mounted({ selectEmitter, constants, state, selectVm, updatePopper, destroyPopper, parent })
  });
};
const initWatch = ({ watch, selectVm, state, nextTick }) => {
  watch(
    () => isBrowser ? selectVm.state.inputWidth : void 0,
    (val) => {
      nextTick(() => {
        state.minWidth = (selectVm && selectVm.$el && selectVm.$el.getBoundingClientRect().width || val) + "px";
      });
    },
    { immediate: true }
  );
  watch(
    () => state.query,
    (val) => {
      selectVm.state.queryValue = val;
    }
  );
  watch(
    () => selectVm.state.visible,
    (val) => {
      state.query = "";
      setTimeout(() => {
        state.visible = val;
        state.zIndex = popup_manager_default.nextZIndex();
      }, 0);
      if (val && selectVm.multiple) {
        state.originValue = selectVm.modelValue && selectVm.modelValue.slice(0);
      }
    }
  );
  watch(
    () => selectVm.shape,
    () => {
      nextTick(() => {
        state.referenceElm = selectVm.$refs.reference && selectVm.$refs.reference.$el;
      });
    }
  );
};
const renderless = (props, { computed, onBeforeUnmount, onDeactivated, onMounted, reactive, toRefs, watch, inject }, { vm, slots, parent, emit, nextTick, isMobileFirstMode }) => {
  const api2 = {};
  const constants = parent.select._constants;
  const selectEmitter = inject("selectEmitter");
  const selectVm = inject("selectVm");
  const popper = vue_popper_default({
    emit,
    nextTick,
    onBeforeUnmount,
    onDeactivated,
    props,
    reactive,
    vm,
    slots,
    toRefs,
    watch
  });
  const state = initState({ reactive, computed, popper, selectVm });
  initApi({ api: api2, popper, state, selectEmitter, constants, selectVm, parent, nextTick, props });
  initWatch({ watch, selectVm, state, nextTick });
  onBeforeUnmount(() => {
    popper.destroyPopper("remove");
    state.popperElm = null;
    state.referenceElm = null;
  });
  onMounted(api2.mounted);
  return api2;
};
export {
  api,
  renderless
};
