import "./core.mjs";
import { AlignStyle, AlignClass } from "./formats/align.mjs";
import { DirectionStyle, DirectionClass, DirectionAttribute } from "./formats/direction.mjs";
import IndentClass from "./formats/indent.mjs";
import Blockquote from "./formats/blockquote.mjs";
import Header from "./formats/header.mjs";
import ListItem from "./formats/list.mjs";
import { BackgroundStyle, BackgroundClass } from "./formats/background.mjs";
import { ColorStyle, ColorClass } from "./formats/color.mjs";
import { FontStyle, FontClass } from "./formats/font.mjs";
import { SizeStyle, SizeClass } from "./formats/size.mjs";
import Bold from "./formats/bold.mjs";
import Italic from "./formats/italic.mjs";
import Link from "./formats/link.mjs";
import Script from "./formats/script.mjs";
import Strike from "./formats/strike.mjs";
import Underline from "./formats/underline.mjs";
import Formula from "./formats/formula.mjs";
import Image from "./formats/image.mjs";
import Video from "./formats/video.mjs";
import CodeBlock, { Code } from "./formats/code.mjs";
import Syntax from "./modules/syntax.mjs";
import Table from "./modules/table.mjs";
import Toolbar from "./modules/toolbar.mjs";
import Icons from "./ui/icons.mjs";
import Picker from "./ui/picker.mjs";
import ColorPicker from "./ui/color-picker.mjs";
import IconPicker from "./ui/icon-picker.mjs";
import Tooltip from "./ui/tooltip.mjs";
import BubbleTheme from "./themes/bubble.mjs";
import SnowTheme from "./themes/snow.mjs";
import Quill from "./core/quill.mjs";
Quill.register({
  "attributors/attribute/direction": DirectionAttribute,
  "attributors/class/align": AlignClass,
  "attributors/class/background": BackgroundClass,
  "attributors/class/color": ColorClass,
  "attributors/class/direction": DirectionClass,
  "attributors/class/font": FontClass,
  "attributors/class/size": SizeClass,
  "attributors/style/align": AlignStyle,
  "attributors/style/background": BackgroundStyle,
  "attributors/style/color": ColorStyle,
  "attributors/style/direction": DirectionStyle,
  "attributors/style/font": FontStyle,
  "attributors/style/size": SizeStyle
}, true);
Quill.register({
  "formats/align": AlignClass,
  "formats/direction": DirectionClass,
  "formats/indent": IndentClass,
  "formats/background": BackgroundStyle,
  "formats/color": ColorStyle,
  "formats/font": FontClass,
  "formats/size": SizeClass,
  "formats/blockquote": Blockquote,
  "formats/code-block": CodeBlock,
  "formats/header": Header,
  "formats/list": ListItem,
  "formats/bold": Bold,
  "formats/code": Code,
  "formats/italic": Italic,
  "formats/link": Link,
  "formats/script": Script,
  "formats/strike": Strike,
  "formats/underline": Underline,
  "formats/formula": Formula,
  "formats/image": Image,
  "formats/video": Video,
  "modules/syntax": Syntax,
  "modules/table": Table,
  "modules/toolbar": Toolbar,
  "themes/bubble": BubbleTheme,
  "themes/snow": SnowTheme,
  "ui/icons": Icons,
  "ui/picker": Picker,
  "ui/icon-picker": IconPicker,
  "ui/color-picker": ColorPicker,
  "ui/tooltip": Tooltip
}, true);
const AttributeMap = DeltaExports.AttributeMap;
const Op = DeltaExports.Op;
export {
  AttributeMap,
  Op,
  Quill as default
};
