"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
require("../../../../../../../../quill@2.0.3/node_modules/quill/quill.js");
const emitter = require("../../../../../../../../quill@2.0.3/node_modules/quill/core/emitter.js");
const base = require("../../../../../../../../quill@2.0.3/node_modules/quill/themes/base.js");
const editor_utils_es = require("../../../config/editor.utils.es.js");
const editor_config_es = require("../../../config/editor.config.es.js");
const debounce_es = require("../../../utils/debounce.es.js");
const link_es = require("../formats/link.es.js");
require("../../../../../../../../quill@2.0.3/node_modules/quill/core.js");
const quill = require("../../../../../../../../quill@2.0.3/node_modules/quill/core/quill.js");
const selection = require("../../../../../../../../quill@2.0.3/node_modules/quill/core/selection.js");
const _Tooltip = class _Tooltip2 extends base.BaseTooltip {
  constructor(quill2, bounds) {
    super(quill2, bounds);
    this.options = {
      autoProtocol: "https"
    };
    this.setTemplate();
    this.isInputFocus = false;
    this.isHover = false;
    this.resolveOptions();
    link_es.default.autoProtocol = this.options.autoProtocol;
    this.debouncedHideToolTip = debounce_es.debounce(this.hideToolTip, 300);
    this.debouncedShowToolTip = debounce_es.debounce(this.showToolTip, 300);
    this.quill.emitter.on(editor_config_es.CHANGE_LANGUAGE_EVENT, () => {
      this.setTemplate();
    });
  }
  setTemplate() {
    this.root.innerHTML = [
      `<input type="text" data-formula="e=mc^2" data-link="${this.quill.getLangText("linkplaceholder")}" data-video="Embed URL" style="width: 225px;">`,
      '<span class="ql-split"></span>',
      '<a class="ql-preview"><i class="icon-share"></i></a>',
      '<a class="ql-remove"><i class="icon-delete"></i></a>'
    ].join("");
    this.textbox = this.root.querySelector('input[type="text"]');
    this.listen();
  }
  resolveOptions() {
    this.options = {
      autoProtocol: "https"
    };
    const value = this.quill.options.autoProtocol;
    if (value && typeof value === "string") {
      this.options.autoProtocol = value;
    } else if (typeof value === "boolean" && !value) {
      this.options.autoProtocol = "";
    }
  }
  shouldHide() {
    return !this.isHover && !this.isInputFocus;
  }
  hideToolTip() {
    if (this.shouldHide()) {
      this.hide();
    }
  }
  showToolTip(name, value, range) {
    if (!this.shouldHide()) {
      this.edit(name, value, range);
    }
  }
  handleMouseLeave() {
    this.isHover = false;
    this.debouncedHideToolTip();
  }
  handleMouseEnter(event) {
    const isTooltipShow = !this.root.classList.contains("ql-hidden");
    if (isTooltipShow) {
      return;
    }
    if (this.isInputFocus) {
      this.save();
    }
    this.isHover = true;
    const linkNode = event.target;
    const preview = link_es.default.formats(linkNode);
    if (preview.startsWith("#")) {
      return;
    }
    const linkBlot = quill.default.find(linkNode);
    const index = this.quill.getIndex(linkBlot);
    const [link, offset] = this.quill.scroll.descendant(
      link_es.default,
      index
    );
    const length = link && link.length();
    this.linkRange = new selection.Range(index - offset, length);
    this.debouncedShowToolTip("link", preview, this.linkRange);
  }
  listen() {
    super.listen();
    this.root.querySelector("a.ql-remove").addEventListener("click", (event) => {
      if (!editor_utils_es.isNullOrUndefined(this.linkRange)) {
        const range = this.linkRange;
        this.restoreFocus();
        this.quill.formatText(range, "link", false, emitter.default.sources.API);
        delete this.linkRange;
      }
      event.preventDefault();
      this.hide();
    });
    this.quill.root.addEventListener(
      "mouseover",
      (event) => {
        if ((event.target.tagName.toUpperCase() !== "A" || !event.target.classList.contains(link_es.default.className)) && !event.target.closest(`a.${link_es.default.className}`)) {
          return;
        }
        this.handleMouseEnter(event);
      },
      false
    );
    this.quill.root.addEventListener(
      "mouseout",
      (event) => {
        if (event.target.tagName.toUpperCase() !== "A" && !event.target.closest(`a.${link_es.default.className}`)) {
          return;
        }
        this.handleMouseLeave();
      },
      false
    );
    this.root.addEventListener(
      "mouseenter",
      () => {
        this.isHover = true;
      },
      false
    );
    this.root.addEventListener("mouseleave", this.handleMouseLeave.bind(this), false);
    this.root.querySelector("a.ql-preview").addEventListener("click", (event) => {
      const link = link_es.default.sanitize(this.textbox.value);
      window.open(link, "_blank");
      event.preventDefault();
    });
    this.root.querySelector('input[type="text"]').addEventListener("focus", () => {
      this.isInputFocus = true;
    });
    this.root.querySelector('input[type="text"]').addEventListener("blur", () => {
      this.isInputFocus = false;
      this.save();
    });
    this.quill.on(
      emitter.default.events.SELECTION_CHANGE,
      (range, _oldRange, source) => {
        if (editor_utils_es.isNullOrUndefined(range)) return;
        if (source === emitter.default.sources.USER) {
          const [link, offset] = this.quill.scroll.descendant(
            link_es.default,
            range.index
          );
          if (!editor_utils_es.isNullOrUndefined(link)) {
            this.linkRange = new selection.Range(range.index - offset, link.length());
            const preview = link_es.default.formats(link.domNode);
            if (!preview.startsWith("#")) {
              this.edit("link", preview, this.linkRange);
            }
            return;
          }
        }
        if (this.shouldHide()) {
          this.hide();
        }
      }
    );
    this.quill.on(
      emitter.default.events.TEXT_CHANGE,
      () => {
        const selection2 = this.quill.getSelection();
        const index = selection2 && selection2.index;
        setTimeout(() => {
          const link = this.quill.scroll.descendant(
            link_es.default,
            index
          )[0];
          if (!link) {
            this.handleMouseLeave();
          }
        });
      }
    );
  }
  save() {
    let value = this.textbox.value;
    if (!value) return;
    this.textbox.value = "";
    switch (this.root.getAttribute("data-mode")) {
      case "link": {
        const { scrollTop } = this.quill.root;
        if (this.options.autoProtocol) {
          value = this.addHttpProtocol(value);
        }
        if (this.linkRange) {
          this.quill.formatText(
            this.linkRange,
            "link",
            value,
            emitter.default.sources.USER
          );
          this.restoreFocus();
        } else {
          this.restoreFocus();
          this.quill.format("link", value, emitter.default.sources.USER);
        }
        this.quill.root.scrollTop = scrollTop;
        break;
      }
      case "formula": {
        const range = this.quill.getSelection(true);
        if (!editor_utils_es.isNullOrUndefined(range)) {
          const index = range.index + range.length;
          this.quill.insertEmbed(
            index,
            this.root.getAttribute("data-mode"),
            value,
            emitter.default.sources.USER
          );
          if (this.root.getAttribute("data-mode") === "formula") {
            this.quill.insertText(index + 1, " ", emitter.default.sources.USER);
          }
          this.quill.setSelection(index + 2, emitter.default.sources.USER);
        }
        break;
      }
      case "video": {
        const range = this.quill.getSelection(true);
        this.quill.insertText(range.index, "\n", emitter.default.sources.USER);
        this.quill.insertEmbed(range.index + 1, "video", { src: value }, emitter.default.sources.USER);
        this.quill.insertText(range.index + 2, "\n", emitter.default.sources.USER);
        this.quill.setSelection(range.index + 3, emitter.default.sources.SILENT);
        this.textbox.value = "";
        this.hide();
        break;
      }
    }
  }
  position(reference) {
    const left = reference.left;
    const top = reference.bottom + this.quill.root.scrollTop;
    this.root.style.left = `${left}px`;
    this.root.style.top = `${top}px`;
    this.root.classList.remove("ql-flip");
    const containerBounds = this.boundsContainer.getBoundingClientRect();
    const rootBounds = this.root.getBoundingClientRect();
    let shift = 0;
    if (rootBounds.right > containerBounds.right) {
      shift = containerBounds.right - rootBounds.right;
      this.root.style.left = `${left + shift}px`;
    }
    if (rootBounds.left < containerBounds.left) {
      shift = containerBounds.left - rootBounds.left;
      this.root.style.left = `${left + shift}px`;
    }
    if (rootBounds.bottom > containerBounds.bottom) {
      const height = rootBounds.bottom - rootBounds.top;
      const verticalShift = reference.bottom - reference.top + height;
      const fixedTop = top - verticalShift;
      this.root.style.top = `${fixedTop < 0 ? this.quill.root.scrollTop + reference.top : fixedTop}px`;
      this.root.classList.add("ql-flip");
    }
    return shift;
  }
  // @ts-expect-error
  edit(mode = "link", preview = null, range) {
    this.linkRange = range || this.quill.selection.savedRange;
    this.root.classList.remove("ql-hidden");
    this.root.classList.add("ql-editing");
    if (!editor_utils_es.isNullOrUndefined(preview)) {
      this.textbox.value = preview;
    } else if (mode !== this.root.getAttribute("data-mode")) {
      this.textbox.value = "";
    }
    this.position(this.quill.getBounds(range || this.quill.selection.savedRange));
    if (this.textbox.value === "") {
      this.textbox.focus();
    }
    this.textbox.setAttribute(
      "placeholder",
      this.textbox.getAttribute(`data-${mode}`) || ""
    );
    this.root.setAttribute("data-mode", mode);
  }
  show() {
    super.show();
    this.root.removeAttribute("data-mode");
  }
  addHttpProtocol(url) {
    let result = url;
    if (!url) {
      return "";
    }
    if (!editor_utils_es.hadProtocol(url)) {
      result = `${this.options.autoProtocol}://${url}`;
    }
    return result;
  }
};
_Tooltip.TEMPLATE = [
  `<input type="text" data-formula="e=mc^2" data-link="${editor_config_es.LANG_CONF["en-US"].linkplaceholder}" data-video="Embed URL" style="width: 225px;">`,
  '<span class="ql-split"></span>',
  '<a class="ql-preview"><i class="icon-share"></i></a>',
  '<a class="ql-remove"><i class="icon-delete"></i></a>'
].join("");
let Tooltip = _Tooltip;
exports.default = Tooltip;
