"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const defendXSS = require("../../util/defendXSS.js");
const chartToken = require("./chartToken.js");
function setStack(baseOption, iChartOption, legendData, seriesData) {
  var type = iChartOption.type;
  if (type && type === "stack") {
    baseOption.series.forEach(function(item) {
      item.itemStyle.borderWidth = chartToken.default.borderWidth;
      item.itemStyle.borderColor = chartToken.default.borderColor;
    });
    var direction = iChartOption.direction;
    iChartOption.data.forEach(function(item, i) {
      for (var j = legendData.length - 1; j >= 0; j--) {
        var name = legendData[j];
        if (item[name]) {
          seriesData[name][i] = {
            value: seriesData[name][i],
            itemStyle: {
              borderRadius: direction === "horizontal" ? [0, chartToken.default.borderRadius, chartToken.default.borderRadius, 0] : [chartToken.default.borderRadius, chartToken.default.borderRadius, 0, 0]
            }
          };
          break;
        }
      }
    });
  }
}
function setAbsoluteYaxisLabel(baseOption) {
  var yAxis = baseOption.yAxis;
  yAxis.forEach(function(item) {
    item.axisLabel.formatter = function(value) {
      return Math.abs(value);
    };
  });
}
function setTipFormatter(params, ticket, callback) {
  var html = "";
  params.forEach(function(item, index) {
    if (index === 0) {
      html += '<div style="margin-bottom:4px;">' + defendXSS.default(item.name) + "</div>";
    }
    html += '<div>\n                <span style="display:inline-block;width:10px;height:10px;border-radius:5px;background-color:' + defendXSS.default(item.color) + ';"></span>\n                <span style="margin-left:5px;color:#000000">\n                    <span style="display:inline-block; margin-right:8px;min-width:48px;">' + defendXSS.default(item.seriesName) + '</span> \n                    <span style="font-weight:bold">' + defendXSS.default(item.value ? Math.abs(item.value) : "-") + "</span>\n                </span>\n            </div>";
  });
  return html;
}
function setDoubleSides(baseOption, iChartOption) {
  var type = iChartOption.type;
  if (type && type === "double-sides") {
    setAbsoluteYaxisLabel(baseOption);
    if (!baseOption.tooltip.formatter) {
      baseOption.tooltip.formatter = setTipFormatter;
    }
  }
}
function setDirection(baseOption, direction) {
  if (direction && direction === "horizontal") {
    var temp = baseOption.xAxis;
    baseOption.xAxis = baseOption.yAxis;
    baseOption.yAxis = temp;
  }
}
exports.setDirection = setDirection;
exports.setDoubleSides = setDoubleSides;
exports.setStack = setStack;
