"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const color = require("../../util/color.js");
const cloneDeep = require("../../util/cloneDeep.js");
const index = require("../../option/config/mark/index.js");
const chartToken = require("./chartToken.js");
const merge = require("../../util/merge.js");
var seriesInit = {
  // 数据
  data: [],
  // 气泡图
  type: "scatter",
  // 鼠标hover时显示label，并且其他legend变成灰色
  emphasis: {
    label: {
      show: true,
      color: "#ffffff",
      fontSize: 14,
      formatter: function formatter(param) {
        return param.data[3];
      },
      position: "top"
    }
  },
  // 气泡失去焦点时，配置样式
  blur: {},
  // 气泡样式
  itemStyle: {}
};
function setSeries(_ref) {
  var legendData = _ref.legendData, data = _ref.data, markLine = _ref.markLine, color$1 = _ref.color, iChartOption = _ref.iChartOption;
  seriesInit.emphasis.label.color = chartToken.default.emphasisLabelColor;
  var series = [];
  legendData.forEach(function(legend, index$1) {
    var seriesUnit = cloneDeep.default(seriesInit);
    var itemBorderColor = color.getColor(color$1, index$1);
    var itemColor = color.codeToRGB(itemBorderColor, iChartOption.symbolOpacity || 0.2);
    if (markLine) {
      seriesUnit.markLine = index.getMarkLineDefault(true);
      if (markLine.y) {
        seriesUnit.markLine.data.push({
          yAxis: markLine.y
        });
      }
      if (markLine.x) {
        seriesUnit.markLine.data.push({
          xAxis: markLine.x
        });
      }
    }
    seriesUnit.name = legend;
    seriesUnit.data = data[legend];
    seriesUnit.itemStyle = {
      color: itemColor,
      borderColor: itemBorderColor,
      borderWidth: 1
    };
    series.push(seriesUnit);
  });
  return series;
}
function handleSeriesExtra(baseOpt, iChartOption) {
  var symbol = iChartOption.symbol, symbolRotate = iChartOption.symbolRotate, symbolOffset = iChartOption.symbolOffset, cursor = iChartOption.cursor, label = iChartOption.label, itemStyle = iChartOption.itemStyle, blur = iChartOption.blur, emphasis = iChartOption.emphasis;
  baseOpt.series.forEach(function(item) {
    item.symbol = symbol;
    item.symbolRotate = symbolRotate;
    item.symbolOffset = symbolOffset;
    item.cursor = cursor;
    item.label = label;
    merge.default(item.itemStyle, itemStyle);
    merge.default(item.blur, blur);
    merge.default(item.emphasis, emphasis);
  });
}
exports.handleSeriesExtra = handleSeriesExtra;
exports.seriesInit = seriesInit;
exports.setSeries = setSeries;
