"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const color = require("../../util/color.js");
const type = require("../../util/type.js");
const chartToken = require("./chartToken.js");
const index = require("../../feature/token/index.js");
function handleSetColor(data, opacity) {
  var colorState = index.default.config.colorState;
  var stateColorGroup = {
    info: colorState.colorInfo,
    error: colorState.colorError,
    warning: colorState.colorAlert,
    subwarning: colorState.colorWarning,
    success: colorState.colorSuccess
  };
  var backgroundColor;
  if (data.color) {
    backgroundColor = data.color;
  } else {
    backgroundColor = stateColorGroup[data.name] ? color.codeToRGB(stateColorGroup[data.name], opacity) : "";
  }
  return backgroundColor;
}
function handleLabel(barData, iChartOpt, baseOpt) {
  var label = iChartOpt.label;
  var colorBoard = index.default.config.colorBoard;
  var orange = colorBoard.orange.colorOrange40;
  var green = colorBoard.green.colorGreen40;
  var labelOption = label;
  if (labelOption && labelOption.show) {
    var showLabelArr = [];
    barData.forEach(function(item, index2) {
      var markLineNum = type.isArray(iChartOpt.markLine.data) ? iChartOpt.markLine.data[index2] : iChartOpt.markLine.data;
      var showLabel = Math.round((item - markLineNum) / markLineNum * 100);
      showLabelArr.push(showLabel);
    });
    var categoryX = {
      type: "category",
      position: "top",
      data: showLabelArr,
      axisLabel: {
        show: true,
        fontSize: chartToken.default.fontSize,
        formatter: function formatter(value) {
          return value + "%";
        },
        color: function color2(value) {
          return value >= 0 ? green : orange;
        }
      },
      axisLine: {
        show: false
      },
      axisTick: {
        show: false
      }
    };
    baseOpt.xAxis.push(categoryX);
  }
}
function handleSeries(baseOpt, iChartOpt, legendData, seriesData) {
  var _iChartOpt$itemStyle, _iChartOpt$itemStyle2;
  var series = [];
  var barWidth = (_iChartOpt$itemStyle = iChartOpt.itemStyle) != null && _iChartOpt$itemStyle.barWidth ? iChartOpt.itemStyle.barWidth : chartToken.default.barWidth;
  var backgroundWidth = (_iChartOpt$itemStyle2 = iChartOpt.itemStyle) != null && _iChartOpt$itemStyle2.backgroundWidth ? iChartOpt.itemStyle.backgroundWidth : chartToken.default.barBgWidth;
  var barData = seriesData[legendData[0]];
  var radius = chartToken.default.borderRadius;
  var borderRadius = [radius, radius, 0, 0];
  if (iChartOpt.direction === "horizontal") {
    borderRadius = [0, radius, radius, 0];
  }
  var barObj = {
    name: legendData[0],
    type: "bar",
    barWidth,
    z: 10,
    data: barData,
    symbol: "rect",
    symbolRepeat: false,
    itemStyle: {
      borderRadius
    }
  };
  handleLabel(barData, iChartOpt, baseOpt);
  var markLineData = type.isArray(iChartOpt.markLine.data) ? iChartOpt.markLine.data : iChartOpt.data.map(function() {
    return iChartOpt.markLine.data;
  });
  var scatterObj = {
    type: "scatter",
    symbol: "rect",
    silent: true,
    symbolSize: iChartOpt.markLine.symbolSize ? iChartOpt.markLine.symbolSize : [32, 2],
    symbolOffset: iChartOpt.markLine.symbolOffset ? iChartOpt.markLine.symbolOffset : [0, 0],
    z: 20,
    data: markLineData,
    color: iChartOpt.markLine.color ? iChartOpt.markLine.color : handleSetColor(iChartOpt.markLine, 1),
    emphasis: {
      scale: false
    }
  };
  if (iChartOpt.background) {
    iChartOpt.background.forEach(function(item) {
      var rectData = type.isArray(item.data) ? item.data : iChartOpt.data.map(function() {
        return item.data;
      });
      var totalData = {
        type: "bar",
        barWidth: backgroundWidth,
        stack: "total",
        data: rectData,
        color: iChartOpt.theme.indexOf("dark") !== -1 ? handleSetColor(item, 0.3) : handleSetColor(item, 0.15),
        barGap: 0
      };
      if (iChartOpt.direction === "horizontal") {
        totalData.yAxisIndex = 1;
      } else {
        totalData.xAxisIndex = 1;
      }
      series.push(totalData);
    });
    var xaxisData = Object.assign({}, baseOpt.xAxis[0]);
    xaxisData.show = false;
    baseOpt.xAxis.push(xaxisData);
  }
  series.unshift(scatterObj);
  series.unshift(barObj);
  baseOpt.series = series;
}
exports.handleSeries = handleSeries;
