"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const color = require("../../util/color.js");
const cloneDeep = require("../../util/cloneDeep.js");
const chartToken = require("./chartToken.js");
const index = require("../../feature/token/index.js");
var emptySeriesUnit = {
  type: "gauge",
  startAngle: 0,
  endAngle: 0,
  center: ["50%", "50%"],
  radius: "50%",
  splitNumber: 2,
  axisLine: {
    show: false
  },
  axisLabel: {
    show: false
  },
  splitLine: {
    show: false
  },
  itemStyle: {
    show: false
  },
  axisTick: {
    show: false
  },
  pointer: {
    show: false
  },
  detail: {
    show: false
  },
  title: {
    show: false
  },
  data: []
};
var seriesInit = {
  name: "",
  type: "gauge",
  // 仪表盘轨道底色
  axisLine: {
    roundCap: true,
    lineStyle: {
      width: 10
    }
  },
  // 仪表盘进度条
  progress: {
    show: true,
    roundCap: true,
    width: 10
  },
  // 大刻度数量
  splitNumber: 4,
  // 大刻度线样式
  splitLine: {
    distance: 0,
    length: 10,
    lineStyle: {
      width: 2
    }
  },
  // 大刻度数值
  axisLabel: {
    distance: 16,
    fontSize: 14,
    color: "red"
  },
  // 小刻度线样式
  axisTick: {
    distance: 0,
    length: 10,
    lineStyle: {
      color: "transparent",
      width: 2
    }
  },
  // 指针样式
  pointer: {
    show: false,
    icon: "path://M4.49 10.21L0.1 1.44C-0.23 0.78 0.25 0 1 0L9.76 0C10.5 0 10.99 0.78 10.65 1.44C9.14 4.47 7.33 8.09 6.28 10.21C5.91 10.94 4.86 10.94 4.49 10.21Z",
    length: "5%",
    width: 16,
    offsetCenter: [0, "-105%"]
  },
  title: {
    show: false
  },
  detail: {
    valueAnimation: true
  },
  data: []
};
function handleSplitLine(iChartOption, seriesUnit) {
  if (iChartOption.splitLine) {
    if (iChartOption.splitLine.show !== void 0) {
      seriesUnit.splitLine.show = iChartOption.splitLine.show;
    } else {
      seriesUnit.splitLine.show = true;
    }
  } else {
    seriesUnit.splitLine.show = true;
  }
  if (seriesUnit.splitLine.show) {
    seriesUnit.splitNumber = iChartOption.splitNumber || 4;
  } else {
    seriesUnit.splitNumber = -1;
  }
  if (iChartOption.itemStyle) {
    var _iChartOption$itemSty = iChartOption.itemStyle, lineStyle = _iChartOption$itemSty.lineStyle, width = _iChartOption$itemSty.width;
    seriesUnit.splitLine.length = lineStyle && lineStyle.length || width || 10;
    seriesUnit.splitLine.distance = width ? width * -1 : -3;
  } else {
    seriesUnit.splitLine.length = 7;
    seriesUnit.splitLine.distance = 0;
  }
  if (iChartOption.itemStyle && iChartOption.itemStyle.lineStyle) {
    if (iChartOption.itemStyle.lineStyle.color) {
      seriesUnit.splitLine.lineStyle.color = iChartOption.itemStyle.lineStyle.color;
    } else {
      seriesUnit.splitLine.lineStyle.color = chartToken.default.splitLineColor;
    }
    seriesUnit.splitLine.lineStyle.width = iChartOption.itemStyle.lineStyle.width || 4;
  }
}
function handleTheme(iChartOption) {
  var orbitalColor = iChartOption.orbitalColor;
  seriesInit.axisLine.lineStyle.color = [[1, orbitalColor || chartToken.default.axisLineColor]];
  seriesInit.splitLine.lineStyle.color = chartToken.default.splitLineColor;
  seriesInit.axisLabel.color = chartToken.default.axisLabelColor;
}
function handleDetail(seriesUnit, text, data) {
  seriesUnit.detail.formatter = text.formatter || function(value) {
    return "{value|" + value + "}\n{name|" + data[0].name + "}";
  };
  seriesUnit.detail.offsetCenter = text.offset || [0, 0];
  seriesUnit.detail.rich = text.formatterStyle || {
    value: {
      fontSize: 60,
      fontWeight: "bolder",
      color: chartToken.default.detailRichColor
    },
    name: {
      fontSize: 14,
      color: chartToken.default.detailRichColor,
      padding: [24, 0, 0, 0]
    }
  };
}
function handleProgress(seriesUnit, iChartOption, data) {
  var itemStyle = iChartOption.itemStyle;
  seriesUnit.progress.width = itemStyle ? itemStyle.width ? itemStyle.width : 16 : 16;
  if (data && data.length !== 0 && data[0].value === 0) {
    seriesUnit.progress.roundCap = false;
  }
}
function handleAxisLine(seriesUnit, iChartOption) {
  var itemStyle = iChartOption.itemStyle;
  seriesUnit.axisLine.lineStyle.width = itemStyle ? itemStyle.width ? itemStyle.width : 16 : 16;
}
function setSplitColor(series, splitColor) {
  series.axisLine.lineStyle.color = splitColor;
  series.axisLine.roundCap = false;
  series.progress.show = false;
  series.pointer.itemStyle = {
    color: "auto"
  };
}
function setGradientColor(series, gradientColor) {
  var linearColor = {
    type: "linear",
    x: 0,
    y: 0,
    x2: 1 / (series.data[0].value / series.max),
    y2: 0,
    colorStops: gradientColor.map(function(item, index2) {
      return {
        offset: index2 === 0 ? 0 : index2 / (gradientColor.length - 1),
        color: item
      };
    }),
    global: false
  };
  if (linearColor.colorStops.length === 1) {
    linearColor.colorStops.push({
      offset: 1,
      color: gradientColor[0]
    });
  }
  series.progress.itemStyle = {
    color: linearColor
  };
  series.pointer.itemStyle = {
    color: linearColor
  };
}
function setOuterHalo(seriesHalo, splitColor) {
  var temp = cloneDeep.default(emptySeriesUnit);
  var outerGaugeHalo = cloneDeep.default(temp);
  outerGaugeHalo.startAngle = seriesHalo.startAngle;
  outerGaugeHalo.endAngle = seriesHalo.endAngle;
  outerGaugeHalo.center = seriesHalo.center;
  var distance = seriesHalo.pointer.lineDistance.slice(0, seriesHalo.pointer.lineDistance.length - 1) - 0;
  outerGaugeHalo.radius = parseFloat(seriesHalo.radius) + distance + "%";
  outerGaugeHalo.axisLine = {
    lineStyle: {
      width: 1,
      opacity: 0.3,
      color: splitColor
    }
  };
  outerGaugeHalo.splitNumber = splitColor.length;
  outerGaugeHalo.splitLine = {
    // 不同颜色之间的中缝
    distance: 8,
    length: 10,
    lineStyle: {
      width: 2,
      color: ""
    }
  };
  outerGaugeHalo.splitLine.lineStyle.color = "transparent";
  return outerGaugeHalo;
}
function setProgressOuterHalo(series, outerGradientColor, mask, iChartOption) {
  var temp = cloneDeep.default(emptySeriesUnit);
  var linearColor = {
    type: "linear",
    x: 0,
    y: 0,
    x2: 1,
    y2: 0,
    colorStops: outerGradientColor.map(function(item, index2) {
      return {
        offset: index2 === 0 ? 0 : index2 / (outerGradientColor.length - 1),
        color: item
      };
    }),
    global: false
  };
  var outerGauge = cloneDeep.default(temp);
  outerGauge.startAngle = series.startAngle;
  outerGauge.endAngle = series.endAngle;
  if (mask && mask.show) {
    outerGauge.startAngle = series.startAngle - 5;
    outerGauge.endAngle = series.endAngle + 5;
  }
  outerGauge.center = series.center;
  var distance = series.pointer.lineDistance.slice(0, series.pointer.lineDistance.length - 1) - 0;
  outerGauge.radius = parseFloat(series.radius) + distance + "%";
  outerGauge.axisLine = {
    lineStyle: {
      width: 1,
      opacity: 0.3,
      color: [[1, linearColor]]
    }
  };
  if (mask && mask.show) {
    var pointerStyle = iChartOption.pointerStyle;
    outerGauge.axisLine = {
      lineStyle: {
        width: mask.width || (pointerStyle && pointerStyle.lineDistance || "5%").replace("%", "") * 2.5,
        opacity: 0.1,
        color: [[1, linearColor]]
      }
    };
  }
  return outerGauge;
}
function setHightLightGauge(series, iChartOption) {
  var gradientColor = iChartOption.gradientColor, mask = iChartOption.mask;
  if (mask && mask.show) {
    var pointerStyle = iChartOption.pointerStyle;
    var outerGauge = cloneDeep.default(emptySeriesUnit);
    outerGauge.startAngle = series.startAngle - 5;
    outerGauge.endAngle = series.endAngle + 5;
    outerGauge.center = series.center;
    var seriesRadius = series.radius.replace("%", "");
    var pointerRadius = pointerStyle && pointerStyle.lineDistance || "5%";
    outerGauge.radius = seriesRadius - 0 + (pointerRadius.replace("%", "") - 0) + "%";
    if (mask.hightLight !== false) {
      mask.hightLight = true;
    }
    if (mask && mask.hightLight) {
      var linearColor = {
        type: "linear",
        x: 0,
        y: 0,
        x2: 1,
        y2: 0,
        colorStops: gradientColor.map(function(item, index2) {
          return {
            offset: index2 === 0 ? 0 : index2 / (gradientColor.length - 1),
            color: color.codeToRGB(color.codeToHex(item), 0.5)
          };
        }),
        global: false
      };
      var max = iChartOption.max || 100;
      outerGauge.axisLine = {
        lineStyle: {
          width: 1,
          opacity: 1,
          color: [
            // [0.28, linearColor],
            [(iChartOption.data[0].value - 5 * max / 100) / max, color.codeToRGB(color.codeToHex(gradientColor[0]), 0.5)],
            [(iChartOption.data[0].value + 5 * max / 100) / max, color.codeToRGB(color.codeToHex(gradientColor[0]), 1)],
            [1, linearColor]
          ]
        }
      };
    } else {
      var _linearColor = {
        type: "linear",
        x: 0,
        y: 0,
        x2: 1,
        y2: 0,
        colorStops: gradientColor.map(function(item, index2) {
          return {
            offset: index2 === 0 ? 0 : index2 / (gradientColor.length - 1),
            color: item
          };
        }),
        global: false
      };
      outerGauge.axisLine = {
        lineStyle: {
          width: 1,
          opacity: 0.3,
          color: [[1, _linearColor]]
        }
      };
    }
    return outerGauge;
  }
}
function setMarkLine(series, markLine, marklineColor) {
  var temp = cloneDeep.default(emptySeriesUnit);
  var markGauge = cloneDeep.default(temp);
  markGauge.name = "markLine";
  markGauge.min = series.min;
  markGauge.max = series.max;
  markGauge.startAngle = series.startAngle;
  markGauge.endAngle = series.endAngle;
  markGauge.center = series.center;
  markGauge.radius = series.radius;
  markGauge.animation = false;
  markGauge.pointer = {
    icon: "path://M0 0 L30 0 L30 100 L0 100 Z",
    width: 3,
    length: 15,
    offsetCenter: [0, "-86%"],
    itemStyle: {
      color: marklineColor
    }
  };
  markGauge.data = [{
    value: markLine
  }];
  return markGauge;
}
function handleOther(iChartOption, seriesUnit, series, data) {
  var marklineColor = index.default.config.colorState.colorError;
  if (iChartOption.splitColor && iChartOption.splitColor.length > 0) {
    setSplitColor(seriesUnit, iChartOption.splitColor);
  } else if (iChartOption.gradientColor && iChartOption.gradientColor.length > 0) {
    setGradientColor(seriesUnit, iChartOption.gradientColor);
  }
  if (iChartOption.markLine && data[0].value >= iChartOption.markLine) {
    seriesUnit.pointer.itemStyle = {
      color: marklineColor
    };
    seriesUnit.progress.itemStyle = {
      color: marklineColor
    };
  }
  series.push(seriesUnit);
  if (iChartOption.splitColor && iChartOption.splitColor.length > 0) {
    var outerGauge = setOuterHalo(seriesUnit, iChartOption.splitColor);
    if (iChartOption.itemStyle && iChartOption.itemStyle.outerGauge) {
      if (iChartOption.itemStyle.outerGauge.show === false) {
        outerGauge.axisLine.lineStyle.width = 0;
        outerGauge.axisLine.lineStyle.opacity = 0;
      }
    }
    series.push(outerGauge);
  } else if (iChartOption.gradientColor && iChartOption.gradientColor.length > 1) {
    var _outerGauge = setProgressOuterHalo(seriesUnit, iChartOption.gradientColor, iChartOption.mask, iChartOption);
    if (iChartOption.itemStyle && iChartOption.itemStyle.outerGauge) {
      if (!iChartOption.itemStyle.outerGauge.show) {
        _outerGauge.axisLine.lineStyle.width = 0;
        _outerGauge.axisLine.lineStyle.opacity = 0;
      }
    }
    series.push(_outerGauge);
    var hightLightGauge = setHightLightGauge(seriesUnit, iChartOption);
    series.push(hightLightGauge);
  }
  if (iChartOption.markLine) {
    var markGauge = setMarkLine(seriesUnit, iChartOption.markLine, marklineColor);
    series.push(markGauge);
  }
}
function setSeriesInit(seriesUnit, iChartOption) {
  var chartPosition = iChartOption.position || iChartOption.chartPosition || {};
  var pointerStyle = iChartOption.pointerStyle, pointer = iChartOption.pointer, min = iChartOption.min, max = iChartOption.max, startAngle = iChartOption.startAngle, endAngle = iChartOption.endAngle;
  seriesUnit.name = iChartOption.seriesName || iChartOption.name;
  seriesUnit.data = iChartOption.data;
  seriesUnit.pointer.show = pointer || false;
  seriesUnit.pointer.width = pointerStyle && pointerStyle.width || 16;
  seriesUnit.pointer.length = pointerStyle && pointerStyle.length || "5%";
  seriesUnit.pointer.offsetCenter[1] = pointerStyle && pointerStyle.pointerDistance || "-108%";
  seriesUnit.pointer.lineDistance = pointerStyle && pointerStyle.lineDistance || "5%";
  seriesUnit.center = chartPosition.center || ["50%", "50%"];
  seriesUnit.radius = chartPosition.radius || "70%";
  seriesUnit.min = min || 0;
  seriesUnit.max = max || 100;
  seriesUnit.startAngle = startAngle === void 0 ? 225 : startAngle;
  seriesUnit.endAngle = endAngle === void 0 ? -45 : endAngle;
}
function handleSeries(iChartOption) {
  var data = iChartOption.data;
  var text = iChartOption.text || {};
  var axisLabelStyle = iChartOption.axisLabelStyle || {};
  var silent = iChartOption.silent;
  handleTheme(iChartOption);
  if (axisLabelStyle["color"]) {
    seriesInit.axisLabel["color"] = axisLabelStyle["color"];
  } else {
    seriesInit.axisLabel["color"] = chartToken.default.axisLabelColor;
  }
  seriesInit.axisLabel["distance"] = axisLabelStyle["distance"] || 22;
  seriesInit.axisLabel["fontWeight"] = axisLabelStyle["fontWeight"] || 400;
  seriesInit.axisLabel["fontSize"] = axisLabelStyle["fontSize"] || 14;
  var series = [];
  var seriesUnit = cloneDeep.default(seriesInit);
  setSeriesInit(seriesUnit, iChartOption);
  handleSplitLine(iChartOption, seriesUnit);
  handleDetail(seriesUnit, text, data);
  handleProgress(seriesUnit, iChartOption, data);
  handleAxisLine(seriesUnit, iChartOption);
  handleOther(iChartOption, seriesUnit, series, data);
  series[0].silent = silent || false;
  return series;
}
exports.default = handleSeries;
exports.emptySeriesUnit = emptySeriesUnit;
exports.seriesInit = seriesInit;
