"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const color = require("../../util/color.js");
const BaseOption = require("./BaseOption.js");
const index$1 = require("../../option/config/xAxis/index.js");
const index$2 = require("../../option/config/yAxis/index.js");
const index = require("../../option/config/grid/index.js");
const index$3 = require("../../option/config/tooltip/index.js");
const defendXSS = require("../../util/defendXSS.js");
const constants = require("../../util/constants.js");
function setHeatMapDeaultIchartOption(iChartOpt) {
  if (!iChartOpt.color) {
    iChartOpt.color = BaseOption.SYMBOLCOLOR[iChartOpt.type];
  }
  var padding = iChartOpt.chartPadding || iChartOpt.padding;
  if (!padding && iChartOpt.type === BaseOption.CHARTTYPE[1]) {
    iChartOpt.padding = iChartOpt.handle ? [50, 120, 20, 20] : [50, 30, 20, 20];
  }
}
function rectangularFormatter(params) {
  var color$1 = params.color;
  var data = params.data;
  var x = data[0], y = data[1], z = data[2], name = data[3];
  var htmlString = '<div style="margin-bottom:4px;">\n                                矩形热力图\n                            </div>';
  htmlString += '<div style="margin-bottom:4px;">\n                            <span style="display:inline-block;width:10px;height:10px;\n                            margin-right:8px;border-style: solid;border-width:1px;\n                            border-color:' + defendXSS.default(color.changeRgbaOpacity(color$1, 1)) + ";background-color:" + defendXSS.default(color$1) + ';"></span>\n                            <span>' + defendXSS.default(name) + "</span>\n                        </div>";
  htmlString += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;min-width:60px;">x维度</span> \n                                <span>' + defendXSS.default(x) + "</span>\n                            </div>";
  htmlString += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;min-width:60px;">y维度</span> \n                                <span>' + defendXSS.default(y) + "</span>\n                            </div>";
  htmlString += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;\n                                min-width:60px;">透明度维度</span> \n                                <span>' + defendXSS.default(z) + "</span>\n                            </div>";
  return htmlString;
}
function calendarFormatter(params) {
  var color$1 = params.color;
  var data = params.data;
  var name = params.name;
  var z = data[2];
  var htmlDom = '<div style="margin-bottom:4px;">\n                                日历热力图\n                            </div>';
  htmlDom += '<div style="margin-bottom:4px;">\n                            <span style="display:inline-block;width:10px;\n                            height:10px;margin-right:8px;border-style: solid;\n                            border-width:1px;border-color:' + defendXSS.default(color.changeRgbaOpacity(color$1, 1)) + ";background-color:" + defendXSS.default(color$1) + ';"></span>\n                            <span>' + defendXSS.default(name) + "</span>\n                        </div>";
  htmlDom += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;\n                                min-width:60px;">Value</span> \n                                <span>' + defendXSS.default(z) + "</span>\n                            </div>";
  return htmlDom;
}
function hexagonFormatter(params) {
  var color$1 = params.color;
  var data = params.data;
  var x = data[0], y = data[1], z = data[2], name = data[3];
  var html = '<div style="margin-bottom:4px;">\n                            蜂窝热力图\n                            </div>';
  html += '<div style="margin-bottom:4px;">\n                            <span style="display:inline-block;width:10px;\n                            height:10px;margin-right:8px;border-style: solid;\n                            border-width:1px;border-color:' + defendXSS.default(color.changeRgbaOpacity(color$1, 1)) + ";background-color:" + defendXSS.default(color$1) + ';"></span>\n                            <span>' + defendXSS.default(name) + "</span>\n                        </div>";
  html += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;min-width:60px;">x维度</span> \n                                <span>' + defendXSS.default(x) + "</span>\n                            </div>";
  html += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;min-width:60px;">y维度</span> \n                                <span>' + defendXSS.default(y) + "</span>\n                            </div>";
  html += '\n                            <div>\n                                <span style="display:inline-block;margin-right:8px;min-width:60px;">颜色维度</span> \n                                <span>' + defendXSS.default(z) + "</span>\n                            </div>";
  return html;
}
var heatMapTooltipFormatter = {
  RectangularHeatMapChart: rectangularFormatter,
  CalendarHeatMapChart: calendarFormatter,
  HexagonHeatMapChart: hexagonFormatter
};
function setTooltip(baseOpt, iChartOpt, type) {
  function setChartTooltip(opt) {
    var _iChartOpt$tooltip;
    if (!iChartOpt.tipHtml && !(iChartOpt != null && (_iChartOpt$tooltip = iChartOpt.tooltip) != null && _iChartOpt$tooltip.formatter)) {
      opt.formatter = heatMapTooltipFormatter[type];
    }
  }
  baseOpt.tooltip = index$3.default(iChartOpt, constants.CHART_TYPE.HEAT_MAP, setChartTooltip);
}
function handleCalendarYaxis(yAxis, data) {
  yAxis.type = "category";
  yAxis.data = data[1];
  yAxis.splitLine.show = false;
  yAxis.axisLine.show = true;
}
function handleHexagonYaxis(yAxis, data) {
  yAxis.splitLine.show = false;
  yAxis.axisLabel.show = false;
  yAxis.min = 0;
  yAxis.max = data[2];
}
var yAxisHandler = {
  CalendarHeatMapChart: handleCalendarYaxis,
  HexagonHeatMapChart: handleHexagonYaxis
};
function setYaxis(baseOpt, iChartOpt, chartType, data) {
  function setChartYaxis(opt, index2) {
    if (chartType !== BaseOption.CHARTTYPE[0] && index2 === 0) {
      yAxisHandler[chartType](opt, data);
    }
  }
  baseOpt.yAxis = index$2.default(baseOpt, iChartOpt, constants.CHART_TYPE.HEAT_MAP, setChartYaxis);
}
function handleRectangularXaxis(xAxis) {
  xAxis.type = "value";
  xAxis.axisLine.show = false;
}
function handleCalendarXaxis(xAxis, data) {
  xAxis.data = data[0];
  xAxis.axisTick.show = false;
  xAxis.axisLabel.padding = [0, 0, 0, 10 * data[0][0].length / 2];
}
function handleHexagonXaxis(xAxis, data) {
  xAxis.type = "value";
  xAxis.axisLine.show = false;
  xAxis.axisTick.show = false;
  xAxis.axisLabel.show = false;
  xAxis.min = 0;
  xAxis.max = data[1];
}
var xAxisHandler = {
  RectangularHeatMapChart: handleRectangularXaxis,
  CalendarHeatMapChart: handleCalendarXaxis,
  HexagonHeatMapChart: handleHexagonXaxis
};
function setXaxis(baseOpt, iChartOpt, chartType, data) {
  function setChartXaxis(opt, index2) {
    if (index2 === 0) xAxisHandler[chartType](opt, data);
  }
  baseOpt.xAxis = index$1.default(iChartOpt, setChartXaxis);
}
function setGrid(baseOpt, iChartOpt) {
  var basicGrid = index.default(iChartOpt);
  baseOpt.grid = basicGrid;
}
function initRectSys(baseOpt, iChartOpt, chartType, data) {
  baseOpt.color = iChartOpt.color;
  setGrid(baseOpt, iChartOpt);
  setXaxis(baseOpt, iChartOpt, chartType, data);
  setYaxis(baseOpt, iChartOpt, chartType, data);
  setTooltip(baseOpt, iChartOpt, chartType);
}
exports.initRectSys = initRectSys;
exports.setHeatMapDeaultIchartOption = setHeatMapDeaultIchartOption;
