"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const cloneDeep = require("../../util/cloneDeep.js");
const color = require("../../util/color.js");
const defendXSS = require("../../util/defendXSS.js");
function setbarWidth(iChartOption, baseOpt) {
  var _iChartOption$barWidt = iChartOption.barWidth, barWidth = _iChartOption$barWidt === void 0 ? 8 : _iChartOption$barWidt;
  baseOpt.series.forEach(function(series) {
    series.barWidth = barWidth;
  });
}
function setStartAngle(iChartOption, baseOpt) {
  var _iChartOption$angleAx, _iChartOption$radiusA, _iChartOption$radiusA2;
  if (!(iChartOption != null && (_iChartOption$angleAx = iChartOption.angleAxis) != null && _iChartOption$angleAx.startAngle)) {
    if (!iChartOption.startAngle) {
      iChartOption.startAngle = 90;
    }
    baseOpt.angleAxis.startAngle = iChartOption.startAngle;
  }
  if (!(iChartOption != null && (_iChartOption$radiusA = iChartOption.radiusAxis) != null && (_iChartOption$radiusA2 = _iChartOption$radiusA.axisLabel) != null && _iChartOption$radiusA2.align)) {
    if (!iChartOption.labelAlign) {
      iChartOption.labelAlign = "right";
    }
    baseOpt.radiusAxis.axisLabel.align = iChartOption.labelAlign;
  }
}
function handleLegendData(iChartOption, baseOpt) {
  var data = iChartOption.data, color$1 = iChartOption.color;
  data.forEach(function(dataItem) {
    if (!dataItem.children || dataItem.children.length === 0) {
      baseOpt.legend.data = [];
    }
  });
  var reverseData = [].concat(data).map(function(item, index) {
    item.beforeReverseIndex = index;
    return item;
  }).reverse();
  if (Array.isArray(baseOpt.legend.data)) {
    reverseData.forEach(function(dataItem) {
      baseOpt.legend.data.push({
        name: dataItem.name,
        itemStyle: {
          color: color.getColor(color$1, dataItem.beforeReverseIndex)
        }
      });
    });
  }
}
function handleJadeJueFormatter(JadeJueTooltip, baseOpt, type) {
  JadeJueTooltip.formatter = function(params) {
    var htmlString = "";
    var value = params.data.beforeChangeValue;
    var name = params.data.name || params.name;
    if (type === "process") {
      value = baseOpt.series[params.seriesIndex].beforeChangeValue;
      name = baseOpt.series[params.seriesIndex].name;
    }
    htmlString += '<span style="display:inline-block;margin-right:5px;margin-left:8px;border-radius:50%;height:10px;">' + defendXSS.default(name) + "</span><br/>" + ('<span style="display:inline-block;margin-right:5px;margin-left:8px;border-radius:50%;height:10px;width:10px;background:' + defendXSS.default(params.color) + '"></span>') + '<span style="display:inline-block;margin-right:5px;border-radius:50%;height:10px;">' + (defendXSS.default(name) + "</span>") + ('<span style="display:inline-block;margin-right:8px;margin-left:25px;font-weight:bold">' + defendXSS.default(value) + "</span>");
    return htmlString;
  };
}
function setTooltip(formatter, baseOpt, type) {
  var JadeJueTooltip = cloneDeep.default(baseOpt.tooltip);
  if (formatter) {
    JadeJueTooltip.formatter = formatter;
  } else {
    handleJadeJueFormatter(JadeJueTooltip, baseOpt, type);
  }
  JadeJueTooltip.trigger = "item";
  return JadeJueTooltip;
}
function handleMinRatio(iChartOption, baseOpt, type) {
  var barMinRatio = iChartOption.barMinRatio, data = iChartOption.data, tipHtml = iChartOption.tipHtml;
  if (barMinRatio) {
    var minValue = barMinRatio * baseOpt.angleAxis.sum / 100;
    if (type === "process") {
      baseOpt.series.forEach(function(item) {
        item.beforeChangeValue = item.data[0];
        if (item.data[0] <= minValue) {
          item.data[0] = minValue;
        }
      });
    } else {
      baseOpt.series.forEach(function(series, index) {
        series.data.forEach(function(dataItem, index_) {
          if (series.name === dataItem.name) {
            dataItem.beforeChangeValue = dataItem.value;
            if (dataItem.value <= minValue) {
              dataItem.value = minValue;
              data[index].value = minValue;
            }
          }
        });
      });
    }
    baseOpt.tooltip = setTooltip(tipHtml, baseOpt, type);
  }
}
exports.handleLegendData = handleLegendData;
exports.handleMinRatio = handleMinRatio;
exports.setStartAngle = setStartAngle;
exports.setTooltip = setTooltip;
exports.setbarWidth = setbarWidth;
