"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const color = require("../../util/color.js");
const cloneDeep = require("../../util/cloneDeep.js");
const handleOption = require("./handleOption.js");
const chartToken = require("./chartToken.js");
function createSeries(iChartOption, baseOpt, sum, legendData) {
  var length = 0;
  iChartOption.data.forEach(function(dataItem) {
    if (length <= dataItem.children.length) {
      length = dataItem.children.length;
    }
  });
  if (baseOpt.series.length < length) {
    for (var i = 0; i <= length - baseOpt.series.length + 1; i++) {
      baseOpt.series.push({
        type: "bar",
        data: "",
        stack: "a",
        roundCap: false,
        z: 2,
        coordinateSystem: "polar"
      });
    }
  }
  baseOpt.series.forEach(function(series, index) {
    series.sum = sum;
    series.itemStyle = {
      borderColor: chartToken.default.itemBorderColor,
      borderWidth: 2
    };
    if (index === 0) {
      series.z = 1;
    } else if (index === baseOpt.series.length - 1) {
      series.z = 0;
    } else {
      series.roundCap = false;
    }
  });
  var _loop = function _loop2(_i2) {
    baseOpt.series[_i2].data = [];
    iChartOption.data.forEach(function(item_) {
      baseOpt.series[_i2].data.push(item_.children[_i2]);
      baseOpt.series[_i2].name = legendData[_i2];
    });
  };
  for (var _i = 0; _i < length; _i++) {
    _loop(_i);
  }
}
var setRadiusAxis = function setRadiusAxis2(baseOpt, data, type) {
  baseOpt.radiusAxis.z = 10;
  baseOpt.radiusAxis.data = [];
  if (type === "process") return;
  data.forEach(function(item, index) {
    baseOpt.radiusAxis.data.push(item.name);
  });
};
function setSeriesData(iChartOption, baseOpt) {
  var _iChartOption$showBac = iChartOption.showBackground, showBackground = _iChartOption$showBac === void 0 ? true : _iChartOption$showBac, data = iChartOption.data, color$1 = iChartOption.color, _iChartOption$roundCa = iChartOption.roundCap, roundCap = _iChartOption$roundCa === void 0 ? true : _iChartOption$roundCa, _iChartOption$type = iChartOption.type, type = _iChartOption$type === void 0 ? "base" : _iChartOption$type;
  if (type === "process") {
    roundCap = false;
    data.forEach(function(dataItem, index) {
      baseOpt.series.push({
        type: "bar",
        data: [dataItem.value],
        stack: "a",
        roundCap,
        z: 2,
        coordinateSystem: "polar",
        name: dataItem.name,
        beforeChangeValue: void 0
      });
    });
  } else if (type === "base") {
    data.forEach(function(dataItem, index) {
      var sum = 0;
      data.forEach(function(dataItem2) {
        sum += dataItem2.value;
      });
      var newData = cloneDeep.default(data);
      newData.forEach(function(newItem) {
        newItem.value = 0;
        if (newItem.name === dataItem.name) {
          newItem.value = dataItem.value;
        }
        newItem.itemStyle = {
          color: Array.isArray(color$1) ? color.getColor(color$1, index) : color$1
        };
        newItem.sum = sum;
        newItem.beforeChangeValue = void 0;
      });
      baseOpt.series.push({
        type: "bar",
        data: newData,
        stack: "a",
        // 配置柱体两端是否圆角（在编辑器修改此属性重新调用echarts.setOption，但是视图未更新。bug）
        roundCap,
        z: 2,
        coordinateSystem: "polar",
        name: dataItem.name
      });
    });
  }
  handleOption.handleMinRatio(iChartOption, baseOpt, type);
  setRadiusAxis(baseOpt, data, type);
  if (type === "process") {
    baseOpt.series.forEach(function(item) {
      item.showBackground = showBackground;
    });
  } else {
    data.forEach(function(dataItem) {
      var sum = 0;
      if (type === "base") {
        if (showBackground) {
          if (baseOpt.angleAxis.sum === 0) {
            baseOpt.angleAxis.sum = 1;
            baseOpt.angleAxis.max = 4 / 3;
          }
          var newData = cloneDeep.default(data);
          newData.forEach(function(newItem) {
            newItem.itemStyle = {
              color: chartToken.default.itemColor
            };
            newItem.value = baseOpt.angleAxis.sum - newItem.value;
            newItem.sum = 0;
          });
          baseOpt.series[data.length] = {
            type: "bar",
            data: newData,
            stack: "a",
            z: 1,
            roundCap,
            coordinateSystem: "polar",
            // 关闭灰色进度鼠标hover事件
            silent: true
          };
        }
      } else {
        var legendData = [];
        data.forEach(function(dataItem2) {
          dataItem2.children.forEach(function(child) {
            sum += child.value;
          });
        });
        data.forEach(function(dataItem2) {
          dataItem2.children.forEach(function(child) {
            child.sum = sum;
            legendData.push(child.type);
          });
        });
        legendData = Array.from(new Set(legendData));
        createSeries(iChartOption, baseOpt, sum, legendData);
        baseOpt.color = color$1;
      }
    });
  }
}
exports.setSeriesData = setSeriesData;
