"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const min = require("../../../util/sort/min.js");
const type = require("../../../util/type.js");
const handleOptipn = require("../handleOptipn.js");
const color = require("../../../util/color.js");
const chartToken = require("./chartToken.js");
const index = require("../../../feature/token/index.js");
function gradientBottomArea(item, percent, colorTo, colorFrom) {
  var newSeries = {
    type: item.type,
    name: item.name,
    data: item.data,
    smooth: item.smooth,
    step: item.step,
    lineStyle: {
      width: 0
    },
    symbol: "none",
    areaStyle: {
      origin: "end",
      color: {
        type: "linear",
        x2: 0,
        y2: 1,
        x: 0,
        y: 0,
        colorStops: [{
          offset: 0,
          color: chartToken.default.colorAreaTP
        }, {
          offset: 1 - percent - 1e-5,
          color: chartToken.default.colorAreaTP
        }, {
          offset: 1 - percent,
          color: colorTo
        }, {
          offset: 1,
          color: colorFrom
        }]
      }
    }
  };
  return newSeries;
}
function splitArea(baseOption, iChartOption, YAxiMax) {
  if (iChartOption.area && iChartOption.splitLine) {
    var temp = [];
    var colors = baseOption.color;
    baseOption.series.forEach(function(item, index2) {
      var minValue = min.default(item.data);
      var percent = (iChartOption.splitLine - minValue) / (YAxiMax - minValue);
      var color$1 = color.getColor(colors, index2);
      var colorTo = color.codeToRGB(color$1, 0.15);
      var colorFrom = color.codeToRGB(color$1, 0.15);
      var newSeries = gradientBottomArea(item, percent, colorTo, colorFrom);
      temp.push(newSeries);
    });
    baseOption.series = baseOption.series.concat(temp);
  }
}
function formatter(baseOption, iChartOption) {
  if (!iChartOption.area) return;
  if (iChartOption.splitLine || iChartOption.markLine && iChartOption.markLine.bottom && type.isNumber(iChartOption.markLine.bottom)) {
    var dataLength = baseOption.legend.data.length;
    var tipFormatter = baseOption.tooltip.formatter;
    baseOption.tooltip.formatter = function(params, ticket, callback) {
      return tipFormatter ? tipFormatter(params.slice(0, dataLength), ticket, callback) : handleOptipn.defaultFormatter(params);
    };
  }
}
function pureBottomArea(itemx, percentx, bottomColorx) {
  var seriesObj = {
    type: itemx.type,
    name: itemx.name,
    data: itemx.data,
    smooth: itemx.smooth,
    step: itemx.step,
    symbol: "none",
    areaStyle: {
      color: {
        x: 0,
        y: 0,
        x2: 0,
        y2: 1,
        colorStops: [{
          offset: 0,
          color: chartToken.default.colorAreaTP
        }, {
          offset: 1 - percentx - 1e-5,
          color: chartToken.default.colorAreaTP
        }, {
          offset: 1 - percentx,
          color: bottomColorx
        }, {
          offset: 1,
          color: bottomColorx
        }],
        type: "linear"
      },
      origin: "end"
    },
    lineStyle: {
      width: 0
    }
  };
  return seriesObj;
}
function markLineArea(baseOption, iChartOption, YAxiMax) {
  if (iChartOption.area && iChartOption.markLine && iChartOption.markLine.bottom && type.isNumber(iChartOption.markLine.bottom)) {
    var temp = [];
    baseOption.series.forEach(function(item) {
      var bottomColor = color.codeToRGB(iChartOption.markLine.bottomColor, 0.15) || color.codeToRGB(index.default.config.colorState.colorError, 0.15);
      var minValue = min.default(item.data);
      var percent = (iChartOption.markLine.bottom - minValue) / (YAxiMax - minValue);
      if (iChartOption.markLine.bottom >= minValue) {
        var newSeries = pureBottomArea(item, percent, bottomColor);
        temp.push(newSeries);
      }
    });
    baseOption.series = baseOption.series.concat(temp);
  }
}
function bottomArea(baseOption, iChartOption, YAxiMax) {
  markLineArea(baseOption, iChartOption, YAxiMax);
  splitArea(baseOption, iChartOption, YAxiMax);
  formatter(baseOption, iChartOption);
}
exports.default = bottomArea;
