"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const cloneDeep = require("../../util/cloneDeep.js");
const defendXSS = require("../../util/defendXSS.js");
const chartToken = require("./chartToken.js");
function handleData(baseOpt, legendData, xAxisData) {
  if (!baseOpt.legend.data) {
    baseOpt.legend.data = legendData;
  }
  baseOpt.xAxis.forEach(function(item) {
    item.data = xAxisData;
  });
}
function onlyOnePoint(baseOption) {
  baseOption.series.forEach(function(itemObj) {
    if (itemObj.data.length === 1) {
      itemObj.showSymbol = true;
    }
  });
}
function defaultFormatter(params) {
  var htmlString = "";
  params = params.slice(0, params.length / 2);
  params.forEach(function(item, index) {
    if (index === 0) {
      htmlString += '<div style="margin-bottom:4px;">' + defendXSS.default(item.name) + "</div>";
    }
    htmlString += '<div>\n                            <span style="display:inline-block;width:10px;height:10px;border-radius:5px;background-color:' + defendXSS.default(item.color) + ';"></span>\n                            <span style="margin-left:5px;>\n                                <span style="display:inline-block; margin-right:8px;min-width:60px;">' + defendXSS.default(item.seriesName) + '</span>\n                                <span style="font-weight:bold">' + defendXSS.default(item.value) + "</span>\n                            </span>\n                        </div>";
  });
  return htmlString;
}
function isNullValue(value) {
  return value === "" || value === void 0 || value === null;
}
function addDiscreteVisualMap(baseOption, iChartOption, actualSeriesIndex, seriesIndex, discreteVisualMap) {
  var area = iChartOption.area;
  var visualMap = baseOption.visualMap;
  if (area) return;
  if (visualMap && visualMap.lenth !== 0) {
    var newVisualMapItem = visualMap[seriesIndex] ? cloneDeep.default(visualMap[seriesIndex]) : null;
    if (newVisualMapItem) {
      newVisualMapItem.seriesIndex = actualSeriesIndex;
      discreteVisualMap.push(newVisualMapItem);
    }
  }
}
function discrete(iChartOption, baseOption) {
  if (iChartOption.discrete) {
    var actualSeriesIndex = baseOption.series.length - 1;
    var discreteSeries = [];
    var discreteVisualMap = [];
    baseOption.series.forEach(function(series, seriesIndex) {
      var newSeries = cloneDeep.default(series);
      newSeries.symbol = "circle";
      newSeries.symbolSize = chartToken.default.symbolSizeSM;
      newSeries.itemStyle.borderWidth = chartToken.default.borderZero;
      newSeries.showSymbol = true;
      newSeries.showAllSymbol = true;
      newSeries.emphasis = {
        itemStyle: {
          opacity: 0
        }
      };
      var discreteData = [];
      for (var index = 0; index < newSeries.data.length; index++) {
        var pre = newSeries.data[index - 1];
        var next = newSeries.data[index];
        var cur = newSeries.data[index + 1];
        if (!isNullValue(pre) || !isNullValue(cur)) {
          discreteData.push(null);
        } else {
          discreteData.push(next);
        }
      }
      newSeries.data = discreteData;
      actualSeriesIndex++;
      addDiscreteVisualMap(baseOption, iChartOption, actualSeriesIndex, seriesIndex, discreteVisualMap);
      discreteSeries.push(newSeries);
    });
    baseOption.series = [].concat(baseOption.series, discreteSeries);
    baseOption.visualMap = [].concat(baseOption.visualMap, discreteVisualMap);
    var tipFormatter = baseOption.tooltip.formatter;
    baseOption.tooltip.formatter = function(params, ticket, callback) {
      if (tipFormatter) {
        return tipFormatter(params.slice(0, params.length / 2), ticket, callback);
      } else {
        return defaultFormatter(params);
      }
    };
  }
}
exports.defaultFormatter = defaultFormatter;
exports.discrete = discrete;
exports.handleData = handleData;
exports.onlyOnePoint = onlyOnePoint;
