"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("../../feature/token/index.js");
const cloneDeep = require("../../util/cloneDeep.js");
const defendXSS = require("../../util/defendXSS.js");
const chartToken = require("./chartToken.js");
function setDashedLineVisualMap(seriesIndex, lineColor, predictIndex) {
  var vm = {
    type: "piecewise",
    seriesIndex,
    dimension: 0,
    show: false,
    pieces: [{
      gte: 0,
      lte: predictIndex,
      color: index.default.config.visualMapPiecesColor
    }, {
      gt: predictIndex,
      color: lineColor
    }]
  };
  return vm;
}
function setToolTip(dataLength, fontColor, selfFormatter) {
  if (selfFormatter) {
    return selfFormatter;
  }
  var tipHtml = function tipHtml2(params) {
    var htmlString = "";
    params.forEach(function(item, index2) {
      if (index2 < dataLength) {
        if (index2 === 0) {
          htmlString += defendXSS.default(item.name) + "<br/>";
        }
        var color_ = item.color.colorStops;
        htmlString += '\n                    <div>\n                        <span style="display:inline-block;width:10px;\n                        height:10px;border-radius:5px;background-color:' + defendXSS.default(color_ ? color_[0].color : item.color) + ';">\n                        </span>\n                        <span style="margin-left:5px;color:' + defendXSS.default(fontColor) + '">\n                            <span style="display:inline-block;width:80px;">' + defendXSS.default(item.seriesName) + '</span> \n                            <span style="font-weight:bold">' + defendXSS.default(item.value) + "</span>\n                        </span>\n                    </div>\n                ";
      }
    });
    return htmlString;
  };
  return tipHtml;
}
function handlePredict(option, predict, tipHtml, lineStyle) {
  if (!predict) return;
  var dashColor = index.default.config.visualMapDashColor;
  if (lineStyle && lineStyle.dashColor) {
    dashColor = lineStyle.dashColor;
  }
  var data = option.series;
  var dataLength = data.length;
  option.xAxis[0].data.length;
  var predictIndex = option.xAxis[0].data.indexOf(predict);
  if (predictIndex > -1) {
    for (var index$1 = 0; index$1 < dataLength; index$1++) {
      var temp = cloneDeep.default(data[index$1]);
      temp.lineStyle = {
        // 为了视觉上看不见盖住粗细+1
        width: chartToken.default.lineWidth + 1,
        type: [5, 8]
      };
      temp.itemStyle = {
        opacity: 0
      };
      temp.silent = true;
      temp.showSymbol = false;
      temp.showAllSymbol = false;
      option.series.push(temp);
      option.visualMap.push(setDashedLineVisualMap(dataLength + index$1, dashColor, predictIndex));
    }
  }
  option.tooltip.formatter = setToolTip(dataLength, index.default.config.tooltipTextColor, tipHtml);
}
exports.handlePredict = handlePredict;
