"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const min = require("../../util/sort/min.js");
const max = require("../../util/sort/max.js");
const type = require("../../util/type.js");
const color = require("../../util/color.js");
const index = require("../../feature/token/index.js");
function handleVisualMapItem(_ref) {
  var index2 = _ref.index, topColor = _ref.topColor, top = _ref.top, bottom = _ref.bottom, colors = _ref.colors, bottomColor = _ref.bottomColor, vmColor = _ref.vmColor;
  var visualMapItem = {
    show: false,
    type: "piecewise",
    dimension: 1,
    seriesIndex: index2,
    pieces: [{
      gte: top,
      // 大于 top 的
      color: topColor
    }, {
      gt: bottom,
      lt: top,
      // 小于 top, 大于 bottom 的，为正常颜色
      color: color.getColor(colors, index2)
    }, {
      lte: bottom,
      // 小于 bottom
      color: bottomColor
    }],
    outOfRange: {
      color: vmColor
    }
  };
  return visualMapItem;
}
function setVisualMap(legendData, seriesData, markLine, colors) {
  var visualMap = [];
  if (markLine) {
    var topValue = markLine.top;
    var bottomValue = markLine.bottom;
    var vmColor = index.default.config.colorState.colorError;
    var topColor = markLine.topColor || vmColor;
    var bottomColor = markLine.bottomColor || vmColor;
    if (!type.isNumber(topValue)) {
      topValue = void 0;
    }
    if (!type.isNumber(bottomValue)) {
      bottomValue = void 0;
    }
    if (topValue === void 0 && bottomValue === void 0) {
      return visualMap;
    }
    if (topValue !== void 0 && bottomValue !== void 0 && bottomValue >= topValue) {
      throw new Error("阈值线bottom的值必须小于阈值线top的值");
    }
    legendData.forEach(function(legendName, index2) {
      var data = seriesData[legendName];
      var minData = min.default(data);
      var maxData = max.default(data);
      var bottom = bottomValue;
      var top = topValue;
      if (markLine.topUse && markLine.topUse.indexOf(legendName) === -1) {
        top = void 0;
      }
      if (markLine.bottomUse && markLine.bottomUse.indexOf(legendName) === -1) {
        bottom = void 0;
      }
      if (top === void 0 && bottom === void 0) {
        return;
      }
      if (bottom === void 0) {
        bottom = Math.min(top - 0.01, minData - 0.01);
      }
      if (top === void 0) {
        top = Math.max(bottom + 0.01, maxData + 0.01);
      }
      var visualMapItem = handleVisualMapItem({
        index: index2,
        topColor,
        top,
        bottom,
        colors,
        bottomColor,
        vmColor
      });
      visualMap.push(visualMapItem);
    });
  }
  return visualMap;
}
exports.setVisualMap = setVisualMap;
