"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const merge = require("../../util/merge.js");
const cloneDeep = require("../../util/cloneDeep.js");
const chartToken = require("./chartToken.js");
const index = require("../../feature/token/index.js");
var seriesInit = {
  type: "liquidFill",
  center: ["50%", "50%"],
  waveAnimation: true,
  color: "",
  backgroundStyle: {
    color: ""
  },
  outline: {
    itemStyle: {
      borderColor: "",
      shadowBlur: 0
    }
  },
  itemStyle: {
    opacity: 1,
    shadowBlur: 5,
    shadowColor: "rgba(0, 0, 0, 0.4)"
  },
  label: {},
  data: []
};
function setLabel(seriesTarget, seriesSource, label) {
  var labelColor = chartToken.default.labelColor;
  if (seriesSource.label !== void 0) {
    seriesTarget.label.color = labelColor;
    seriesTarget.label.position = "inside";
    seriesTarget.label.show = true;
    merge.default(seriesTarget.label, seriesSource.label);
  } else if (seriesSource.label === void 0 && label) {
    seriesTarget.label.color = labelColor;
    seriesTarget.label.position = "inside";
    seriesTarget.label.show = true;
    merge.default(seriesTarget.label, label);
  } else {
    seriesTarget.label.color = labelColor;
    seriesTarget.label.position = "inside";
    seriesTarget.label.show = true;
  }
}
function setOutline(seriesTarget, seriesSource, outline) {
  var colorGroup = index.default.config.colorGroup;
  var itemStyle = seriesTarget.outline.itemStyle;
  if (seriesSource.outline !== void 0) {
    itemStyle.borderColor = colorGroup[0];
    merge.default(seriesTarget.outline, seriesSource.outline);
  } else if (seriesSource.outline === void 0 && outline) {
    itemStyle.borderColor = colorGroup[0];
    merge.default(seriesTarget.outline, outline);
  } else {
    itemStyle.borderColor = colorGroup[0];
  }
}
function setColor(seriesTarget, seriesSource, color) {
  var colorGroup = index.default.config.colorGroup;
  if (seriesSource.color !== void 0) {
    seriesTarget.color = [].concat(colorGroup);
    merge.default(seriesTarget.color, seriesSource.color);
  } else if (seriesSource.color === void 0 && color) {
    seriesTarget.color = [].concat(colorGroup);
    merge.default(seriesTarget.color, color);
  } else {
    seriesTarget.color = [].concat(colorGroup);
  }
}
function setBackgroundStyle(seriesTarget, seriesSource, backgroundStyle) {
  var backgroundColor = chartToken.default.backgroundColor;
  if (seriesSource.backgroundStyle !== void 0) {
    seriesTarget.backgroundStyle.color = backgroundColor;
    merge.default(seriesTarget.backgroundStyle, seriesSource.backgroundStyle);
  } else if (seriesSource.backgroundStyle === void 0 && backgroundStyle) {
    seriesTarget.backgroundStyle.color = backgroundColor;
    merge.default(seriesTarget.backgroundStyle, backgroundStyle);
  } else {
    seriesTarget.backgroundStyle.color = backgroundColor;
  }
}
function setSeries(iChartOption) {
  var data = iChartOption.data, shape = iChartOption.shape, color = iChartOption.color, name = iChartOption.name, label = iChartOption.label, outline = iChartOption.outline, backgroundStyle = iChartOption.backgroundStyle, waveAnimation = iChartOption.waveAnimation;
  var series = [];
  var selfSeries = iChartOption.series;
  if (selfSeries !== void 0 && selfSeries.length != 0) {
    selfSeries.forEach(function(item) {
      var seriesUnit2 = cloneDeep.default(seriesInit);
      item.data && (seriesUnit2.data = item.data);
      item.shape && (seriesUnit2.shape = item.shape);
      item.waveAnimation !== void 0 && (seriesUnit2.waveAnimation = item.waveAnimation);
      setColor(seriesUnit2, item, color);
      setBackgroundStyle(seriesUnit2, item, backgroundStyle);
      setOutline(seriesUnit2, item, outline);
      setLabel(seriesUnit2, item, label);
      merge.default(seriesUnit2, item);
      series.push(seriesUnit2);
    });
  } else {
    var seriesUnit = cloneDeep.default(seriesInit);
    data && (seriesUnit.data = data);
    name && (seriesUnit.name = name);
    shape && (seriesUnit.shape = shape);
    waveAnimation !== void 0 && (seriesUnit.waveAnimation = waveAnimation);
    setColor(seriesUnit, iChartOption);
    setBackgroundStyle(seriesUnit, iChartOption);
    setOutline(seriesUnit, iChartOption);
    setLabel(seriesUnit, iChartOption);
    series.push(seriesUnit);
  }
  return series;
}
exports.seriesInit = seriesInit;
exports.setSeries = setSeries;
