"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const merge = require("../../util/merge.js");
const math = require("../../util/math.js");
const defendXSS = require("../../util/defendXSS.js");
var INIT_RADIUS = {
  min: 200,
  gap: 200,
  angle: 30,
  direction: "bottom"
};
var DIRECTION_ANGLE = {
  top: 180,
  left: -90,
  right: 90,
  bottom: 0
};
var LINE_STYLE = {
  color: "#BBBBBB"
};
var NodeManager = /* @__PURE__ */ function() {
  function NodeManager2(dom, option, selected) {
    this.dom = void 0;
    this.data = void 0;
    this.radius = void 0;
    this.selected = void 0;
    this.angles = void 0;
    this.dom = dom;
    this.selected = selected;
    this.data = option.data;
    this.radius = merge.default(INIT_RADIUS, option.radius);
    this.render = option.render;
    this.autoRotate = option.autoRotate;
    this.lineStyle = merge.default(LINE_STYLE, option.lineStyle);
    this.angles = [];
    this.distance = math.getEdge(this.radius.min, this.radius.min, this.radius.angle);
    this.createWarppers();
    this.createInitNodes();
  }
  var _proto = NodeManager2.prototype;
  _proto.createWarppers = function createWarppers() {
    var _this = this;
    this.depth = this.getDataDepth(this.data, 0);
    for (var index = 0; index < this.depth; index++) {
      var _this$autoRotate;
      var radius = this.radius.min + this.radius.gap * index;
      var width = defendXSS.default(radius * 2);
      var zindex = defendXSS.default(100 - index);
      var inertia = (_this$autoRotate = this.autoRotate) == null ? void 0 : _this$autoRotate.inertia;
      var warpper = '<div class="ozc_warpper ' + (inertia === false ? "ozc_no_inertia" : "") + `" style='width: ` + width + "px;height: " + width + "px;z-index: " + zindex + "; transform: translateX(-50%) translateY(-50%) rotate(0deg);'></div>";
      this.dom.insertAdjacentHTML("beforeend", warpper);
    }
    this.warppers = this.dom.getElementsByClassName("ozc_warpper");
    Array.from(this.warppers).forEach(function(element) {
      element.style.setProperty("--lineColor", _this.lineStyle.color);
    });
  };
  _proto.createInitNodes = function createInitNodes() {
    this.createBoss();
    var data = [this.data];
    for (var level = 0; level < this.depth; level++) {
      var _data$preSelect;
      var preSelect = this.selected[level - 1] || 0;
      var nextSelect = this.selected[level] || 0;
      data = ((_data$preSelect = data[preSelect]) == null ? void 0 : _data$preSelect.children) || [];
      this.createLevel(data, nextSelect, level);
    }
  };
  _proto.createBoss = function createBoss() {
    var boss = this.createNode(this.data, "left: 50%;top: 50%;transform: translateX(-50%) translateY(-50%);");
    this.dom.appendChild(boss);
  };
  _proto.createLevel = function createLevel(data, selectedIndex, warpperIndex) {
    var _this2 = this;
    var warpper = this.warppers[warpperIndex];
    if (!data || data.length == 0) {
      return;
    } else {
      warpper.style.display = "block";
    }
    var radius = this.radius.min + warpperIndex * this.radius.gap;
    var distance = this.distance + warpperIndex * 10;
    var angleUnit = math.getAngle(radius, radius, distance)[2];
    var initAngle = selectedIndex * angleUnit * -1 + DIRECTION_ANGLE[this.radius.direction];
    this.angles[warpperIndex] = angleUnit;
    data.forEach(function(user, i) {
      var center = {
        x: radius,
        y: radius
      };
      var angle = initAngle + angleUnit * i;
      var left = center.x + radius * Math.sin(Math.PI * angle / 180);
      var top = center.y + radius * Math.cos(Math.PI * angle / 180);
      var style = "left: " + left + "px;top: " + top + "px;";
      var node = _this2.createNode(user, style, warpperIndex, i);
      warpper.appendChild(node);
    });
  };
  _proto.refreshWarpper = function refreshWarpper(warpperIndex, selected) {
    this.selected = selected;
    for (var level = warpperIndex + 1; level < this.depth; level++) {
      this.warppers[level].style.transform = "translateX(-50%) translateY(-50%) rotate(0deg)";
      this.warppers[level].innerHTML = "";
      var depthData = this.getDepthData(this.data, level);
      var nextSelect = this.selected[level] || 0;
      this.createLevel(depthData, nextSelect, level);
    }
  };
  _proto.createNode = function createNode(data, style, warpperIndex, nodeIndex) {
    var _this$autoRotate2;
    var node = document.createElement("div");
    node.setAttribute("style", style);
    var inertia = (_this$autoRotate2 = this.autoRotate) == null ? void 0 : _this$autoRotate2.inertia;
    var nodeClass = inertia === false ? "ozc_card ozc_no_inertia" : "ozc_card";
    node.setAttribute("class", nodeClass);
    node.setAttribute("data-node-index", nodeIndex);
    node.setAttribute("data-warpper-index", warpperIndex);
    var state = {
      boss: warpperIndex == void 0,
      selected: this.selected[warpperIndex] === nodeIndex ? true : false
    };
    this.render && this.render(node, data, state);
    return node;
  };
  _proto.getDepthData = function getDepthData(data, targetDepth) {
    var result = [data];
    for (var level = 0; level < this.depth; level++) {
      var _result$preSelect;
      var preSelect = this.selected[level - 1] || 0;
      result = ((_result$preSelect = result[preSelect]) == null ? void 0 : _result$preSelect.children) || [];
      if (level === targetDepth) {
        break;
      }
    }
    return result;
  };
  _proto.getDataDepth = function getDataDepth(data, depth) {
    var _this3 = this;
    if (data && data.children && data.children.length > 0) {
      depth++;
      var childDepths = data.children.map(function(child) {
        return _this3.getDataDepth(child, depth);
      });
      depth = Math.max.apply(Math, childDepths);
    }
    return depth;
  };
  return NodeManager2;
}();
exports.INIT_RADIUS = INIT_RADIUS;
exports.default = NodeManager;
