"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const dom = require("../../util/dom.js");
const math = require("../../util/math.js");
var RotateManager = /* @__PURE__ */ function() {
  function RotateManager2(dom2, option, selected, nodeManager) {
    this.currentTarget = null;
    this.dom = dom2;
    this.data = option.data;
    this.selected = selected;
    this.nodeManager = nodeManager;
    this.angles = this.nodeManager.angles;
    this.drag = false;
    if (option.autoRotate) {
      this.autoRotate(option.autoRotate);
    } else {
      this.initWarppersEvent();
      this.initNodesEvent(this.dom);
    }
  }
  var _proto = RotateManager2.prototype;
  _proto.initWarppersEvent = function initWarppersEvent() {
    var _this = this;
    this.warppers = this.dom.getElementsByClassName("ozc_warpper");
    Array.from(this.warppers).forEach(function(element, index) {
      element.addEventListener("mousedown", function(e) {
        var currentTargetRect = e.currentTarget.getBoundingClientRect();
        _this.drag = false;
        _this.currentTarget = {
          dom: e.currentTarget,
          // 旋转元素
          index,
          // 旋转层数
          center: {
            // 旋转中心位置
            x: currentTargetRect.x + currentTargetRect.width / 2,
            y: currentTargetRect.y + currentTargetRect.height / 2
          },
          startRotate: _this.getRotateDeg(e.currentTarget),
          // 初始旋转角度
          startTouchPosition: {
            // 鼠标拖动的起始位置
            x: e.clientX,
            y: e.clientY
          },
          clockwise: void 0
          // true 表示顺时针方向
        };
      });
      element.addEventListener("mousemove", function(e) {
        if (_this.currentTarget) {
          _this.drag = true;
          var curLevel = _this.currentTarget.index;
          _this.currentTarget.endTouchPosition = {
            x: e.clientX,
            y: e.clientY
          };
          var clockwise = math.pointsDirection(_this.currentTarget.center, _this.currentTarget.startTouchPosition, _this.currentTarget.endTouchPosition);
          var angle = math.getAngleByPoints(_this.currentTarget.center, _this.currentTarget.startTouchPosition, _this.currentTarget.endTouchPosition)[1];
          if (_this.currentTarget.clockwise === void 0) {
            _this.currentTarget.clockwise = clockwise;
          } else if (_this.currentTarget.clockwise !== clockwise) {
            if (angle > 10) {
              angle = 360 - angle;
            } else {
              _this.currentTarget.clockwise = clockwise;
            }
          }
          var rotate = angle;
          rotate = angle * (_this.currentTarget.clockwise ? 1 : -1);
          rotate = _this.currentTarget.startRotate + rotate;
          var selected = _this.getSelected(rotate).selected;
          var prevSelected = _this.selected[curLevel];
          _this.setRotateDeg(_this.currentTarget.dom, curLevel, rotate, selected, prevSelected);
        }
      });
      element.addEventListener("mouseup", function(e) {
        if (_this.drag && _this.currentTarget) {
          var curIndex = _this.currentTarget.index;
          var endRotate = _this.getRotateDeg(_this.currentTarget.dom);
          var result = _this.getSelected(endRotate, true);
          endRotate = result.rotate;
          _this.selected[curIndex] = result.selected;
          _this.nodeManager.refreshWarpper(curIndex, _this.selected);
          _this.setRotateDeg(_this.currentTarget.dom, curIndex, endRotate, _this.selected[curIndex], -1);
        }
        _this.currentTarget = null;
      });
    });
    this.dom.addEventListener("mouseleave", function(e) {
      if (_this.drag && _this.currentTarget) {
        var mouseupEvent = new Event("mouseup");
        _this.currentTarget.dom.dispatchEvent(mouseupEvent);
        e.stopPropagation();
        e.preventDefault();
      }
    });
  };
  _proto.initNodesEvent = function initNodesEvent(container) {
    var _this2 = this;
    container.addEventListener("click", function(e) {
      var parent = dom.isParent(e.target, "ozc_card");
      if (!_this2.drag && parent) {
        var warpperIndex = parseInt(parent.getAttribute("data-warpper-index"));
        var nodeIndex = parseInt(parent.getAttribute("data-node-index"));
        var warpperDom = _this2.nodeManager.warppers[warpperIndex];
        var moveIndex = _this2.selected[warpperIndex] - nodeIndex;
        var rotate = _this2.angles[warpperIndex] * moveIndex * -1;
        rotate = _this2.getRotateDeg(warpperDom) + rotate;
        _this2.selected[warpperIndex] = nodeIndex;
        _this2.setRotateDeg(warpperDom, warpperIndex, rotate, nodeIndex, -1);
        _this2.nodeManager.refreshWarpper(warpperIndex, _this2.selected);
      }
    });
  };
  _proto.setRotateDeg = function setRotateDeg(dom2, domIndex, rotate, selected, prevSelected) {
    var _this3 = this;
    dom2.style.transform = " translateX(-50%) translateY(-50%) rotate(" + rotate + "deg)";
    var data = this.nodeManager.getDepthData(this.data, domIndex);
    Array.from(dom2.getElementsByClassName("ozc_card")).forEach(function(element, index) {
      element.style.transform = "rotate(" + -rotate + "deg)";
      element.innerHTML = "";
      _this3.nodeManager.render && _this3.nodeManager.render(element, data[index], {
        boss: false,
        selected: selected === index ? true : false,
        prevSelected: prevSelected === index ? true : false
      });
    });
  };
  _proto.getRotateDeg = function getRotateDeg(element) {
    var transform = element.style.transform;
    var degIndex = transform.indexOf("deg)");
    var rotateIndex = transform.indexOf("rotate(");
    var targetRotate = parseFloat(transform.slice(rotateIndex + "rotate(".length, degIndex));
    return targetRotate;
  };
  _proto.getSelected = function getSelected(rotate, correct) {
    var curIndex = this.currentTarget.index;
    var unitAngle = this.angles[curIndex];
    if (Math.abs(rotate % unitAngle) >= unitAngle / 2) {
      if (rotate >= 0) {
        rotate = (Math.floor(rotate / unitAngle) + 1) * unitAngle;
      } else {
        rotate = (Math.ceil(rotate / unitAngle) - 1) * unitAngle;
      }
    } else {
      if (rotate >= 0) {
        rotate = Math.floor(rotate / unitAngle) * unitAngle;
      } else {
        rotate = Math.ceil(rotate / unitAngle) * unitAngle;
      }
    }
    var dataLength = this.nodeManager.getDepthData(this.data, curIndex).length;
    var changeIndex = (rotate - this.currentTarget.startRotate) / unitAngle;
    var endIndex = parseInt(changeIndex.toFixed()) + this.selected[curIndex];
    var selectedIndex = 0;
    if (endIndex >= dataLength) {
      selectedIndex = dataLength - 1;
      correct && (rotate = rotate - (endIndex - (dataLength - 1)) * unitAngle);
    } else if (endIndex < 0) {
      selectedIndex = 0;
      correct && (rotate = rotate + Math.abs(endIndex) * unitAngle);
    } else {
      selectedIndex = endIndex;
    }
    return {
      rotate,
      selected: selectedIndex
    };
  };
  _proto.autoRotate = function autoRotate(option) {
    var current = 0;
    var speed = option.speed || 1;
    var direction = option.direction || "clockwise";
    var warppers = this.dom.getElementsByClassName("ozc_warpper");
    function rotate() {
      if (direction === "clockwise") {
        current += speed;
      } else {
        current -= speed;
      }
      Array.from(warppers).forEach(function(element) {
        element.style.transform = " translateX(-50%) translateY(-50%) rotate(" + current + "deg)";
        Array.from(element.getElementsByClassName("ozc_card")).forEach(function(card) {
          card.style.transform = "rotate(" + -current + "deg)";
        });
      });
      window.requestAnimationFrame(rotate);
    }
    window.requestAnimationFrame(rotate);
  };
  return RotateManager2;
}();
exports.default = RotateManager;
