"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const color = require("../../util/color.js");
const chartToken = require("./chartToken.js");
function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t = arguments[e];
      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
function getSeriesInit(type) {
  var baseSeries = {
    type: "bar",
    coordinateSystem: "polar",
    emphasis: {
      focus: "series"
    },
    animation: true,
    data: []
  };
  return type === "normal" ? _extends({}, baseSeries, {
    barGap: "-100%",
    barCategoryGap: "40%",
    itemStyle: {
      borderRadius: [0, 0, 2, 2]
    }
  }) : _extends({}, baseSeries, {
    barWidth: "96%",
    itemStyle: {}
  });
}
function getPieInit() {
  return {
    type: "pie",
    label: {
      show: true,
      fontSize: 12,
      color: chartToken.default.labelColor,
      position: "inside"
    },
    itemStyle: {
      color: "transparent"
    },
    animation: false,
    silent: true,
    data: void 0,
    center: void 0
  };
}
function handleBarItemStyle(iChartOption, seriesUnit) {
  var color$1 = iChartOption.color, data = iChartOption.data;
  var seriesColor = {};
  var typeArr = [];
  data.forEach(function(item, index) {
    if (typeArr.indexOf(item.type) === -1) {
      seriesColor[item.type] = [];
      typeArr.push(item.type);
    }
    seriesColor[item.type].push(index);
  });
  seriesUnit.itemStyle.color = function(_ref) {
    var dataIndex = _ref.dataIndex;
    var num = 0;
    for (var i in seriesColor) {
      if (Object.hasOwnProperty.call(seriesColor, i)) {
        if (seriesColor[i].indexOf(dataIndex) !== -1) {
          return color.getColor(color$1, num);
        }
        num++;
      }
    }
  };
}
function setSeries(seriesData, labelData, iChartOption, polar, type) {
  var data = iChartOption.data, label = iChartOption.label;
  var series = [];
  if (type === "normal") {
    data.forEach(function(item, i) {
      var seriesUnit2 = getSeriesInit(type);
      seriesUnit2.name = item.name;
      seriesUnit2.data = seriesData[i];
      series.push(seriesUnit2);
    });
  } else {
    var seriesUnit = getSeriesInit(type);
    seriesUnit.data = seriesData;
    handleBarItemStyle(iChartOption, seriesUnit);
    series.push(seriesUnit);
  }
  var showLabel = label ? label.show : true;
  if (showLabel && type === "normal") {
    var pieUnit = getPieInit();
    pieUnit.data = labelData;
    pieUnit.center = polar.center;
    var radius = polar.radius[1];
    var radiusN = Number(radius.substring(0, radius.length - 1));
    pieUnit.radius = [radius, radiusN + 8 + "%"];
    series.push(pieUnit);
  }
  return series;
}
exports.handleBarItemStyle = handleBarItemStyle;
exports.setSeries = setSeries;
