"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const BaseOption = require("./BaseOption.js");
const cloneDeep = require("../../util/cloneDeep.js");
const color = require("../../util/color.js");
const merge = require("../../util/merge.js");
const dom = require("../../util/dom.js");
const chartToken = require("./chartToken.js");
const index = require("../../feature/token/index.js");
function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t = arguments[e];
      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
function setStateBarColor(data, stateColorGroup, successColor, stateList) {
  var min = stateList[0];
  var max = stateList[stateList.length - 1];
  if (min.value === max.value) {
    var resState = min.state;
    return data < min.value ? successColor : stateColorGroup[resState];
  } else {
    if (data < min.value) {
      return successColor;
    }
    if (data > max.value) {
      var _resState = max.state;
      return stateColorGroup[_resState];
    }
    var len = stateList.length;
    var color2;
    for (var i = 0; i < len; i++) {
      if (stateList[i].value >= data) {
        var index2 = i > 0 ? i - 1 : 0;
        var _resState2 = stateList[index2].state;
        color2 = stateColorGroup[_resState2];
        break;
      }
    }
    return color2;
  }
}
function getStateList(state) {
  var stateList = [];
  for (var k in state) {
    if (Object.hasOwnProperty.call(state, k)) {
      var stateItem = {
        state: k,
        value: state[k]
      };
      stateList.push(stateItem);
    }
  }
  stateList.sort(function(a, b) {
    return a.value - b.value;
  });
  return stateList;
}
function handleBarColor(params, iChartOpt, dataSet, doubleSide, isTip) {
  if (isTip === void 0) {
    isTip = false;
  }
  var state = iChartOpt.state, color$1 = iChartOpt.color;
  if (state) {
    var colorState = index.default.config.colorState;
    var stateColorGroup = {
      error: colorState.colorError,
      warning: colorState.colorAlert,
      subwarning: colorState.colorWarning,
      success: colorState.colorSuccess
    };
    var successColor = stateColorGroup.success;
    var stateList = getStateList(state);
    return setStateBarColor(isTip ? dataSet.barData[params.dataIndex].value : params.data.value, stateColorGroup, successColor, stateList);
  } else {
    return color.getColor(color$1, doubleSide ? Math.floor(params.seriesIndex / 2) : params.dataIndex);
  }
}
function handleBasicProcessSeries(baseOpt, iChartOpt, dataSet, doubleSide) {
  baseOpt.series = doubleSide ? cloneDeep.default(BaseOption.DOUBLEBASICOPTION.series) : cloneDeep.default(BaseOption.BASICSERIES);
  if (doubleSide) {
    baseOpt.series[0].data = dataSet.barData[0];
    baseOpt.series[2].data = dataSet.barData[1];
    baseOpt.series[0].name = dataSet.seriesName[0];
    baseOpt.series[2].name = dataSet.seriesName[1];
    baseOpt.series[0].itemStyle.color = color.getColor(iChartOpt.color, 0);
    baseOpt.series[2].itemStyle.color = color.getColor(iChartOpt.color, 1);
    baseOpt.series[1].data = new Array(dataSet.barData[0].length).fill(dataSet.maxValue);
    baseOpt.series[3].data = new Array(dataSet.barData[1].length).fill(dataSet.maxValue);
  } else {
    baseOpt.series[0].data = new Array(dataSet.barData.length).fill(0);
    baseOpt.series[1].data = new Array(dataSet.barData.length).fill(dataSet.maxValue);
    var dataSreies = cloneDeep.default(BaseOption.BASICDATASERIES);
    dataSreies.name = "data";
    dataSreies.data = dataSet.barData;
    dataSreies.itemStyle.color = function(params) {
      return handleBarColor(params, iChartOpt, dataSet, doubleSide);
    };
    baseOpt.series.push(dataSreies);
  }
}
function setBarBordRadius(barData, seriesName) {
  var total = [];
  var _loop = function _loop2(rowIndex2) {
    var rowData = [];
    barData.forEach(function(item, columnIndex) {
      if (item[rowIndex2].value) {
        var dataInfo = {
          columnIndex,
          rowIndex: rowIndex2,
          value: item[rowIndex2].value
        };
        rowData.push(dataInfo);
      }
    });
    total.push(rowData);
  };
  for (var rowIndex = 0; rowIndex < seriesName.length; rowIndex++) {
    _loop(rowIndex);
  }
  total.forEach(function(to) {
    var tolen = to.length;
    if (tolen !== 0) {
      if (tolen === 1) {
        barData[to[0].columnIndex][to[0].rowIndex] = _extends({}, barData[to[0].columnIndex][to[0].rowIndex], {
          value: barData[to[0].columnIndex][to[0].rowIndex].value,
          itemStyle: {
            borderRadius: [4, 4, 4, 4]
          }
        });
      } else {
        barData[to[0].columnIndex][to[0].rowIndex] = _extends({}, barData[to[0].columnIndex][to[0].rowIndex], {
          value: barData[to[0].columnIndex][to[0].rowIndex].value,
          itemStyle: {
            borderRadius: [4, 0, 0, 4]
          }
        });
        barData[to[to.length - 1].columnIndex][to[to.length - 1].rowIndex] = _extends({}, barData[to[to.length - 1].columnIndex][to[to.length - 1].rowIndex], {
          value: barData[to[to.length - 1].columnIndex][to[to.length - 1].rowIndex].value,
          itemStyle: {
            borderRadius: [0, 4, 4, 0]
          }
        });
      }
    }
  });
}
function handleStackProcessSeries(baseOpt, iChartOpt, dataSet) {
  setBarBordRadius(dataSet.barData, dataSet.seriesName);
  baseOpt.series = cloneDeep.default(BaseOption.BASICSERIES);
  baseOpt.series[0].data = new Array(dataSet.seriesName.length).fill(0);
  baseOpt.series[1].data = new Array(dataSet.seriesName.length).fill(dataSet.maxValue);
  dataSet.barData.forEach(function(item, index2) {
    var unitSeries = cloneDeep.default(BaseOption.BASICDATASERIES);
    unitSeries.data = item;
    unitSeries.itemStyle.borderRadius = void 0;
    unitSeries.name = dataSet.barName[index2];
    unitSeries.stack = "total";
    baseOpt.series.push(unitSeries);
  });
}
function handleBasicProcessTheme(baseOpt, doubleSide) {
  var itemBgEmpty = chartToken.default.itemBgEmpty, axisLabelColor = chartToken.default.axisLabelColor, labeColor = chartToken.default.labeColor;
  if (doubleSide) {
    baseOpt.series.forEach(function(ser, serIndex) {
      if (serIndex % 2 !== 0) {
        ser.itemStyle.color = itemBgEmpty;
        ser.label.color = axisLabelColor;
      }
    });
  } else {
    baseOpt.series[0].label.color = axisLabelColor;
    baseOpt.series[1].label.color = labeColor;
    baseOpt.series[1].itemStyle.color = itemBgEmpty;
  }
}
function handleStackProcessTheme(baseOpt, iChartOpt) {
  var itemBgEmpty = chartToken.default.itemBgEmpty, axisLabelColor = chartToken.default.axisLabelColor, labeColor = chartToken.default.labeColor;
  baseOpt.series[1].itemStyle.color = itemBgEmpty;
  baseOpt.series[1].label.color = labeColor;
  baseOpt.series[0].label.color = axisLabelColor;
  baseOpt.series.forEach(function(serie, index2) {
    if (index2 > 1) {
      serie.itemStyle.color = function(params) {
        return color.getColor(iChartOpt.color, params.seriesIndex - 2);
      };
      serie.label.show = true;
      serie.label.color = labeColor;
    }
  });
}
function handleDoubleLabel(params, iChartOpt, dataSet, position, index2) {
  if (index2 === void 0) {
    index2 = 0;
  }
  var name = params.name, dataIndex = params.dataIndex;
  var innerVal = dataSet.barData[index2][dataIndex]._initValue || dataSet.barData[index2][dataIndex].value;
  if (innerVal === null || innerVal === void 0) return "";
  var innerUnit = iChartOpt.unit || iChartOpt.unit === "" ? iChartOpt.unit : BaseOption.BASICUNIT;
  var val = "" + innerVal + innerUnit;
  return "{name" + position + "|" + name + "}\n{value" + position + "|" + val + "}";
}
function getMaxValLength(dataSet, iChartOpt, index2) {
  var maxLength = 0;
  var innerUnit = iChartOpt.unit || iChartOpt.unit === "" ? iChartOpt.unit : BaseOption.BASICUNIT;
  dataSet.barData[index2].forEach(function(item) {
    var val = "" + (item._initValue || item.value) + innerUnit;
    var len = dom.getTextWidth(val, 14);
    if (len >= maxLength) maxLength = len;
  });
  return maxLength;
}
function setTitle(baseOpt, iChartOpt) {
  if (iChartOpt.title) merge.default(baseOpt.series[0].label, iChartOpt.title);
}
function setText(baseOpt, iChartOpt, dataSet) {
  if (iChartOpt.text) {
    merge.default(baseOpt.series[1].label, iChartOpt.text);
    var formatter = iChartOpt.text.formatter;
    if (formatter) {
      baseOpt.series[1].label.formatter = function(params) {
        var dataIndex = params.dataIndex;
        var value = dataSet.barData[dataIndex]._initValue || dataSet.barData[dataIndex].value;
        var inerParams = _extends({}, params, {
          value,
          data: value
        });
        return formatter(inerParams);
      };
    }
  }
}
function setBarLabel(baseOpt, iChartOpt, dataSet) {
  baseOpt.series[0].label.formatter = function(params) {
    return params.name === "null" || params.name === "undefined" ? "" : params.name;
  };
  baseOpt.series[1].label.formatter = function(params) {
    var innerVal = dataSet.barData[params.dataIndex]._initValue || dataSet.barData[params.dataIndex].value;
    if (innerVal === null || innerVal === void 0) return "";
    var innerUnit = iChartOpt.unit || iChartOpt.unit === "" ? iChartOpt.unit : BaseOption.BASICUNIT;
    return "" + innerVal + innerUnit;
  };
}
function handleBasicProcessLabel(baseOpt, iChartOpt, dataSet, doubleSide) {
  if (doubleSide) {
    var labeColor = chartToken.default.labeColor, disabledColor = chartToken.default.disabledColor;
    baseOpt.series[1].label.formatter = function(params) {
      return handleDoubleLabel(params, iChartOpt, dataSet, "Left");
    };
    baseOpt.series[3].label.formatter = function(params) {
      return handleDoubleLabel(params, iChartOpt, dataSet, "Right", 1);
    };
    var valWidthL = getMaxValLength(dataSet, iChartOpt, 0);
    var valWidthR = getMaxValLength(dataSet, iChartOpt, 1);
    baseOpt.series[1].label.rich.nameLeft.color = disabledColor;
    baseOpt.series[1].label.rich.valueLeft.color = labeColor;
    baseOpt.series[1].label.offset = [-valWidthL - 10, 9];
    baseOpt.series[3].label.rich.nameRight.color = disabledColor;
    baseOpt.series[3].label.rich.valueRight.color = labeColor;
    baseOpt.series[3].label.offset = [valWidthR + 10, 9];
  } else {
    setBarLabel(baseOpt, iChartOpt, dataSet);
    setTitle(baseOpt, iChartOpt);
    setText(baseOpt, iChartOpt, dataSet);
  }
}
function handleStackProcessLabel(baseOpt, iChartOpt, dataSet) {
  baseOpt.series[0].label.formatter = function(params) {
    return params.name === "null" || params.name === "undefined" ? "" : params.name;
  };
  if (iChartOpt.title) {
    merge.default(baseOpt.series[0].label, iChartOpt.title);
  }
  baseOpt.series[1].label.formatter = function(params) {
    var name = dataSet.seriesName[params.dataIndex];
    if (name === null || name === void 0) return "";
    if (iChartOpt.text && iChartOpt.text.labelText) return iChartOpt.text.labelText[params.dataIndex];
    if (iChartOpt.unit) return "" + dataSet.sumValue[params.dataIndex] + iChartOpt.unit;
    return "" + dataSet.sumValue[params.dataIndex];
  };
  if (iChartOpt.text) {
    merge.default(baseOpt.series[1].label, iChartOpt.text);
  }
  baseOpt.series.forEach(function(serie, index2) {
    if (index2 > 1) serie.label.formatter = function(params) {
      var value = params.data._initValue || params.data.value;
      return value || "";
    };
  });
  if (iChartOpt.label) {
    baseOpt.series.forEach(function(serie, serieIndex) {
      if (serieIndex > 1) merge.default(serie.label, iChartOpt.label);
    });
  }
}
function handleLabel(baseOpt, iChartOpt, dataSet, doubleSide) {
  if (iChartOpt.name === BaseOption.CHARTTYPENAME.ProcessBarChart) {
    handleBasicProcessLabel(baseOpt, iChartOpt, dataSet, doubleSide);
  } else {
    handleStackProcessLabel(baseOpt, iChartOpt, dataSet);
  }
}
function handleTheme(baseOpt, iChartOpt, doubleSide) {
  if (iChartOpt.name === BaseOption.CHARTTYPENAME.ProcessBarChart) {
    handleBasicProcessTheme(baseOpt, doubleSide);
  } else {
    handleStackProcessTheme(baseOpt, iChartOpt);
  }
}
function handleBarWidth(baseOpt, iChartOpt) {
  var name = iChartOpt.name;
  var barWidth = iChartOpt.barWidth || BaseOption.BASICBARWIDTH[name];
  baseOpt.series.forEach(function(serie) {
    serie.barWidth = barWidth;
  });
}
function handleSeries(baseOpt, iChartOpt, dataSet, doubleSide) {
  if (iChartOpt.name === BaseOption.CHARTTYPENAME.ProcessBarChart) {
    handleBasicProcessSeries(baseOpt, iChartOpt, dataSet, doubleSide);
  } else {
    handleStackProcessSeries(baseOpt, iChartOpt, dataSet);
  }
  handleTheme(baseOpt, iChartOpt, doubleSide);
  handleLabel(baseOpt, iChartOpt, dataSet, doubleSide);
  handleBarWidth(baseOpt, iChartOpt);
}
exports.default = handleSeries;
exports.getStateList = getStateList;
exports.handleBarColor = handleBarColor;
exports.setStateBarColor = setStateBarColor;
