"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const paint = require("../../../util/paint.js");
const dom = require("../../../util/dom.js");
const type = require("../../../util/type.js");
const handleSeries = require("../handleSeries.js");
const index = require("../../../feature/token/index.js");
const handleSeries$1 = require("../../ProcessChart/handleSeries.js");
const color = require("../../../util/color.js");
const handleOptipn = require("../handleOptipn.js");
const defendXSS = require("../../../util/defendXSS.js");
const merge = require("../../../util/merge.js");
const chartToken = require("../chartToken.js");
const BaseOption = require("../BaseOption.js");
function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t = arguments[e];
      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
var SUFFIX_CLASSNAME = "radarCanvas";
var GradientRadar = /* @__PURE__ */ function() {
  function GradientRadar2(echartsIns, params) {
    this.painter = null;
    this.centerPoint = {
      x: 0,
      y: 0
    };
    this.canvasDom = null;
    this.echartsIns = null;
    this.iChartOption = null;
    this.containerDom = {
      target: null,
      width: 0,
      height: 0
    };
    this.radiusScale = 0;
    this.radius = {
      isArray: false,
      value: void 0
    };
    this.pointsMap = null;
    this.baseOption = null;
    this.radarKeys = null;
    this.echartsIns = echartsIns;
    var container = echartsIns.getDom();
    this.containerDom.target = container;
    this.getContainerSize();
    var iChartOption = params.iChartOption, baseOption = params.baseOption, radarKeys = params.radarKeys;
    this.iChartOption = iChartOption;
    this.baseOption = baseOption;
    this.radarKeys = radarKeys;
  }
  var _proto = GradientRadar2.prototype;
  _proto.init = function init() {
    if (!this.iChartOption.gradient) return;
    if (!this.containerDom.target) return;
    this.initCanvas();
    this.moveStartToRadarCenter();
    this.getRadius();
    this.getPointsMap();
    this.paintRadarArea();
  };
  _proto.getContainerSize = function getContainerSize() {
    this.containerDom.width = this.echartsIns.getWidth();
    this.containerDom.height = this.echartsIns.getHeight();
  };
  _proto.setCanvasSize = function setCanvasSize() {
    this.canvasDom.width = this.containerDom.width;
    this.canvasDom.height = this.containerDom.height;
  };
  _proto.initCanvas = function initCanvas() {
    if (this.containerDom.target) {
      var dom$1 = this.containerDom.target.getElementsByClassName("huicharts-" + SUFFIX_CLASSNAME)[0];
      if (dom$1) this.containerDom.target.removeChild(dom$1);
    }
    var canvas = document.createElement("canvas");
    this.canvasDom = canvas;
    canvas.id = SUFFIX_CLASSNAME;
    canvas.className = "huicharts-" + SUFFIX_CLASSNAME;
    this.setCanvasSize();
    canvas.style.width = "100%";
    canvas.style.height = "100%";
    canvas.style.position = "absolute";
    canvas.style.left = 0;
    canvas.style.top = 0;
    canvas.style["pointer-events"] = "none";
    dom.appendDom(this.containerDom.target, canvas);
    this.initPainter();
  };
  _proto.initPainter = function initPainter() {
    var ctx = this.canvasDom.getContext("2d");
    this.painter = new paint.default(ctx);
    this.setPainter();
  };
  _proto.setPainter = function setPainter() {
    this.painter.lineCap("round");
    this.painter.lineJoin("round");
    this.painter.lineWidth(chartToken.default.lineWidth);
  };
  _proto.convertsRadiusToNub = function convertsRadiusToNub(val) {
    var r = this.formatterVal(val);
    return this.isPctString(val) ? r * this.radiusScale / 2 : r;
  };
  _proto.getRadius = function getRadius() {
    this.getRadiusScale();
    var position = this.iChartOption.position;
    var radius = (position == null ? void 0 : position.radius) || "50%";
    if (type.isArray(radius)) {
      this.radius.isArray = true;
      var inner = radius[0], outer = radius[1];
      var innerR = this.convertsRadiusToNub(inner);
      var outerR = this.convertsRadiusToNub(outer);
      this.radius.value = [innerR, outerR];
    } else {
      this.radius.value = this.convertsRadiusToNub(radius);
    }
  };
  _proto.getRadiusScale = function getRadiusScale() {
    this.radiusScale = this.containerDom.width >= this.containerDom.height ? this.containerDom.height : this.containerDom.width;
  };
  _proto.moveStartToRadarCenter = function moveStartToRadarCenter() {
    this.getCenterPoint();
    this.painter.save();
    this.painter.translate(this.centerPoint.x, this.centerPoint.y);
  };
  _proto.formatterVal = function formatterVal(val) {
    if (this.isPctString(val)) {
      return Number.parseFloat(val) / 100;
    }
    return Number.parseFloat(val);
  };
  _proto.getPoint = function getPoint(val, isY) {
    var size = isY ? this.containerDom.height : this.containerDom.width;
    var actualVal = this.formatterVal(val);
    return this.isPctString(val) ? actualVal * size : actualVal;
  };
  _proto.isPctString = function isPctString(val) {
    return type.isString(val) && val.includes("%");
  };
  _proto.getCenterPoint = function getCenterPoint() {
    var position = this.iChartOption.position;
    var center = (position == null ? void 0 : position.center) || ["50%", "50%"];
    var x = this.getPoint(center[0]);
    var y = this.getPoint(center[1], true);
    this.centerPoint.x = x;
    this.centerPoint.y = y;
  };
  _proto.clearCanvas = function clearCanvas() {
    this.painter.restore();
    this.painter.clearRect(0, 0, this.containerDom.width, this.containerDom.height);
  };
  _proto.paintRadarArea = function paintRadarArea() {
    var _this = this;
    if (!this.pointsMap) return;
    var pointsCoordinate = this.pointsMap.map(function(item, index2) {
      var point = _this.getPointCoordinate(item, index2);
      return _extends({}, item, {
        point
      });
    });
    pointsCoordinate.forEach(function(point, pointIndex) {
      pointIndex === _this.pointsMap.length - 1 ? _this.paintRadarSide(point, pointsCoordinate[0]) : _this.paintRadarSide(point, pointsCoordinate[pointIndex + 1]);
    });
    pointsCoordinate.forEach(function(point) {
      _this.paintPoint(point);
    });
  };
  _proto.paintPoint = function paintPoint(point) {
    var x = point.point[0];
    var y = point.point[1];
    var fillColor = chartToken.default.gradientItemBorderColor;
    var r = (chartToken.default.symbolSize - 2 * chartToken.default.lineWidth) / 2 < 0 ? 0 : (chartToken.default.symbolSize - 2 * chartToken.default.lineWidth) / 2;
    this.painter.fillStyle(fillColor);
    this.painter.beginPath();
    this.painter.moveTo(x, y);
    this.painter.arc(x, y, r, 0, 2 * Math.PI);
    this.painter.fill();
    this.painter.closePath();
  };
  _proto.judgePointAtCanvasCoordinateAxis = function judgePointAtCanvasCoordinateAxis(angle) {
    return angle === 0 || angle === 90 || angle === 270 || angle === 360;
  };
  _proto.paintRadarSide = function paintRadarSide(point, nextPoint) {
    var x = point.point[0];
    var y = point.point[1];
    var x1 = nextPoint.point[0];
    var y1 = nextPoint.point[1];
    var linearGradient = this.painter.createLinearGradient(x, y, x1, y1);
    linearGradient.addColorStop(0, point.color);
    linearGradient.addColorStop(1, nextPoint.color);
    this.painter.strokeStyle(linearGradient);
    this.painter.beginPath();
    this.painter.moveTo(x, y);
    this.painter.lineTo(x1, y1);
    this.painter.stroke();
    this.painter.closePath();
  };
  _proto.getPointDistanceFromCenter = function getPointDistanceFromCenter(dataItem) {
    var dataName = dataItem.dataName, dataValue = dataItem.dataValue;
    var max = this.baseOption.radar[0].indicator.find(function(item) {
      return item.name === dataName;
    }).max;
    if (this.radius.isArray) {
      return (this.radius.value[1] - this.radius.value[0]) * (dataValue / max) + this.radius.value[0];
    }
    return this.radius.value * (dataValue / max);
  };
  _proto.getCoefficient = function getCoefficient(multiple, isYaxis) {
    if (isYaxis === void 0) {
      isYaxis = false;
    }
    if (isYaxis) {
      return multiple === 2 ? 1 : -1;
    } else {
      return multiple === 1 ? -1 : 1;
    }
  };
  _proto.judgeQuadrants = function judgeQuadrants(angle) {
    if (angle > 0 && angle < 90) {
      return 4;
    } else if (angle > 90 && angle < 180) {
      return 3;
    } else if (angle > 180 && angle < 270) {
      return 2;
    } else if (angle > 270 && angle < 360) {
      return 1;
    }
  };
  _proto.getPointCoordinate = function getPointCoordinate(dataItem, dataIndex) {
    var distance = this.getPointDistanceFromCenter(dataItem);
    var angle = 360 / this.radarKeys.length * dataIndex;
    var x = 0;
    var y = 0;
    if (this.judgePointAtCanvasCoordinateAxis(angle)) {
      var multiple = angle / 90;
      var isYaxis = angle / 90 % 2 === 0;
      x = isYaxis ? 0 : distance * this.getCoefficient(multiple);
      y = isYaxis ? distance * this.getCoefficient(multiple, isYaxis) : 0;
    } else {
      var quadrant = this.judgeQuadrants(angle);
      var actualAngle = (angle - 90 * (4 - quadrant)) * (Math.PI / 180);
      if (quadrant === 1) {
        x = Math.cos(actualAngle);
        y = Math.sin(actualAngle) * -1;
      } else if (quadrant === 2) {
        x = Math.sin(actualAngle);
        y = Math.cos(actualAngle);
      } else if (quadrant === 3) {
        x = Math.cos(actualAngle) * -1;
        y = Math.sin(actualAngle);
      } else {
        x = Math.sin(actualAngle) * -1;
        y = Math.cos(actualAngle) * -1;
      }
      x = x * distance;
      y = y * distance;
    }
    return [x, y];
  };
  _proto.setSeries = function setSeries() {
    var _this2 = this;
    if (!this.pointsMap) return;
    var seriesUnit = this.baseOption.series[0];
    seriesUnit.lineStyle.opacity = 0;
    seriesUnit.areaStyle.opacity = 0;
    seriesUnit.emphasis = {};
    this.pointsMap.forEach(function(item, index2) {
      var seriesName = item.seriesName, dataName = item.dataName, dataValue = item.dataValue, color2 = item.color;
      var dataNameIndex = _this2.radarKeys.indexOf(dataName);
      var unitRadar = handleSeries.handleRedPointerRadar(_this2.baseOption, _this2.radarKeys, dataNameIndex, dataName);
      var unitPointerSeries = BaseOption.handleRedPointerSeries(index2, dataValue, seriesName, true);
      unitPointerSeries.itemStyle.color = color2;
      unitPointerSeries.itemStyle.shadowBlur = 15;
      unitPointerSeries.itemStyle.shadowColor = color2;
      unitPointerSeries.emphasis.itemStyle.color = color2;
      unitPointerSeries.emphasis.itemStyle.borderColor = color2;
      _this2.baseOption.radar.push(unitRadar);
      _this2.baseOption.series.push(unitPointerSeries);
    });
  };
  _proto.getPointColor = function getPointColor(value, index$1) {
    var _this$iChartOption = this.iChartOption, state = _this$iChartOption.state, color$1 = _this$iChartOption.color;
    if (state) {
      var colorState = index.default.config.colorState;
      var stateColorGroup = {
        error: colorState.colorError,
        warning: colorState.colorAlert,
        subwarning: colorState.colorWarning,
        success: colorState.colorSuccess
      };
      var successColor = stateColorGroup.success;
      var stateList = handleSeries$1.getStateList(state);
      return handleSeries$1.setStateBarColor(value, stateColorGroup, successColor, stateList);
    } else {
      return color.getColor(color$1, index$1);
    }
  };
  _proto.getPointsMap = function getPointsMap() {
    var _this3 = this;
    var data = this.iChartOption.data;
    var exceeded = handleSeries.getExceededMarkLineValue(data, 0, false);
    this.pointsMap = exceeded.map(function(item, index2) {
      var color2 = _this3.getPointColor(item.dataValue, index2);
      return _extends({}, item, {
        color: color2
      });
    });
  };
  _proto.setLegend = function setLegend() {
    this.baseOption.legend.show = false;
  };
  _proto.tipFormatter = function tipFormatter() {
    var _this4 = this;
    this.baseOption.tooltip.formatter = function(params) {
      var seriesdata = params.data;
      var dataName = seriesdata.name;
      var value = seriesdata.value;
      if (params.seriesName === "threshold") {
        value = _this4.radarKeys.map(function(key) {
          return _this4.iChartOption.data[dataName][key];
        });
      }
      var htmlString = '<div style="margin-bottom:4px;">' + defendXSS.default(dataName) + "</div>";
      value.forEach(function(item, index2) {
        var color2 = _this4.pointsMap[index2].color;
        htmlString += '<div style="margin-bottom:4px;">\n            <span style="display:inline-block;width:8px;\n            height:8px;margin-right:8px;border-radius:5px;\n            background-color:' + defendXSS.default(color2) + ';"></span>\n            <span style="display:inline-block;margin-right:8px;\n            min-width:60px;font-size:12px">' + defendXSS.default(_this4.radarKeys[index2]) + '</span>\n            <span style="font-size:14px">' + defendXSS.default(handleOptipn.checkValue(item)) + "</span>\n            </div>";
      });
      return htmlString;
    };
  };
  _proto.setTooltip = function setTooltip() {
    var _this5 = this;
    var _this$iChartOption2 = this.iChartOption, tipHtml = _this$iChartOption2.tipHtml, tooltip = _this$iChartOption2.tooltip;
    tipHtml ? handleOptipn.handletipHtml(this.baseOption.tooltip, tipHtml, this.radarKeys) : this.tipFormatter();
    if (tooltip) {
      merge.default(this.baseOption.tooltip, tooltip);
    }
    if (tooltip != null && tooltip.formatter) {
      this.baseOption.tooltip.formatter = function(params, ticket, callback) {
        return tooltip.formatter(params, _this5.radarKeys, ticket, callback);
      };
    }
  };
  _proto.resize = function resize() {
    if (this.containerDom.target && this.canvasDom) {
      if (this.painter) this.clearCanvas();
      this.getContainerSize();
      this.setCanvasSize();
      this.setPainter();
      this.moveStartToRadarCenter();
      this.getRadius();
      this.paintRadarArea();
    }
  };
  return GradientRadar2;
}();
exports.default = GradientRadar;
