"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const type = require("../../util/type.js");
const cloneDeep = require("../../util/cloneDeep.js");
const defendXSS = require("../../util/defendXSS.js");
const merge = require("../../util/merge.js");
const tips = require("../../util/tips.js");
const index$1 = require("../../option/config/polarTitle/index.js");
const index = require("../../option/config/tooltip/index.js");
const index$2 = require("../../option/config/legend/index.js");
const index$3 = require("../../feature/token/index.js");
const BaseOption = require("./BaseOption.js");
const color = require("../../util/color.js");
function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t = arguments[e];
      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
function initRadarSys(baseOpt, iChartOpt) {
  baseOpt.color = iChartOpt.color;
  baseOpt.tooltip = index.default(iChartOpt, "RadarChart");
  baseOpt.title = index$1.default(iChartOpt);
  baseOpt.legend = index$2.default(iChartOpt);
  baseOpt.radar = [];
  baseOpt.series = [];
}
function handleCusMax(iChartOpt) {
  var max = 0;
  var radarMax = iChartOpt.radarMax, radar = iChartOpt.radar;
  if (type.isArray(radarMax)) {
    radarMax.forEach(function(item) {
      if (item.max > max) max = item.max;
    });
  }
  if (radar != null && radar.indicator) {
    radar.indicator.forEach(function(el) {
      if (el.max > max) max = el.max;
    });
  }
  return max;
}
function findDataMax(keys, data) {
  var max;
  for (var j = 0; j < keys.length; j++) {
    var key = keys[j];
    if (max === void 0) {
      max = data[key];
    }
    max = Math.max(max, data[key]);
  }
  return max;
}
function getRadarMax(data, iChartOpt, isCustomMaxVal) {
  var max;
  var cusDataName;
  var radarMax = iChartOpt.radarMax, radar = iChartOpt.radar;
  if (isCustomMaxVal) {
    if (type.isArray(radarMax)) {
      cusDataName = radarMax.map(function(item) {
        return item.name;
      });
    }
    if (radar != null && radar.indicator) {
      cusDataName = radar.indicator.map(function(i2) {
        return i2.name;
      });
    }
  }
  var names = Object.keys(data);
  var _loop = function _loop2() {
    var name = names[i];
    var keys = Object.keys(data[name]);
    if (isCustomMaxVal) {
      var dataKeys = [];
      keys.forEach(function(name2) {
        if (!cusDataName.includes(name2)) {
          dataKeys.push(name2);
        }
      });
      var seriesDataMax = findDataMax(dataKeys, data[name]);
      if (max === void 0) max = seriesDataMax;
      if (max === void 0) {
        max = handleCusMax(iChartOpt);
      }
    } else {
      var _seriesDataMax = findDataMax(keys, data[name]);
      if (max === void 0) max = _seriesDataMax;
      if (max || max === 0) {
        max = Math.max(max, _seriesDataMax);
      }
    }
  };
  for (var i = 0; i < names.length; i++) {
    _loop();
  }
  return max;
}
function getRadarKeys(data) {
  var radarKeys = [];
  var seriesNames = Object.keys(data);
  for (var i = 0; i < seriesNames.length; i++) {
    var seriesName = seriesNames[i];
    var seriesData = data[seriesName];
    var dataNames = Object.keys(seriesData);
    for (var j = 0; j < dataNames.length; j++) {
      var dataName = dataNames[j];
      if (radarKeys.indexOf(dataName) === -1) {
        radarKeys.push(dataName);
      }
    }
  }
  return radarKeys;
}
function setCenterAndRadius(radar, iChartOpt) {
  var chartPosition = iChartOpt.chartPosition || iChartOpt.position || {};
  if (chartPosition.center) radar.center = chartPosition.center;
  if (chartPosition.radius) radar.radius = chartPosition.radius;
}
function setIndicator(radar, radarKeys, isCustomMaxVal, iChartOpt) {
  var radarMax = iChartOpt.radarMax, data = iChartOpt.data, radarMark = iChartOpt.radarMark;
  var isRadarMaxArr = type.isArray(radarMax);
  radar.indicator = radarKeys.map(function(name, index2) {
    if (!isRadarMaxArr) {
      return index2 === 0 ? {
        name,
        max: radarMax
      } : {
        name,
        max: radarMax,
        axisLabel: {
          show: false
        }
      };
    } else {
      var inerMax = getRadarMax(data, iChartOpt, isCustomMaxVal);
      var isName = radarMax.find(function(item) {
        return item.name === name;
      });
      var cusIndicator = isName || {};
      return _extends({
        name,
        max: inerMax,
        axisLabel: {
          show: !!radarMark
        }
      }, cusIndicator);
    }
  });
}
function mergeRadar(radar, iChartOpt, isCommon) {
  var _iChartOpt$radar;
  if (iChartOpt.radar) merge.default(radar, iChartOpt.radar);
  if ((_iChartOpt$radar = iChartOpt.radar) != null && _iChartOpt$radar.indicator) {
    var mixinIndicator = radar.indicator.map(function(i) {
      var coveredIndicator = iChartOpt.radar.indicator.find(function(indicate) {
        return indicate.name === i.name;
      });
      return coveredIndicator || i;
    });
    radar.indicator = mixinIndicator;
  }
}
function setAxisLabel(radar, iChartOpt) {
  var radarMark = iChartOpt.radarMark;
  radar.axisLabel.show = radarMark === void 0 ? true : !!radarMark;
}
function setRadarShape(radar, baseOpt) {
  radar.shape = baseOpt.radar[0].shape;
}
function setRadar(baseOpt, iChartOpt, radarKeys, isCustomMaxVal) {
  var radar = BaseOption.default();
  setAxisLabel(radar, iChartOpt);
  setCenterAndRadius(radar, iChartOpt);
  setIndicator(radar, radarKeys, isCustomMaxVal, iChartOpt);
  mergeRadar(radar, iChartOpt);
  baseOpt.radar.push(radar);
}
function handletipHtml(tooltip, tipHtml, radarKeys) {
  tooltip.formatter = function(params, ticket, callback) {
    return tipHtml(params, radarKeys, ticket, callback);
  };
}
function checkValue(val) {
  return val || val === 0 ? val : "-";
}
function getCommonDataColor(iChartOpt, data, dataName) {
  var dataNames = Object.keys(data);
  var curNameIndex = dataNames.indexOf(dataName);
  if (curNameIndex !== -1) return color.getColor(iChartOpt.color, curNameIndex);
}
function handleFormatter(tooltip, iChartOpt, radarKeys, data) {
  var markLine = iChartOpt.markLine;
  var alarmColor = index$3.default.config.colorState.colorError;
  var isThreshold = !!(type.isObject(markLine) && markLine != null && markLine.threshold);
  tooltip.formatter = function(params) {
    var seriesdata = params.data;
    var dataName = seriesdata.name;
    var tipData = seriesdata.value;
    var dataColor = params.color;
    if (params.seriesName === "threshold") {
      tipData = radarKeys.map(function(key) {
        return data[dataName][key];
      });
      dataColor = getCommonDataColor(iChartOpt, data, dataName);
    }
    var htmlString = '<div style="margin-bottom:4px;">' + defendXSS.default(dataName) + "</div>";
    tipData.forEach(function(item, index2) {
      var color2 = dataColor;
      if (markLine) {
        var markVal = isThreshold ? markLine.threshold[radarKeys[index2]] : markLine;
        if (item >= markVal) color2 = alarmColor;
      }
      htmlString += '<div style="margin-bottom:4px;">\n      <span style="display:inline-block;width:8px;\n      height:8px;margin-right:8px;border-radius:5px;\n      background-color:' + defendXSS.default(color2) + ';"></span>\n      <span style="display:inline-block;margin-right:8px;\n      min-width:60px;font-size:12px">' + defendXSS.default(radarKeys[index2]) + '</span>\n      <span style="font-size:14px">' + defendXSS.default(checkValue(item)) + "</span>\n      </div>";
    });
    return htmlString;
  };
}
function setTooltip(baseOpt, iChartOpt, radarKeys, data) {
  var tipHtml = iChartOpt.tipHtml, tooltip = iChartOpt.tooltip, gradient = iChartOpt.gradient;
  if (gradient) return;
  tipHtml ? handletipHtml(baseOpt.tooltip, tipHtml, radarKeys) : handleFormatter(baseOpt.tooltip, iChartOpt, radarKeys, data);
  if (tooltip != null && tooltip.formatter) {
    baseOpt.tooltip.formatter = function(params, ticket, callback) {
      return tooltip.formatter(params, radarKeys, ticket, callback);
    };
  }
}
function setMarkCenterAndRadius(radar, baseOpt) {
  radar.center = baseOpt.radar[0].center;
  radar.radius = baseOpt.radar[0].radius;
}
function setCustomMarkLine(markRadar, baseOpt, iChartOpt, radarKeys) {
  var threshold = iChartOpt.markLine.threshold;
  markRadar.indicator = cloneDeep.default(baseOpt.radar[0].indicator);
  markRadar.splitLine.show = false;
  var thresholdData = radarKeys.map(function(item) {
    var data = threshold[item] || 0;
    return data;
  });
  var thresholdSeries = BaseOption.getThresholdSeries();
  thresholdSeries.data = [{
    name: "",
    value: thresholdData
  }];
  baseOpt.series.push(thresholdSeries);
}
function setCommonMarkLine(markRadar, baseOpt, iChartOpt, radarKeys) {
  var radius = baseOpt.radar[0].radius;
  var radiusIsArray = type.isArray(radius);
  var radarMax = iChartOpt.radarMax, markLine = iChartOpt.markLine;
  if (type.isObject(radarMax)) {
    tips.default.error("If markLine is of the number type, radarMax must be of the number type. If radarMax is of the object type, set markLine.threshold");
    return;
  }
  var markLineValue = Number.parseFloat(markLine);
  var marklineRadius;
  if (radiusIsArray) {
    var disRadius = Number.parseFloat(radius[1]) - Number.parseFloat(radius[0]);
    marklineRadius = disRadius * (markLineValue / radarMax) + Number.parseFloat(radius[0]) + (radius[0].toString().indexOf("%") !== -1 ? "%" : "");
  } else {
    marklineRadius = markLineValue / radarMax * Number.parseFloat(radius) + (radius.toString().indexOf("%") !== -1 ? "%" : "");
  }
  markRadar.radius = marklineRadius;
  markRadar.indicator = Array(radarKeys.length).fill({
    name: ""
  });
}
function setMarkLine(baseOpt, iChartOpt, radarKeys) {
  var markLine = iChartOpt.markLine, gradient = iChartOpt.gradient;
  if (gradient) return;
  if (markLine) {
    var markRadar = BaseOption.getMarkRadarOption();
    setRadarShape(markRadar, baseOpt);
    setMarkCenterAndRadius(markRadar, baseOpt);
    var isThreshold = !!(type.isObject(markLine) && markLine != null && markLine.threshold);
    isThreshold ? setCustomMarkLine(markRadar, baseOpt, iChartOpt, radarKeys) : setCommonMarkLine(markRadar, baseOpt, iChartOpt, radarKeys);
    baseOpt.radar.push(markRadar);
  }
}
exports.checkValue = checkValue;
exports.getRadarKeys = getRadarKeys;
exports.getRadarMax = getRadarMax;
exports.handletipHtml = handletipHtml;
exports.initRadarSys = initRadarSys;
exports.setMarkLine = setMarkLine;
exports.setRadar = setRadar;
exports.setRadarShape = setRadarShape;
exports.setTooltip = setTooltip;
