"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const type = require("../../util/type.js");
const BaseOption = require("./BaseOption.js");
const handleOptipn = require("./handleOptipn.js");
const color = require("../../util/color.js");
const merge = require("../../util/merge.js");
function setEmphasisItemStyleBorderColor(iChartOpt, nameIndex) {
  var borderColor = color.getColor(iChartOpt.color, nameIndex);
  return {
    itemStyle: {
      borderColor
    }
  };
}
function setAreaStyleOpacity(seriesUnit, iChartOpt) {
  var area = iChartOpt.area;
  if (area) {
    merge.default(seriesUnit.areaStyle, area);
  }
  if (area && area.show === false) {
    seriesUnit.areaStyle.opacity = 0;
    seriesUnit.emphasis.areaStyle.opacity = 0;
  }
}
function setSeries(baseOpt, iChartOpt, radarKeys, data) {
  var dataNames = Object.keys(data);
  var seriesUnit = BaseOption.getSeriesUnit();
  setAreaStyleOpacity(seriesUnit, iChartOpt);
  var seriesData = dataNames.map(function(name, nameIndex) {
    var radarData = {
      name,
      value: radarKeys.map(function(key) {
        return data[name][key];
      }),
      emphasis: setEmphasisItemStyleBorderColor(iChartOpt, nameIndex)
    };
    return radarData;
  });
  seriesUnit.data = seriesData;
  baseOpt.series.push(seriesUnit);
}
function handleRedPointerRadar(baseOpt, radarKeys, dataNameIndex, dataName) {
  var max = baseOpt.radar[0].indicator.find(function(item) {
    return item.name === dataName;
  }).max;
  var redPointerRadar = BaseOption.getRedPointerRadar();
  handleOptipn.setRadarShape(redPointerRadar, baseOpt);
  redPointerRadar.center = baseOpt.radar[0].center;
  redPointerRadar.radius = baseOpt.radar[0].radius;
  redPointerRadar.startAngle = 90 + 360 / radarKeys.length * dataNameIndex;
  redPointerRadar.indicator = [{
    name: "",
    max
  }];
  return redPointerRadar;
}
function getExceededMarkLineValue(data, markLine, isThreshold) {
  var thresholdPoint = [];
  var names = Object.keys(data);
  for (var i = 0; i < names.length; i++) {
    var name = names[i];
    var keys = Object.keys(data[name]);
    for (var j = 0; j < keys.length; j++) {
      var key = keys[j];
      var markLineVal = isThreshold ? markLine.threshold[key] : markLine;
      if ((markLineVal || markLineVal === 0) && data[name][key] >= markLineVal) {
        thresholdPoint.push({
          seriesName: name,
          dataName: key,
          dataValue: data[name][key]
        });
      }
    }
  }
  return thresholdPoint;
}
function setMarkLineSeries(baseOpt, iChartOpt, radarKeys) {
  var markLine = iChartOpt.markLine, gradient = iChartOpt.gradient;
  if (gradient) return;
  if (markLine) {
    var data = iChartOpt.data;
    var isThreshold = !!(type.isObject(markLine) && markLine != null && markLine.threshold);
    var exceeded = getExceededMarkLineValue(data, markLine, isThreshold);
    exceeded.forEach(function(item, index) {
      var seriesName = item.seriesName;
      var dataName = item.dataName;
      var dataNameIndex = radarKeys.indexOf(dataName);
      var dataValue = item.dataValue;
      var redPointerRadar = handleRedPointerRadar(baseOpt, radarKeys, dataNameIndex, dataName);
      var redPointerSeries = BaseOption.handleRedPointerSeries(index, dataValue, seriesName);
      baseOpt.radar.push(redPointerRadar);
      baseOpt.series.push(redPointerSeries);
    });
  }
}
exports.handleRedPointerSeries = BaseOption.handleRedPointerSeries;
exports.getExceededMarkLineValue = getExceededMarkLineValue;
exports.handleRedPointerRadar = handleRedPointerRadar;
exports.setMarkLineSeries = setMarkLineSeries;
exports.setSeries = setSeries;
