"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const util = require("./util.js");
const LeafManager = require("./LeafManager.js");
const NodeManager = require("./NodeManager.js");
const index = require("./line/index.js");
const TagManager = require("./TagManager.js");
function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t = arguments[e];
      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
var BranchManager = /* @__PURE__ */ function() {
  function BranchManager2(data, container, snowInstance, subRootConfig, parentInstance) {
    if (subRootConfig === void 0) {
      subRootConfig = {};
    }
    if (parentInstance === void 0) {
      parentInstance = {};
    }
    var option = snowInstance.option, imageList = snowInstance.imageList, drag = snowInstance.drag;
    var _subRootConfig = subRootConfig, _subRootConfig$parent = _subRootConfig.parentDeg, parentDeg = _subRootConfig$parent === void 0 ? 0 : _subRootConfig$parent, _subRootConfig$isSubR = _subRootConfig.isSubRoot, isSubRoot = _subRootConfig$isSubR === void 0 ? false : _subRootConfig$isSubR, _subRootConfig$style = _subRootConfig.style, style = _subRootConfig$style === void 0 ? {} : _subRootConfig$style, _subRootConfig$equalD = _subRootConfig.equalDeg, equalDeg = _subRootConfig$equalD === void 0 ? 0 : _subRootConfig$equalD;
    var _parentInstance = parentInstance, _parentInstance$leafs = _parentInstance.leafsArr, leafsArr = _parentInstance$leafs === void 0 ? [] : _parentInstance$leafs, _parentInstance$cente = _parentInstance.centerDomArr, centerDomArr = _parentInstance$cente === void 0 ? [] : _parentInstance$cente, _parentInstance$cente2 = _parentInstance.centerLineArr, centerLineArr = _parentInstance$cente2 === void 0 ? [] : _parentInstance$cente2, _parentInstance$cente3 = _parentInstance.centerTagArr, centerTagArr = _parentInstance$cente3 === void 0 ? [] : _parentInstance$cente3, _parentInstance$activ = _parentInstance.activeMac, activeMac = _parentInstance$activ === void 0 ? null : _parentInstance$activ, _parentInstance$leafT = _parentInstance.leafTrigger, leafTrigger = _parentInstance$leafT === void 0 ? null : _parentInstance$leafT, _parentInstance$dista = _parentInstance.distance, distance = _parentInstance$dista === void 0 ? null : _parentInstance$dista;
    this.data = data;
    this.container = container;
    this.snowInstance = snowInstance;
    this.option = option;
    this.imageList = imageList;
    this.drag = drag;
    this.chartContainer = snowInstance.container;
    this.parentDeg = parentDeg;
    this.isSubRoot = isSubRoot;
    this.leafsArr = leafsArr;
    this.centerDomArr = centerDomArr;
    this.centerLineArr = centerLineArr;
    this.centerTagArr = centerTagArr;
    this.activeMac = activeMac;
    this.leafTrigger = leafTrigger;
    this.distance = distance;
    this.leafData = [];
    this.branchData = [];
    this.equalDeg = equalDeg;
    this.mount(data, style);
  }
  var _proto = BranchManager2.prototype;
  _proto.mount = function mount(data, style) {
    var group = this.grouping(style);
    this.initCenterDom(data, group);
    this.loopData(data.isRoot ? _extends({}, data, {
      level: 0
    }) : data, group);
  };
  _proto.grouping = function grouping(style) {
    var wrapper = util.insertDom(this.container, {
      "class": "sfc-branch " + (!this.isSubRoot ? "rootBranch" : "")
    }, _extends({}, style));
    wrapper.setAttribute("data-mac", this.data.mac);
    var coreWrapper = util.insertDom(wrapper, {
      "class": "sfc-core"
    }, {
      transform: "rotate(" + (360 - this.parentDeg) + "deg)"
    });
    var nodeWrapper = util.insertDom(coreWrapper, {
      "class": "sfc-node-container"
    });
    var lineWrapper = util.insertDom(coreWrapper, {
      "class": "sfc-line-container"
    });
    var tagWrapper = util.insertDom(coreWrapper, {
      "class": "sfc-tag-container"
    });
    this.wrapper = wrapper;
    return {
      wrapper,
      coreWrapper,
      nodeWrapper,
      lineWrapper,
      tagWrapper
    };
  };
  _proto.initCenterDom = function initCenterDom(data, _ref) {
    var nodeWrapper = _ref.nodeWrapper;
    this.centerDom = new NodeManager.default(nodeWrapper, data, 0, true, this);
    this.centerDomArr.push(this.centerDom);
    this.centerDom.setSubRootWidth();
  };
  _proto.loopData = function loopData(data, group) {
    var _data$children, _this = this;
    (_data$children = data.children) == null ? void 0 : _data$children.forEach(function(dataItem) {
      var _dataItem$children;
      if (dataItem.isAP || (_dataItem$children = dataItem.children) != null && _dataItem$children.length) {
        _this.branchData.push(_extends({}, dataItem, {
          level: data.level + 1
        }));
      } else {
        _this.leafData.push(_extends({}, dataItem, {
          level: data.level + 1
        }));
      }
    });
    var coreWrapper = group.coreWrapper;
    if (!this.leafData.length && !this.option.overAll) {
      coreWrapper.classList.add("noLeafChild");
    }
    var degInterval = this.arrangSubRoot(this.branchData, group);
    this.arrangeLeaf(degInterval, group);
  };
  _proto.arrangSubRoot = function arrangSubRoot(data, group) {
    var _this2 = this;
    var startAngle = 0;
    var splitNum = data.length;
    var degInterval = [this.isSubRoot ? util.degPositive(this.parentDeg + 180) : 0];
    if (this.isSubRoot) {
      splitNum = data.length + 1;
      startAngle = 180 + this.parentDeg - 360 / splitNum * data.length;
      degInterval.push(util.degPositive(startAngle));
    } else {
      if (this.option.isLink) {
        splitNum = data.length + 1;
        startAngle = 360 / splitNum;
      }
      degInterval.push(0);
    }
    data.forEach(function(dataItem, index2) {
      var deg = dataItem.deg !== void 0 && dataItem.deg !== null ? dataItem.deg : 360 / splitNum * index2 + startAngle;
      degInterval.push(util.degPositive(deg));
      var equalDeg = Math.min(360 / splitNum, 180);
      _this2.loopSubRootData(dataItem, group, deg, equalDeg);
    });
    degInterval.sort(function(a, b) {
      return a - b;
    });
    return Array.from(new Set(degInterval));
  };
  _proto.arrangeLeaf = function arrangeLeaf(degInterval, group) {
    var _this3 = this;
    var intervalNum = degInterval.length;
    var intervalLeafNum = new Array(intervalNum).fill(1);
    this.leafData.forEach(function(item, index2) {
      if (index2 >= intervalNum) {
        intervalLeafNum[index2 % intervalNum]++;
      }
    });
    var leafDegArr = [];
    var cloneLeafsArr = [].concat(this.leafsArr);
    this.leafData.forEach(function(item, index2) {
      var degSpace = (degInterval[index2 % intervalNum + 1] || degInterval[0] + 360) - degInterval[index2 % intervalNum];
      var deg = degSpace / (intervalLeafNum[index2 % intervalNum] + 1) * (parseInt(index2 / intervalNum) + 1) + degInterval[index2 % intervalNum];
      leafDegArr.push(deg);
      _this3.loopLeafData(item, group, deg);
    });
    this.multipleTerminal(leafDegArr, group, cloneLeafsArr, intervalNum, degInterval, intervalLeafNum);
  };
  _proto.multipleTerminal = function multipleTerminal(leafDegArr, group, cloneLeafsArr, intervalNum, degInterval, intervalLeafNum) {
    var _this4 = this;
    if (!leafDegArr.length) return;
    leafDegArr.sort(function(a, b) {
      return a - b;
    });
    var minAngleDiff = leafDegArr[1] - leafDegArr[0];
    leafDegArr.forEach(function(item, index2) {
      if (index2 <= leafDegArr.length - 2) {
        if (leafDegArr[index2 + 1] - leafDegArr[index2] < minAngleDiff) {
          minAngleDiff = leafDegArr[index2 + 1] - leafDegArr[index2];
        }
      }
    });
    var wrapper = group.wrapper;
    var waistLength = util.pxToNumber(util.getStyle(wrapper, "width")) / 2;
    var sideLength = util.getSideLength(minAngleDiff, waistLength);
    var leafDom = this.leafsArr[0].node.returnWrapper();
    var leafDomWidth = util.pxToNumber(util.getStyle(leafDom, "width"));
    if (sideLength < leafDomWidth + 20) {
      sideLength = leafDomWidth + 20;
      for (var i = cloneLeafsArr.length; i < this.leafsArr.length; i++) {
        this.leafsArr[i].node.container.removeChild(this.leafsArr[i].node.dom);
        this.leafsArr[i].tag.container.removeChild(this.leafsArr[i].tag.dom);
        this.leafsArr[i].line.container.removeChild(this.leafsArr[i].line.canvas);
      }
      this.leafsArr.splice(cloneLeafsArr.length);
      if (this.option.overAll) {
        minAngleDiff = util.getTriangleAngle(sideLength, waistLength);
        var maxLeafNum = Math.floor(360 / intervalNum / minAngleDiff) * intervalNum;
        var newLeafDegArr = [];
        var _intervalLeafNum = new Array(intervalNum).fill(1);
        this.leafData.forEach(function(item, index2) {
          if (index2 > maxLeafNum - 1) return;
          if (index2 >= intervalNum) {
            _intervalLeafNum[index2 % intervalNum]++;
          }
        });
        this.leafData.forEach(function(item, index2) {
          var degSpace = (degInterval[index2 % intervalNum + 1] || degInterval[0] + 360) - degInterval[index2 % intervalNum];
          var deg = degSpace / (_intervalLeafNum[index2 % intervalNum] + 1) * (parseInt(index2 / intervalNum) + 1) + degInterval[index2 % intervalNum];
          newLeafDegArr.push(deg);
        });
        this.leafData.forEach(function(item, index2) {
          var deg = newLeafDegArr[index2];
          if (index2 >= maxLeafNum) return;
          _this4.loopLeafData(item, group, deg);
        });
      } else {
        waistLength = util.getWaistLength(minAngleDiff, sideLength);
        var wrapperSize = util.pxToNumber(util.getStyle(wrapper, "width"));
        util.setStyle(wrapper, {
          width: waistLength * 2 + "px",
          height: waistLength * 2 + "px",
          // 多终端的话，根节点定位中心还是50%，其它次级根节点需要额外计算自身宽度的变化
          left: this.isSubRoot ? "calc(100% + " + (this.distance + waistLength - wrapperSize / 2) + "px)" : "50%"
        });
        this.leafData.forEach(function(item, index2) {
          _this4.loopLeafData(item, group, leafDegArr[index2]);
        });
      }
    }
  };
  _proto.loopSubRootData = function loopSubRootData(dataItem, _ref2, deg, equalDeg) {
    var wrapper = _ref2.wrapper, tagWrapper = _ref2.tagWrapper, lineWrapper = _ref2.lineWrapper;
    var _this$option = this.option, overAll = _this$option.overAll, distance = _this$option.distance;
    var minDistance = distance.minDistance, minDistanceDrill = distance.minDistanceDrill;
    this.distance = overAll ? minDistance : minDistanceDrill;
    this.subBranch = new BranchManager2(dataItem, wrapper, this.snowInstance, {
      style: {
        position: "absolute",
        transform: "translate(-50%,-50%) rotate(" + (deg - this.parentDeg) + "deg)"
      },
      parentDeg: deg,
      isSubRoot: true,
      equalDeg
    }, this);
    this.upDateSubRootStyle(equalDeg, wrapper, dataItem, overAll);
    this.initCenterLine(dataItem, deg, {
      tagWrapper,
      lineWrapper
    });
  };
  _proto.upDateSubRootStyle = function upDateSubRootStyle(equalDeg, wrapper, dataItem, overAll) {
    var waistLength = util.getWaistLength(equalDeg, util.getCssNumber(this.subBranch.wrapper, "width") + 50 * 2);
    this.distance = dataItem.distance || Math.max(waistLength - util.getCssNumber(wrapper, "width"), this.distance);
    var sizeRatio = util.pxToNumber(util.getStyle(this.subBranch.wrapper, "width")) / util.pxToNumber(util.getStyle(wrapper, "width"));
    var transformOrigin = "calc(" + (50 * (1 - 1 / sizeRatio) + "%") + " - " + this.distance + "px) center";
    var subBranchStyle = {};
    if (overAll) {
      subBranchStyle = {
        left: "calc(100% + " + this.distance + "px)",
        transformOrigin
      };
    } else {
      subBranchStyle = {
        transformOrigin: "unset"
        // 设置旋转中心，如果是家庭拓扑，会在setTransformOrigin中处理。
      };
      if (util.getCssNumber(this.subBranch.wrapper, "left") < util.getCssNumber(this.subBranch.wrapper.parentNode, "width") + this.distance) {
        subBranchStyle.left = "calc(100% + " + this.distance + "px)";
      }
    }
    util.setStyle(this.subBranch.wrapper, subBranchStyle);
  };
  _proto.loopLeafData = function loopLeafData(dataItem, group, deg) {
    this.leaf = new LeafManager.default(dataItem, group, deg, this);
    this.leafsArr.push(this.leaf);
  };
  _proto.initCenterLine = function initCenterLine(data, deg, _ref3) {
    var tagWrapper = _ref3.tagWrapper, lineWrapper = _ref3.lineWrapper;
    var leafNode = this.centerDomArr.filter(function(item) {
      return item.data.mac === data.mac;
    })[0];
    this.centerLineArr.push(new index.default({
      lineWrapper,
      leafNode
    }, data, deg, this, true));
    this.centerTagArr.push(new TagManager.default(tagWrapper, data, this.option, deg, true, this.drag));
  };
  _proto.reRender = function reRender(activeMac, leafTrigger) {
    var _this5 = this;
    this.branchData = [];
    this.leafData = [];
    this.leafsArr = [];
    this.centerLineArr = [];
    this.centerTagArr = [];
    this.centerDomArr = [];
    this.container.innerHTML = null;
    this.activeMac = activeMac;
    this.leafTrigger = leafTrigger;
    this.mount(this.snowInstance.data[0], {});
    if (!this.option.overAll) {
      this.snowInstance.setPositionLeft();
      this.snowInstance.setTransformOrigin();
      this.snowInstance.setTagPosition();
    }
    var _this$snowInstance$br = this.snowInstance.branchs[0], centerDomArr = _this$snowInstance$br.centerDomArr, leafsArr = _this$snowInstance$br.leafsArr;
    if (leafTrigger) {
      leafsArr.forEach(function(leaf) {
        if (leaf.node.data.mac === activeMac) {
          _this5.drag.moveTargetToCenter(leaf.node.wrapper);
        }
      });
    } else {
      centerDomArr.forEach(function(node) {
        if (node.data.mac === activeMac) {
          _this5.drag.moveTargetToCenter(node.returnWrapper().parentNode);
        }
      });
    }
  };
  return BranchManager2;
}();
exports.default = BranchManager;
