"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const util = require("../util.js");
const CommonConstant = require("../CommonConstant.js");
const LineStraight = require("./LineStraight.js");
const LineArrow = require("./LineArrow.js");
const LineDashed = require("./LineDashed.js");
const LineDotted = require("./LineDotted.js");
var _LineFactory;
function _extends() {
  return _extends = Object.assign ? Object.assign.bind() : function(n) {
    for (var e = 1; e < arguments.length; e++) {
      var t = arguments[e];
      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);
    }
    return n;
  }, _extends.apply(null, arguments);
}
var LineFactory = (_LineFactory = {}, _LineFactory[CommonConstant.radioType.TYPE_Straight] = LineStraight.default, _LineFactory[CommonConstant.radioType.TYPE_Arrow] = LineArrow.default, _LineFactory[CommonConstant.radioType.TYPE_Dashed] = LineDashed.default, _LineFactory[CommonConstant.radioType.TYPE_Dotted] = LineDotted.default, _LineFactory);
var LineManager = /* @__PURE__ */ function() {
  function LineManager2(_ref, data, deg, _ref2, isSubRoot, tagWidth) {
    var lineWrapper = _ref.lineWrapper, leafNode = _ref.leafNode;
    var imageList = _ref2.imageList, option = _ref2.option, centerDom = _ref2.centerDom, distance = _ref2.distance, drag = _ref2.drag;
    this.isSubRoot = isSubRoot;
    this.container = lineWrapper;
    this.data = data;
    this.option = option;
    this.deg = util.degPositive(deg);
    this.imageList = imageList;
    this.canvasWidth;
    this.canvasHeight = 150;
    this.direction = util.setDirection(this.deg);
    this.canvas;
    this.ctx;
    this.iconSize = 10;
    this.distance = distance;
    this.centerDom = centerDom.dom;
    this.leafNode = leafNode.dom;
    this.drag = drag;
    this.tagWidth = isSubRoot ? 0 : tagWidth;
    this.createCanvas();
    this.draw();
  }
  var _proto = LineManager2.prototype;
  _proto.createCanvas = function createCanvas() {
    var containerWidth = util.pxToNumber(util.getStyle(this.container, "width"));
    var centerDomWidth = util.pxToNumber(util.getStyle(this.centerDom, "width"));
    var centerparentWidth = util.pxToNumber(util.getStyle(this.centerDom.parentNode, "width"));
    var leafNodeWidth = util.pxToNumber(util.getStyle(this.leafNode, "width"));
    this.canvasWidth = containerWidth / 2 - centerDomWidth / 2;
    if (this.isSubRoot) {
      this.canvasWidth = (this.data.distance || this.distance) - leafNodeWidth / 2 + centerparentWidth / 2 - centerDomWidth / 2;
    }
    var left = CommonConstant.direction.left;
    var canvasStyle;
    if (this.direction === left) {
      canvasStyle = {
        right: "calc(50% + " + centerDomWidth / 2 + "px)",
        transformOrigin: "calc(100% + " + centerDomWidth / 2 + "px) center",
        transform: "translate(0,-50%) rotate(" + (this.deg - 180) + "deg)"
      };
    } else {
      canvasStyle = {
        left: "calc(50% + " + centerDomWidth / 2 + "px)",
        transformOrigin: -1 * centerDomWidth / 2 + "px center",
        transform: "translate(0,-50%) rotate(" + this.deg + "deg)"
      };
    }
    this.canvas = util.insertDom(this.container, {
      "class": "sfc-line-item",
      width: this.canvasWidth * 2,
      height: this.canvasHeight * 2
    }, _extends({}, canvasStyle), "canvas");
    this.ctx = this.canvas.getContext("2d");
    this.canvas.style.width = this.canvasWidth + "px";
    this.canvas.style.height = this.canvasHeight + "px";
    this.ctx.scale(2, 2);
  };
  _proto.destroyCanvas = function destroyCanvas() {
    this.container.removeChild(this.canvas);
  };
  _proto.draw = function draw() {
    util.judgeDisabled(this.option, [this.canvas]);
    if (this.option.overAll && !this.isSubRoot) {
      return;
    }
    this.ctx.clearRect(0, 0, 1e4, 1e4);
    this.ctx.beginPath();
    var verCenterPoint = (this.canvasHeight - this.iconSize) / 2;
    this.drawLine(verCenterPoint);
    if (!this.option.overAll && this.option.scaleAdaptive) {
      var scale = this.drag.scale;
      if (scale >= 0.5 && scale <= 0.75) {
        !this.isSubRoot && util.setStyle(this.canvas, {
          display: "none"
        });
      } else if (scale > 0.75 && scale < 0.95) {
        util.setStyle(this.canvas, {
          display: "block"
        });
      } else if (scale >= 0.95 && scale <= 1.5) {
        this.drawText(verCenterPoint);
        util.setStyle(this.canvas, {
          display: "block"
        });
      }
    } else {
      this.drawText(verCenterPoint);
      util.setStyle(this.canvas, {
        display: "block"
      });
    }
    this.ctx.closePath();
  };
  _proto.drawLine = function drawLine(verCenterPoint) {
    var _this$data = this.data, connectInterface = _this$data.connectInterface, lineStrokeColor = _this$data.lineStrokeColor;
    var lineType;
    var intervalType;
    if (connectInterface.indexOf("SSID") !== -1) {
      lineType = "Arrow";
      intervalType = connectInterface.slice(5);
    } else if (connectInterface.indexOf("wireless") !== -1) {
      lineType = "Arrow";
      intervalType = "wireless";
    } else if (connectInterface.indexOf("Dashed") !== -1) {
      lineType = "Dashed";
    } else if (connectInterface.indexOf("Dotted") !== -1) {
      lineType = "Dotted";
    } else {
      lineType = "Straight";
    }
    var line = new LineFactory[lineType](this, verCenterPoint, CommonConstant.interval[intervalType], lineStrokeColor, intervalType);
    line.draw();
  };
  _proto.drawText = function drawText(verCenterPoint) {
    var _this$imageList = this.imageList, imageUp = _this$imageList.imageUp, imageDown = _this$imageList.imageDown;
    var _this$data2 = this.data, upRate = _this$data2.upRate, downRate = _this$data2.downRate;
    var left = CommonConstant.direction.left;
    var maxTextWidth = Math.max(util.getTextWidth(upRate), util.getTextWidth(downRate));
    var actualCanvasRatio = (100 - (100 - CommonConstant.tagPosition) * 2) / 100;
    var horCenterPoint;
    if (this.direction === left) {
      horCenterPoint = this.canvasWidth - maxTextWidth - (this.canvasWidth * actualCanvasRatio - maxTextWidth - this.tagWidth) / 2;
    } else {
      horCenterPoint = (this.canvasWidth * actualCanvasRatio - maxTextWidth - this.tagWidth) / 2;
    }
    if (this.isSubRoot) {
      horCenterPoint = (this.canvasWidth - maxTextWidth) / 2;
    }
    var _textStyle = CommonConstant.textStyle(this.option.theme), font = _textStyle.font, fillStyle = _textStyle.fillStyle, unitStyle = _textStyle.unitStyle;
    this.ctx.font = font;
    if (upRate) {
      this.ctx.fillStyle = fillStyle;
      this.ctx.drawImage(imageUp.image, horCenterPoint - 16, verCenterPoint - 12, this.iconSize, this.iconSize);
      this.ctx.fillText(util.extractNumbersAndDots(upRate), horCenterPoint, verCenterPoint - 2);
      this.ctx.fillStyle = unitStyle;
      this.ctx.fillText(util.getUnitText(upRate), horCenterPoint + util.getTextWidth(util.extractNumbersAndDots(upRate)) + 4, verCenterPoint - 2);
    }
    if (downRate) {
      this.ctx.fillStyle = fillStyle;
      this.ctx.drawImage(imageDown.image, horCenterPoint - 16, verCenterPoint + 12, this.iconSize, this.iconSize);
      this.ctx.fillText(util.extractNumbersAndDots(downRate), horCenterPoint, verCenterPoint + 22);
      this.ctx.fillStyle = unitStyle;
      this.ctx.fillText(util.getUnitText(downRate), horCenterPoint + util.getTextWidth(util.extractNumbersAndDots(downRate)) + 4, verCenterPoint + 22);
    }
  };
  _proto.refreshData = function refreshData(data) {
    this.data = data !== void 0 ? data : this.data;
    this.draw();
  };
  return LineManager2;
}();
exports.default = LineManager;
