"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const CommonConstant = require("./CommonConstant.js");
var setAttribute = function setAttribute2(dom, attribute) {
  for (var i in attribute) {
    if (Object.hasOwnProperty.call(attribute, i)) {
      dom.setAttribute(i, attribute[i]);
    }
  }
};
var setStyle = function setStyle2(dom, styles) {
  for (var i in styles) {
    if (Object.hasOwnProperty.call(styles, i)) {
      dom.style[i] = styles[i];
    }
  }
};
var onImageLoad = function onImageLoad2(imageList, callback) {
  var onloadNum = 0;
  var loadEvent = function loadEvent2() {
    onloadNum++;
    if (onloadNum === Object.keys(imageList).length) {
      callback();
    }
  };
  for (var item in imageList) {
    if (Object.hasOwnProperty.call(imageList, item)) {
      imageList[item]["image"].addEventListener("load", loadEvent);
    }
  }
};
var pxToNumber = function pxToNumber2(pxText) {
  return pxText.split("px")[0] - 0;
};
var createDom = function createDom2(type) {
  if (type === void 0) {
    type = "div";
  }
  return document.createElement(type);
};
var insertDom = function insertDom2(container, attribute, styles, type) {
  var dom = createDom(type);
  setAttribute(dom, attribute);
  setStyle(dom, styles);
  container.appendChild(dom);
  return dom;
};
var getStyle = function getStyle2(dom, cssName) {
  return window.getComputedStyle(dom)[cssName];
};
var getCssNumber = function getCssNumber2(dom, cssName) {
  return pxToNumber(getStyle(dom, cssName));
};
var degPositive = function degPositive2(angle) {
  var deg;
  if (angle >= 0 && angle <= 360) {
    deg = angle;
  } else if (angle > 360) {
    deg = angle % 360;
  } else {
    deg = 360 + angle % 360;
  }
  return deg;
};
function extractNumbersAndDots(str) {
  return str.match(/[\d.]+/g)[0];
}
var getUnitText = function getUnitText2(text) {
  var valString = extractNumbersAndDots(text).toString();
  var unit = text.slice(valString.length);
  return unit;
};
var setDefault = function setDefault2(userOption, defaultOption) {
  for (var key in defaultOption) {
    if (userOption[key] === void 0 || userOption[key] === null) {
      userOption[key] = defaultOption[key];
    }
  }
};
var judgeValue = function judgeValue2(value, min, max) {
  var flag = true;
  if (parseFloat(value) <= parseFloat(max) && parseFloat(value) >= parseFloat(min)) {
    flag = true;
  } else {
    flag = false;
  }
  return flag;
};
var _loopData = function loopData(data, newData) {
  data.forEach(function(item, index) {
    newData.push(item);
    if (item.children && item.children.length) {
      _loopData(item.children, newData);
    }
  });
};
var arrFlat = function arrFlat2(data) {
  var newData = [];
  _loopData(data, newData);
  return newData;
};
var isDataDisabled = function isDataDisabled2(data, container, option) {
  var disabledData = data.filter(function(item) {
    return item.disabled === true;
  });
  if (disabledData.length && !option.overAll) {
    container.classList.add("disabled");
  } else {
    container.classList.remove("disabled");
  }
};
var compareRefresh = function compareRefresh2(data, branchs, _ref, that) {
  var centerArr = _ref.centerArr, target = _ref.target;
  var flatData = arrFlat(data);
  branchs.forEach(function(item) {
    flatData.forEach(function(dataItem) {
      item[centerArr].forEach(function(val) {
        if (val.data.mac === dataItem.mac) {
          val.refreshData(dataItem);
        }
      });
      item.leafsArr.forEach(function(val) {
        if (val.data.mac === dataItem.mac) {
          val[target].refreshData(dataItem);
        }
      });
    });
  });
  isDataDisabled(flatData, that.container, that.option);
};
var judgeDisabled = function judgeDisabled2(option, domArr, isLeaf, dataItem, isNode) {
  var data = option.data, overAll = option.overAll;
  if (overAll) {
    return;
  }
  var hasDisabled = false;
  var tagDisabled = false;
  var flatData = arrFlat(data);
  if (!isLeaf) {
    if (!isNode) {
      flatData.forEach(function(item) {
        if (item.disabled) {
          hasDisabled = true;
        }
      });
    } else {
      if (dataItem.disabled) {
        hasDisabled = true;
      }
    }
  } else if (isLeaf) {
    flatData.forEach(function(item) {
      if (item.disabled) {
        tagDisabled = true;
      }
    });
    if (dataItem.disabled) {
      hasDisabled = true;
    }
  }
  domArr.forEach(function(item) {
    setStyle(item, {
      opacity: hasDisabled ? CommonConstant.opacity : 1
    });
  });
  if (isLeaf && tagDisabled) {
    setStyle(domArr[1], {
      opacity: CommonConstant.opacity
    });
  }
};
var moveToCenter = function moveToCenter2(_ref2, that, isNode) {
  var activeMac = _ref2.activeMac, leafTrigger = _ref2.leafTrigger, drag = _ref2.drag, chartContainer = _ref2.chartContainer;
  if (activeMac && that.data.mac === activeMac) {
    if (that.option.overAll) {
      drag.moveTargetToCenter(chartContainer);
    } else {
      if (isNode) {
        drag.moveTargetToCenter(leafTrigger ? that.wrapper : that.wrapper.parentNode.parentNode);
      } else {
        drag.moveTargetToCenter(leafTrigger ? that.node.returnWrapper() : that.node.returnWrapper().parentNode.parentNode);
      }
    }
  }
};
var getRootTextColor = function getRootTextColor2(data) {
  return judgeValue(data.rxPower, data.rxPowerMin, data.rxPowerMax) ? CommonConstant.singlecolor : CommonConstant.singleWarnColor;
};
var getLeafTextColor = function getLeafTextColor2(data) {
  var _data$connectInterfac;
  return judgeValue(data.signalStrength, ((_data$connectInterfac = data.connectInterface) == null ? void 0 : _data$connectInterfac.indexOf("2.4G")) !== -1 ? CommonConstant.singleMin2G : CommonConstant.singleMin5G, CommonConstant.singleMax) ? CommonConstant.singlecolor : CommonConstant.singleWarnColor;
};
var getAPTextColor = function getAPTextColor2(data) {
  var _data$connectInterfac2;
  var single;
  var singleTextColor;
  if (((_data$connectInterfac2 = data.connectInterface) == null ? void 0 : _data$connectInterfac2.indexOf("SSID")) !== -1) {
    single = data.signalStrength;
    singleTextColor = getLeafTextColor(data);
  } else {
    single = data.rxPower;
    singleTextColor = getRootTextColor(data);
  }
  return {
    single,
    singleTextColor
  };
};
var getSingle = function getSingle2(data, type) {
  var single;
  var singleTextColor;
  switch (type) {
    case "root":
      single = data.rxPower;
      singleTextColor = getRootTextColor(data);
      break;
    case "leaf":
      single = data.signalStrength;
      singleTextColor = getLeafTextColor(data);
      break;
    case "ap":
      single = getAPTextColor(data).single;
      singleTextColor = getAPTextColor(data).singleTextColor;
      break;
  }
  return {
    single,
    singleTextColor
  };
};
var setDirection = function setDirection2(deg) {
  var flag;
  var left = CommonConstant.direction.left, right = CommonConstant.direction.right;
  if (deg >= 90 && deg < 270) {
    flag = left;
  } else {
    flag = right;
  }
  return flag;
};
var getTextWidth = function getTextWidth2(text, fontSize) {
  if (fontSize === void 0) {
    fontSize = 12;
  }
  var result = 0;
  var ele = document.createElement("span");
  ele.innerText = text;
  ele.setAttribute("style", "font-size: " + fontSize + "px");
  document.documentElement.append(ele);
  result = ele.offsetWidth;
  document.documentElement.removeChild(ele);
  return result;
};
var getSideLength = function getSideLength2(arc, r) {
  return Math.sin(arc / 2 / 180 * Math.PI) * r * 2;
};
var getWaistLength = function getWaistLength2(arc, sideLength) {
  return sideLength / 2 / Math.sin(arc / 2 / 180 * Math.PI);
};
function radiansToDegrees(radian) {
  return 180 * radian / Math.PI;
}
var getTriangleAngle = function getTriangleAngle2(sideLength, r) {
  return radiansToDegrees(Math.asin(sideLength / 2 / r)) * 2;
};
var getDistance = function getDistance2(dom1, dom2) {
  var _dom1$getBoundingClie = dom1.getBoundingClientRect(), x1 = _dom1$getBoundingClie.x, y1 = _dom1$getBoundingClie.y, width1 = _dom1$getBoundingClie.width, height1 = _dom1$getBoundingClie.height;
  var _dom2$getBoundingClie = dom2.getBoundingClientRect(), x2 = _dom2$getBoundingClie.x, y2 = _dom2$getBoundingClie.y, width2 = _dom2$getBoundingClie.width, height2 = _dom2$getBoundingClie.height;
  var center1 = {
    x: x1 + width1 / 2,
    y: y1 + height1 / 2
  };
  var center2 = {
    x: x2 + width2 / 2,
    y: y2 + height2 / 2
  };
  var distance = Math.sqrt(Math.pow(center2.x - center1.x, 2) + Math.pow(center2.y - center1.y, 2));
  return distance;
};
var getAreaScope = function getAreaScope2(nodeArr) {
  var scope = {
    left: nodeArr[0].getBoundingClientRect().left,
    right: nodeArr[0].getBoundingClientRect().right,
    top: nodeArr[0].getBoundingClientRect().top,
    bottom: nodeArr[0].getBoundingClientRect().bottom
  };
  nodeArr.forEach(function(node) {
    var _node$getBoundingClie = node.getBoundingClientRect(), left = _node$getBoundingClie.left, right = _node$getBoundingClie.right, top = _node$getBoundingClie.top, bottom = _node$getBoundingClie.bottom;
    if (left < scope.left) {
      scope.left = left;
    }
    if (right > scope.right) {
      scope.right = right;
    }
    if (top < scope.top) {
      scope.top = top;
    }
    if (bottom > scope.bottom) {
      scope.bottom = bottom;
    }
  });
  return {
    width: scope.right - scope.left,
    height: scope.bottom - scope.top
  };
};
var boundaryJudge = function boundaryJudge2(e, container, toolTip) {
  var distance = 20;
  var x = e.clientX + distance;
  var y = e.clientY + distance;
  var _container$getBoundin = container.getBoundingClientRect(), containerWidth = _container$getBoundin.width, containerHeight = _container$getBoundin.height, containerLeft = _container$getBoundin.x, containerTop = _container$getBoundin.y;
  var _toolTip$getBoundingC = toolTip.getBoundingClientRect(), toolTipWidth = _toolTip$getBoundingC.width, toolTipHeight = _toolTip$getBoundingC.height;
  if (x + toolTipWidth > containerLeft + containerWidth) {
    x = e.clientX - toolTipWidth - distance / 2;
  }
  if (y + toolTipHeight > containerTop + containerHeight) {
    y = e.clientY - toolTipHeight - distance / 2;
  }
  return {
    x,
    y
  };
};
exports.arrFlat = arrFlat;
exports.boundaryJudge = boundaryJudge;
exports.compareRefresh = compareRefresh;
exports.createDom = createDom;
exports.degPositive = degPositive;
exports.extractNumbersAndDots = extractNumbersAndDots;
exports.getAreaScope = getAreaScope;
exports.getCssNumber = getCssNumber;
exports.getDistance = getDistance;
exports.getSideLength = getSideLength;
exports.getSingle = getSingle;
exports.getStyle = getStyle;
exports.getTextWidth = getTextWidth;
exports.getTriangleAngle = getTriangleAngle;
exports.getUnitText = getUnitText;
exports.getWaistLength = getWaistLength;
exports.insertDom = insertDom;
exports.isDataDisabled = isDataDisabled;
exports.judgeDisabled = judgeDisabled;
exports.judgeValue = judgeValue;
exports.moveToCenter = moveToCenter;
exports.onImageLoad = onImageLoad;
exports.pxToNumber = pxToNumber;
exports.setAttribute = setAttribute;
exports.setDefault = setDefault;
exports.setDirection = setDirection;
exports.setStyle = setStyle;
