"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const convert = require("../../util/convert.js");
const defendXSS = require("../../util/defendXSS.js");
const index = require("../../feature/token/index.js");
const chartToken = require("./chartToken.js");
function _createForOfIteratorHelperLoose(r, e) {
  var t = "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"];
  if (t) return (t = t.call(r)).next.bind(t);
  if (Array.isArray(r) || (t = _unsupportedIterableToArray(r)) || e) {
    t && (r = t);
    var o = 0;
    return function() {
      return o >= r.length ? { done: true } : { done: false, value: r[o++] };
    };
  }
  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _unsupportedIterableToArray(r, a) {
  if (r) {
    if ("string" == typeof r) return _arrayLikeToArray(r, a);
    var t = {}.toString.call(r).slice(8, -1);
    return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0;
  }
}
function _arrayLikeToArray(r, a) {
  (null == a || a > r.length) && (a = r.length);
  for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e];
  return n;
}
var TimeLine = /* @__PURE__ */ function() {
  function TimeLine2(canvasId, currentTime, option) {
    this.showElement = function(element) {
      element.style.display = "block";
    };
    this.hideElement = function(element) {
      element.style.display = "none";
    };
    this.canvas = document.getElementById(canvasId);
    this.ctx = this.canvas.getContext("2d");
    this.option = option;
    this.currentTime = currentTime;
    this.canvas.style.background = this.option.background || "transparent";
    this.axisLineColor = this.option.axisLineStyle && this.option.axisLineStyle.color || index.default.config.xAxisTickLineColor;
    this.labelColor = this.option.labelStyle && this.option.labelStyle.color || index.default.config.xAxisLabelColor;
    this.totalHours = null;
    this.minuteStep = [1, 2, 5, 10, 15, 20, 30, 60, 120, 180, 240, 360, 720, 1440];
    this.minScaleSpacing = 20;
    this.minLargeScaleSpacing = 80;
    this.zoom = 24;
    this.selectedPosition = [];
    this.setTipTimeOut = null;
    this.init();
    var _this = this;
    this.eventListener = {
      mouseup: function mouseup(event) {
        _this.clickEvent(event);
        _this.hoverMove(event);
      },
      mousemove: function mousemove(event) {
        _this.hoverMove(event);
      },
      mouseleave: function mouseleave(event) {
        _this.init();
        document.getElementsByClassName("timeline_hoverPointer")[0].style.display = "none";
        document.getElementsByClassName("timeline_hoverTooltip")[0].style.display = "none";
      }
    };
    this.canvas.addEventListener("mousemove", this.eventListener.mousemove);
    this.canvas.addEventListener("mouseup", this.eventListener.mouseup);
    this.canvas.addEventListener("mouseleave", this.eventListener.mouseleave);
  }
  var _proto = TimeLine2.prototype;
  _proto.init = function init() {
    this.refreshStartTimestamp();
    this.clearCanvas();
    this.drawScale();
    this.option.currentTime && this.drawCursor(this.option.currentTime);
    this.option.alarmData && this.drawAlarm(this.option);
    this.drawRoundRect(20, this.canvas.height - 40, this.canvas.width - 40, 8, [4, 4, 4, 4]);
    this.option.statusRangeData && this.drawStatusRange(this.option);
  };
  _proto.refreshStartTimestamp = function refreshStartTimestamp() {
    if (this.option.timeRange !== void 0) {
      this.totalHours = (this.option.timeRange.endTime - this.option.timeRange.startTime) / (60 * 60 * 1e3);
    }
    this.startTimestamp = this.option.timeRange && this.option.timeRange.startTime || this.currentTime - this.totalHours * 60 * 60 * 1e3;
  };
  _proto.clearCanvas = function clearCanvas() {
    this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
  };
  _proto.drawScale = function drawScale() {
    var oneMinutePx = (this.canvas.width - 40) / (this.totalHours * 60);
    var oneMSPx = oneMinutePx / (60 * 1e3);
    var scaleSpacing = this.minScaleSpacing;
    var scaleUnit = scaleSpacing / oneMinutePx;
    var len = this.minuteStep.length;
    if (this.option.interval !== void 0) {
      scaleUnit = this.option.interval;
      scaleSpacing = oneMinutePx * scaleUnit;
    } else {
      for (var i = 0; i < len; i++) {
        if (scaleUnit < this.minuteStep[i]) {
          scaleUnit = this.minuteStep[i];
          scaleSpacing = oneMinutePx * scaleUnit;
          break;
        }
      }
    }
    var mediumStep = 30;
    if (this.option.mediumStep !== void 0) {
      mediumStep = this.option.mediumStep;
    } else {
      for (var _i = 0; _i < len; _i++) {
        if (this.minLargeScaleSpacing / oneMinutePx <= this.minuteStep[_i]) {
          mediumStep = this.minuteStep[_i];
          break;
        }
      }
    }
    var totalScales = (this.canvas.width - 40) / scaleSpacing;
    var scaleLeft = 0;
    var scaleTime = this.startTimestamp;
    var lineHeight;
    var leftOffsetMs = scaleUnit * 60 * 1e3 - this.startTimestamp % (scaleUnit * 60 * 1e3);
    var leftOffsetPx = leftOffsetMs * oneMSPx;
    var oneScalesMS = scaleSpacing / oneMSPx;
    var scaleLineHeight = this.canvas.height - 30;
    for (var _i2 = 0; _i2 < totalScales + 1; _i2++) {
      var date = new Date(scaleTime);
      if (scaleTime / (60 * 1e3) % mediumStep === 0) {
        lineHeight = 6;
        var scaleText = this.createScaleText(date);
        this.ctx.font = "normal 12px HarmonyOS_Sans";
        this.ctx.fillStyle = this.labelColor;
        this.ctx.fillText(scaleText, scaleLeft + 20 - this.ctx.measureText(scaleText).width / 2, this.canvas.height - 4);
      } else {
        lineHeight = 3;
      }
      this.ctx.beginPath();
      this.ctx.moveTo(scaleLeft + 20, scaleLineHeight);
      this.ctx.lineTo(scaleLeft + 20, scaleLineHeight + lineHeight);
      this.ctx.closePath();
      this.ctx.lineWidth = 1;
      this.ctx.strokeStyle = this.axisLineColor;
      this.ctx.stroke();
      scaleLeft = leftOffsetPx + _i2 * scaleSpacing;
      scaleTime = this.startTimestamp + leftOffsetMs + _i2 * oneScalesMS;
    }
  };
  _proto.drawCursor = function drawCursor(currentTime) {
    this.selectedPosition.push(this.getTimePos(currentTime), convert.formatDate(currentTime, "hh:mm"));
    this.drawSelectedPointer(this.selectedPosition);
  };
  _proto.drawAlarm = function drawAlarm(option) {
    var alarmData = option.alarmData;
    var background = option.customBackground;
    for (var i = 0; i < alarmData.length; i++) {
      var x = this.getTimePos(alarmData[i].time);
      var type = alarmData[i].type;
      this.ctx.beginPath();
      this.ctx.arc(x, this.canvas.height - 60, 7, 0, 2 * Math.PI);
      this.ctx.closePath();
      this.ctx.fillStyle = background[type] || index.default.config.colorState.colorNone;
      this.ctx.fill();
    }
  };
  _proto.drawStatusRange = function drawStatusRange(option) {
    var statusRangeData = option.statusRangeData;
    var timeRange = option.timeRange;
    var background = option.customBackground;
    for (var i = 0; i < statusRangeData.length; i++) {
      var startPosition = this.getTimePos(statusRangeData[i].time[0]);
      var endPosition = this.getTimePos(statusRangeData[i].time[1]);
      var type = statusRangeData[i].type;
      this.ctx.beginPath();
      if (statusRangeData[i].time[0] === timeRange.startTime && statusRangeData[i].time[1] === timeRange.endTime) {
        this.roundRect(startPosition, this.canvas.height - 40, endPosition - startPosition, 8, [4, 4, 4, 4]);
      } else if (statusRangeData[i].time[0] === timeRange.startTime) {
        this.roundRect(startPosition, this.canvas.height - 40, endPosition - startPosition, 8, [4, 0, 0, 4]);
      } else if (statusRangeData[i].time[1] === timeRange.endTime) {
        this.roundRect(startPosition, this.canvas.height - 40, endPosition - startPosition, 8, [0, 4, 4, 0]);
      } else {
        this.ctx.rect(startPosition, this.canvas.height - 40, endPosition - startPosition, 8);
      }
      this.ctx.closePath();
      this.ctx.fillStyle = background[type] || index.default.config.colorState.colorNone;
      this.ctx.fill();
    }
  };
  _proto.drawRoundRect = function drawRoundRect(x, y, width, height, radius) {
    this.ctx.beginPath();
    this.roundRect(x, y, width, height, radius);
    this.ctx.closePath();
    this.ctx.fillStyle = chartToken.default.roundRectBg;
    this.ctx.fill();
  };
  _proto.roundRect = function roundRect(x, y, width, height, radiusArray) {
    this.ctx.moveTo(x + radiusArray[0], y);
    this.ctx.lineTo(x + width - radiusArray[1], y);
    this.ctx.arcTo(x + width, y, x + width, y + radiusArray[1], radiusArray[1]);
    this.ctx.lineTo(x + width, y + height - radiusArray[2]);
    this.ctx.arcTo(x + width, y + height, x + width - radiusArray[2], y + height, radiusArray[2]);
    this.ctx.lineTo(x + radiusArray[3], y + height);
    this.ctx.arcTo(x, y + height, x, y + height - radiusArray[3], radiusArray[3]);
    this.ctx.lineTo(x, y + radiusArray[0]);
    this.ctx.arcTo(x, y, x + radiusArray[0], y, radiusArray[0]);
  };
  _proto.hoverMove = function hoverMove(event) {
    var mouseX = this.getMouseXRelativePos(event);
    var hoverPointer = document.getElementsByClassName("timeline_hoverPointer")[0];
    var hoverTooltip = document.getElementsByClassName("timeline_hoverTooltip")[0];
    var hoverTime = this.getMousePosTime(event);
    if (mouseX > 0 && mouseX < this.canvas.width - 40) {
      this.showElement(hoverPointer);
      if (this.option.timeTooltip === true) {
        this.showElement(hoverTooltip);
        hoverTooltip.innerHTML = "<p>" + convert.formatDate(hoverTime, "yy-MM-dd hh:mm") + "</p>";
        if (mouseX + 142 > this.canvas.width - 40) {
          hoverTooltip.style.left = mouseX - 1 + 20 - 142 + "px";
        } else {
          hoverTooltip.style.left = mouseX - 1 + 30 + "px";
        }
        hoverTooltip.style.top = this.canvas.height - 30 + "px";
      }
      hoverPointer.style.left = mouseX - 1 + 20 + "px";
      hoverPointer.style.top = this.canvas.height - 30 + "px";
    } else {
      this.hideElement(hoverPointer);
      this.hideElement(hoverTooltip);
    }
    this.setAlarmTip(event, mouseX);
  };
  _proto.setAlarmTip = function setAlarmTip(event, mouseX) {
    var _this2 = this;
    var mouseY = this.getMouseYRelativePos(event);
    var alarmTooltip = document.getElementsByClassName("timeline_alarmTooltip")[0];
    alarmTooltip.addEventListener("mouseenter", function(e) {
      if (_this2.setTipTimeOut) {
        clearTimeout(_this2.setTipTimeOut);
      }
    });
    alarmTooltip.addEventListener("mouseleave", function(e) {
      _this2.closeAlarmTip(alarmTooltip);
    });
    var alarmData = this.option.alarmData;
    if (alarmData) {
      this.processAlarmData(alarmData, mouseX, mouseY, alarmTooltip);
    }
  };
  _proto.closeAlarmTip = function closeAlarmTip(alarmTooltip) {
    var _this3 = this;
    this.setTipTimeOut = setTimeout(function() {
      _this3.hideElement(alarmTooltip);
    }, 200);
  };
  _proto.processAlarmData = function processAlarmData(alarmData, mouseX, mouseY, alarmTooltip) {
    for (var _iterator = _createForOfIteratorHelperLoose(alarmData), _step; !(_step = _iterator()).done; ) {
      var data = _step.value;
      var x = this.getTimePos(data.time);
      if (this.isMouseInCircle(mouseX + 20, mouseY, x, this.canvas.height - 60)) {
        if (this.setTipTimeOut) {
          clearTimeout(this.setTipTimeOut);
        }
        this.showElement(alarmTooltip);
        this.option.onCircleHover(alarmTooltip, data);
        if (x - alarmTooltip.offsetWidth / 2 < 0) {
          alarmTooltip.style.left = "0px";
        } else if (x - alarmTooltip.offsetWidth / 2 > this.canvas.width - alarmTooltip.offsetWidth - 40) {
          alarmTooltip.style.left = this.canvas.width - alarmTooltip.offsetWidth + "px";
        } else {
          alarmTooltip.style.left = x - alarmTooltip.offsetWidth / 2 + "px";
        }
        alarmTooltip.style.top = this.canvas.height - 80 - alarmTooltip.clientHeight + "px";
        return;
      }
      if (this.setTipTimeOut) {
        clearTimeout(this.setTipTimeOut);
      }
      this.closeAlarmTip(alarmTooltip);
    }
  };
  _proto.isMouseInCircle = function isMouseInCircle(mouseX, mouseY, x, y) {
    var distance = Math.sqrt(Math.pow(mouseX - x, 2) + Math.pow(mouseY - y, 2));
    return distance <= 8;
  };
  _proto.clickEvent = function clickEvent(event) {
    this.isSelected = true;
    var mouseX = this.getMouseXRelativePos(event);
    var time = this.getMousePosTime(event);
    var seletedTime = time.getTime();
    var currentTime = (/* @__PURE__ */ new Date()).getTime();
    if (seletedTime <= currentTime) {
      this.option.onClick && this.option.onClick(time.getTime());
      this.selectedPosition.splice(0, this.selectedPosition.length);
      this.selectedPosition.push(mouseX + 20, convert.formatDate(time, "hh:mm"));
      var selectedPointer = document.getElementsByClassName("timeline_selectedPointer")[0];
      var selectedTooltip = document.getElementsByClassName("timeline_selectedTooltip")[0];
      var selectedCircle = document.getElementsByClassName("timeline_selectedCircle")[0];
      if (mouseX > 0 && mouseX < this.canvas.width - 40) {
        this.drawSelectedPointer(this.selectedPosition);
      } else {
        this.hideElement(selectedPointer);
        this.hideElement(selectedTooltip);
        this.hideElement(selectedCircle);
      }
    }
  };
  _proto.drawSelectedPointer = function drawSelectedPointer(selectedPosition) {
    var selectedPointer = document.getElementsByClassName("timeline_selectedPointer")[0];
    var selectedTooltip = document.getElementsByClassName("timeline_selectedTooltip")[0];
    var selectedCircle = document.getElementsByClassName("timeline_selectedCircle")[0];
    selectedPointer.style.left = selectedPosition[0] - 1 + "px";
    selectedPointer.style.top = this.canvas.height - 42 + "px";
    this.showElement(selectedPointer);
    selectedTooltip.innerHTML = "<p>" + defendXSS.default(selectedPosition[1]) + "</p>";
    selectedTooltip.style.left = selectedPosition[0] - 20 + "px";
    selectedTooltip.style.top = this.canvas.height - 16 + "px";
    this.showElement(selectedTooltip);
    selectedCircle.style.left = selectedPosition[0] - 4 + "px";
    selectedCircle.style.top = this.canvas.height - 50 + "px";
    this.showElement(selectedCircle);
  };
  _proto.createScaleText = function createScaleText(time) {
    if (time.getTime() === this.option.timeRange.startTime) {
      return convert.formatDate(time, "00:00");
    } else if (time.getTime() === this.option.timeRange.endTime && (this.option.timeRange.endTime - this.option.timeRange.startTime) % (24 * 60 * 60 * 1e3) === 0) {
      return convert.formatDate(time, "24:00");
    }
    return convert.formatDate(time, "hh:mm");
  };
  _proto.getMouseXRelativePos = function getMouseXRelativePos(event) {
    var scrollX = document.documentElement.scrollLeft || document.body.scrollLeft;
    var x = event.pageX || event.clientX + scrollX;
    var baseLeft = this.canvas.getBoundingClientRect().x;
    return x - baseLeft - 20;
  };
  _proto.getMouseYRelativePos = function getMouseYRelativePos(event) {
    var scrollY = document.documentElement.scrollTop || document.body.scrollTop;
    var y = event.pageY || event.clientY + scrollY;
    var baseTop = this.canvas.getBoundingClientRect().y;
    return y - baseTop;
  };
  _proto.getMousePosTime = function getMousePosTime(event) {
    var posX = this.getMouseXRelativePos(event);
    var onePxsMS = (this.canvas.width - 40) / (this.totalHours * 60 * 60 * 1e3);
    var time = new Date(this.startTimestamp + posX / onePxsMS);
    return time;
  };
  _proto.getTimePos = function getTimePos(time) {
    var onePxsMS = (this.canvas.width - 40) / (this.totalHours * 60 * 60 * 1e3);
    var posX = (time - this.startTimestamp) * onePxsMS + 20;
    return posX;
  };
  _proto.setCurrentTime = function setCurrentTime(time) {
    var newTime;
    if (typeof time === "string") {
      newTime = new Date(time).getTime();
    } else if (typeof time === "object") {
      newTime = time.getTime && time.getTime();
    } else if (typeof time === "number") {
      newTime = time;
    }
    this.currentTime = newTime;
    this.init();
  };
  return TimeLine2;
}();
exports.default = TimeLine;
