"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("../../feature/token/index.js");
const chartToken = require("./chartToken.js");
const handleOptipn = require("./handleOptipn.js");
const merge = require("../../util/merge.js");
var LAYOUT_CONFIG = {
  LineTreeChart: "orthogonal",
  RingTreeChart: "radial"
};
function getSeriesUnit(type) {
  return {
    type: "tree",
    data: void 0,
    name: void 0,
    // symbol: 'circle',
    layout: LAYOUT_CONFIG[type],
    // 文本样式
    label: {
      // 图元的距离
      distance: 10,
      fontSize: 12,
      color: chartToken.default.labelColor
    },
    // 设置连线的样式
    lineStyle: {
      width: 1,
      // 连线的曲度
      curveness: 0.5,
      color: chartToken.default.lineColor
    },
    // 图元设置
    itemStyle: {
      color: index.default.config.colorState.colorInfo
    },
    symbolSize: 10,
    initialTreeDepth: 1,
    // 初始动画的时长，
    animationDuration: 550,
    // 数据更新动画的时长。
    animationDurationUpdate: 750
  };
}
function setLineTree(seriesItem, iChartOpt) {
  var lineType = iChartOpt.lineType, direction = iChartOpt.direction;
  seriesItem.edgeShape = lineType || "curve";
  var dir = direction || "left";
  var configMap = {
    top: {
      label: {
        position: "top",
        rotate: -90,
        align: "right",
        verticalAlign: "middle"
      },
      leaves: {
        label: {
          position: "bottom",
          align: "left"
        }
      },
      orient: "TB"
    },
    right: {
      label: {
        position: "right"
      },
      leaves: {
        label: {
          position: "left"
        }
      },
      orient: "RL"
    },
    bottom: {
      label: {
        position: "bottom",
        rotate: 90,
        align: "right",
        verticalAlign: "middle"
      },
      leaves: {
        label: {
          position: "top",
          align: "left"
        }
      },
      orient: "BT"
    },
    left: {
      label: {
        position: "left"
      },
      leaves: {
        label: {
          position: "right"
        }
      },
      orient: "LR"
    }
  };
  merge.default(seriesItem, configMap[dir]);
}
function setSeries(baseOpt, type, iChartOpt) {
  var data = iChartOpt.data;
  if (!data) return;
  if (data && data.length !== 0) {
    var series = [];
    data.forEach(function(dataItem) {
      var seriesItem = getSeriesUnit(type);
      handleOptipn.setChartPadding(seriesItem, type, iChartOpt);
      seriesItem.name = dataItem.name;
      seriesItem.data = dataItem.data;
      var symbolSize = iChartOpt.symbolSize, initialTreeDepth = iChartOpt.initialTreeDepth;
      if (symbolSize) seriesItem.symbolSize = symbolSize;
      if (initialTreeDepth) seriesItem.initialTreeDepth = initialTreeDepth;
      if (type === "LineTreeChart") setLineTree(seriesItem, iChartOpt);
      series.push(seriesItem);
    });
    baseOpt.series = series;
  }
}
exports.setSeries = setSeries;
