"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const color = require("../../util/color.js");
const cloneDeep = require("../../util/cloneDeep.js");
const chartToken = require("./chartToken.js");
const random = require("../../util/random.js");
var seriesInit = {
  type: "wordCloud",
  // 网格大小，各项之间间距
  gridSize: 16,
  // 形状支持 circle 圆，cardioid  心， diamond 菱形， triangle-forward 、triangle 三角，star五角星
  shape: "circle",
  // 字体大小范围
  sizeRange: [16, 64],
  // 文字旋转角度范围
  rotationRange: [0, 0],
  // 旋转步值
  rotationStep: 0,
  // 画布位置
  top: "center",
  left: "center",
  right: null,
  bottom: null,
  // 画布宽
  width: "75%",
  // 画布高
  height: "80%",
  // 是否渲染超出画布的文字
  drawOutOfBound: false,
  // 文字显示时动画
  layoutAnimation: true,
  // 文字颜色
  textStyle: {
    color: function color2() {
      return "rgb(" + [Math.round(random.default() * 256), Math.round(random.default() * 256), Math.round(random.default() * 256)].join(",") + ")";
    }
  },
  // hover时样式
  emphasis: {
    // focus: 'self',
    textStyle: {
      textShadowBlur: 20,
      textShadowOffsetY: 2,
      textShadowOffsetX: 2,
      textShadowColor: "#191919"
    }
  },
  data: []
};
function handleSeries(params) {
  var data = params.data, width = params.width, height = params.height, gridSize = params.gridSize, sizeRange = params.sizeRange, rotationRange = params.rotationRange, rotationStep = params.rotationStep, shape = params.shape, maskImage = params.maskImage, textColor = params.textColor, colors = params.colors;
  seriesInit.emphasis.textStyle.textShadowColor = chartToken.default.emphasisTextShadowColor;
  var series = [];
  var seriesUnit = cloneDeep.default(seriesInit);
  if (maskImage) {
    seriesUnit.maskImage = maskImage;
  }
  seriesUnit.width = width || "75%";
  seriesUnit.height = height || "80%";
  seriesUnit.gridSize = gridSize || 16;
  seriesUnit.sizeRange = sizeRange || [16, 64];
  seriesUnit.rotationRange = rotationRange || [0, 0];
  seriesUnit.rotationStep = rotationStep || 0;
  seriesUnit.shape = shape || "circle";
  if (textColor) {
    seriesUnit.textStyle.color = textColor;
  } else {
    seriesUnit.textStyle.color = function(data2) {
      var index = data2.dataIndex;
      return color.getColor(colors, index);
    };
  }
  seriesUnit.data = data;
  series.push(seriesUnit);
  return series;
}
exports.default = handleSeries;
exports.seriesInit = seriesInit;
