"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const index = require("./lints/charts/index.js");
const index$1 = require("./dialog/index.js");
const index$2 = require("./lints/modules/index.js");
const type = require("../../util/type.js");
const color = require("../../util/color.js");
function chartLinter(name, eChartOption, dom, action, displayMode) {
  var standardOption = index.default(name, eChartOption);
  var differences = getObjectDifferences(standardOption, eChartOption, action === "fix");
  if (!differences.length) {
    differences = ["无不符合规范项！"];
  }
  var existingDialog = dom.querySelector(".linter-dialog");
  if (existingDialog) {
    dom.removeChild(existingDialog);
  }
  if (action === "check" && !displayMode || displayMode === "dialog") {
    new index$1.default(dom, differences);
  } else if (displayMode === "console") {
    console.warn(differences);
  }
}
function getObjectDifferences(seriesOpt, echartsOpt, isfix) {
  var differences = [];
  Object.keys(echartsOpt).sort().forEach(function(key) {
    if (key === "color") {
      var standardColor = index$2.default(echartsOpt[key], "color");
      if (!type.isArray(echartsOpt.color)) {
        differences.push("主题颜色与规范不匹配");
        if (isfix) {
          echartsOpt.color = standardColor;
        }
        return false;
      } else {
        if (JSON.stringify(standardColor) !== JSON.stringify(echartsOpt.color)) {
          differences.push("主题颜色与规范不匹配");
          if (isfix) {
            echartsOpt.color = standardColor;
          }
          return false;
        }
      }
    } else {
      if (!type.isArray(echartsOpt[key])) {
        key === "series" ? compare(seriesOpt.series, echartsOpt[key], key) : compare(index$2.default(echartsOpt[key], key), echartsOpt[key], key);
      } else {
        var isOne = echartsOpt[key].length === 1;
        echartsOpt[key].forEach(function(item, num) {
          var order = isOne ? "" : num;
          key === "series" ? compare(seriesOpt.series, item, key + order) : compare(index$2.default(item, key), item, key + order);
        });
      }
    }
  });
  function compare(obj1, obj2, path) {
    if (path === void 0) {
      path = "";
    }
    if (obj1) {
      Object.keys(obj1).forEach(function(key) {
        var p = path ? path + "." + key : key;
        if (obj1[key] === void 0 || obj1[key] === null) {
          return false;
        } else if (obj2[key] === void 0 || obj2[key] === null) {
          differences.push(p + " 在eChartOption无此属性");
          if (isfix) {
            obj2[key] = obj1[key];
          }
          return false;
        } else if (typeof obj1[key] === "object" && typeof obj2[key] === "object") {
          if (key === "padding" && type.isArray(obj1[key]) && type.isArray(obj2[key])) {
            if (!(JSON.stringify(obj1[key]) === JSON.stringify(obj2[key]))) {
              differences.push("" + p + "值跟规范不同");
              if (isfix) {
                obj2[key] = obj1[key];
              }
            }
          } else {
            compare(obj1[key], obj2[key], p);
          }
        } else if (key === "color" && typeof obj1[key] === "string" && typeof obj2[key] === "string") {
          if (color.transColor(obj1[key]).toUpperCase() !== color.transColor(obj2[key]).toUpperCase()) {
            differences.push("" + p + "值跟规范不同");
            if (isfix) {
              obj2[key] = obj1[key];
            }
          }
        } else if (obj1[key] !== obj2[key]) {
          differences.push("" + p + "值跟规范不同");
          if (isfix) {
            obj2[key] = obj1[key];
          }
        }
      });
    }
  }
  return differences;
}
exports.default = chartLinter;
