"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const type = require("../../util/type.js");
const index = require("../../feature/token/index.js");
const constants = require("../../util/constants.js");
function setDefaultTheme(iChartOption) {
  iChartOption.theme = iChartOption.theme || index.default.themeName || constants.DEFAULT_THEME_NAME;
}
function setDefaultColor(iChartOption) {
  if (!iChartOption.color) {
    iChartOption.color = index.default.config.colorGroup;
  }
}
function setXAxisKeyName(xAxisItem, defaultKey) {
  var keyName = defaultKey;
  if (type.isString(xAxisItem.data)) {
    keyName = xAxisItem.data;
    delete xAxisItem.data;
  }
  if (xAxisItem.keyName) {
    keyName = xAxisItem.keyName;
  }
  xAxisItem.keyName = keyName;
}
function setDefaultXAxis(iChartOption) {
  var data = iChartOption.data;
  var keyName;
  if (data && data.length > 0) {
    var keys = Object.keys(data[0]);
    if (keys.length > 0) {
      keyName = keys[0];
    }
  }
  if (type.isArray(iChartOption.xAxis)) {
    iChartOption.xAxis.forEach(function(xAxisItem) {
      setXAxisKeyName(xAxisItem, keyName);
    });
  } else if (typeof iChartOption.xAxis == "object") {
    setXAxisKeyName(iChartOption.xAxis, keyName);
    iChartOption.xAxis = [iChartOption.xAxis];
  } else {
    iChartOption.xAxis = [{
      keyName
    }];
  }
}
function setChartPadding(iChartOption) {
  var defaultPadding = [50, 20, 50, 20];
  var padding = iChartOption.padding || iChartOption.chartPadding;
  if (!padding) {
    iChartOption.padding = defaultPadding;
  } else if (padding.length === 1) {
    iChartOption.padding = [padding[0], 20, padding[0], 20];
  } else if (padding.length === 2) {
    iChartOption.padding = [padding[0], padding[1], padding[0], padding[1]];
  } else if (padding.length === 3) {
    iChartOption.padding = [padding[0], padding[1], padding[2], padding[1]];
  } else {
    iChartOption.padding = padding;
  }
  delete iChartOption.chartPadding;
}
function setDefaultLegend(iChartOption) {
  if (!iChartOption.legend) {
    iChartOption.legend = {
      show: true,
      position: {
        left: "center",
        bottom: 14
      },
      orient: "horizontal"
    };
  }
  if (iChartOption.legend.show === void 0) {
    iChartOption.legend.show = false;
  }
  if (!iChartOption.legend.orient) {
    iChartOption.legend.orient = "horizontal";
  }
  if (!iChartOption.legend.position) {
    iChartOption.legend.position = {
      left: "center",
      bottom: 15
    };
  }
}
function setDefaultDataZoom(iChartOption) {
  if (!iChartOption.dataZoom) {
    iChartOption.dataZoom = {
      show: false,
      position: {
        left: 40,
        bottom: 18
      }
    };
  }
  if (iChartOption.dataZoom.show === void 0) {
    iChartOption.dataZoom.show = false;
  }
  if (!iChartOption.dataZoom.position) {
    iChartOption.dataZoom.position = {
      left: "center",
      bottom: 20
    };
  }
}
function filterDisplayData(iChartOption) {
  if (!iChartOption.data || !iChartOption.dataRules || !iChartOption.dataRules.display) return;
  var displayData = iChartOption.dataRules.display;
  var data = iChartOption.data;
  var newData = [];
  if (displayData.length > 0) {
    data.forEach(function(item) {
      var obj = {};
      for (var key in item) {
        if (Object.hasOwnProperty.call(item, key)) {
          var element = item[key];
          if (iChartOption.xAxis && (key === iChartOption.xAxis[0].keyName || key === iChartOption.xAxis.name)) {
            obj[key] = element;
          }
          if (displayData.includes(key)) {
            obj[key] = element;
          }
        }
      }
      newData.push(obj);
    });
    iChartOption.data = newData;
  }
}
function init(iChartOption) {
  setDefaultTheme(iChartOption);
  setDefaultColor(iChartOption);
  setDefaultXAxis(iChartOption);
  setChartPadding(iChartOption);
  setDefaultLegend(iChartOption);
  setDefaultDataZoom(iChartOption);
  filterDisplayData(iChartOption);
  return iChartOption;
}
exports.default = init;
