"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
function getColor(colors, index) {
  return colors[index % colors.length];
}
function codeToRGB(code, opacity) {
  if (code === void 0) {
    return void 0;
  }
  var result = [];
  result.push(parseInt(code.substring(1, 3), 16));
  result.push(parseInt(code.substring(3, 5), 16));
  result.push(parseInt(code.substring(5), 16));
  return "rgba(" + result.join(",") + "," + opacity + ")";
}
function codeToHex(color) {
  switch (color) {
    case "red":
      return "#ff0000";
    case "blue":
      return "#0000ff";
    case "green":
      return "#00ff00";
    case "pink":
      return "#FFC0CB";
    case "yellow":
      return "#FFFF00";
    case "orange":
      return "#FFA500";
    case "black":
      return "#000000";
    case "white":
      return "#ffffff";
    case "gray":
      return "#808080";
    case "purple":
      return "#800080";
  }
  if (color.includes("#")) {
    if (color.length === 7) {
      return color;
    } else if (color.length === 4) {
      return color[0] + color[1] + color[1] + color[2] + color[2] + color[3] + color[3];
    }
  }
  var values = color.replace(/rgba?\(/, "").replace(/\)/, "").replace(/[\s+]/g, "").split(",");
  var a = parseFloat(values[3] || 1);
  var r = Math.floor(a * parseInt(values[0]) + (1 - a) * 255);
  var g = Math.floor(a * parseInt(values[1]) + (1 - a) * 255);
  var b = Math.floor(a * parseInt(values[2]) + (1 - a) * 255);
  return "#" + ("0" + r.toString(16)).slice(-2) + ("0" + g.toString(16)).slice(-2) + ("0" + b.toString(16)).slice(-2);
}
function changeRgbaOpacity(rgba, opacity) {
  var _rgba$match = rgba.match(/\d+(\.\d+)?/g), r = _rgba$match[0], g = _rgba$match[1], b = _rgba$match[2];
  return "rgba(" + r + "," + g + "," + b + "," + opacity + ")";
}
function transColor(colorStr) {
  var regex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
  var matches = colorStr.match(regex);
  if (matches) {
    return "#" + matches[1] + matches[1] + matches[2] + matches[2] + matches[3] + matches[3];
  }
  return colorStr;
}
exports.changeRgbaOpacity = changeRgbaOpacity;
exports.codeToHex = codeToHex;
exports.codeToRGB = codeToRGB;
exports.getColor = getColor;
exports.transColor = transColor;
