"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const chunkG2ADBYYC = require("../chunk-G2ADBYYC.js");
const index = require("./index.js");
const api = [
  "t",
  "popInput",
  "state",
  "editorState",
  "save",
  "reset",
  "inputBlur",
  "init",
  "toggleVisible",
  "inputFocus",
  "closePopper",
  "onInput",
  "getAmountText",
  "handleClearClick",
  "handleChange",
  "openDetailEditor",
  "closeDetailEidtor",
  "uiMode",
  "getAmountTextWithoutCurrncy",
  "watchUiMode"
];
const initState = ({
  reactive,
  computed,
  props,
  $service,
  editorState,
  vm,
  api: api2,
  constants,
  parent
}) => {
  const state = reactive({
    visible: false,
    amount: props.modelValue || "",
    currency: props.currency,
    date: props.date,
    overMaxLen: false,
    isFocus: false,
    lock: false,
    amountText: "",
    lastInput: props.modelValue,
    lastCurrency: props.currency,
    lastDate: props.date,
    format: computed(() => {
      return chunkG2ADBYYC.__spreadProps(chunkG2ADBYYC.__spreadValues({}, index.getPrecision({ service: $service, props, currency: editorState.currency })), {
        prefix: state.currency
      });
    }),
    disabled: computed(() => props.disabled || (parent.tinyForm || {}).disabled),
    displayOnly: computed(() => props.displayOnly || (parent.tinyForm || {}).displayOnly),
    theme: vm.theme,
    radioVal: "",
    clearValues: false,
    amountValue: computed(() => api2.getAmountText()),
    amountNumberValue: computed(() => api2.getAmountTextWithoutCurrncy()),
    filterMenu: constants.FILTER_OPTION,
    editorPhase: "close"
    // 'close' | 'selection' | 'currency' | 'date',
  });
  return state;
};
const initEditorState = ({
  reactive,
  props
}) => {
  return reactive({
    amount: "",
    date: "",
    currency: props.currency,
    lastInput: props.modelValue
  });
};
const initUiMode = ({ useBreakpoint }) => {
  return useBreakpoint().current;
};
const initApi = ({
  api: api2,
  t,
  editorState,
  props,
  state,
  emit,
  vm,
  uiMode,
  isMobileFirstMode
}) => {
  Object.assign(api2, {
    state,
    t,
    editorState,
    uiMode,
    getDecimal: index.getDecimal(props),
    innerFormat: index.innerFormat({ state, props }),
    getAmountText: index.getAmountText({ state, props }),
    getAmountTextWithoutCurrncy: index.getAmountTextWithoutCurrncy({ state }),
    initAmount: index.initAmount({ props, api: api2 }),
    onInputPreprocess: index.onInputPreprocess(props),
    onInput: index.onInput({ state, props, api: api2 }),
    initText: index.initText({ state }),
    inputFocus: index.inputFocus({ state, props }),
    inputBlur: index.inputBlur({ api: api2, props, state }),
    closePopper: index.closePopper(state),
    emitChange: index.emitChange({ emit, state, props, api: api2 }),
    popInput: index.popInput({ editorState, api: api2, state, props }),
    save: index.save({ api: api2, state, editorState }),
    reset: index.reset({ state, editorState }),
    handelClick: index.handelClick({ api: api2, vm }),
    addOutSideEvent: index.addOutSideEvent(api2),
    watchModelValue: index.watchModelValue({ api: api2, state }),
    watchCurrency: index.watchCurrency({ api: api2, state, editorState }),
    watchUiMode: index.watchUiMode({ api: api2, isMobileFirstMode }),
    toggleVisible: index.toggleVisible({ api: api2, props, state, editorState, uiMode, isMobileFirstMode }),
    handleClearClick: index.handleClearClick({ state, emit, editorState }),
    handleChange: index.handleChange({ state, emit }),
    openDetailEditor: index.openDetailEditor({ state }),
    closeDetailEidtor: index.closeDetailEidtor({ state, props, api: api2 })
  });
};
const initWatch = ({
  watch,
  props,
  state,
  api: api2,
  uiMode
}) => {
  watch(() => props.modelValue, api2.watchModelValue, { immediate: true });
  watch(() => props.currency, api2.watchCurrency, { immediate: true });
  watch(
    () => props.date,
    (value) => {
      state.date = value;
      state.lastDate = value;
    },
    { immediate: true }
  );
  watch(
    () => props.rounding,
    (value) => {
      state.format.rounding = value;
    }
  );
  watch(
    () => props.digits,
    (value) => {
      state.format.fraction = value;
    }
  );
  watch(uiMode, api2.watchUiMode);
};
const renderless = (props, { onUnmounted, computed, reactive, watch, inject }, { t, emit, vm, service, parent, constants, isMobileFirstMode, useBreakpoint }) => {
  var _a;
  const api2 = {};
  const $service = index.initService(service);
  const editorState = initEditorState({ reactive, props });
  const state = initState({ reactive, computed, props, $service, editorState, vm, api: api2, constants, parent });
  const uiMode = initUiMode({ useBreakpoint });
  initApi({ api: api2, t, editorState, props, state, emit, vm, uiMode, isMobileFirstMode });
  parent.tinyForm = parent.tinyForm || inject("form", null);
  (_a = api2 == null ? void 0 : api2.getDecimal) == null ? void 0 : _a.call(api2, 0);
  initWatch({ watch, props, state, api: api2, uiMode });
  onUnmounted(() => {
    var _a2;
    return (_a2 = api2 == null ? void 0 : api2.addOutSideEvent) == null ? void 0 : _a2.call(api2, false);
  });
  parent.$on("handle-clear", (event) => {
    api2.handleClearClick(event);
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
