"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const debounce = require("../common/deps/debounce.js");
const vuePopper = require("../common/deps/vue-popper.js");
const string = require("../common/string.js");
const index = require("./index.js");
const api = [
  "state",
  "suggestionState",
  "getInput",
  "handleChange",
  "close",
  "open",
  "handleFocus",
  "handleBlur",
  "handleClear",
  "handleKeyEnter",
  "highlight",
  "select",
  "doDestroy"
];
const initState = ({
  reactive,
  $prefix,
  computed,
  props,
  inject
}) => {
  var _a;
  const state = reactive({
    showAutoWidth: inject("showAutoWidth", null),
    popperElm: null,
    activated: false,
    suggestions: [],
    loading: false,
    highlightedIndex: -1,
    suggestionDisabled: false,
    id: $prefix + "-" + string.guid(),
    suggestionVisible: computed(() => index.computedVisible(state)),
    // props.validateEvent优先级大于inject，都没有配置默认为true
    validateEvent: (_a = props.validateEvent) != null ? _a : inject("validateEvent", true)
  });
  return state;
};
const initSuggestionState = ({
  reactive,
  parent,
  showPopper,
  popperElm,
  referenceElm
}) => {
  return reactive({
    parent,
    dropdownWidth: "",
    showPopper,
    popperElm,
    referenceElm
  });
};
const initApi = ({
  api: api2,
  state,
  doDestroy,
  suggestionState,
  emit,
  vm,
  props,
  updatePopper,
  nextTick,
  constants,
  dispatch
}) => {
  api2.getData = index.getData({ props, state, updatePopper, nextTick });
  Object.assign(api2, {
    state,
    doDestroy,
    suggestionState,
    close: index.close(state),
    open: index.open(api2),
    handleBlur: index.handleBlur({ emit, state, dispatch, props }),
    mounted: index.mounted({ vm, state, suggestionState }),
    highlight: index.highlight({ constants, vm, state }),
    handleClear: index.handleClear({ emit, state }),
    select: index.select({ emit, nextTick, props, state, dispatch }),
    watchVisible: index.watchVisible({ suggestionState, vm }),
    handleChange: index.handleChange({ api: api2, emit, state, props, dispatch }),
    handleFocus: index.handleFocus({ api: api2, emit, props, state }),
    handleKeyEnter: index.handleKeyEnter({ api: api2, emit, nextTick, props, state }),
    debouncedGetData: debounce.default(props.debounce, api2.getData)
  });
};
const renderless = (props, {
  computed,
  onBeforeUnmount,
  onMounted,
  reactive,
  watch,
  toRefs,
  onDeactivated,
  inject
}, { $prefix, vm, parent, emit, constants, nextTick, slots, dispatch }) => {
  const api2 = {};
  const state = initState({ reactive, $prefix, computed, props, inject });
  const { showPopper, popperElm, referenceElm, doDestroy, updatePopper } = vuePopper.default({
    reactive,
    watch,
    vm,
    emit,
    slots,
    nextTick,
    props,
    onBeforeUnmount,
    toRefs,
    onDeactivated
  });
  const suggestionState = initSuggestionState({ reactive, parent, showPopper, popperElm, referenceElm });
  initApi({
    api: api2,
    state,
    doDestroy,
    suggestionState,
    emit,
    vm,
    props,
    updatePopper,
    nextTick,
    constants,
    dispatch
  });
  watch(() => state.suggestionVisible, api2.watchVisible);
  onMounted(api2.mounted);
  return api2;
};
exports.api = api;
exports.initSuggestionState = initSuggestionState;
exports.renderless = renderless;
