"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const throttle = require("../common/deps/throttle.js");
const api = [
  "state",
  "handleDraggerClick",
  "handleClickDay",
  "handleDraggerMousedown",
  "handleDraggerTouchstart",
  "handleDraggerTouchend",
  "throttledHandleDraggerTouchmove",
  "setCascaderVisible",
  "handleCascaderChange",
  "i18nYearMonth",
  "touchstart",
  "touchmove",
  "touchend"
];
const initWatch = ({ watch, api: api2, state, emit }) => {
  watch(
    () => state.weekDates,
    (value, oldValue) => {
      if (state.dragging || state.touching)
        return;
      state.prevWeekDates = api2.getWeekOfDate("prev", state.weekDates[3].dateStr);
      state.nextWeekDates = api2.getWeekOfDate("next", state.weekDates[3].dateStr);
      state.copyWeekDates = state.weekDates.slice(0);
      emit("week-change", value, oldValue);
    },
    { deep: true }
  );
  watch(
    () => state.showWeek,
    (value) => {
      api2.showWeekChange(value);
    }
  );
};
const renderless = (props, { reactive, computed, onBeforeMount, onMounted, watch }, { emit, vm, t }) => {
  const state = reactive({
    minSize2: "min-width: 3rem; min-height: 2rem",
    minSize3: "min-width: 3rem; min-height: 3rem",
    dayOfWeek: 7,
    showRows: 0,
    weekDays: [],
    prevWeekDates: [],
    weekDates: [],
    nextWeekDates: [],
    lastCalendarDays: [],
    calendarDays: [],
    nextCalendarDays: [],
    threeMonthDays: [],
    copyWeekDates: [],
    calendarDaysCopy: [],
    currentYear: "",
    currentMonth: "",
    data: computed(() => api2.computedData()),
    currentRow: computed(() => api2.computedCurrentRow()),
    filteredCalendarDays: computed(() => api2.computedFilteredCalendarDays()),
    visibleRows: [],
    itemHeight: 0,
    totalRows: computed(() => api2.computedTotalRows()),
    showWeek: true,
    dragging: false,
    touching: false,
    delta: 0,
    duration: 300,
    offsetPos: {
      X: 0,
      Y: 0
    },
    deltaPos: {
      X: 0,
      Y: 0
    },
    startPos: {
      X: 0,
      Y: 0
    },
    activeRow: 0,
    activeDate: "",
    cascaderCurrent: [],
    cascaderOptions: computed(() => api2.computeCascaderOptions()),
    cascaderVisible: false
  });
  const api2 = {
    state,
    touchstart: index.touchstart({ state }),
    touchmove: index.touchmove({ state }),
    getCalendarDays: index.getCalendarDays(state),
    computedData: index.computedData(state),
    computedCurrentRow: index.computedCurrentRow(state),
    computedFilteredCalendarDays: index.computedFilteredCalendarDays(state),
    handleDraggerClick: index.handleDraggerClick(state, emit),
    calcCalendarItemHeight: index.calcCalendarItemHeight({ state, vm }),
    computedTotalRows: index.computedTotalRows(state),
    throttledHandleMousemove: throttle.default(20, index.handleMousemove({ state, vm })),
    handleDraggerTouchstart: index.handleDraggerTouchstart({ state, vm }),
    handleDraggerTouchend: index.handleDraggerTouchend({ state, vm, emit }),
    throttledHandleDraggerTouchmove: throttle.default(20, index.handleDraggerTouchmove({ state, vm })),
    setCascaderVisible: index.setCascaderVisible(state),
    computeCascaderOptions: index.computeCascaderOptions(t),
    i18nYearMonth: index.i18nYearMonth({ state, t })
  };
  Object.assign(api2, {
    getPrevWeek: index.getPrevWeek({ props, emit, api: api2, state }),
    getNextWeek: index.getNextWeek({ props, emit, api: api2, state }),
    showWeekChange: index.showWeekChange({ state }),
    updateCalendarDays: index.updateCalendarDays({ props, state, api: api2, emit }),
    getAllDatesOfCurrWeek: index.getAllDatesOfCurrWeek({ state, props }),
    getWeekOfDate: index.getWeekOfDate({ api: api2 }),
    touchend: index.touchend({ state, api: api2, vm }),
    handleClickDay: index.handleClickDay({ api: api2, emit, props, state }),
    handleDraggerMousedown: index.handleDraggerMousedown({ api: api2, state, vm }),
    handleMouseup: index.handleMouseup({ api: api2, state, vm, emit }),
    handleCascaderChange: index.handleCascaderChange({ api: api2, emit, props, state })
  });
  initWatch({ watch, api: api2, state, emit });
  onBeforeMount(() => {
    api2.getCalendarDays(props.modelValue, props.config, "cur");
    api2.getCalendarDays(props.modelValue, props.config, "last");
    api2.getCalendarDays(props.modelValue, props.config, "next");
    state.weekDates = api2.getAllDatesOfCurrWeek(props.modelValue || /* @__PURE__ */ new Date());
  });
  onMounted(() => {
    api2.calcCalendarItemHeight();
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
