"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const throttle = require("../common/deps/throttle.js");
const api = [
  "state",
  "renderProxy",
  "isToday",
  "dateIsToday",
  "getEventByTime",
  "selectDay",
  "toToday",
  "getPrevWeek",
  "getNextWeek",
  "goPrevMonth",
  "goNextMonth",
  "currentDateChange",
  "newSchedule",
  "handleMouseenter",
  "handleMouseleave",
  "isSelectedDate",
  "isStartOrEndDay",
  "getDayBgColor",
  "isShowNewSchedule",
  "isShowMark",
  "handleCascaderChange",
  "setCascaderVisible",
  "handleDropBtnClick",
  "getEventShowTime",
  "touchstart",
  "touchmove",
  "touchend"
];
const initState = ({ reactive, props, computed, api: api2, images, modesIcon }) => {
  const state = reactive({
    dayStartTime: props._constants.DAY_START_TIME,
    dayEndTime: props._constants.DAY_END_TIME,
    wednesday: props._constants.WEDNESDAY,
    modesIcon,
    images,
    weekDays: [0, 1, 2, 3, 4, 5, 6],
    dayTimes: computed(() => api2.genDayTimes()),
    mode: props.mode,
    events: [],
    eventsChangeHash: 0,
    weekDates: [],
    prevWeekDates: [],
    nextWeekDates: [],
    monthEvents: [],
    nextMonthEvents: [],
    prevMonthEvents: [],
    selectedDateEvents: [],
    monthWednesdays: [],
    wholeYearWednesdays: [],
    index: 0,
    delta: 0,
    startPos: {
      X: 0,
      Y: 0
    },
    deltaPos: {
      X: 0,
      Y: 0
    },
    offsetPos: {
      X: 0,
      Y: 0
    },
    curWeekEvents: {},
    scrollStartTime: 0,
    debounceTime: 100,
    touchTime: 0,
    direction: "",
    monthEventsLength: 0,
    activedWeekIndex: -1,
    selectedTip: "",
    selectedDate: "",
    selectedDates: [],
    showYear: false,
    showMonth: false,
    showSelectedDateEvents: false,
    multiSelect: computed(() => props.multiSelect),
    eventTipVisible: false,
    cascaderVisible: false,
    eventTipContent: {},
    activeYear: props.year,
    displayMode: props.mode,
    activeMonth: props.month,
    currentDate: props.year + "-" + props.month,
    cascaderCurrent: [props.year || (/* @__PURE__ */ new Date()).getFullYear(), props.month || (/* @__PURE__ */ new Date()).getMonth + 1],
    cascaderOptions: computed(() => api2.computeCascaderOptions()),
    calendar: computed(() => api2.computedCalendar("cur")),
    prevCalendar: computed(() => api2.computedCalendar("prev")),
    nextCalendar: computed(() => api2.computedCalendar("next"))
  });
  return state;
};
const initWatch = ({ watch, props, state, emit, api: api2, nextTick }) => {
  watch(
    () => props.modelValue,
    (value) => {
      if (value) {
        if (props.multiSelect && Array.isArray(value)) {
          state.selectedDates = value;
        } else {
          state.selectedDate = value;
        }
      }
    },
    { immediate: true, deep: true }
  );
  watch(
    () => state.mode,
    (val) => {
      emit("mode-change", val);
      if (val === "schedule") {
        api2.getCurWeekEvent();
      }
    }
  );
  watch(
    () => props.events,
    () => {
      api2.handleEvents();
    },
    { deep: true }
  );
  watch(
    () => state.selectedDate,
    () => {
      emit("selected-date-change", state.selectedDate);
    },
    { deep: true }
  );
  watch(
    () => state.weekDates,
    (value, oldValue) => {
      state.prevWeekDates = api2.getWeekOfDate("prev", state.weekDates[3].value);
      state.nextWeekDates = api2.getWeekOfDate("next", state.weekDates[3].value);
      api2.getCurWeekEvent();
      emit("week-change", value, oldValue);
    },
    { deep: true }
  );
  watch(
    () => state.eventsChangeHash,
    () => {
      if (state.mode === "schedule" || state.mode === "timeline") {
        api2.getCurWeekEvent();
      }
    }
  );
  watch(
    () => state.activedWeekIndex,
    () => {
      nextTick(() => {
        state.showSelectedDateEvents = true;
      });
    }
  );
  addWatch({ watch, props, state, emit, api: api2 });
};
const addWatch = ({ watch, props, state, emit, api: api2 }) => {
  watch(
    () => props.month,
    (val, oldVal) => {
      if (val !== oldVal) {
        state.activeMonth = val;
      }
    },
    { immediate: true }
  );
  watch(
    () => props.year,
    (val, oldVal) => {
      if (val !== oldVal) {
        state.activeYear = val;
      }
    },
    { immediate: true }
  );
  watch(
    () => state.activeYear,
    (value, oldValue) => {
      if (value !== oldValue) {
        state.currentDate = state.activeYear + "-" + state.activeMonth;
        state.cascaderCurrent = [Number(state.activeYear), Number(state.activeMonth)];
        api2.getAllDatesOfCurrWeek(state.currentDate + "-4");
        emit("year-change", value, oldValue);
        emit("update:modelValue", value, oldValue);
        api2.handleEvents();
      }
    }
  );
  watch(
    () => state.activeMonth,
    (value, oldValue) => {
      if (value !== oldValue) {
        state.currentDate = state.activeYear + "-" + state.activeMonth;
        state.cascaderCurrent = [Number(state.activeYear), Number(state.activeMonth)];
        api2.getAllDatesOfCurrWeek(state.currentDate + "-4");
        emit("month-change", value, oldValue);
        api2.handleEvents();
      }
    }
  );
};
const initApi = ({ vm, api: api2, state, t, props, emit, nextTick }) => {
  Object.assign(api2, {
    state,
    touchstart: index.touchstart({ state }),
    touchmove: index.touchmove({ state }),
    touchend: index.touchend({ state, api: api2, vm }),
    handleEvents: index.handleEvents({ props, state }),
    computedCalendar: index.computedCalendar({ props, state }),
    computeCascaderOptions: index.computeCascaderOptions(t),
    isToday: index.isToday(state),
    dateIsToday: index.dateIsToday(),
    selectDay: index.selectDay({ state, emit }),
    getEventByTime: index.getEventByTime({ props, state }),
    toToday: index.toToday({ state, api: api2, nextTick }),
    getAllWednesdaysInYear: index.getAllWednesdaysInYear({ state }),
    getAllWednesdaysInMonth: index.getAllWednesdaysInMonth({ state }),
    getAllDatesOfCurrWeek: throttle.default(50, true, index.getAllDatesOfCurrWeek({ props, state })),
    initWeeklyCalendar: index.initWeeklyCalendar({ api: api2, state }),
    getDatesOfPreviousWeek: index.getDatesOfPreviousWeek({ api: api2, state }),
    getDatesOfNextWeek: index.getDatesOfNextWeek({ api: api2, state }),
    currentDateChange: index.currentDateChange({ api: api2, state }),
    newSchedule: index.newSchedule({ emit }),
    getPrevWeek: throttle.default(50, true, index.getPrevWeek({ api: api2, state, emit })),
    getNextWeek: throttle.default(50, true, index.getNextWeek({ api: api2, state, emit })),
    goPrevMonth: throttle.default(50, true, index.goPrevMonth({ state })),
    goNextMonth: throttle.default(50, true, index.goNextMonth({ state })),
    handleMouseenter: index.handleMouseenter({ vm, state }),
    handleMouseleave: index.handleMouseleave({ state }),
    isSelectedDate: index.isSelectedDate({ state }),
    isStartOrEndDay: index.isStartOrEndDay({ state }),
    getDayBgColor: index.getDayBgColor({ props }),
    isShowNewSchedule: index.isShowNewSchedule({ props }),
    genDayTimes: index.genDayTimes({ props }),
    isShowMark: index.isShowMark({ props }),
    handleCascaderChange: index.handleCascaderChange({ state }),
    setCascaderVisible: index.setCascaderVisible(state),
    getEventByMonth: index.getEventByMonth({ state }),
    getEventByDate: index.getEventByDate({ state }),
    getWeekOfDate: index.getWeekOfDate(),
    getEventShowTime: index.getEventShowTime({ state }),
    getCurWeekEvent: index.getCurWeekEvent({ state, api: api2 }),
    handleDropBtnClick: index.handleDropBtnClick({ api: api2, state })
  });
};
const renderless = (props, { computed, reactive, watch, onMounted }, { vm, t, emit, nextTick }, { images, modesIcon }) => {
  const api2 = {};
  const state = initState({ reactive, props, computed, api: api2, images, modesIcon });
  initWatch({ api: api2, watch, props, state, emit, nextTick });
  initApi({ vm, api: api2, state, t, props, emit, nextTick });
  onMounted(() => {
    api2.initWeeklyCalendar(`${state.activeYear}-${state.activeMonth}-4`);
    api2.handleEvents();
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
