"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const object = require("../common/object.js");
const calendar = require("../common/calendar/calendar.js");
const computedCalendar = ({ state }) => () => {
  const calendar$1 = calendar.getCalendar(state.activeYear, state.activeMonth);
  const result = [];
  if (calendar$1) {
    const { last, current, next } = calendar$1;
    if (last && calendar$1.last.end >= calendar$1.last.start) {
      for (let i = last.start; i <= last.end; i++) {
        result.push({ value: i, isLast: true });
      }
    }
    if (current) {
      for (let i = current.start; i <= current.end; i++) {
        result.push({ value: i });
      }
    }
    if (next.end >= next.start) {
      for (let i = next.start; i <= next.end; i++) {
        result.push({ value: i, isNext: true });
      }
    }
  }
  return calendar.transformArray(result);
};
const computedEventList = ({ props, state }) => () => {
  let result = [];
  if (props.events && props.events.length) {
    result = props.events.filter((item) => {
      const { year, month } = calendar.parseDate(item.time);
      return Number(state.activeYear) === year && Number(state.activeMonth) === month;
    });
  }
  return result;
};
const toggeModel = (state) => (mode) => {
  const isYearOrMonth = /^(year|month)$/.test(mode);
  state.displayMode = isYearOrMonth ? mode : state.displayMode === "year" ? "month" : "year";
};
const selectMonth = (state) => (month) => {
  if (month && typeof month === "number" && month <= 12) {
    state.activeMonth = month;
  }
};
const selectDay = (state) => (day) => {
  if (day && day.value) {
    if (day.isLast) {
      const { year, month } = calendar.lastMonth(state.activeYear, state.activeMonth);
      state.activeMonth = month;
      state.activeYear = year;
    }
    if (day.isNext) {
      const { year, month } = calendar.nextMonth(state.activeYear, state.activeMonth);
      state.activeMonth = month;
      state.activeYear = year;
    }
    state.selectedTip = `You selected date: ${state.activeYear}-${state.activeMonth}-${day.value}`;
    state.selectedDate = Number(/* @__PURE__ */ new Date(`${state.activeYear}/${state.activeMonth}/${day.value}`));
  }
};
const getEventByDay = (state) => (day) => {
  let events = [];
  if (state.eventList && state.eventList.length) {
    events = object.copyArray(state.eventList).filter(({ time }) => {
      const date = new Date(time);
      return date.getDate() === day && date.getFullYear() === Number(state.activeYear) && date.getMonth() + 1 === Number(state.activeMonth);
    }).map((event) => {
      event.parseTime = calendar.parseDate(event.time);
      return event;
    });
  }
  return events;
};
const getEventByMonth = ({ props, state }) => (month) => {
  let events = [];
  if (props.events) {
    events = object.copyArray(props.events).filter(({ time }) => {
      const date = new Date(time);
      return date.getFullYear() === Number(state.activeYear) && date.getMonth() + 1 === month;
    }).map((event) => {
      event.parseTime = calendar.parseDate(event.time);
      return event;
    });
  }
  return events;
};
const getTime = (state) => (day) => Number(/* @__PURE__ */ new Date(`${state.activeYear}/${state.activeMonth}/${day}`));
const getYearList = () => () => {
  const date = /* @__PURE__ */ new Date();
  const year = date.getFullYear();
  const yesrs = [];
  for (let i = year - 10; i < year + 10; i++) {
    yesrs.push(i);
  }
  return yesrs;
};
const isToday = (state) => (day) => {
  const date = /* @__PURE__ */ new Date();
  let year = Number(state.activeYear);
  let month = Number(state.activeMonth);
  if (day.isLast) {
    const lastDate = calendar.lastMonth(state.activeYear, state.activeMonth);
    year = lastDate.year;
    month = lastDate.month;
  }
  if (day.isNext) {
    const nextDate = calendar.nextMonth(state.activeYear, state.activeMonth);
    year = nextDate.year;
    month = nextDate.month;
  }
  return date.getDate() === day.value && date.getFullYear() === year && date.getMonth() + 1 === month;
};
const isThisMonth = (state) => (month) => {
  const date = /* @__PURE__ */ new Date();
  const year = Number(state.activeYear);
  return date.getMonth() + 1 === month && date.getFullYear() === year;
};
const genMonths = () => {
  const result = [];
  let index = 1;
  for (let i = 0; i < 3; i++) {
    result[i] = [];
    for (let j = 0; j < 4; j++) {
      result[i][j] = index++;
    }
  }
  return result;
};
const toToday = (state) => () => {
  const year = (/* @__PURE__ */ new Date()).getFullYear();
  const month = (/* @__PURE__ */ new Date()).getMonth() + 1;
  const day = (/* @__PURE__ */ new Date()).getDate();
  state.activeMonth = month;
  state.activeYear = year;
  state.selectedTip = `You selected date: ${year}-${month}-${day}`;
  state.selectedDate = Number(/* @__PURE__ */ new Date(`${year}/${month}/${day}`));
};
const getCurrentDate = () => {
  const day = (/* @__PURE__ */ new Date()).getDate();
  const month = (/* @__PURE__ */ new Date()).getMonth() + 1;
  const year = (/* @__PURE__ */ new Date()).getFullYear();
  return Number(/* @__PURE__ */ new Date(`${year}/${month}/${day}`));
};
exports.computedCalendar = computedCalendar;
exports.computedEventList = computedEventList;
exports.genMonths = genMonths;
exports.getCurrentDate = getCurrentDate;
exports.getEventByDay = getEventByDay;
exports.getEventByMonth = getEventByMonth;
exports.getTime = getTime;
exports.getYearList = getYearList;
exports.isThisMonth = isThisMonth;
exports.isToday = isToday;
exports.selectDay = selectDay;
exports.selectMonth = selectMonth;
exports.toToday = toToday;
exports.toggeModel = toggeModel;
