"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = [
  "state",
  "renderProxy",
  "isToday",
  "isThisMonth",
  "getEventByMonth",
  "getEventByDay",
  "selectDay",
  "selectMonth",
  "toggeModel",
  "getTime",
  "genMonths",
  "toToday"
];
const initState = ({ reactive, props, computed, api: api2 }) => {
  const state = reactive({
    index: 0,
    selectedTip: "",
    selectedDate: index.getCurrentDate(),
    showYear: false,
    showMonth: false,
    activeYear: props.year,
    displayMode: props.mode,
    activeMonth: props.month,
    dropdownYear: computed(() => api2.getYearList()),
    calendar: computed(() => api2.computedCalendar()),
    eventList: computed(() => api2.computedEventList())
  });
  return state;
};
const initWatch = ({ watch, props, state, emit }) => {
  watch(
    () => props.year,
    (value, oldValue) => {
      if (value !== oldValue) {
        state.activeYear = value;
      }
    },
    { immediate: true }
  );
  watch(
    () => props.month,
    (value, oldValue) => {
      if (value !== oldValue) {
        state.activeMonth = value;
      }
    },
    { immediate: true }
  );
  watch(
    () => state.activeYear,
    (value, oldValue) => {
      if (value !== oldValue) {
        emit("year-change", value, oldValue);
        emit("update:modelValue", value, oldValue);
      }
    }
  );
  watch(
    () => state.activeMonth,
    (value, oldValue) => {
      if (value !== oldValue) {
        emit("month-change", value, oldValue);
      }
    }
  );
};
const initApi = ({ api: api2, state, t, props }) => {
  Object.assign(api2, {
    state,
    genMonths: index.genMonths,
    computedCalendar: index.computedCalendar({ state }),
    computedEventList: index.computedEventList({ props, state }),
    getTime: index.getTime(state),
    isToday: index.isToday(state),
    selectDay: index.selectDay(state),
    getYearList: index.getYearList(),
    toggeModel: index.toggeModel(state),
    selectMonth: index.selectMonth(state),
    isThisMonth: index.isThisMonth(state),
    getEventByDay: index.getEventByDay(state),
    getEventByMonth: index.getEventByMonth({ props, state }),
    toToday: index.toToday(state)
  });
};
const renderless = (props, { computed, reactive, watch }, { t, emit }) => {
  const api2 = {};
  const state = initState({ reactive, props, computed, api: api2 });
  initWatch({ watch, props, state, emit });
  initApi({ api: api2, state, t, props });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
