"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const resizeEvent = require("../common/deps/resize-event.js");
const api = [
  "state",
  "items",
  "activeIndex",
  "containerWidth",
  "hover",
  "hasLabel",
  "onComplete",
  "handleMouseEnter",
  "handleMouseLeave",
  "itemInStage",
  "handleButtonEnter",
  "handleButtonLeave",
  "updateItems",
  "resetItemPosition",
  "playSlides",
  "pauseTimer",
  "startTimer",
  "setActiveItem",
  "prev",
  "next",
  "handleIndicatorClick",
  "handleIndicatorHover",
  "throttledArrowClick",
  "throttledIndicatorHover",
  "touchstart",
  "touchmove",
  "touchend"
];
const initState = ({ reactive, computed, api: api2 }) => {
  const state = reactive({
    itemsTranslate: [],
    items: [],
    timer: null,
    hover: false,
    activeIndex: -1,
    completed: false,
    delta: 0,
    containerWidth: 0,
    deltaPos: {
      X: 0,
      Y: 0
    },
    startPos: {
      X: 0,
      Y: 0
    },
    offsetPos: {
      X: 0,
      Y: 0
    },
    touchTime: 0,
    size: 0,
    direction: "",
    moving: false,
    isCorrectDirection: false,
    moveDisable: false,
    noTouchNode: ["svg", "BUTTON", "path", "g"],
    style: computed(() => api2.computedStyle()),
    hasLabel: computed(() => api2.computedHasLabel(state.items)),
    hasButtons: computed(() => api2.computedHasButtons()),
    hasIndicators: computed(() => api2.computedHasIndicators())
  });
  return state;
};
const initApi = ({ vm, api: api2, state, props, emit, mode }) => {
  Object.assign(api2, {
    state,
    computedHasLabel: index.computedHasLabel,
    touchend: index.touchend({ state, api: api2 }),
    touchstart: index.touchstart({ props, state, api: api2 }),
    touchmove: index.touchmove({ props, state, vm }),
    playSlides: index.playSlides({ api: api2, props, state }),
    onComplete: index.onComplete({ api: api2, count: 0, emit, props, state }),
    pauseTimer: index.pauseTimer(state),
    itemInStage: index.itemInStage(state),
    resetItemPosition: index.resetItemPosition(state),
    watchItems: index.watchItems({ api: api2, props }),
    handleButtonLeave: index.handleButtonLeave(state),
    handleIndicatorClick: index.handleIndicatorClick({ api: api2, state }),
    handleIndicatorHover: index.handleIndicatorHover({ api: api2, props, state }),
    watchActiveIndex: index.watchActiveIndex({ api: api2, emit }),
    watchAutoplay: index.watchAutoplay(api2),
    startTimer: index.startTimer({ api: api2, props, state }),
    prev: index.prev({ api: api2, state }),
    next: index.next({ api: api2, state }),
    throttledArrowClick: index.throttledArrowClick(api2),
    setActiveItem: index.setActiveItem({ api: api2, props, state }),
    handleMouseEnter: index.handleMouseEnter({ api: api2, state }),
    handleMouseLeave: index.handleMouseLeave({ api: api2, state }),
    throttledIndicatorHover: index.throttledIndicatorHover(api2),
    handleButtonEnter: index.handleButtonEnter({ api: api2, state }),
    computedStyle: index.computedStyle({ props }),
    simulateTouch: index.simulateTouch({ props, vm }),
    computedHasButtons: index.computedHasButtons({ props, state, mode }),
    computedHasIndicators: index.computedHasIndicators({ props, state, mode }),
    canActive: index.canActive(props)
  });
};
const initWatch = ({ watch, props, api: api2, state }) => {
  watch(() => props.autoplay, api2.watchAutoplay);
  watch(
    () => props.loop,
    () => api2.setActiveItem(state.activeIndex)
  );
};
const renderless = (props, { computed, onMounted, onBeforeUnmount, reactive, watch, provide }, { vm, parent, emit, mode }) => {
  const api2 = {};
  const state = initState({ reactive, computed, api: api2 });
  initApi({ vm, api: api2, state, props, emit, mode });
  const { updateItems } = useItems({ api: api2, onMounted, state, watch });
  api2.updateItems = updateItems;
  useActiveIndex({ api: api2, state, watch });
  useResizeListener({ api: api2, onBeforeUnmount, onMounted, parent });
  provide("CarouselVm", vm);
  initWatch({ watch, props, api: api2, state });
  onMounted(() => {
    api2.startTimer();
    api2.onComplete(state.items.length);
    api2.simulateTouch();
  });
  parent.$on("updateItems", api2.updateItems);
  parent.$on("complete", () => {
    api2.onComplete(state.items.length);
  });
  return api2;
};
const useItems = ({ api: api2, onMounted, state, watch }) => {
  const updateItems = (carouselItemVm) => {
    if (carouselItemVm && !state.items.includes(carouselItemVm)) {
      state.items.push(carouselItemVm);
    }
  };
  watch(
    () => state.items,
    (value) => state.completed && api2.watchItems(value)
  );
  onMounted(updateItems);
  return {
    updateItems
  };
};
const useActiveIndex = ({ api: api2, state, watch }) => {
  watch(
    () => state.activeIndex,
    (value, oldValue) => api2.watchActiveIndex({ value, oldValue }),
    { immediate: true }
  );
};
const useResizeListener = ({ api: api2, onBeforeUnmount, onMounted, parent }) => {
  onMounted(() => {
    resizeEvent.addResizeListener(parent.$el, api2.resetItemPosition);
  });
  onBeforeUnmount(() => {
    if (parent.$el) {
      resizeEvent.removeResizeListener(parent.$el, api2.resetItemPosition);
    }
  });
};
exports.api = api;
exports.renderless = renderless;
exports.useActiveIndex = useActiveIndex;
exports.useItems = useItems;
exports.useResizeListener = useResizeListener;
