"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const node = require("./node.js");
const index = require("./index.js");
const flatNodes = (data, leafOnly) => data.reduce((prev, node2) => {
  if (node2.isLeaf) {
    prev.push(node2);
  } else {
    !leafOnly && prev.push(node2);
    prev = prev.concat(flatNodes(node2.children, leafOnly));
  }
  return prev;
}, []);
class Store {
  constructor(data, config) {
    this.config = config;
    this.initNodes(data);
  }
  getNodes() {
    return this.nodes;
  }
  initNodes(data) {
    data = index.coerceTruthyValueToArray(data);
    this.nodes = data.map((nodeData) => new node.default(nodeData, this.config));
    this.flattedNodes = this.getFlattedNodes(false, false);
    this.leafNodes = this.getFlattedNodes(true, false);
  }
  /**
   * 添加节点到parentNode
   */
  appendNode(nodeData, parentNode) {
    const { config, nodes } = this;
    const node$1 = new node.default(nodeData, config, parentNode);
    const children = parentNode ? parentNode.children : nodes;
    children.push(node$1);
  }
  /**
   * 添加节点到parentNode
   */
  appendNodes(nodeDataList, parentNode) {
    nodeDataList = index.coerceTruthyValueToArray(nodeDataList);
    nodeDataList.forEach((nodeData) => this.appendNode(nodeData, parentNode));
  }
  /**
   * 获取节点值
   * @param leafOnly 只包含叶子节点
   * @param cached 取缓存（flattedNodes或者leafNodes）中的数据
   * @returns
   */
  getFlattedNodes(leafOnly, cached = true) {
    const { leafNodes, flattedNodes } = this;
    const cachedNodes = leafOnly ? leafNodes : flattedNodes;
    return cached ? cachedNodes : flatNodes(this.nodes, leafOnly);
  }
  /**
   * 通过节点值获取Node节点
   * @param value
   * @returns
   */
  getNodeByValue(value) {
    if (value) {
      const lazy = !this.config.lazy;
      const nodes = this.getFlattedNodes(false, lazy).filter(
        (node2) => index.valueEquals(node2.path, value) || node2.value === value
      );
      return nodes && nodes.length ? nodes[0] : null;
    }
    return null;
  }
}
exports.default = Store;
