"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const chunkG2ADBYYC = require("../chunk-G2ADBYYC.js");
const index$1 = require("./index.js");
const resizeEvent = require("../common/deps/resize-event.js");
const vuePopper = require("../common/deps/vue-popper.js");
const index = require("../common/index.js");
const api = [
  "state",
  "getCheckedNodes",
  "computePresentText",
  "handleSuggestionClick",
  "handleSuggestionKeyDown",
  "handleExpandChange",
  "handleDropdownLeave",
  "handleDelete",
  "deleteTag",
  "handleClear",
  "handleInput",
  "handleBlur",
  "handleFocus",
  "toggleDropDownVisible",
  "handleKeyDown",
  "handleMouseenter",
  "handleMouseleave",
  "computePresentContent"
];
const initState = ({
  reactive,
  props,
  computed,
  parent,
  api: api2,
  t,
  constants,
  vm,
  inject,
  designConfig
}) => {
  const state = reactive({
    showAutoWidth: inject("showAutoWidth", null),
    /** popper 元素是否显示。 它是通过v-show 绑定到页面上，造成隐藏时，popperJs并没有destory,有一定的性能影响 */
    dropDownVisible: false,
    checkedValue: props.modelValue || null,
    inputHover: false,
    inputValue: null,
    presentText: null,
    presentTags: [],
    checkedNodes: [],
    filtering: false,
    suggestions: [],
    inputInitialHeight: 0,
    pressDeleteCount: 0,
    realSize: computed(() => props.size),
    formDisabled: computed(() => (parent.tinyForm || {}).disabled),
    displayOnly: computed(() => props.displayOnly),
    disabled: computed(() => props.disabled || state.isDisplayOnly || state.formDisabled),
    tagSize: computed(() => ~["small", "mini"].indexOf(state.realSize) ? "mini" : "small"),
    isDisabled: computed(() => state.disabled),
    isDisplayOnly: computed(() => state.displayOnly || (parent.tinyForm || {}).displayOnly),
    config: computed(() => api2.getConfig()),
    multiple: computed(() => state.config.multiple),
    leafOnly: computed(() => !state.config.checkStrictly),
    readonly: computed(() => !props.filterable || state.multiple),
    clearBtnVisible: computed(() => api2.computClearVisible()),
    panel: computed(() => vm.$refs.panel),
    placeholder: computed(() => props.placeholder || t(constants.placeholder)),
    inputValues: computed(() => state.multiple ? state.presentText : state.inputValue),
    collapseTagsLength: 0,
    isHidden: false,
    tooltipVisible: false,
    tooltipContent: "",
    tagTypeWhenMultiple: (designConfig == null ? void 0 : designConfig.tagTypeWhenMultiple) || ""
  });
  return state;
};
const initApi = ({
  api: api2,
  state,
  constants,
  dispatch,
  emit,
  vm,
  props,
  updatePopper,
  nextTick,
  parent,
  t
}) => {
  Object.assign(api2, {
    state,
    handleFocus: index$1.handleFocus(emit),
    handleClear: index$1.handleClear(state),
    getCheckedNodes: index$1.getCheckedNodes(state),
    deleteTag: index$1.deleteTag({ emit, state, api: api2 }),
    handleDropdownLeave: index$1.handleDropdownLeave(state),
    focusFirstNode: index$1.focusFirstNode({ vm, state }),
    computClearVisible: index$1.computClearVisible({ props, state }),
    showPlaceholder: index$1.showPlaceholder({ props, state, t, constants }),
    updateStyle: index$1.updateStyle({ parent, vm, state, updatePopper, nextTick, props }),
    getSuggestions: index$1.getSuggestions({ nextTick, props, state, updatePopper }),
    handleExpandChange: index$1.handleExpandChange({ constants, emit, dispatch, nextTick, state, updatePopper }),
    getConfig: index$1.getConfig({ parent, props }),
    setInputHeightToTag: index$1.setInputHeightToTag({ nextTick, parent, state }),
    handleSuggestionClick: index$1.handleSuggestionClick({ api: api2, state }),
    handleDelete: index$1.handleDelete({ api: api2, state }),
    computePresentText: index$1.computePresentText({ props, state }),
    handleSuggestionKeyDown: index$1.handleSuggestionKeyDown({ api: api2 }),
    handleInput: index$1.handleInput({ api: api2, state, vm }),
    handleKeyDown: index$1.handleKeyDown({ api: api2 }),
    watchValue: index$1.watchValue({ api: api2, state }),
    computePresentTags: index$1.computePresentTags({ api: api2, props, state }),
    computePresentContent: index$1.computePresentContent({ api: api2, state }),
    watchCheckedValue: index$1.watchCheckedValue({ constants, dispatch, api: api2, nextTick, emit, state }),
    toggleDropDownVisible: index$1.toggleDropDownVisible({ emit, vm, state, updatePopper }),
    selfMounted: index$1.selfMounted({ api: api2, parent, props, vm, state }),
    handleBeforeUpdate: index$1.handleBeforeUpdate({ props, api: api2, state }),
    handleBlur: index$1.handleBlur({ emit, api: api2, props }),
    calcCollapseTags: index$1.calcCollapseTags({ vm, state, nextTick }),
    watchInputHover: index$1.watchInputHover({ vm }),
    handleMouseenter: index$1.handleMouseenter({ vm, state }),
    handleMouseleave: index$1.handleMouseleave({ state })
  });
};
const initWatch = ({
  watch,
  state,
  api: api2,
  props,
  nextTick,
  updatePopper
}) => {
  watch(() => state.disabled, api2.computePresentContent);
  watch(() => props.modelValue, api2.watchValue);
  watch(() => state.checkedValue, api2.watchCheckedValue);
  watch(
    () => props.options,
    () => nextTick(api2.computePresentContent),
    { deep: true, immediate: true }
  );
  watch(
    () => state.presentText,
    (value) => index$1.presentTextHandler({ state, value })
  );
  watch(
    () => state.presentTags,
    (value, oldvalue) => {
      state.multiple && (value.length || oldvalue.length) && nextTick(() => api2.updateStyle());
    }
  );
  watch(
    () => state.filtering,
    () => nextTick(() => updatePopper())
  );
  watch(() => state.multiple, api2.setInputHeightToTag);
  watch(
    () => state.dropDownVisible,
    (value) => value && setTimeout(() => updatePopper())
  );
  watch(
    () => Array.isArray(state.checkedValue) && state.checkedValue.length,
    () => setTimeout(() => updatePopper())
  );
  watch([() => state.inputHover, () => state.dropDownVisible], api2.watchInputHover);
};
const renderless = (props, {
  computed,
  onMounted,
  onBeforeUnmount,
  onDeactivated,
  onUpdated,
  onBeforeUpdate,
  reactive,
  toRefs,
  watch,
  inject
}, { t, emit, nextTick, constants, parent, slots, dispatch, vm, designConfig }) => {
  parent.tinyForm = parent.tinyForm || inject("form", null);
  const { updatePopper } = vuePopper.default({
    reactive,
    watch,
    emit,
    props: chunkG2ADBYYC.__spreadProps(chunkG2ADBYYC.__spreadValues({}, props), {
      placement: index.DATEPICKER.PlacementMap.left,
      popperOptions: { boundariesPadding: 0, gpuAcceleration: false },
      visibleArrow: false,
      offset: 0,
      boundariesPadding: 5,
      arrowOffset: 35
    }),
    toRefs,
    vm,
    slots,
    nextTick,
    onBeforeUnmount,
    onDeactivated
  });
  const api2 = {};
  const state = initState({
    reactive,
    props,
    computed,
    parent,
    api: api2,
    t,
    constants,
    vm,
    inject,
    designConfig
  });
  initApi({ api: api2, state, constants, dispatch, emit, vm, props, updatePopper, nextTick, parent, t });
  initWatch({ watch, state, api: api2, props, nextTick, updatePopper });
  onBeforeUpdate(api2.handleBeforeUpdate);
  onUpdated(api2.updateStyle);
  onMounted(api2.selfMounted);
  parent.$on("handle-clear", (event) => {
    api2.handleClear(event);
  });
  onBeforeUnmount(() => resizeEvent.removeResizeListener(parent.$el, api2.updateStyle));
  return api2;
};
exports.api = api;
exports.renderless = renderless;
