"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = ["state", "handleChange", "computedStore"];
const initState = ({ reactive, computed, parent, api: api2, inject, props }) => {
  const state = reactive({
    size: computed(() => props.size || inject("size", null) || (parent.tinyForm || {}).size),
    vertical: inject("vertical", null),
    iconPosition: props.iconPosition || inject("iconPosition", "center"),
    focus: false,
    selfModel: false,
    showLabel: false,
    isLimitExceeded: false,
    checkboxGroup: null,
    store: computed(() => api2.computedStore()),
    isGroup: computed(() => api2.computedIsGroup()),
    isChecked: computed(() => api2.computedIsChecked()),
    isDisabled: computed(() => api2.computedIsDisabled()),
    checkboxSize: computed(() => api2.computedCheckboxSize()),
    isLimitDisabled: computed(() => api2.computedIsLimitDisabled()),
    formDisabled: computed(() => (parent.tinyForm || {}).disabled),
    formDisplayOnly: computed(() => (parent.tinyForm || {}).displayOnly),
    isDisplayOnly: computed(() => api2.computedIsDisplayOnly()),
    isGroupDisplayOnly: computed(() => api2.computedIsGroupDisplayOnly()),
    displayLabel: computed(() => api2.computedDisplayLabel()),
    inputDisabled: computed(() => state.isDisabled || state.isDisplayOnly || state.isGroupDisplayOnly),
    model: computed({
      get: () => api2.computedGetModelGet(),
      set: (value) => api2.computedGetModelSet(value)
    }),
    showText: computed(() => api2.computedShowText()),
    isShowText: computed(() => api2.computedIsShowText()),
    shape: inject("shape", null) || props.shape
  });
  return state;
};
const initApi = ({
  api: api2,
  state,
  dispatch,
  props,
  parent,
  constants,
  formItemSize,
  emit,
  nextTick,
  t,
  vm
}) => {
  Object.assign(api2, {
    state,
    addToStore: index.addToStore({ state, props }),
    removeFromStore: index.removeFromStore({ state, props }),
    computedStore: index.computedStore({ state, props }),
    computedFormItemSize: index.computedFormItemSize(props),
    computedIsChecked: index.computedIsChecked({ state, props }),
    computedIsLimitDisabled: index.computedIsLimitDisabled(state),
    computedIsDisabled: index.computedIsDisabled({ state, props }),
    computedIsDisplayOnly: index.computedIsDisplayOnly({ state, props }),
    computedIsGroupDisplayOnly: index.computedIsGroupDisplayOnly({ state }),
    computedGetModelGet: index.computedGetModelGet({ state, props }),
    computedIsGroup: index.computedIsGroup({ state, parent, constants }),
    computedCheckboxSize: index.computedCheckboxSize({ state, props, formItemSize }),
    computedGetModelSet: index.computedGetModelSet({ state, dispatch, emit, constants }),
    mounted: index.mounted({ emit, props, api: api2, parent }),
    handleChange: index.handleChange({ state, props, emit, nextTick, dispatch, constants }),
    computedDisplayLabel: index.computedDisplayLabel({ state, props, t }),
    computedIsShowText: index.computedIsShowText({ props }),
    computedShowText: index.computedShowText({ props })
  });
};
const renderless = (props, { computed, onMounted, onBeforeUnmount, reactive, watch, inject }, { vm, parent, emit, constants, nextTick, dispatch, t }) => {
  const api2 = { dispatch };
  const formItemSize = computed(() => api2.computedFormItemSize());
  const state = initState({ reactive, computed, parent, api: api2, inject, props });
  parent.tinyForm = parent.tinyForm || inject("form", null);
  initApi({ api: api2, state, dispatch, props, parent, constants, formItemSize, emit, nextTick, t, vm });
  watch(
    () => props.modelValue,
    (value) => props.validateEvent && api2.dispatch(constants.FORM_ITEM, constants.FORM_CHANGE, value)
  );
  watch(
    () => props.checked,
    (value) => {
      value ? api2.addToStore() : api2.removeFromStore();
    }
  );
  onBeforeUnmount(() => {
    index.toggleEvent({ parent, props, type: "remove" });
  });
  onMounted(() => {
    dispatch("Tooltip", "tooltip-update");
    index.toggleEvent({ parent, props, type: "add" });
    api2.mounted();
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
