"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const date = require("./date.js");
const type = require("../type.js");
const date$1 = require("../date.js");
const index = require("../index.js");
const weeks = index.DATEPICKER.Weeks;
const months = index.DATEPICKER.MonhtList;
const defaultYMD = index.DATEPICKER.DateFormats.date;
const defaultHMS = index.DATEPICKER.DateFormats.time;
const newArray = (start, end) => {
  let res = [];
  for (let i = start; i <= end; i++) {
    res.push(i);
  }
  return res;
};
const getI18nSettings = (t) => ({
  dayNamesShort: weeks.map((week) => t(`ui.datepicker.weeks.${week}`)),
  dayNames: weeks.map((week) => t(`ui.datepicker.weeks.${week}`)),
  monthNamesShort: months.map((month) => t(`ui.datepicker.months.${month}`)),
  monthNames: months.map((month, index2) => t(`ui.datepicker.month${index2 + 1}`)),
  amPm: ["am", "pm"]
});
const isDate = function(date2) {
  if (type.isNull(date2)) {
    return false;
  }
  if (isNaN(new Date(date2).getTime())) {
    return false;
  }
  if (Array.isArray(date2)) {
    return false;
  }
  return true;
};
const toDate = (date2) => isDate(date2) ? new Date(date2) : null;
const isDateObject = (val) => val instanceof Date;
const formatDate = (date$12, format, t) => {
  date$12 = toDate(date$12);
  if (!date$12) {
    return "";
  }
  return date.default.format(date$12, format || defaultYMD, getI18nSettings(t));
};
const parseDate = (string, format, t) => date.default.parse(string, format || defaultYMD, getI18nSettings(t));
const getDayCountOfMonth = (year, month) => {
  if (~[3, 5, 8, 10].indexOf(month)) {
    return 30;
  }
  if (month === 1) {
    return date$1.isLeapYear(year) ? 29 : 28;
  }
  return 31;
};
const getFirstDayOfMonth = (date2) => {
  const temp = new Date(date2.getTime());
  temp.setDate(1);
  return temp.getDay();
};
const prevDate = (date2, amount = 1) => new Date(date2.getFullYear(), date2.getMonth(), date2.getDate() - amount);
const nextDate = (date2, amount = 1) => new Date(date2.getFullYear(), date2.getMonth(), date2.getDate() + amount);
const getStartDateOfMonth = (year, month, offsetDay = 0) => {
  const res = new Date(year, month, 1);
  const day = res.getDay();
  const _day = day === 0 ? 7 : day;
  const offset = _day + offsetDay <= 0 ? 7 + _day : _day;
  return prevDate(res, offset);
};
const getWeekNumber = (src) => {
  if (!isDate(src)) {
    return null;
  }
  const date2 = new Date(src.getTime());
  date2.setHours(0, 0, 0, 0);
  date2.setDate(date2.getDate() + 3 - (date2.getDay() + 6) % 7);
  const week1 = new Date(date2.getFullYear(), 0, 4);
  return 1 + Math.round(((date2.getTime() - week1.getTime()) / 864e5 - 3 + (week1.getDay() + 6) % 7) / 7);
};
const getRangeHours = (ranges = []) => {
  const hours = [];
  let disHours = [];
  ranges.forEach((range2) => {
    const value = range2.map((date2) => date2.getHours());
    disHours = disHours.concat(newArray(value[0], value[1]));
  });
  let isDisabled;
  if (disHours.length) {
    isDisabled = (i) => !~disHours.indexOf(i);
  } else {
    isDisabled = () => false;
  }
  for (let i = 0; i < 24; i++) {
    hours[i] = isDisabled(i);
  }
  return hours;
};
const setRangeData = (arr, start, end, value) => {
  for (let i = start; i < end; i++) {
    arr[i] = value;
  }
};
const range = (length) => Array.apply(null, { length }).map((_, n) => n);
const getRangeMinutes = (ranges, hour) => {
  const sixty = 60;
  const minutes = new Array(sixty);
  if (ranges.length > 0) {
    ranges.forEach((range2) => {
      const [startDate, endDate] = range2;
      const startHour = startDate.getHours();
      const startMinute = startDate.getMinutes();
      const endHour = endDate.getHours();
      const endMinute = endDate.getMinutes();
      const equealStartHour = startHour === hour;
      if (equealStartHour && endHour !== hour) {
        setRangeData(minutes, startMinute, sixty, true);
      } else if (equealStartHour && endHour === hour) {
        setRangeData(minutes, startMinute, endMinute + 1, true);
      } else if (!equealStartHour && endHour === hour) {
        setRangeData(minutes, 0, endMinute + 1, true);
      } else if (startHour < hour && endHour > hour) {
        setRangeData(minutes, 0, sixty, true);
      }
    });
  } else {
    setRangeData(minutes, 0, sixty, true);
  }
  return minutes;
};
const modifyDate = (date2, y, m, d) => {
  date2 = toDate(date2);
  return new Date(y, m, d, date2.getHours(), date2.getMinutes(), date2.getSeconds(), date2.getMilliseconds());
};
const modifyTime = (date2, h, m, s) => {
  date2 = toDate(date2);
  return new Date(date2.getFullYear(), date2.getMonth(), date2.getDate(), h, m, s, date2.getMilliseconds());
};
const modifyWithTimeString = (date2, time, t) => {
  if (type.isNull(date2) || !time) {
    return date2;
  }
  time = parseDate(time, defaultHMS, t);
  return modifyTime(date2, time.getHours(), time.getMinutes(), time.getSeconds());
};
const clearTime = (date2) => new Date(date2.getFullYear(), date2.getMonth(), date2.getDate());
const clearMilliseconds = (date2) => new Date(
  date2.getFullYear(),
  date2.getMonth(),
  date2.getDate(),
  date2.getHours(),
  date2.getMinutes(),
  date2.getSeconds(),
  0
);
const limitTimeRange = (date$12, ranges, format = defaultHMS) => {
  if (ranges.length === 0) {
    return date$12;
  }
  const normalizeDate = (date2) => date.default.parse(date.default.format(date2, format), format);
  const ndate = normalizeDate(date$12);
  const nranges = ranges.map((range2) => range2.map(normalizeDate));
  if (nranges.some((nrange) => ndate >= nrange[0] && ndate <= nrange[1])) {
    return date$12;
  }
  let minDate = nranges[0][0];
  let maxDate = minDate;
  nranges.forEach((nrange) => {
    let minTempDate = minDate = new Date(Math.min(nrange[0], minDate));
    maxDate = new Date(Math.max(nrange[1], minTempDate));
  });
  const ret = ndate < minDate ? minDate : maxDate;
  return modifyDate(ret, date$12.getFullYear(), date$12.getMonth(), date$12.getDate());
};
const timeWithinRange = (date2, selectableRange, format) => {
  const limitedDate = limitTimeRange(date2, selectableRange, format);
  return limitedDate.getTime() === date2.getTime();
};
const changeYearMonthAndClampDate = (date2, year, month) => {
  const monthDate = Math.min(date2.getDate(), getDayCountOfMonth(year, month));
  return modifyDate(date2, year, month, monthDate);
};
const nextMonth = (date2) => {
  const year = date2.getFullYear();
  const month = date2.getMonth();
  const isLast = month === 11;
  return isLast ? changeYearMonthAndClampDate(date2, year + 1, 0) : changeYearMonthAndClampDate(date2, year, month + 1);
};
const prevMonth = (date2) => {
  const year = date2.getFullYear();
  const month = date2.getMonth();
  const isFirst = month === 0;
  return isFirst ? changeYearMonthAndClampDate(date2, year - 1, 11) : changeYearMonthAndClampDate(date2, year, month - 1);
};
const nextYear = (date2, next = 1) => {
  const year = date2.getFullYear();
  const month = date2.getMonth();
  return changeYearMonthAndClampDate(date2, year + next, month);
};
const prevYear = (date2, prev = 1) => {
  const year = date2.getFullYear();
  const month = date2.getMonth();
  return changeYearMonthAndClampDate(date2, year - prev, month);
};
const extractTimeFormat = (dateFormat) => dateFormat.replace(/\W?D{1,2}|\W?Do|\W?d{1,4}|\W?M{1,4}|\W?y{2,4}/g, "").trim();
const extractDateFormat = (dateFormat) => dateFormat.replace(/\W?m{1,2}|\W?ZZ/g, "").replace(/\W?h{1,2}|\W?s{1,3}|\W?a/gi, "").trim();
exports.changeYearMonthAndClampDate = changeYearMonthAndClampDate;
exports.clearMilliseconds = clearMilliseconds;
exports.clearTime = clearTime;
exports.extractDateFormat = extractDateFormat;
exports.extractTimeFormat = extractTimeFormat;
exports.formatDate = formatDate;
exports.getDayCountOfMonth = getDayCountOfMonth;
exports.getFirstDayOfMonth = getFirstDayOfMonth;
exports.getI18nSettings = getI18nSettings;
exports.getRangeHours = getRangeHours;
exports.getRangeMinutes = getRangeMinutes;
exports.getStartDateOfMonth = getStartDateOfMonth;
exports.getWeekNumber = getWeekNumber;
exports.isDate = isDate;
exports.isDateObject = isDateObject;
exports.limitTimeRange = limitTimeRange;
exports.modifyDate = modifyDate;
exports.modifyTime = modifyTime;
exports.modifyWithTimeString = modifyWithTimeString;
exports.nextDate = nextDate;
exports.nextMonth = nextMonth;
exports.nextYear = nextYear;
exports.parseDate = parseDate;
exports.prevDate = prevDate;
exports.prevMonth = prevMonth;
exports.prevYear = prevYear;
exports.range = range;
exports.timeWithinRange = timeWithinRange;
exports.toDate = toDate;
