"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const chunkG2ADBYYC = require("../../../chunk-G2ADBYYC.js");
const type = require("../../type.js");
const util = require("./util.js");
const node = require("./node.js");
class TreeStore {
  constructor(options) {
    this.currentNode = null;
    this.currentNodeKey = null;
    for (let option in options) {
      if (type.hasOwn.call(options, option)) {
        this[option] = options[option];
      }
    }
    this.nodesMap = {};
    this.root = new node.default({ data: this.data, store: this });
    if (this.lazy && this.load) {
      this.load(this.root, (data) => {
        this.root.doCreateChildren(data);
        this._initDefaultCheckedNodes();
        typeof this.afterLoad === "function" && this.afterLoad({ data, init: true });
      });
    } else {
      this._initDefaultCheckedNodes();
    }
  }
  getMappingData(data) {
    const props = this.props || {};
    const mapping = {};
    for (let key in props) {
      if (type.hasOwn.call(props, key)) {
        mapping[key] = data[props[key]];
      }
    }
    return chunkG2ADBYYC.__spreadValues(chunkG2ADBYYC.__spreadValues({}, data), mapping);
  }
  filter(value) {
    const { lazy, filterNodeMethod, getMappingData } = this;
    const walkTree = (node2) => {
      const childNodes = node2.root ? node2.root.childNodes : node2.childNodes;
      childNodes.forEach((child) => {
        const mappingData = getMappingData.call(this, child.data);
        child.visible = filterNodeMethod.call(child, value, mappingData, child);
        walkTree(child);
      });
      if (!node2.visible && childNodes.length) {
        let allHidden = !childNodes.some(({ visible }) => visible);
        if (node2.root) {
          node2.root.visible = allHidden === false;
        } else {
          node2.visible = allHidden === false;
        }
      }
      if (!value) {
        return;
      }
      if (node2.visible && !node2.isLeaf && !lazy) {
        node2.expand();
      }
    };
    walkTree(this);
  }
  setData(newVal) {
    if (newVal !== this.root.data) {
      this.root.setData(newVal);
      this._initDefaultCheckedNodes();
    } else {
      this.root.updateChildren();
    }
  }
  getNode(data) {
    if (data instanceof node.default) {
      return data;
    }
    const nodeKey = typeof data !== "object" ? data : util.getNodeKey(this.key, data);
    return this.nodesMap[nodeKey] || null;
  }
  insertBefore(data, insertData) {
    const refNode = this.getNode(insertData);
    refNode.parent.insertBefore({ data }, refNode);
  }
  insertAfter(data, insertData) {
    const refNode = this.getNode(insertData);
    refNode.parent.insertAfter({ data }, refNode);
  }
  remove(data, isSaveChildNode, isNode) {
    const treeNode = isNode ? data : this.getNode(data);
    if (treeNode && treeNode.parent) {
      if (treeNode === this.currentNode) {
        this.currentNode = null;
      }
      if (isSaveChildNode && treeNode.childNodes) {
        treeNode.childNodes.forEach((child) => {
          treeNode.parent.insertChild({ data: child.data });
        });
      }
      treeNode.parent.removeChild(treeNode);
    }
  }
  append(data, parentData, index) {
    const parentNode = parentData ? this.getNode(parentData) : this.root;
    if (parentNode) {
      const child = parentNode.insertChild({ data }, index);
      data._isNewNode && this.registerNode(child);
    }
  }
  setDefaultCheckedKey(newValue) {
    if (newValue !== this.defaultCheckedKeys) {
      this.defaultCheckedKeys = newValue;
      this._initDefaultCheckedNodes();
    }
  }
  _initDefaultCheckedNodes() {
    const defaultCheckedKeys = this.defaultCheckedKeys || [];
    const nodesMap = this.nodesMap;
    defaultCheckedKeys.forEach((checkedKey) => {
      const node2 = nodesMap[checkedKey];
      node2 && node2.setChecked(true, !this.checkStrictly);
    });
  }
  _initDefaultCheckedNode(node2) {
    const defaultCheckedKeys = this.defaultCheckedKeys || [];
    ~defaultCheckedKeys.indexOf(node2.key) && node2.setChecked(true, !this.checkStrictly);
  }
  getCheckedKeys(leafOnly = false) {
    return this.getCheckedNodes(leafOnly).map((node2) => (node2 || {})[this.key]);
  }
  getHalfCheckedKeys() {
    return this.getHalfCheckedNodes().map((node2) => (node2 || {})[this.key]);
  }
  deregisterNode(node2) {
    const key = this.key;
    if (!key || !node2 || !node2.data) {
      return;
    }
    node2.childNodes.forEach((child) => {
      this.deregisterNode(child);
    });
    delete this.nodesMap[node2.key];
  }
  registerNode(node2) {
    const key = this.key;
    if (!key || !node2 || !node2.data) {
      return;
    }
    const nodeKey = node2.key;
    if (nodeKey !== void 0) {
      this.nodesMap[nodeKey] = node2;
    }
  }
  getCheckedNodes(leafOnly = false, includeHalfChecked = false, isNode = false) {
    const checkedNodes = [];
    const walkTree = (node2) => {
      const childNodes = node2.root ? node2.root.childNodes : node2.childNodes;
      childNodes.forEach((child) => {
        const { checked, indeterminate, isLeaf, data } = child;
        if ((checked || includeHalfChecked && indeterminate) && (!leafOnly || leafOnly && isLeaf)) {
          checkedNodes.push(isNode ? child : data);
        }
        walkTree(child);
      });
    };
    walkTree(this);
    return checkedNodes;
  }
  getHalfCheckedNodes() {
    const nodes = [];
    const walkTree = (node2) => {
      const childNodes = node2.root ? node2.root.childNodes : node2.childNodes;
      childNodes.forEach((child) => {
        const { indeterminate, data } = child;
        indeterminate && nodes.push(data);
        walkTree(child);
      });
    };
    walkTree(this);
    return nodes;
  }
  _getAllNodes() {
    const allNodes = [];
    const nodesMap = this.nodesMap;
    Object.keys(nodesMap).forEach((nodeKey) => {
      type.hasOwn.call(nodesMap, nodeKey) && allNodes.push(nodesMap[nodeKey]);
    });
    return allNodes;
  }
  updateChildren(key, data) {
    const node2 = this.nodesMap[key];
    if (!node2) {
      return;
    }
    const childNodes = node2.childNodes;
    for (let i = childNodes.length - 1; i >= 0; i--) {
      this.remove(childNodes[i].data);
    }
    for (let i = 0, len = data.length; i < len; i++) {
      const child = data[i];
      this.append(child, node2.data);
    }
  }
  _setCheckedKeys(key, leafOnly = false, checkedKeys = {}) {
    const nodes = this._getAllNodes().sort((prevNode, nextNode) => nextNode.level - prevNode.level);
    const cache = /* @__PURE__ */ Object.create(null);
    const keys = Object.keys(checkedKeys);
    nodes.forEach((node2) => {
      node2.setChecked(false, false);
    });
    for (let i = 0, len = nodes.length; i < len; i++) {
      const node2 = nodes[i];
      const nodeKey = node2.data[key].toString();
      let checked = ~keys.indexOf(nodeKey);
      if (!checked) {
        if (node2.checked && !cache[nodeKey]) {
          node2.setChecked(false, false);
        }
      } else {
        let parentNode = node2.parent;
        while (parentNode && parentNode.level > 0) {
          cache[parentNode.data[key]] = true;
          parentNode = parentNode.parent;
        }
        if (node2.isLeaf || this.checkStrictly) {
          node2.setChecked(true, false);
        } else if (leafOnly) {
          node2.setChecked(false, false);
          const walkTree = (node22) => {
            const childNodes = node22.childNodes;
            childNodes.forEach((child) => {
              !child.isLeaf && child.setChecked(false, false);
              walkTree(child);
            });
          };
          walkTree(node2);
        } else {
          node2.setChecked(true, true);
        }
      }
    }
  }
  setDefaultExpandedKeys(keys) {
    keys = keys || [];
    this.defaultExpandedKeys = keys;
    keys.forEach((key) => {
      const node2 = this.getNode(key);
      node2 && node2.expand(null, this.autoExpandParent);
    });
  }
  setCheckedKeys(keys, leafOnly = false) {
    this.defaultCheckedKeys = keys;
    const checkedKeys = {};
    keys.forEach((key) => {
      checkedKeys[key] = true;
    });
    this._setCheckedKeys(this.key, leafOnly, checkedKeys);
  }
  setCheckedNodes(array, leafOnly = false) {
    const key = this.key;
    const checkedKeys = {};
    array.forEach((item) => {
      checkedKeys[(item || {})[key]] = true;
    });
    this._setCheckedKeys(key, leafOnly, checkedKeys);
  }
  setChecked(data, checked, deep) {
    const node2 = this.getNode(data);
    node2 && node2.setChecked(!!checked, deep);
  }
  setCurrentNode(currentNode) {
    const prevNode = this.currentNode;
    if (prevNode) {
      prevNode.isCurrent = false;
    }
    this.currentNode = currentNode;
    if (currentNode) {
      this.currentNode.isCurrent = true;
    }
  }
  getCurrentNode() {
    return this.currentNode;
  }
  setCurrentNodeKey(key) {
    if (type.isNull(key)) {
      this.currentNode && (this.currentNode.isCurrent = false);
      this.currentNode = null;
      return;
    }
    const node2 = this.getNode(key);
    node2 && this.setCurrentNode(node2);
  }
  setUserCurrentNode(node2) {
    const key = node2[this.key];
    const currNode = this.nodesMap[key];
    this.setCurrentNode(currNode);
  }
  getData(data) {
    return (this.getNode(data) || {}).data;
  }
  getAllData() {
    const children = this.props.children;
    const walkTree = (nodes) => {
      return nodes.map((node2) => {
        return chunkG2ADBYYC.__spreadProps(chunkG2ADBYYC.__spreadValues({}, node2.data), { [children]: walkTree(node2.childNodes) });
      });
    };
    return walkTree(this.root.childNodes);
  }
}
exports.default = TreeStore;
