"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const chunkG2ADBYYC = require("../../chunk-G2ADBYYC.js");
const object = require("../object.js");
const popupManager = require("./popup-manager.js");
const dom = require("./dom.js");
let idSeed = 1;
const isServer = typeof window === "undefined";
const setWatchFn = ({
  onMounted,
  onBeforeUnmount,
  watch,
  vm,
  api,
  props,
  state,
  nextTick
}) => {
  onMounted(() => {
    vm._popupId = `popup-${idSeed++}`;
    popupManager.default.register(vm._popupId, vm);
  });
  onBeforeUnmount(() => {
    popupManager.default.deregister(vm._popupId);
    popupManager.default.closeModal(vm._popupId);
  });
  watch(
    () => props.visible,
    (val) => {
      if (val) {
        if (vm._opening) {
          return;
        }
        if (state.rendered) {
          api.open();
        } else {
          state.rendered = true;
          nextTick(() => {
            api.open();
          });
        }
      } else {
        api.close();
      }
    }
  );
};
const openFn = ({ state, vm }) => (options) => {
  if (!state.rendered) {
    state.rendered = true;
  }
  const props = object.merge({}, vm.$props || vm, options);
  if (vm._closeTimer) {
    clearTimeout(vm._closeTimer);
    vm._closeTimer = null;
  }
  clearTimeout(vm._openTimer);
  const doOpen = () => {
    if (isServer || state.opened) {
      return;
    }
    vm._opening = true;
    const dom$1 = vm.$el;
    const modal = props.modal;
    const zIndex = props.zIndex;
    if (zIndex) {
      popupManager.default.zIndex = zIndex;
    }
    if (modal) {
      if (vm._closing) {
        popupManager.default.closeModal(vm._popupId);
        vm._closing = false;
      }
      popupManager.default.openModal(
        vm._popupId,
        popupManager.default.nextZIndex(),
        props.modalAppendToBody ? void 0 : dom$1,
        props.modalClass,
        props.modalFade
      );
      if (props.lockScroll) {
        popupManager.default.fixBodyBorder();
        dom.addClass(document.body, popupManager.default.popLockClass);
      }
    }
    if (getComputedStyle(dom$1).position === "static") {
      dom$1.style.position = "absolute";
    }
    dom$1.style.zIndex = popupManager.default.nextZIndex().toString();
    state.opened = true;
    vm._opening = false;
  };
  const openDelay = Number(props.openDelay);
  if (openDelay > 0) {
    vm._openTimer = setTimeout(() => {
      vm._openTimer = null;
      doOpen();
    }, openDelay);
  } else {
    doOpen();
  }
};
const closeFn = ({ state, vm }) => () => {
  if (vm._openTimer !== null) {
    clearTimeout(vm._openTimer);
    vm._openTimer = null;
  }
  clearTimeout(vm._closeTimer);
  const doClose = () => {
    vm._closing = true;
    state.opened = false;
    popupManager.default.closeModal(vm._popupId);
    vm._closing = false;
  };
  const closeDelay = Number(vm.closeDelay);
  if (closeDelay > 0) {
    vm._closeTimer = setTimeout(() => {
      vm._closeTimer = null;
      doClose();
    }, closeDelay);
  } else {
    doClose();
  }
};
var vue_popup_default = (options) => {
  const { api, nextTick, onBeforeUnmount, onMounted, props, reactive, toRefs, vm, watch } = options;
  const state = reactive({
    opened: false,
    rendered: false
  });
  setWatchFn({ onMounted, onBeforeUnmount, watch, vm, api, props, state, nextTick });
  const open = openFn({ state, vm });
  const close = closeFn({ state, vm });
  return chunkG2ADBYYC.__spreadValues({ open, close, PopupManager: popupManager.default }, toRefs(state));
};
exports.default = vue_popup_default;
