"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const type = require("./type.js");
exports.extend = void 0;
const getObj = (data, names, isExceptRoot) => {
  if (!data || !type.isPlainObject(data) || !names || typeof names !== "string") {
    return;
  }
  const nameArr = names.split(".");
  let obj = data;
  const len = nameArr.length;
  if (len > 1) {
    const startIndex = 0;
    for (let i = startIndex; i < len; i++) {
      obj = obj[nameArr[i]];
      if (type.isNull(obj)) {
        return obj;
      }
    }
    return obj;
  } else {
    return obj[nameArr[0]];
  }
};
const setObj = (data, names, value, isMerge) => {
  if (!data || !type.isPlainObject(data) || !names || typeof names !== "string") {
    return data;
  }
  const nameArr = names.split(".");
  const obj = data;
  let len = nameArr.length;
  let item = nameArr[0];
  if (len > 1) {
    len--;
    let tmpl = obj;
    let name, target;
    for (let i = 0; i < len; i++) {
      name = nameArr[i];
      target = tmpl[name];
      if (target === null || !type.isPlainObject(target)) {
        tmpl[name] = {};
        target = tmpl[name];
      }
      tmpl = target;
    }
    item = nameArr[len];
    tmpl[item] = value;
  } else {
    obj[item] = value;
  }
  return obj;
};
const copyField = (data, fields, isMerge, isExclude) => {
  const innerCopyFields = (obj, fields2, isMerge2, isExclude2) => {
    const result = {};
    {
      fields2.forEach((field) => setObj(result, field, getObj(obj, field)));
    }
    return result;
  };
  if (type.isPlainObject(data)) {
    return Array.isArray(fields) ? innerCopyFields(data, fields) : exports.extend(isMerge !== false, {}, data);
  }
  return data;
};
const copyArray = (arr) => {
  return Array.isArray(arr) ? arr.map((item) => copyField(item)) : arr;
};
const deepCopy = (target, name, deep, copy, src) => {
  let copyIsArray;
  if (deep && copy && (type.isPlainObject(copy) || (copyIsArray = Array.isArray(copy)))) {
    if (copyIsArray) {
      copyIsArray = false;
      target[name] = copyArray(copy);
    } else {
      const clone = src && type.isPlainObject(src) ? src : {};
      target[name] = exports.extend(deep, clone, copy);
    }
  } else if (copy !== void 0) {
    try {
      target[name] = copy;
    } catch (e) {
    }
  }
};
exports.extend = function(...args) {
  const length = args.length;
  let target = args[0] || {};
  let i = 1;
  let deep = false;
  if (type.typeOf(target) === "boolean") {
    deep = target;
    target = args[i] || {};
    i++;
  }
  if (!type.isObject(target) && type.typeOf(target) !== "function") {
    target = {};
  }
  for (; i < length; i++) {
    const options = args[i];
    if (options !== null && type.isObject(options)) {
      const names = Object.keys(options);
      for (const name of names) {
        const src = target[name];
        const copy = options[name];
        if (target !== copy) {
          deepCopy(target, name, deep, copy, src);
        }
      }
    }
  }
  return target;
};
exports.isEachEqual = void 0;
const isEqual = (sourceData, targetData, deep, fields) => {
  if (type.typeOf(sourceData) === type.typeOf(targetData)) {
    deep = deep !== false;
    if (Array.isArray(fields)) {
      const _sourceData = copyField(sourceData, fields);
      const _targetData = copyField(targetData, fields);
      return isEqual(_sourceData, _targetData, deep);
    }
    const source = exports.isEachEqual(sourceData, targetData, deep);
    const target = exports.isEachEqual(targetData, sourceData, deep);
    return source && target;
  }
  return false;
};
exports.isEachEqual = (data1, data2, deep) => {
  if (!type.isPlainObject(data1)) {
    if (!Array.isArray(data1)) {
      return data1 === data2;
    }
    if (data1.length !== data2.length) {
      return false;
    }
    for (let i = 0, length = data1.length; i < length; i++) {
      const result = isEqual(data1[i], data2[i], deep);
      if (!result) {
        return false;
      }
    }
    return true;
  }
  let bEqual = true;
  const names = Object.keys(data1);
  for (const name of names) {
    if (type.hasOwn.call(data2, name)) {
      const _data1 = data1[name];
      const _data2 = data2[name];
      if (deep && type.isObject(_data1) || Array.isArray(_data1)) {
        bEqual = exports.isEachEqual(_data1, _data2, deep);
      } else {
        bEqual = _data1 === _data2;
      }
    } else {
      bEqual = false;
    }
    if (bEqual === false) {
      break;
    }
  }
  return bEqual;
};
const toJsonStr = (obj) => {
  try {
    return JSON.stringify(obj);
  } catch (e) {
    return void 0;
  }
};
const merge = function(target, ...rest) {
  for (let i = 0, len = rest.length; i < len; i++) {
    const source = rest[i] || {};
    for (const prop in source) {
      if (type.hasOwn.call(source, prop)) {
        const value = source[prop];
        if (value !== void 0) {
          target[prop] = value;
        }
      }
    }
  }
  return target;
};
exports.copyArray = copyArray;
exports.copyField = copyField;
exports.getObj = getObj;
exports.isEqual = isEqual;
exports.merge = merge;
exports.setObj = setObj;
exports.toJsonStr = toJsonStr;
