"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = [
  "state",
  "clear",
  "replace",
  "getCroppedCanvas",
  "setCanvasData",
  "setDragMode",
  "setAspectRatio",
  "getCanvasData",
  "getImageData",
  "getContainerData",
  "scaleY",
  "scale",
  "scaleX",
  "destroy",
  "enable",
  "closeCropArea",
  "disable",
  "closeCrop",
  "getCropBoxData",
  "getData",
  "move",
  "reset",
  "rotate",
  "setCropBoxData",
  "setData",
  "showFileChooser",
  "setImage",
  "cropImage",
  "zoom"
];
const initRenderIcon = (api2, t) => {
  const renderIcon = [
    {
      method: () => api2.showFileChooser(),
      icon: "IconNew",
      title: t("ui.crop.choose")
    },
    { split: true },
    {
      method: () => api2.zoom(-0.1),
      icon: "IconZoomOut",
      title: t("ui.crop.zoomOut")
    },
    {
      method: () => api2.zoom(0.1),
      icon: "IconZoomIn",
      title: t("ui.crop.zoomIn")
    },
    {
      method: () => api2.rotate(-45),
      icon: "IconRepeat",
      title: t("ui.crop.rotate_45")
    },
    {
      method: () => api2.rotate(45),
      icon: "IconRefres",
      title: t("ui.crop.rotate45")
    },
    { split: true },
    {
      method: () => api2.closeCropArea(),
      icon: "IconCrop",
      title: t("ui.crop.closeCropArea")
    },
    {
      method: () => api2.reset(),
      icon: "IconConmentRefresh",
      title: t("ui.crop.reset")
    },
    { split: true },
    {
      method: () => api2.closeCrop(),
      icon: "IconClose",
      title: t("ui.crop.closeCrop")
    },
    {
      method: () => api2.cropImage(),
      icon: "IconYes",
      title: t("ui.crop.cropImage")
    }
  ];
  return renderIcon;
};
const initState = ({ reactive, props, api: api2, t }) => {
  const state = reactive({
    src: props.src,
    cropper: "",
    data: null,
    cropvisible: props.cropvisible,
    renderIcon: initRenderIcon(api2, t)
  });
  return state;
};
const initApi = ({ api: api2, state, emit, refs, props }) => {
  Object.assign(api2, {
    state,
    zoom: index.zoom(state),
    move: index.move(state),
    clear: index.clear(state),
    reset: index.reset(state),
    scale: index.scale(state),
    rotate: index.rotate(state),
    enable: index.enable(state),
    scaleX: index.scaleX(state),
    scaleY: index.scaleY(state),
    replace: index.replace(state),
    setData: index.setData(state),
    getData: index.getData(state),
    disable: index.disable(state),
    destroy: index.destroy(state),
    setDragMode: index.setDragMode(state),
    getImageData: index.getImageData(state),
    watchImageSrc: index.watchImageSrc(state),
    setCanvasData: index.setCanvasData(state),
    getCanvasData: index.getCanvasData(state),
    closeCrop: index.closeCrop({ emit, state }),
    setCropBoxData: index.setCropBoxData(state),
    setAspectRatio: index.setAspectRatio(state),
    getCropBoxData: index.getCropBoxData(state),
    showFileChooser: index.showFileChooser(refs),
    getCroppedCanvas: index.getCroppedCanvas(state),
    getContainerData: index.getContainerData(state),
    createCrop: index.createCrop({ emit, props, refs, state }),
    // computedCropImages: computedCropImages({ constants, t }),
    shortcutKeys: index.shortcutKeys(api2),
    closeCropArea: index.closeCropArea(api2),
    setImage: index.setImage({ api: api2, state, props }),
    cropImage: index.cropImage({ api: api2, emit, props, state }),
    watchVisible: index.watchVisible({ api: api2, state })
  });
};
const initWatch = ({ watch, props, api: api2 }) => {
  watch(() => props.cropvisible, api2.watchVisible);
  watch(() => props.src, api2.watchImageSrc, { immediate: true });
};
const renderless = (props, { onMounted, reactive, watch }, { emit, refs, t }) => {
  const api2 = {};
  const state = initState({ reactive, props, api: api2, t });
  initApi({ api: api2, state, emit, refs, props });
  onMounted(api2.createCrop);
  initWatch({ watch, props, api: api2 });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
