"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const chunkG2ADBYYC = require("../chunk-G2ADBYYC.js");
const index$1 = require("../picker/index.js");
const debounce = require("../common/deps/debounce.js");
const dateUtil = require("../common/deps/date-util.js");
const index = require("../common/index.js");
const dom = require("../common/deps/dom.js");
const date = require("../common/date.js");
const string = require("../common/string.js");
const getYearLabel = ({ state, t }) => () => {
  const { YearI18n, Year, PanelYearNum } = index.DATEPICKER;
  const yearTranslation = t(YearI18n);
  if (state.currentView === Year) {
    const startYear = state.startYear;
    if (yearTranslation) {
      return startYear + " " + yearTranslation + " - " + (startYear + PanelYearNum - 1) + " " + yearTranslation;
    }
    return startYear + " - " + (startYear + PanelYearNum - 1);
  }
  return state.year + " " + yearTranslation;
};
const watchValue = ({ api, state }) => (value) => {
  if ([index.DATEPICKER.Dates, index.DATEPICKER.Years].includes(state.selectionMode) && state.value) {
    return;
  }
  if (dateUtil.isDate(value)) {
    state.date = state.selectionMode === "week" ? index$1.getWeekData(value) : new Date(value);
  } else {
    state.date = api.getDefaultValue();
  }
};
const watchDefaultValue = ({ state }) => (value) => {
  if (!dateUtil.isDate(state.value)) {
    state.date = value ? new Date(value) : /* @__PURE__ */ new Date();
  }
};
const watchTimePickerVisible = ({ nextTick, vm }) => (value) => {
  if (value) {
    nextTick(() => {
      vm.$refs.timepicker.adjustSpinners();
    });
  }
};
const watchSelectionMode = ({ state }) => (value) => {
  if (value === index.DATEPICKER.Month) {
    if (state.currentView !== index.DATEPICKER.Year || state.currentView !== index.DATEPICKER.Month) {
      state.currentView = index.DATEPICKER.Month;
    }
  } else if (value === index.DATEPICKER.Dates) {
    state.currentView = index.DATEPICKER.Date;
  } else if ([index.DATEPICKER.Year, index.DATEPICKER.Years, index.DATEPICKER.YearRange].includes(value)) {
    state.currentView = index.DATEPICKER.Year;
  }
};
const proxyTimePickerDataProperties = ({ vm, state, watch }) => () => {
  const format = (timeFormat) => {
    if (vm.$refs.timepicker) {
      vm.$refs.timepicker.state.format = timeFormat;
    }
  };
  const value = (value2) => {
    if (vm.$refs.timepicker) {
      vm.$refs.timepicker.state.value = null;
      if (value2) {
        vm.$refs.timepicker.state.value = value2;
      }
    }
  };
  const date2 = (date22) => {
    if (vm.$refs.timepicker) {
      vm.$refs.timepicker.state.date = date22;
    }
  };
  const selectableRange = (selectableRange2) => {
    if (vm.$refs.timepicker) {
      vm.$refs.timepicker.state.selectableRange = selectableRange2;
    }
  };
  watch(() => state.value, value);
  watch(() => state.date, date2, { deep: true });
  watch(() => state.selectableRange, selectableRange, { deep: true });
  format(state.timeFormat);
  value(state.value);
  date2(state.date);
  selectableRange(state.selectableRange);
};
const handleClear = ({ api, state, emit }) => () => {
  state.date = api.getDefaultValue();
  emit("pick", null);
};
const cusEmit = ({ state, emit }) => (value, ...args) => {
  if (!value) {
    emit("pick", value, ...args);
  } else if (Array.isArray(value)) {
    const dates = value.map((date2) => state.showTime ? dateUtil.clearMilliseconds(date2) : dateUtil.clearTime(date2));
    emit("pick", dates, ...args);
  } else {
    emit("pick", state.showTime ? dateUtil.clearMilliseconds(value) : dateUtil.clearTime(value), ...args);
  }
  state.userInputDate = null;
  state.userInputTime = null;
};
const showMonthPicker = ({ state }) => () => state.currentView = index.DATEPICKER.Month;
const showYearPicker = ({ state }) => () => state.currentView = index.DATEPICKER.Year;
const cusPrevMonth = ({ state }) => () => state.date = dateUtil.prevMonth(state.date);
const cusNextMonth = ({ state }) => () => state.date = dateUtil.nextMonth(state.date);
const cusPrevYear = ({ state }) => () => {
  if (state.currentView === index.DATEPICKER.Year) {
    state.startYear = state.startYear - index.DATEPICKER.PanelYearNum;
  } else {
    state.date = dateUtil.prevYear(state.date);
  }
};
const cusNextYear = ({ state }) => () => {
  if (state.currentView === index.DATEPICKER.Year) {
    state.startYear = state.startYear + index.DATEPICKER.PanelYearNum;
  } else {
    state.date = dateUtil.nextYear(state.date);
  }
};
const handleShortcutClick = (api) => (shortcut) => {
  if (shortcut.onClick) {
    const picker = {
      $emit: (type, start, end) => {
        api.doPick(start, end);
      }
    };
    shortcut.onClick(picker);
  }
};
const doPick = (emit) => (date2) => {
  emit("pick", date2, false);
};
const handleTimePick = ({ api, state, t }) => (value, visible, first) => {
  if (dateUtil.isDate(value)) {
    const newDate = state.value ? dateUtil.modifyTime(state.value, value.getHours(), value.getMinutes(), value.getSeconds()) : dateUtil.modifyWithTimeString(api.getDefaultValue(), state.defaultTime, t);
    state.date = newDate;
    api.cusEmit(state.date, true);
  } else {
    api.cusEmit(value, true);
  }
  if (!first) {
    state.timePickerVisible = visible;
  }
};
const handleTimePickClose = (state) => () => {
  state.timePickerVisible = false;
};
const handleMonthPick = ({ api, state }) => (month) => {
  if (state.selectionMode === index.DATEPICKER.Month) {
    state.date = dateUtil.modifyDate(state.date, state.year, month, 1);
    api.cusEmit(state.date);
  } else {
    state.date = dateUtil.changeYearMonthAndClampDate(state.date, state.year, month);
    state.currentView = index.DATEPICKER.Date;
  }
};
const handleDatePick = ({ api, state, t }) => (value) => {
  if (state.selectionMode === index.DATEPICKER.Day) {
    let newDate = state.value ? dateUtil.modifyDate(state.value, value.getFullYear(), value.getMonth(), value.getDate()) : dateUtil.modifyWithTimeString(value, state.defaultTime, t);
    if (!api.checkDateWithinRange(newDate)) {
      newDate = dateUtil.modifyDate(state.selectableRange[0][0], value.getFullYear(), value.getMonth(), value.getDate());
    }
    state.date = newDate;
    api.cusEmit(state.date, state.showTime);
  } else if (state.selectionMode === index.DATEPICKER.Week) {
    api.cusEmit(value.date);
  } else if (state.selectionMode === index.DATEPICKER.Dates) {
    api.cusEmit(value, true);
  }
};
const handleYearPick = ({ api, state }) => (value) => {
  if (state.selectionMode === index.DATEPICKER.Year) {
    state.date = dateUtil.modifyDate(state.date, value, 0, 2);
    api.cusEmit(state.date);
  } else if ([index.DATEPICKER.Years].includes(state.selectionMode)) {
    state.date = value.map((year) => new Date(year, 0, 2));
    api.cusEmit(state.date, state.selectionMode === index.DATEPICKER.YearRange ? value.length < 2 : true);
  } else {
    state.date = dateUtil.changeYearMonthAndClampDate(state.date, value, state.month);
    state.currentView = index.DATEPICKER.Month;
  }
};
const getDisabledNow = ({ state }) => () => {
  let disabledDate = state.disabledDate;
  if (!disabledDate)
    return false;
  return disabledDate(/* @__PURE__ */ new Date());
};
const getDisabledConfirm = ({ state }) => () => {
  let disabledDate = state.disabledDate;
  if (!disabledDate)
    return false;
  if (!state.value)
    return true;
  return disabledDate(state.value);
};
const dateToLocaleStringForIE = (timezone, value) => {
  const localTimezone = date.getLocalTimezone();
  const offsetTimezone = timezone - localTimezone;
  const offsetTime = new Date(value).getTime() + offsetTimezone * 36e5;
  return new Date(offsetTime);
};
const getNowTime = ({ props }) => () => {
  return new Promise((resolve) => {
    resolve(props.nowClick());
  }).then((res) => {
    return res;
  });
};
const changeToNow = ({ api, state, props }) => async () => {
  let now = /* @__PURE__ */ new Date();
  if (props.nowClick !== void 0) {
    now = await api.getNowTime();
  }
  const timezone = state.timezone;
  const isServiceTimezone = timezone.isServiceTimezone;
  let disabledDate = !state.disabledDate;
  if (isServiceTimezone) {
    const date$1 = date.getDateWithNewTimezone(now, date.getLocalTimezone(), timezone.to);
    state.date = date$1;
    disabledDate = disabledDate || !state.disabledDate(date$1);
  } else {
    disabledDate = disabledDate || !state.disabledDate(now);
  }
  if (disabledDate && api.checkDateWithinRange(now)) {
    if (!isServiceTimezone) {
      if (state.showTimezone && state.selectedTz) {
        state.date = dateToLocaleStringForIE(state.selectedTz.offset, now);
      } else {
        state.date = now;
      }
    }
    api.cusEmit(state.date);
    state.emitDbTime(now);
  }
};
const confirm = ({ api, state, t }) => () => {
  if ([index.DATEPICKER.Dates, index.DATEPICKER.Years].includes(state.selectionMode)) {
    api.cusEmit(state.value);
  } else {
    let value = state.value ? state.value : dateUtil.modifyWithTimeString(api.getDefaultValue(), state.defaultTime, t);
    const timezone = state.timezone;
    if (!state.value && timezone.isServiceTimezone) {
      value = state.date = date.getDateWithNewTimezone(value, date.getLocalTimezone(), timezone.to);
    } else {
      state.date = new Date(value);
    }
    api.cusEmit(value);
  }
};
const resetView = ({ state }) => () => {
  if (state.selectionMode === index.DATEPICKER.Month) {
    state.currentView = index.DATEPICKER.Month;
  } else if ([index.DATEPICKER.Year, index.DATEPICKER.Years, index.DATEPICKER.YearRange].includes(state.selectionMode)) {
    state.currentView = index.DATEPICKER.Year;
  } else {
    state.currentView = index.DATEPICKER.Date;
  }
};
const handleEnter = (api) => () => {
  dom.on(document.body, "keydown", api.handleKeydown);
};
const handleLeave = ({ api, emit }) => () => {
  emit("dodestroy");
  dom.off(document.body, "keydown", api.handleKeydown);
};
const handleKeydown = ({ api, state }) => (event) => {
  const keyCode = event.keyCode;
  if (state.visible && !state.timePickerVisible) {
    if (index.DATEPICKER.List.includes(keyCode)) {
      api.handleKeyControl(keyCode);
      event.stopPropagation();
      event.preventDefault();
    }
    if (keyCode === 13 && state.userInputDate === null && state.userInputTime === null) {
      api.cusEmit(state.date, false);
    }
  }
};
const handleKeyControl = ({ state, emit }) => (keyCode) => {
  const mapping = {
    year: chunkG2ADBYYC.__spreadProps(chunkG2ADBYYC.__spreadValues({}, index.DATEPICKER.YearObj), {
      offset: (date2, step) => date2.setFullYear(date2.getFullYear() + step)
    }),
    month: chunkG2ADBYYC.__spreadProps(chunkG2ADBYYC.__spreadValues({}, index.DATEPICKER.YearObj), {
      offset: (date2, step) => date2.setMonth(date2.getMonth() + step)
    }),
    week: chunkG2ADBYYC.__spreadProps(chunkG2ADBYYC.__spreadValues({}, index.DATEPICKER.WeekObj), {
      offset: (date2, step) => date2.setDate(date2.getDate() + step * 7)
    }),
    day: chunkG2ADBYYC.__spreadProps(chunkG2ADBYYC.__spreadValues({}, index.DATEPICKER.DayObj), {
      offset: (date2, step) => date2.setDate(date2.getDate() + step)
    })
  };
  const mode = state.selectionMode;
  const year = 31536e6;
  const now = state.date.getTime();
  const newDate = new Date(state.date.getTime());
  while (Math.abs(now - newDate.getTime()) <= year) {
    const map = mapping[mode];
    map.offset(newDate, map[keyCode]);
    if (!(typeof state.disabledDate === "function" && state.disabledDate(newDate))) {
      state.date = newDate;
      emit("pick", newDate, true);
      break;
    }
  }
};
const handleVisibleTimeChange = ({ api, vm, state, t }) => (value) => {
  const time = dateUtil.parseDate(value, state.timeFormat, t);
  if (time && api.checkDateWithinRange(time)) {
    state.date = dateUtil.modifyDate(time, state.year, state.month, state.monthDate);
    state.userInputTime = null;
    if (vm.$refs.timepicker) {
      vm.$refs.timepicker.state.value = state.date;
    }
    state.timePickerVisible = false;
    api.cusEmit(state.date, true);
  }
};
const handleVisibleDateChange = ({ api, state, t }) => (value) => {
  const date2 = dateUtil.parseDate(value, state.dateFormat, t);
  if (date2) {
    if (typeof state.disabledDate === "function" && state.disabledDate(date2)) {
      return;
    }
    state.date = dateUtil.modifyTime(date2, state.date.getHours(), state.date.getMinutes(), state.date.getSeconds());
    state.userInputDate = null;
    api.resetView();
    api.cusEmit(state.date, true);
  }
};
const isValidValue = ({ api, state }) => (value) => value && !isNaN(value) && (typeof state.disabledDate === "function" ? !state.disabledDate(value) : true) && api.checkDateWithinRange(value);
const getDefaultValue = (state) => () => state.defaultValue ? new Date(state.defaultValue) : /* @__PURE__ */ new Date();
const checkDateWithinRange = ({ state }) => (date2) => state.selectableRange.length > 0 ? dateUtil.timeWithinRange(date2, state.selectableRange, state.format || "HH:mm:ss") : true;
const selectTz = ({ emit, state }) => (tz) => {
  if (state.timezone.isServiceTimezone) {
    return;
  }
  state.tz = tz.name;
  state.selectedTz = tz;
  state.showpopup = false;
  state.value = dateToLocaleStringForIE(tz.offset, state.date);
  emit("select-change", { tz, date: state.value });
};
const searchTz = ({ api, state }) => (input) => api.debounceChange(state, input);
const debounceChange = debounce.default(500, (state, input) => {
  if (!input) {
    state.showpopup = false;
    return;
  }
  const tzList = state.renderTzdata.slice();
  let count = 0;
  const filterAry = [];
  state.renderTzdata = tzList.map((tz) => {
    if (tz.name.toLowerCase().includes(input)) {
      tz.visible = false;
      filterAry.push(tz);
      count++;
    } else {
      tz.visible = true;
    }
    return tz;
  });
  if (count === 1) {
    state.tz = filterAry[0].name;
    state.showpopup = false;
  } else {
    state.showpopup = true;
  }
});
const toggleTz = (state) => () => {
  if (state.timezone.isServiceTimezone) {
    return;
  }
  state.renderTzdata = state.renderTzdata && state.renderTzdata.map((item) => {
    item.visible = false;
    return item;
  });
  state.showpopup = true;
};
const handleTzPickClose = (state) => () => state.showpopup = false;
const getRenderTz = ({ state }) => (value) => {
  if (!state.showTimezone || !value) {
    return;
  }
  state.renderTzdata = value[state.lang];
  if (state.renderTzdata) {
    const { isServiceTimezone, to } = state.timezone;
    const selectedTz = state.selectedTz || {};
    const defaultTimezone = selectedTz.tz ? selectedTz.tz : state.defaultTimezone;
    let findTimezoneKey;
    if (to === 0) {
      findTimezoneKey = "";
    } else {
      findTimezoneKey = to > 0 ? "+" : "-";
      findTimezoneKey = findTimezoneKey + string.fillChar(String(Math.abs(to)), 2) + ":00";
    }
    findTimezoneKey = `(UTC${findTimezoneKey})`;
    state.renderTzdata.some((item) => {
      if (~[selectedTz.tz, defaultTimezone].indexOf(item.tz) || isServiceTimezone && ~item.name.indexOf(findTimezoneKey)) {
        state.selectedTz = item;
        state.tz = state.selectedTz.name;
        return true;
      }
      return false;
    }) || (state.selectedTz = void 0);
  }
};
const computerVisibleTime = ({ state, t }) => () => state.userInputTime !== null ? state.userInputTime : dateUtil.formatDate(state.value || state.defaultValue, state.timeFormat, t);
const computerVisibleDate = ({ state, t }) => () => state.userInputDate !== null ? state.userInputDate : dateUtil.formatDate(state.value || state.defaultValue, state.dateFormat, t);
const computerTimeFormat = ({ state }) => () => state.format ? dateUtil.extractTimeFormat(state.timefmt || state.format) : index.DATEPICKER.DateFormats.timerange;
const watchVisible = ({ api, state }) => () => {
  if (state.needChangeTimezoneData) {
    api.getRenderTz(state.timezoneData);
    state.needChangeTimezoneData = false;
  }
};
exports.changeToNow = changeToNow;
exports.checkDateWithinRange = checkDateWithinRange;
exports.computerTimeFormat = computerTimeFormat;
exports.computerVisibleDate = computerVisibleDate;
exports.computerVisibleTime = computerVisibleTime;
exports.confirm = confirm;
exports.cusEmit = cusEmit;
exports.cusNextMonth = cusNextMonth;
exports.cusNextYear = cusNextYear;
exports.cusPrevMonth = cusPrevMonth;
exports.cusPrevYear = cusPrevYear;
exports.debounceChange = debounceChange;
exports.doPick = doPick;
exports.getDefaultValue = getDefaultValue;
exports.getDisabledConfirm = getDisabledConfirm;
exports.getDisabledNow = getDisabledNow;
exports.getNowTime = getNowTime;
exports.getRenderTz = getRenderTz;
exports.getYearLabel = getYearLabel;
exports.handleClear = handleClear;
exports.handleDatePick = handleDatePick;
exports.handleEnter = handleEnter;
exports.handleKeyControl = handleKeyControl;
exports.handleKeydown = handleKeydown;
exports.handleLeave = handleLeave;
exports.handleMonthPick = handleMonthPick;
exports.handleShortcutClick = handleShortcutClick;
exports.handleTimePick = handleTimePick;
exports.handleTimePickClose = handleTimePickClose;
exports.handleTzPickClose = handleTzPickClose;
exports.handleVisibleDateChange = handleVisibleDateChange;
exports.handleVisibleTimeChange = handleVisibleTimeChange;
exports.handleYearPick = handleYearPick;
exports.isValidValue = isValidValue;
exports.proxyTimePickerDataProperties = proxyTimePickerDataProperties;
exports.resetView = resetView;
exports.searchTz = searchTz;
exports.selectTz = selectTz;
exports.showMonthPicker = showMonthPicker;
exports.showYearPicker = showYearPicker;
exports.toggleTz = toggleTz;
exports.watchDefaultValue = watchDefaultValue;
exports.watchSelectionMode = watchSelectionMode;
exports.watchTimePickerVisible = watchTimePickerVisible;
exports.watchValue = watchValue;
exports.watchVisible = watchVisible;
