"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index$1 = require("./index.js");
const dateUtil = require("../common/deps/date-util.js");
const index = require("../common/index.js");
const api = [
  "state",
  "isValidValue",
  "handleTzPickClose",
  "toggleTz",
  "searchTz",
  "selectTz",
  "confirm",
  "cusPrevYear",
  "cusNextYear",
  "cusPrevMonth",
  "cusNextMonth",
  "changeToNow",
  "handleMonthPick",
  "handleYearPick",
  "handleDatePick",
  "showMonthPicker",
  "showYearPicker",
  "handleTimePick",
  "handleEnter",
  "handleVisibleTimeChange",
  "handleVisibleDateChange",
  "handleLeave",
  "handleShortcutClick",
  "handleTimePickClose",
  "getNowTime"
];
const initState = ({ reactive, computed, api: api2, i18n, designConfig }) => {
  var _a, _b;
  const state = reactive({
    popperClass: "",
    date: /* @__PURE__ */ new Date(),
    value: "",
    defaultValue: null,
    defaultTime: null,
    showTime: false,
    selectionMode: index.DATEPICKER.Day,
    shortcuts: "",
    visible: false,
    currentView: index.DATEPICKER.Date,
    disabledDate: "",
    cellClassName: "",
    selectableRange: [],
    firstDayOfWeek: 7,
    showWeekNumber: false,
    timePickerVisible: false,
    format: "",
    timeFmt: "",
    timezone: {},
    arrowControl: false,
    userInputDate: null,
    userInputTime: null,
    showTimezone: false,
    showpopup: false,
    tz: "",
    selectedTz: null,
    animationName: "",
    startYear: Math.floor((/* @__PURE__ */ new Date()).getFullYear() / 10) * 10,
    disabledNow: computed(() => api2.getDisabledNow()),
    disabledConfirm: computed(() => api2.getDisabledConfirm()),
    year: computed(() => !Array.isArray(state.date) && state.date.getFullYear()),
    month: computed(() => !Array.isArray(state.date) && state.date.getMonth()),
    week: computed(() => dateUtil.getWeekNumber(state.date)),
    monthDate: computed(() => state.date.getDate()),
    footerVisible: computed(() => state.showTime || [index.DATEPICKER.Dates, index.DATEPICKER.Years].includes(state.selectionMode)),
    visibleTime: computed(() => api2.computerVisibleTime()),
    visibleDate: computed(() => api2.computerVisibleDate()),
    yearLabel: computed(() => api2.getYearLabel()),
    timeFormat: computed(() => api2.computerTimeFormat()),
    dateFormat: computed(() => state.format ? dateUtil.extractDateFormat(state.format.replace(state.timefmt, "")) : index.DATE.Date),
    lang: computed(() => i18n ? i18n.locale.replace(/_/g, "-") : "zh-CN"),
    isShowTz: computed(() => state.showTimezone && state.showTime),
    isShowFooter: computed(() => state.footerVisible && [index.DATEPICKER.Date, index.DATEPICKER.Year].includes(state.currentView)),
    buttonType: ((_a = designConfig == null ? void 0 : designConfig.state) == null ? void 0 : _a.buttonType) || "default",
    buttonSize: ((_b = designConfig == null ? void 0 : designConfig.state) == null ? void 0 : _b.buttonSize) || ""
  });
  state.needChangeTimezoneData = true;
  return state;
};
const initWatch = ({ watch, state, api: api2, nextTick }) => {
  watch(
    () => state.isShowTz,
    () => {
      setTimeout(() => state.fnUpdatePopper && state.fnUpdatePopper());
    }
  );
  watch(
    () => state.isShowFooter,
    () => {
      setTimeout(() => state.fnUpdatePopper && state.fnUpdatePopper());
    }
  );
  watch(
    () => state.lang,
    () => state.needChangeTimezoneData = true
  );
  watch(() => state.defaultValue, api2.watchDefaultValue);
  watch(() => state.value, api2.watchValue);
  watch(() => state.timePickerVisible, api2.watchTimePickerVisible);
  watch(() => state.selectionMode, api2.watchSelectionMode);
  watch(
    () => state.timePickerVisible,
    () => nextTick(api2.proxyTimePickerDataProperties)
  );
  watch(() => state.visible, api2.watchVisible);
};
const initApi = ({ api: api2, state, t, emit, nextTick, vm, watch, props }) => {
  Object.assign(api2, {
    t,
    state,
    debounceChange: index$1.debounceChange,
    toggleTz: index$1.toggleTz(state),
    watchVisible: index$1.watchVisible({ api: api2, state }),
    getRenderTz: index$1.getRenderTz({ state }),
    selectTz: index$1.selectTz({ emit, state }),
    handleTzPickClose: index$1.handleTzPickClose(state),
    getDefaultValue: index$1.getDefaultValue(state),
    showYearPicker: index$1.showYearPicker({ state }),
    handleTimePickClose: index$1.handleTimePickClose(state),
    cusNextMonth: index$1.cusNextMonth({ state }),
    cusPrevMonth: index$1.cusPrevMonth({ state }),
    resetView: index$1.resetView({ state }),
    showMonthPicker: index$1.showMonthPicker({ state }),
    cusNextYear: index$1.cusNextYear({ state }),
    cusPrevYear: index$1.cusPrevYear({ state }),
    watchDefaultValue: index$1.watchDefaultValue({ state }),
    getYearLabel: index$1.getYearLabel({ state, t }),
    cusEmit: index$1.cusEmit({ state, emit }),
    watchTimePickerVisible: index$1.watchTimePickerVisible({ nextTick, vm }),
    checkDateWithinRange: index$1.checkDateWithinRange({ state }),
    watchSelectionMode: index$1.watchSelectionMode({ state }),
    proxyTimePickerDataProperties: index$1.proxyTimePickerDataProperties({ vm, state, watch }),
    handleKeyControl: index$1.handleKeyControl({ state, emit }),
    doPick: index$1.doPick(emit),
    searchTz: index$1.searchTz({ api: api2, state }),
    handleEnter: index$1.handleEnter(api2),
    handleLeave: index$1.handleLeave({ api: api2, emit }),
    changeToNow: index$1.changeToNow({ api: api2, state, props }),
    isValidValue: index$1.isValidValue({ api: api2, state }),
    handleClear: index$1.handleClear({ api: api2, state, emit }),
    watchValue: index$1.watchValue({ api: api2, state }),
    handleKeydown: index$1.handleKeydown({ api: api2, state }),
    confirm: index$1.confirm({ api: api2, state, t }),
    handleMonthPick: index$1.handleMonthPick({ api: api2, state }),
    handleVisibleDateChange: index$1.handleVisibleDateChange({ api: api2, state, t }),
    handleTimePick: index$1.handleTimePick({ api: api2, state, t }),
    handleYearPick: index$1.handleYearPick({ api: api2, state }),
    handleDatePick: index$1.handleDatePick({ api: api2, state, t }),
    computerVisibleTime: index$1.computerVisibleTime({ state, t }),
    handleShortcutClick: index$1.handleShortcutClick(api2),
    computerVisibleDate: index$1.computerVisibleDate({ state, t }),
    handleVisibleTimeChange: index$1.handleVisibleTimeChange({ api: api2, vm, state, t }),
    computerTimeFormat: index$1.computerTimeFormat({ state }),
    getDisabledNow: index$1.getDisabledNow({ state }),
    getDisabledConfirm: index$1.getDisabledConfirm({ state }),
    getNowTime: index$1.getNowTime({ props })
  });
};
const renderless = (props, { computed, reactive, watch, nextTick }, { t, emit: $emit, vm, i18n, designConfig }) => {
  const api2 = {};
  const emit = props.emitter ? props.emitter.emit : $emit;
  const state = initState({ reactive, computed, api: api2, i18n, designConfig });
  initApi({ api: api2, state, t, emit, nextTick, vm, watch, props });
  initWatch({ watch, state, api: api2, nextTick });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
