"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const yearMonth = require("./year-month.js");
const api = [
  "state",
  "getDate",
  "getWeeksByMonth",
  "getDaysByWeek",
  "getSelectedPosition",
  "selectOption",
  "confirm",
  "timeToggle",
  "timeConfirm",
  "scrollStart",
  "scrollEnd",
  "loadingDate",
  "clear",
  "close",
  "formatDate"
];
const renderless = (props, { computed, reactive, watch, onMounted }, { emit, vm, nextTick, constants }) => {
  const { DATE_RANGE, DATE_TIME_RANGE, YEAR_MONTH_RANGE, YEAR_MONTH } = constants.TYPE;
  const api2 = {};
  const state = reactive({
    scrollerHeight: window.innerHeight || 1e3,
    toggle: false,
    date: null,
    loading: false,
    ready: false,
    dateList: {},
    timeVisible: false,
    showTimeIndex: 0,
    time: ["00", "00", "00"],
    timeList: [
      ["00", "00", "00"],
      ["00", "00", "00"]
    ],
    years: [],
    isYearMonthPanel: computed(() => [YEAR_MONTH_RANGE, YEAR_MONTH].includes(props.type)),
    yearNum: computed(() => [YEAR_MONTH, YEAR_MONTH_RANGE].includes(props.type) ? 40 : 10),
    itemSize: computed(() => [YEAR_MONTH, YEAR_MONTH_RANGE].includes(props.type) ? 150 : 300),
    buffer: 200,
    selected: computed(() => api2.selectedComputed()),
    months: computed(() => Object.keys(state.dateList).map((item) => ({ id: item, yearMonth: item }))),
    btnDisabled: computed(
      () => [DATE_RANGE, DATE_TIME_RANGE, YEAR_MONTH_RANGE].includes(props.type) && state.date && state.date.length === 1
    ),
    computedYears: computed(() => Object.keys(state.years).map((item) => ({ id: item, year: item })))
  });
  Object.assign(api2, {
    state,
    initPanel: index.initPanel({ state, api: api2 }),
    initYearMonthPanel: yearMonth.initYearMonthPanel({ state, props }),
    getCurrentDate: index.getCurrentDate({ api: api2, props }),
    getWeeksByMonth: index.getWeeksByMonth({ state }),
    getDate: index.getDate({ state }),
    getDaysByWeek: index.getDaysByWeek({ state, api: api2 }),
    getSelectedPosition: index.getSelectedPosition({ state, api: api2 }),
    confirm: index.confirm({ state, emit, props, api: api2, constants }),
    timeConfirm: index.timeConfirm({ state, emit }),
    scrollStart: index.scrollStart({ state, api: api2, props }),
    scrollEnd: index.scrollEnd({ state, api: api2 }),
    selectOption: index.selectOption({ state, emit, props, constants }),
    timeToggle: index.timeToggle({ state }),
    watchVisible: index.watchVisible({ emit, api: api2, state }),
    scrollToCurrentDate: index.scrollToCurrentDate({ state, vm, nextTick }),
    watchModelValue: index.watchModelValue({ props, state, constants }),
    selectedComputed: index.selectedComputed({ props, state, constants, api: api2 }),
    loadingDate: index.loadingDate({ state, api: api2 }),
    loadYearMonth: yearMonth.loadYearMonth({ state, api: api2 }),
    getOffsetMonth: yearMonth.getOffsetMonth(),
    formatDate: index.formatDate({ props, constants }),
    clear: index.clear({ state, emit, api: api2 }),
    close: index.close({ emit, vm })
  });
  watch(() => props.visible, api2.watchVisible);
  watch(() => props.modelValue, api2.watchModelValue, { immediate: true });
  onMounted(() => {
    state.isYearMonthPanel ? api2.initYearMonthPanel({ isInit: true }) : api2.initPanel({ isInit: true });
    props.visible && api2.scrollToCurrentDate({ date: (Array.isArray(state.date) ? state.date[0] : state.date) || /* @__PURE__ */ new Date() });
    nextTick(() => {
      state.ready = true;
    });
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
