"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const chunkG2ADBYYC = require("../chunk-G2ADBYYC.js");
const index = require("./index.js");
const dateUtil = require("../common/deps/date-util.js");
const api = [
  "state",
  "handleShortcutClick",
  "handleConfirm",
  "handleClear",
  "handleChangeRange",
  "rightNextMonth",
  "rightNextYear",
  "rightPrevMonth",
  "rightPrevYear",
  "handleRangePick",
  "leftNextMonth",
  "leftNextYear",
  "leftPrevMonth",
  "leftPrevYear",
  "handleMaxTimePick",
  "handleDateInput",
  "handleMinTimePick",
  "handleTimeChange",
  "handleTimeInput",
  "handleMinTimeClose",
  "handleDateChange",
  "handleMaxTimeClose",
  "isValidValue"
];
const initState = ({ reactive, computed, api: api2, constants, designConfig }) => {
  var _a;
  const state = reactive({
    popperElm: null,
    popperClass: "",
    value: [],
    defaultValue: null,
    defaultTime: null,
    minDate: "",
    maxDate: "",
    shortcutType: "",
    shortcutText: "",
    singleSelect: false,
    minRangeDate: constants.startDate,
    maxRangeDate: constants.endDate,
    leftDate: /* @__PURE__ */ new Date(),
    rightDate: dateUtil.nextMonth(/* @__PURE__ */ new Date()),
    rangeState: { endDate: null, selecting: false, row: null, column: null },
    showTime: false,
    format: "",
    arrowControl: false,
    unlinkPanels: false,
    firstDayOfWeek: 7,
    minTimePickerVisible: false,
    maxTimePickerVisible: false,
    shortcuts: "",
    visible: "",
    disabledDate: "",
    cellClassName: "",
    dateUserInput: { min: null, max: null },
    timeUserInput: { min: null, max: null },
    btnDisabled: computed(() => api2.computerBtnDisabled()),
    leftLabel: computed(() => api2.computerLabel("leftDate")),
    rightLabel: computed(() => api2.computerLabel("rightDate")),
    leftYear: computed(() => state.leftDate.getFullYear()),
    leftMonth: computed(() => state.leftDate.getMonth()),
    leftMonthDate: computed(() => state.leftDate.getDate()),
    rightYear: computed(() => state.rightDate.getFullYear()),
    rightMonth: computed(() => state.rightDate.getMonth()),
    rightMonthDate: computed(() => state.rightDate.getDate()),
    minVisibleDate: computed(() => api2.getMinVisibleDate()),
    maxVisibleDate: computed(() => api2.getMaxVisibleDate()),
    minVisibleTime: computed(() => api2.getMinVisibleTime()),
    maxVisibleTime: computed(() => api2.getMaxVisibleTime()),
    timeFormat: computed(() => state.format ? dateUtil.extractTimeFormat(state.format) : "HH:mm:ss"),
    dateFormat: computed(() => state.format ? dateUtil.extractDateFormat(state.format) : "yyyy-MM-dd"),
    enableMonthArrow: computed(() => api2.getEnableMonthArrow()),
    enableYearArrow: computed(() => api2.computerEnableYearArrow()),
    // tiny 新增
    confirmButtonProps: chunkG2ADBYYC.__spreadValues({
      plain: true,
      type: "default"
    }, (_a = designConfig == null ? void 0 : designConfig.state) == null ? void 0 : _a.confirmButtonProps)
  });
  return state;
};
const initWatch = ({ watch, state, api: api2 }) => {
  watch(() => state.minDate, api2.watchMinDate);
  watch(() => state.maxDate, api2.watchMaxDate);
  watch(() => state.minTimePickerVisible, api2.watchMinTimePickerVisible);
  watch(() => state.maxTimePickerVisible, api2.watchMaxTimePickerVisible);
  watch(() => state.value, api2.watchValue);
  watch(() => state.defaultValue, api2.watchDefault);
  watch(() => state.showTime, api2.setTimeFormat);
  watch(() => state.visible, api2.watchPickerVisible);
};
const initApi = ({ api: api2, state, t, vm, nextTick, emit, constants }) => {
  Object.assign(api2, {
    t,
    state,
    computerLabel: index.computerLabel({ state, t }),
    resetView: index.resetView({ state }),
    watchMaxDate: index.watchMaxDate({ state, vm }),
    handleChangeRange: index.handleChangeRange(state),
    handleMaxTimeClose: index.handleMaxTimeClose(state),
    handleMinTimeClose: index.handleMinTimeClose(state),
    isValidValue: index.isValidValue({ state }),
    leftNextYear: index.leftNextYear({ state }),
    getEnableMonthArrow: index.getEnableMonthArrow(state),
    leftNextMonth: index.leftNextMonth({ state }),
    rightPrevYear: index.rightPrevYear({ state }),
    rightPrevMonth: index.rightPrevMonth({ state }),
    rightNextMonth: index.rightNextMonth({ state }),
    leftPrevYear: index.leftPrevYear({ state }),
    handleMaxTimePick: index.handleMaxTimePick({ state }),
    handleMinTimePick: index.handleMinTimePick({ state }),
    getMaxVisibleTime: index.getMaxVisibleTime({ state, t }),
    getMinVisibleTime: index.getMinVisibleTime({ state, t }),
    getMaxVisibleDate: index.getMaxVisibleDate({ state, t }),
    getMinVisibleDate: index.getMinVisibleDate({ state, t }),
    rightNextYear: index.rightNextYear({ state }),
    leftPrevMonth: index.leftPrevMonth({ state }),
    handleDateChange: index.handleDateChange({ state, t }),
    handleTimeChange: index.handleTimeChange({ state, t, vm }),
    watchMinDate: index.watchMinDate({ state, t, vm }),
    handleTimeInput: index.handleTimeInput({ state, t }),
    watchMaxTimePickerVisible: index.watchMaxTimePickerVisible({ nextTick, state, vm }),
    watchMinTimePickerVisible: index.watchMinTimePickerVisible({ nextTick, state, vm }),
    handleDateInput: index.handleDateInput({ state, t }),
    doPick: index.doPick(emit),
    watchValue: index.watchValue({ state }),
    watchDefault: index.watchDefault({ state }),
    handleClear: index.handleClear({ emit, state }),
    setTimeFormat: index.setTimeFormat({ nextTick, vm, state }),
    handleConfirm: index.handleConfirm({ api: api2, emit, state }),
    handleRangePick: index.handleRangePick({ api: api2, state, t }),
    handleShortcutClick: index.handleShortcutClick(state, api2),
    computerBtnDisabled: index.computerBtnDisabled({ state, api: api2 }),
    computerEnableYearArrow: index.computerEnableYearArrow(state),
    watchPickerVisible: index.watchPickerVisible({ state, constants })
  });
};
const renderless = (props, { computed, reactive, watch, nextTick }, { t, emit: $emit, vm, constants, designConfig }) => {
  const api2 = {};
  const emit = props.emitter ? props.emitter.emit : $emit;
  const state = initState({ reactive, computed, api: api2, constants, designConfig });
  initApi({ api: api2, state, t, vm, nextTick, emit, constants });
  initWatch({ watch, state, api: api2 });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
