"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const dateUtil = require("../common/deps/date-util.js");
const api = ["state", "getCellClasses", "getCssToken", "isWeekActive", "handleMouseMove", "handleClick"];
const initState = ({ reactive, computed, api: api2, props }) => {
  const state = reactive({
    tableRows: [[], [], [], [], [], []],
    lastRow: null,
    lastColumn: null,
    constWeeks: ["sun", "mon", "tue", "wed", "thu", "fri", "sat"],
    rows: computed(() => api2.getRows()),
    weeks: computed(() => api2.getWeeks()),
    month: computed(() => !Array.isArray(props.date) && props.date.getMonth()),
    offsetDay: computed(() => api2.getOffsetDay()),
    year: computed(() => !Array.isArray(props.date) && props.date.getFullYear()),
    startDate: computed(() => dateUtil.getStartDateOfMonth(state.year, state.month, state.offsetDay)),
    date: props.value
  });
  return state;
};
const initApi = ({ api: api2, state, props, emit, t, vm }) => {
  Object.assign(api2, {
    t,
    state,
    getOffsetDay: index.getOffsetDay(props),
    getWeeks: index.getWeeks({ props, state }),
    markRange: index.markRange({ props, state }),
    cellMatchesDate: index.cellMatchesDate(state),
    getDateOfCell: index.getDateOfCell({ props, state }),
    getCell: index.getCell({ state, props }),
    isWeekActive: index.isWeekActive({ props, state }),
    watchMaxDate: index.watchMaxDate({ api: api2, props }),
    watchMinDate: index.watchMinDate({ api: api2, props }),
    handleMouseMove: index.handleMouseMove({ api: api2, emit, props, state }),
    getRows: index.getRows({ api: api2, props, state, t, vm }),
    handleClick: index.handleClick({ api: api2, emit, props, state }),
    getCellClasses: index.getCellClasses({ api: api2, props, state }),
    getCssToken: index.getCssToken({ api: api2 }),
    getCellRangeClasses: index.getCellRangeClasses({ props })
  });
};
const initWatch = ({ watch, props, api: api2 }) => {
  watch(
    () => props.rangeState,
    (value) => api2.markRange(props.minDate, value.endDate),
    { deep: true }
  );
  watch(() => props.minDate, api2.watchMinDate);
  watch(() => props.maxDate, api2.watchMaxDate);
};
const renderless = (props, { computed, reactive, watch }, { t, vm, emit }) => {
  const api2 = {};
  const state = initState({ reactive, computed, api: api2, props });
  initApi({ api: api2, state, props, emit, t, vm });
  initWatch({ watch, props, api: api2 });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
