"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = [
  "select",
  "state",
  "deptState",
  "openDialog",
  "getCurrentList",
  "confirm",
  "searchMethod",
  "searchChange",
  "selectChange",
  "closeDialog",
  "cancel"
];
const initState = ({ reactive, computed, parent, props, inject, lastDeptState }) => {
  const state = reactive({
    lastDeptState,
    lastLabels: [],
    labels: [],
    display: "",
    open: false,
    loading: false,
    selectChanged: false,
    searchValue: "",
    searchOptions: [],
    current: "",
    formDisabled: computed(() => (parent.tinyForm || {}).disabled),
    disabled: props.disabled,
    autoSelect: props.autoSelect,
    hrapprover: inject("hrapprover", null)
  });
  return state;
};
const initApi = ({ api: api2, state, deptState, emit, $service, props }) => {
  Object.assign(api2, {
    state,
    deptState,
    closeDialog: index.closeDialog({ emit, state, deptState }),
    resetDeptState: index.resetDeptState(deptState),
    fetchDept: $service.fetchDept,
    fetchDeptList: $service.fetchDeptList,
    fetchDeptByValue: $service.fetchDeptByValue,
    setSelectData: index.setSelectData({ deptState, state }),
    cancel: index.cancel({ emit, state, deptState }),
    doConfirm: index.doConfirm({ emit, props, state, deptState }),
    searchChange: index.searchChange(api2),
    searchMethod: index.searchMethod({ api: api2, state }),
    openDialog: index.openDialog({ api: api2, emit, props, state }),
    selectChange: index.selectChange({ api: api2, emit, state, deptState }),
    setDefaultSearch: index.setDefaultSearch({ api: api2, state }),
    getCurrentList: index.getCurrentList({ api: api2, deptState }),
    confirm: index.confirm({ api: api2, props, state }),
    getDeptList: index.getDeptList({ api: api2, deptState, state }),
    getDisplay: index.getDisplay({ api: api2, deptState, state }),
    setDeptState: index.setDeptState({ api: api2, state, deptState })
  });
};
const renderless = (props, { onBeforeUnmount, reactive, watch, inject, computed }, { service, emit, parent }) => {
  parent.tinyForm = parent.tinyForm || inject("form", null);
  const api2 = {};
  const $service = index.initService({ props, service });
  const deptLevels = ["company", "dept1", "dept2", "dept3", "dept4", "dept5", "dept6", "dept7", "dept8"];
  const deptState = deptLevels.map((title) => reactive({ title, options: [], value: "", disable: true }));
  const lastDeptState = deptLevels.map((title) => reactive({ title, options: [], value: "", disable: true }));
  const state = initState({ reactive, computed, parent, props, inject, lastDeptState });
  initApi({ api: api2, state, deptState, emit, $service, props });
  watch(() => props.modelValue, api2.getDisplay, { immediate: true });
  watch(
    () => state.open,
    (val) => {
      if (!val) {
        state.selectChanged = false;
      }
    }
  );
  onBeforeUnmount(() => {
    state.open = false;
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
