"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index$1 = require("../upload-list/index.js");
const string = require("../common/string.js");
const index = require("./index.js");
const type = require("../common/type.js");
const api = [
  "state",
  "getNewTabPreviewUrl",
  "previewFile",
  "downloadFile",
  "abort",
  "clearFiles",
  "getFile",
  "handleStart",
  "handleProgress",
  "handleSuccess",
  "handleError",
  "handleRemove",
  "handleReUpload",
  "handleReUploadTotal",
  "submit",
  "handleClick",
  "handleFileClick",
  "getFileUploadUrl",
  "updateUrl",
  "previewImage",
  "updateFile",
  "handleChange",
  "abortDownload",
  "handleClickFileList",
  "handleTriggerClick",
  "closeRecordPanel",
  "encryptDialogConfirm",
  "formatFileSize",
  "getTipMessage"
];
const initState = ({
  api: api2,
  reactive,
  computed,
  inject,
  ref,
  vm,
  props,
  httpRequest,
  service,
  useBreakpoint
}) => {
  const { current } = useBreakpoint();
  const state = reactive({
    url: "",
    updateId: "",
    currentDownloadFiles: "",
    tempIndex: 1,
    draging: false,
    uploadFiles: [],
    dragOver: false,
    httpRequest,
    form: inject("form", ref({ default: "" })),
    listeners: vm.$listeners,
    docSize: 0,
    // unit(B)
    chunkSize: 0,
    chunkBatchLimit: 5,
    downloadChunkLimit: 5,
    batchQueue: {},
    batchQueueListen: {},
    batchCancelToken: {},
    replayAtoms: {},
    chunkUploadUrl: "",
    largeFileInfo: {},
    headers: {},
    accept: "",
    edmToken: {},
    isSuccess: false,
    singleMaxSize: 200,
    formData: {},
    showPreview: false,
    iframeUrl: "",
    fileWater: false,
    tabUrl: "",
    cacheDocuments: {},
    isEdm: computed(() => !type.isEmptyObject(props.edm)),
    uploadDisabled: computed(() => api2.computedUploadDisabled()),
    dialogConfigObj: computed(() => api2.getDialogConfigObj()),
    uploadingFiles: [],
    currentUploadingFileUids: [],
    uploadingSize: computed(() => api2.computedUploadingSize()),
    isEntireCheckCode: computed(() => !("isEntireCheckCode" in props.edm && props.edm.isEntireCheckCode !== true)),
    downloadBatchQueue: {},
    downloadBatchQueueListen: {},
    downloadChunkFile: {},
    downloadReplayAtoms: {},
    errorStatusCodes: [0, 401, 429],
    // 0：上传异常 401：没权限（token过期）429：超限
    hasFileInfoInterface: computed(() => service.setting.services.EDM && service.setting.services.EDM.DocumentInfoUrl),
    currentDownloadFile: "",
    isDragover: false,
    downloadCancelToken: {},
    // 取消下载token
    downloadCancelData: {},
    // 取消下载时需要清空的缓存数据
    isHwh5: computed(() => !type.isEmptyObject(props.hwh5)),
    selected: null,
    types: computed(() => api2.computedSourcetype()),
    triggerClickType: "",
    visible: false,
    downloadParamsWhitelist: ["docId", "wmType", "docVersion"],
    encryptDialogConfig: {
      show: false,
      selectFileMethod: null
    },
    current
  });
  return state;
};
const initApi = ({ api: api2, state, props, constants, vm, $service, t, Modal, emit }) => {
  Object.assign(api2, {
    state,
    sliceChunk: index.sliceChunk({ state }),
    getFormData: index.getFormData({ constants, props, state }),
    abort: index.abort({ constants, vm, state }),
    handleClick: index.handleClick({ constants, vm }),
    handleFileClick: index.handleFileClick({ props, emit }),
    getFile: index.getFile(state),
    clearFiles: index.clearFiles(state),
    watchFileList: index.watchFileList({ constants, state, props, api: api2 }),
    watchListType: index.watchListType({ constants, state, api: api2 }),
    onBeforeDestroy: index.onBeforeDestroy(state),
    computedUploadDisabled: index.computedUploadDisabled({ props, state }),
    computedUploadingSize: index.computedUploadingSize({ state, constants }),
    getFileUploadUrl: index.getFileUploadUrl($service),
    getToken: index.getToken({ constants, props, state, t, Modal }),
    getDialogConfigObj: index.getDialogConfigObj({ props, state, t, constants }),
    computeDocChunkSize: index.computeDocChunkSize({ props, state, constants }),
    updateFile: index.updateFile({ constants, vm }),
    getPreviewUrlSync: index.getPreviewUrlSync({ constants, props, state }),
    ordinaryDownload: index$1.downloadFile($service),
    clearUploadingFiles: index.clearUploadingFiles({ constants, state }),
    calcUploadingFilesInfo: index.calcUploadingFilesInfo({ state, constants }),
    properFileSize: index.properFileSize({ props, state, api: api2, constants, Modal, t }),
    mounted: index.mounted({ vm, state }),
    previewFileSingle: index.previewFileSingle({ api: api2, state, props, constants, service: $service }),
    previewFileBatch: index.previewFileBatch({ service: $service, props, state, api: api2 }),
    previewImageSingle: index.previewImageSingle({ state, props, service: $service }),
    previewImageBatch: index.previewImageBatch({ service: $service, api: api2 }),
    abortDownload: index.abortDownload({ state }),
    createDownloadCancelToken: index.createDownloadCancelToken({ state, service: $service }),
    computedSourcetype: index.computedSourcetype({ props, constants }),
    getFileSourceType: index.getFileSourceType({ state, props, constants }),
    encryptDialogConfirm: index.encryptDialogConfirm({ state }),
    getTipMessage: index.getTipMessage({ t, api: api2, constants }),
    formatFileSize: string.formatFileSize
  });
};
const mergeApi = ({ api: api2, props, $service, state, constants, emit, mode, Modal, t, vm, Streamsaver }) => {
  Object.assign(api2, {
    segmentUploadInit: index.segmentUploadInit({ api: api2, props, service: $service, state, constants }),
    segmentUpload: index.segmentUpload({ api: api2, props, service: $service, state, emit, constants }),
    addFileToList: index.addFileToList({ api: api2, constants, emit, props, state, mode }),
    downloadFile: index.downloadFile({ api: api2, state }),
    downloadFileSingleInner: index.downloadFileSingleInner({ props, state, api: api2, constants }),
    previewImage: index.previewImage({ api: api2, props, service: $service }),
    previewFile: index.previewFile({ api: api2, props }),
    getNewTabPreviewUrl: index.getNewTabPreviewUrl({ api: api2 }),
    submit: index.submit({ api: api2, constants, vm, props, state }),
    handleStart: index.handleStart({ api: api2, constants, props, state, vm }),
    batchSegmentUpload: index.batchSegmentUpload({ api: api2, constants, props, vm, state }),
    largeDocumentUpload: index.largeDocumentUpload({ api: api2, Modal, state, emit, constants, t }),
    handleProgress: index.handleProgress({ api: api2, constants, emit, state }),
    handleSuccess: index.handleSuccess({ api: api2, constants, emit, Modal, props, state, t }),
    handleError: index.handleError({ api: api2, constants, emit, state, props }),
    handleRemove: index.handleRemove({ api: api2, emit, props, state, constants }),
    handleReUpload: index.handleReUpload({ vm, constants }),
    handleReUploadTotal: index.handleReUploadTotal(api2),
    updateUrl: index.updateUrl({ api: api2, props, state }),
    startUpload: index.startUpload({ api: api2, state, constants, vm, Modal, t }),
    beforeUpload: index.beforeUpload({ api: api2, props, Modal, constants, t, state }),
    getDownloadFileInfo: index.getDownloadFileInfo({ api: api2, props, state, service: $service }),
    largeDocumentDownload: index.largeDocumentDownload({ api: api2, state }),
    sliceDownloadChunk: index.sliceDownloadChunk({ state }),
    batchSegmentDownload: index.batchSegmentDownload({ state, api: api2 }),
    downloadFileInner: index.downloadFileInner({ api: api2, props, state }),
    setWriterFile: index.setWriterFile({ state, emit, Streamsaver }),
    afterDownload: index.afterDownload({ api: api2, state }),
    getFileHash: index.getFileHash({ emit, Modal, constants, t, state }),
    modifyServiceUrlSingle: index.modifyServiceUrlSingle({ state, props, constants }),
    getKiaScanTip: index.getKiaScanTip({ Modal, constants, t }),
    downloadFileSingle: index.downloadFileSingle({ service: $service, constants, props, state, api: api2, emit }),
    downloadFileBatch: index.downloadFileBatch({ api: api2, service: $service, props, state, emit }),
    downloadFileSingleHwh5: index.downloadFileSingleHwh5({ state, props, emit, constants }),
    downloadAsyncPackage: index.downloadAsyncPackage({ state, props, api: api2, constants, service: $service }),
    validateDownloadStatus: index.validateDownloadStatus({ state, Modal }),
    handleChange: index.handleChange({ vm, constants }),
    handleClickFileList: index.handleClickFileList({ state, emit }),
    handleTriggerClick: index.handleTriggerClick({ vm, state, constants, props, emit }),
    closeRecordPanel: index.closeRecordPanel({ vm, constants, state, props })
  });
};
const initWatch = ({ watch, state, api: api2, props, $service }) => {
  watch(
    () => {
      var _a;
      return (_a = props.edm) == null ? void 0 : _a.upload;
    },
    (value) => value && api2.getToken({ token: value.token, isinit: true }),
    { immediate: true, deep: true }
  );
  watch(() => props.listType, api2.watchListType);
  watch(
    () => props.fileList,
    (value) => api2.watchFileList(value),
    { immediate: true, deep: true }
  );
  watch(
    () => props.action,
    () => {
      if (!props.httpRequest && !state.isEdm) {
        api2.updateUrl();
      }
    },
    { immediate: true }
  );
  watch(
    () => state.isSuccess,
    (value) => value && $service.getSingleUploadUrl().then((url) => state.url = url),
    { immediate: true }
  );
  watch(() => props.edm, api2.computeDocChunkSize, { deep: true, immediate: true });
};
exports.getApi = () => ({});
const renderless = (props, { computed, inject, onBeforeUnmount, provide, reactive, ref, watch, onMounted }, { t, vm, parent, emit, service, mode, constants, useBreakpoint }, { Modal, Streamsaver }) => {
  let api2 = {};
  const $service = index.initService({ props, service });
  const httpRequest = $service.httpRequest;
  const state = initState({
    reactive,
    computed,
    api: api2,
    inject,
    ref,
    vm,
    props,
    httpRequest,
    service,
    useBreakpoint
  });
  initApi({ api: api2, state, props, constants, vm, $service, t, Modal, emit });
  mergeApi({ api: api2, props, $service, state, constants, emit, mode, Modal, t, vm, Streamsaver });
  exports.getApi = () => api2;
  provide("uploader", parent);
  onMounted(api2.mounted);
  onBeforeUnmount(() => {
    api2.onBeforeDestroy();
    api2 = null;
    vm.$off("drag-over");
  });
  initWatch({ watch, state, api: api2, props, $service });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
