"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const options = require("./options.js");
const api = ["state", "init", "initContent", "selectionChange", "textChange", "doPreview", "handleDblclick"];
const initState = ({ api: api2, reactive, computed, props }) => {
  const state = reactive({
    promises: [],
    promisesData: [],
    cbNum: 0,
    innerOptions: {},
    innerContent: "",
    fileUploadUrl: props.fileUpload && props.fileUpload.url || "",
    quill: null,
    fileInput: null,
    previewOptions: computed(() => api2.computePreviewOptions()),
    previewImgUrl: "",
    showPreview: false,
    content: props.modelValue || props.content,
    isFullscreen: false,
    zIndex: computed(() => api2.computeZIndex()),
    icons: [
      "IconEditorAlignCenter",
      "IconEditorAlignLeft",
      "IconEditorAlignRight",
      "IconEditorBackground",
      "IconEditorBold",
      "IconEditorCode",
      "IconEditorDeleteline",
      "IconEditorEraser",
      "IconEditorItalic",
      "IconEditorLeftBorder",
      "IconEditorList",
      "IconEditorListDot",
      "IconEditorListNum",
      "IconEditorMenuLeft",
      "IconEditorMenuRight",
      "IconEditorQuote",
      "IconEditorRedo",
      "IconEditorRightBorder",
      "IconEditorSub",
      "IconEditorSubtitle",
      "IconEditorSuper",
      "IconEditorTable",
      "IconEditorTextcolor",
      "IconEditorTitle",
      "IconEditorUnderline",
      "IconEditorUndo",
      "IconEditorVideo",
      "IconLink",
      "IconPicture",
      "IconCloudUpload",
      "IconFullscreen"
    ]
  });
  return state;
};
const initApi = ({ api: api2, state, service, emit, props, nextTick, FluentEditor, UploaderDfls, Delta, vm, t, mode }) => {
  Object.assign(api2, {
    state,
    getFileUploadUrl: index.getFileUploadUrl({ service }),
    selectionChange: index.selectionChange({ emit, state }),
    fileOperationToSev: index.fileOperationToSev({ state }),
    undoHandler: index.undoHandler({ state }),
    redoHandler: index.redoHandler({ state }),
    lineheightHandler: index.lineheightHandler({ state, FluentEditor }),
    inputFileHandler: index.inputFileHandler({ state, UploaderDfls }),
    insertFileToEditor: index.insertFileToEditor({ state, FluentEditor, Delta }),
    insertImageToEditor: index.insertImageToEditor({ state, FluentEditor, Delta }),
    uploadImageToSev: index.uploadImageToSev({ state }),
    doPreview: index.doPreview({ props, state, nextTick }),
    stringToJson: index.stringToJson(),
    setToolbarTips: index.setToolbarTips({ api: api2, vm, FluentEditor, iconOption: mode === "mobile-first" ? options.iconOptionMobileFirst : options.iconOption }),
    getOuterHTML: index.getOuterHTML(),
    setToolbarTitle: index.setToolbarTitle({ state, t })
  });
};
const mergeApi = (args) => {
  let { api: api2, state, service, emit, props, vm, i18n, watch, nextTick, useBreakpoint } = args;
  let { constants, FluentEditor, UploaderDfls, Delta, defaultOptions } = args;
  Object.assign(api2, {
    init: index.init({ api: api2, emit, props, service, state, FluentEditor, UploaderDfls, defaultOptions, vm, useBreakpoint, simpleToolbar: options.simpleToolbar }),
    initContent: index.initContent({ state, props, api: api2, nextTick }),
    fileHandler: index.fileHandler({ api: api2, state }),
    imageHandler: index.imageHandler({ api: api2, state }),
    uploaderDflsHandler: index.uploaderDflsHandler({ api: api2, modules: defaultOptions.modules }),
    handleUploadFile: index.handleUploadFile({ api: api2, UploaderDfls }),
    handleUploadImage: index.handleUploadImage({ state, api: api2, FluentEditor, Delta, UploaderDfls }),
    handlers: index.handlers({ api: api2 }),
    mounted: index.mounted({ api: api2, props, state }),
    beforeUnmount: index.beforeUnmount({ api: api2, state, vm }),
    computePreviewOptions: index.computePreviewOptions({ props, state, constants, api: api2 }),
    handleDblclick: index.handleDblclick({ props, api: api2 }),
    textChange: index.textChange({ emit, vm, state, props }),
    keyDownHandler: index.keyDownHandler({ state }),
    computeZIndex: index.computeZIndex({ constants, props }),
    addFullscreenchange: index.addFullscreenchange({ api: api2 }),
    removeFullscreenchange: index.removeFullscreenchange({ api: api2 }),
    handleComposition: index.handleComposition({ state, api: api2 }),
    handleCompositionstart: index.handleCompositionstart({ state }),
    handleCompositionend: index.handleCompositionend({ state }),
    removeHandleComposition: index.removeHandleComposition({ state, api: api2 })
  });
};
const initWatch = ({ watch, state, api: api2, props, vm }) => {
  watch(
    () => props.modelValue,
    (value) => {
      if (index.checkTableISEndElement(vm.$refs.editor.children[0])) {
        state.content = value + "<p><br></p><p><br></p>";
      } else {
        state.content = value;
      }
      api2.initContent();
    }
  );
  watch(
    () => props.content,
    (value) => {
      state.content = value;
      api2.initContent();
    }
  );
  watch(
    () => props.disabled,
    (value) => {
      if (state.quill) {
        state.quill.enable(!value);
      }
    }
  );
};
const renderless = (props, { reactive, watch, onMounted, onBeforeUnmount, computed }, { service, emit, i18n, constants, nextTick, vm, t, useBreakpoint, mode }, { FluentEditor }) => {
  const api2 = {};
  const { DEFAULTS: UploaderDfls } = FluentEditor.imports["modules/uploader"];
  const Delta = FluentEditor.imports["delta"];
  const state = initState({ reactive, computed, api: api2, props });
  const defaultOptions = options.defaultOption({ FluentEditor, state, mentionObj: props.mentionObj });
  initApi({ api: api2, state, service, emit, props, nextTick, FluentEditor, UploaderDfls, Delta, vm, t, mode });
  const args = { api: api2, state, service, emit, props, vm, i18n, watch, nextTick, useBreakpoint };
  Object.assign(args, { constants, FluentEditor, UploaderDfls, Delta, defaultOptions });
  mergeApi(args);
  initWatch({ watch, state, api: api2, props, vm });
  onMounted(api2.mounted);
  onBeforeUnmount(() => {
    UploaderDfls.handler = null;
    UploaderDfls.imagePasteFailCallback = null;
    api2.beforeUnmount();
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
