"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = [
  "state",
  "validate",
  "clearValidate",
  "resetField",
  "getRules",
  "getFilteredRule",
  "onFieldBlur",
  "onFieldChange",
  "updateComputedLabelWidth",
  "addValidateEvents",
  "removeValidateEvents",
  "updateTip",
  "getDisplayedValue",
  "handleLabelMouseenter",
  "handleMouseenter",
  "handleMouseleave"
];
const initState = ({
  reactive,
  computed,
  api: api2,
  mode,
  inject,
  props
}) => {
  const state = reactive({
    mode,
    validateState: "",
    validateMessage: "",
    validateDisabled: false,
    validator: {},
    isNested: false,
    computedLabelWidth: "",
    initialValue: null,
    canShowTip: false,
    // 兼容 2.0 validation 的 required
    validationRequired: false,
    validateType: "text",
    tooltip: "",
    displayedValue: "",
    isBasicComp: false,
    showTooltip: false,
    typeName: "",
    formInstance: inject("form"),
    labelFor: computed(() => props.for || props.prop || ""),
    labelStyle: computed(() => api2.computedLabelStyle()),
    valueStyle: computed(() => api2.computedValueStyle()),
    contentStyle: computed(() => api2.computedContentStyle()),
    form: computed(() => api2.computedForm()),
    fieldValue: computed(() => api2.computedFieldValue()),
    isRequired: computed(() => api2.computedIsRequired()),
    formInline: computed(() => state.formInstance.inline),
    formSize: computed(() => state.formInstance.size),
    formItemSize: computed(() => props.size || state.formSize),
    isDisplayOnly: computed(() => state.formInstance.displayOnly),
    labelPosition: computed(() => state.formInstance.labelPosition),
    hideRequiredAsterisk: computed(() => state.formInstance.state.hideRequiredAsterisk),
    labelSuffix: computed(() => state.formInstance.labelSuffix),
    labelWidth: computed(() => state.formInstance.labelWidth),
    showMessage: computed(() => state.formInstance.showMessage),
    sizeClass: computed(() => state.formItemSize),
    getValidateType: computed(() => api2.computedGetValidateType()),
    validateIcon: computed(() => api2.computedValidateIcon()),
    isErrorInline: computed(() => api2.computedIsErrorInline()),
    isErrorBlock: computed(() => api2.computedIsErrorBlock()),
    disabled: computed(() => state.formInstance.disabled),
    tooltipType: computed(() => state.formInstance.state.tooltipType),
    // 标记表单项下是否有多个子节点
    isMultiple: false
  });
  return state;
};
const initApi = ({ api: api2, state, dispatch, broadcast, props, constants, vm, t, nextTick, slots }) => {
  Object.assign(api2, {
    state,
    dispatch,
    broadcast,
    watchError: index.watchError(state),
    updateTip: index.updateTip({ vm, state }),
    watchValidateStatus: index.watchValidateStatus(state),
    computedLabelStyle: index.computedLabelStyle({ props, state }),
    computedValueStyle: index.computedValueStyle({ props, state }),
    computedContentStyle: index.computedContentStyle({ props, state }),
    computedForm: index.computedForm({ constants, vm, state }),
    computedFieldValue: index.computedFieldValue({ props, state }),
    computedGetValidateType: index.computedGetValidateType({ props, state }),
    computedValidateIcon: index.computedValidateIcon({ props, state }),
    computedIsErrorInline: index.computedIsErrorInline({ props, state }),
    computedIsErrorBlock: index.computedIsErrorBlock({ props, state }),
    clearValidate: index.clearValidate(state),
    getRules: index.getRules({ props, state }),
    updateComputedLabelWidth: index.updateComputedLabelWidth(state),
    removeValidateEvents: index.removeValidateEvents(vm),
    unmounted: index.unmounted({ api: api2, vm, state }),
    mounted: index.mounted({ api: api2, vm, props, state }),
    computedIsRequired: index.computedIsRequired({ api: api2, state }),
    resetField: index.resetField({ api: api2, nextTick, props, state }),
    getFilteredRule: index.getFilteredRule(api2),
    onFieldBlur: index.onFieldBlur(api2),
    onFieldChange: index.onFieldChange({ api: api2, state }),
    addValidateEvents: index.addValidateEvents({ api: api2, vm, props, state }),
    validate: index.wrapValidate({ validateFunc: index.validate({ api: api2, props, state, t }), props }),
    getDisplayedValue: index.getDisplayedValue({ state }),
    clearDisplayedValue: index.clearDisplayedValue({ state }),
    handleLabelMouseenter: index.handleLabelMouseenter({ props, state, slots }),
    handleMouseenter: index.handleMouseenter({ state }),
    handleMouseleave: index.handleMouseleave(state)
  });
};
const initWatch = ({ watch, api: api2, props, state }) => {
  watch(() => props.error, api2.watchError, { immediate: true });
  watch(() => props.validateStatus, api2.watchValidateStatus);
  watch(() => state.formInstance.displayOnly, api2.clearDisplayedValue);
};
const renderless = (props, { computed, inject, onMounted, onUnmounted, provide, reactive, watch }, { vm, constants, t, nextTick, broadcast, dispatch, mode, slots }) => {
  const api2 = {};
  const state = initState({ reactive, computed, api: api2, mode, inject, props });
  provide("formItem", vm);
  initApi({ api: api2, state, dispatch, broadcast, props, constants, vm, t, nextTick, slots });
  initWatch({ watch, api: api2, props, state });
  onMounted(api2.mounted);
  vm.$on("displayed-value-changed", (param) => {
    api2.getDisplayedValue(param);
  });
  onUnmounted(api2.unmounted);
  return api2;
};
exports.api = api;
exports.renderless = renderless;
