"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = [
  "state",
  "resetFields",
  "clearValidate",
  "validate",
  "validateField",
  "getLabelWidthIndex",
  "registerLabelWidth",
  "deregisterLabelWidth",
  "updateTip",
  "showTooltip",
  "hideTooltip"
];
const renderless = (props, { computed, inject, provide, reactive, watch, onBeforeUnmount }, { vm, parent, designConfig }) => {
  const api2 = {};
  const dialog = inject("dialog", null);
  const state = reactive({
    showAutoWidth: props.showAutoWidth,
    fields: [],
    timer: 0,
    tooltipVisible: false,
    displayedValue: "",
    potentialLabelWidthArr: [],
    autoLabelWidth: computed(() => api2.computedAutoLabelWidth()),
    hideRequiredAsterisk: computed(() => api2.computedHideRequiredAsterisk()),
    validateIcon: computed(() => api2.computedValidateIcon()),
    isErrorInline: computed(() => api2.computedIsErrorInline()),
    isErrorBlock: computed(() => api2.computedIsErrorBlock()),
    isDisplayOnly: computed(() => props.displayOnly),
    hasRequired: computed(() => {
      if (props.rules) {
        return Object.values(props.rules).find((ruleOrRules) => {
          if (Array.isArray(ruleOrRules)) {
            return ruleOrRules.some((r) => r.required);
          } else {
            return ruleOrRules.required;
          }
        });
      } else {
        return false;
      }
    }),
    labelWidth: computed(() => {
      var _a;
      return props.labelWidth || ((_a = designConfig == null ? void 0 : designConfig.state) == null ? void 0 : _a.labelWidth) || "84px";
    }),
    tooltipType: computed(() => {
      var _a;
      return ((_a = designConfig == null ? void 0 : designConfig.state) == null ? void 0 : _a.tooltipType) || "normal";
    })
  });
  Object.assign(api2, {
    state,
    updateTip: index.updateTip({ props, state }),
    computedAutoLabelWidth: index.computedAutoLabelWidth({ state }),
    computedHideRequiredAsterisk: index.computedHideRequiredAsterisk({ props, designConfig }),
    computedValidateIcon: index.computedValidateIcon({ props, designConfig }),
    computedIsErrorInline: index.computedIsErrorInline({ props, designConfig }),
    computedIsErrorBlock: index.computedIsErrorBlock({ props, designConfig }),
    created: index.created({ parent, state }),
    resetFields: index.resetFields({ props, state }),
    clearValidate: index.clearValidate(state),
    validate: index.validate({ props, state }),
    validateField: index.validateField(state),
    getLabelWidthIndex: index.getLabelWidthIndex(state),
    registerLabelWidth: index.registerLabelWidth({ api: api2, state }),
    deregisterLabelWidth: index.deregisterLabelWidth({ api: api2, state }),
    watchRules: index.watchRules({ api: api2, props, state }),
    showTooltip: index.showTooltip({ vm, state }),
    hideTooltip: index.hideTooltip({ state })
  });
  api2.created();
  provide("form", parent);
  provide("showAutoWidth", state.showAutoWidth);
  const unbindDialogEvent = index.bindDialogEvent({ api: api2, dialog, state });
  onBeforeUnmount(unbindDialogEvent);
  watch(() => props.rules, api2.watchRules, { deep: props.validateOnRuleChange === "deep" });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
