"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const each = require("../static/base/each.js");
const remove = require("../static/base/remove.js");
const toString = require("../static/string/toString.js");
const toType = (type) => toString.default(type).replace("_", "").toLowerCase();
const eventTypes = [
  "created",
  "mounted",
  "activated",
  "beforeDestroy",
  "destroyed",
  "event.clearActived",
  "event.clearFilter",
  "event.showMenu",
  "event.keydown"
].map(toType);
const _storeMap = {};
const Interceptor = {
  mixin(map) {
    each.default(map, (callback, type) => Interceptor.add(type, callback));
    return Interceptor;
  },
  get(type) {
    return _storeMap[toType(type)] || [];
  },
  add(type, callback) {
    const selfType = toType(type);
    if (callback && ~eventTypes.indexOf(selfType)) {
      _storeMap[selfType] = _storeMap[selfType] ? _storeMap[selfType] : [];
      _storeMap[selfType].push(callback);
    }
    return Interceptor;
  },
  delete(type, callback) {
    const eList = _storeMap[toType(type)];
    if (eList) {
      remove.default(eList, (cb) => cb === callback);
    }
    return Interceptor;
  }
};
var interceptor_default = Interceptor;
exports.default = interceptor_default;
