"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const chunkG2ADBYYC = require("../../chunk-G2ADBYYC.js");
const type = require("../../common/type.js");
const array = require("../../common/array.js");
const isFunction = require("../static/base/isFunction.js");
const get = require("../static/base/get.js");
const set = require("../static/base/set.js");
const getFuncText = (content) => isFunction.default(content) ? content() : content;
const getRowkey = ($table) => $table.rowId;
const getRowid = ($table, row) => {
  const rowId = get.default(row, getRowkey($table));
  return rowId ? encodeURIComponent(rowId) : "";
};
const getColumnList = (columns) => {
  const result = [];
  columns.forEach((column) => {
    if (column.children && column.children.length) {
      result.push(...getColumnList(column.children));
    } else {
      result.push(column);
    }
  });
  return result;
};
const getClass = (property, params) => property ? isFunction.default(property) ? property(params) : property : "";
const getFilters = (filters) => (filters || []).map(({ label, value, data, checked }) => ({
  label,
  value,
  data,
  _data: data,
  checked: !!checked
}));
const initFilter = (filter) => {
  return chunkG2ADBYYC.__spreadValues({
    condition: {
      input: "",
      relation: "equals",
      empty: null,
      type: null,
      value: []
    },
    hasFilter: false,
    custom: null
  }, filter);
};
const formatText = (value) => `${type.isNull(value) ? "" : value}`;
const setCellValue = (row, column, value) => {
  const { format, property } = column;
  if (format && format.async && Array.isArray(format.data) && format.data.length > 0 && value) {
    let labelText = "";
    const { enabled, valueSplit, textSplit } = format.async.splitConfig || {};
    const findCellValue = (optionValue) => array.find(format.data, (col) => {
      if (typeof col === "object") {
        const colLabel = get.default(col, format.async.text || "label");
        const colValue = get.default(col, format.async.value || "value");
        col.label = colLabel;
        return optionValue === colValue || optionValue === colLabel;
      }
      return optionValue === col;
    });
    if (enabled) {
      const labelTexts = [];
      value.split(valueSplit || ",").forEach((item) => {
        const findValue = findCellValue(item);
        if (findValue) {
          labelTexts.push(findValue.label);
        }
      });
      labelText = labelTexts.join(textSplit || ",");
    } else {
      labelText = findCellValue(value);
    }
    set.default(row, column.asyncPrefix + property, labelText ? labelText.label : labelText);
  }
  set.default(row, property, value);
};
const hasChildrenList = (item) => item && item.children && item.children.length > 0;
const emitEvent = (vm, type2, args) => {
  if (vm.tableListeners[type2]) {
    const params = [].concat(args);
    vm.$emit(type2, ...params);
  }
};
const assemColumn = ($table) => {
  const collectColumn = [];
  const assem = (columnVms, columns) => {
    if (Array.isArray(columnVms)) {
      columnVms.forEach((columnVm) => {
        const column = columnVm.columnConfig;
        const children = [];
        if (column) {
          columns.push(column);
          assem(columnVm.childColumns, children);
          column.children = children.length > 0 ? children : null;
        }
      });
    }
  };
  assem($table.childColumns, collectColumn);
  $table.collectColumn = collectColumn;
};
const getCellValue = (row, column) => get.default(row, column.own.field);
const getListeners = ($attrs, $listeners) => {
  const regHyphenate = /\B([A-Z])/g;
  const regEventPrefix = /^on[A-Z]/;
  const listeners = {};
  if ($listeners) {
    return $listeners;
  }
  Object.keys($attrs).forEach((name) => {
    const event = $attrs[name];
    if (regEventPrefix.test(name) && typeof event === "function") {
      listeners[name.slice(2).replace(regHyphenate, "-$1").toLowerCase()] = event;
    }
  });
  return listeners;
};
exports.assemColumn = assemColumn;
exports.emitEvent = emitEvent;
exports.formatText = formatText;
exports.getCellValue = getCellValue;
exports.getClass = getClass;
exports.getColumnList = getColumnList;
exports.getFilters = getFilters;
exports.getFuncText = getFuncText;
exports.getListeners = getListeners;
exports.getRowid = getRowid;
exports.getRowkey = getRowkey;
exports.hasChildrenList = hasChildrenList;
exports.initFilter = initFilter;
exports.setCellValue = setCellValue;
