"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const browser = require("../../common/browser.js");
const remove = require("../static/base/remove.js");
const dom = require("../../common/deps/dom.js");
const wheelName = browser.default.isDoc && /Firefox/i.test(navigator.userAgent) ? "DOMMouseScroll" : "mousewheel";
const eventStore = [];
const invoke = ({ comp, type, cb }, event) => {
  if (type === event.type || type === "mousewheel" && event.type === wheelName) {
    cb.call(comp, event);
  }
};
const GlobalEvent = {
  on(comp, type, cb, capture = false) {
    if (cb) {
      eventStore.push({ comp, type, cb, capture });
    }
  },
  off(comp, type, capture = false) {
    remove.default(eventStore, (item) => item.comp === comp && item.type === type && item.capture === capture);
  },
  trigger(event) {
    eventStore.filter((item) => !item.capture).forEach((item) => invoke(item, event));
  },
  capture(event) {
    eventStore.filter((item) => item.capture).forEach((item) => invoke(item, event));
  }
};
if (browser.default.isDoc) {
  dom.on(document, "keydown", GlobalEvent.trigger);
  dom.on(document, "contextmenu", GlobalEvent.trigger);
  dom.on(window, "mousedown", GlobalEvent.trigger);
  dom.on(window, "mousedown", GlobalEvent.capture, true);
  dom.on(window, "blur", GlobalEvent.trigger);
  dom.on(window, "resize", GlobalEvent.trigger);
  dom.on(window, wheelName, GlobalEvent.trigger);
}
var event_default = GlobalEvent;
exports.default = event_default;
