"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = [
  "state",
  "zIndex",
  "touchstart",
  "touchmove",
  "touchend",
  "urlList",
  "hide",
  "prev",
  "next",
  "handleActions",
  "toggleMode",
  "handleImgLoad",
  "handleImgError",
  "handleMouseDown",
  "handleVisible",
  "swipeLeft",
  "swipeRight",
  "handleDelete",
  "activeItems",
  "imagePreview",
  "itemClick",
  "selectOption",
  "langClick"
];
const initState = ({ reactive, computed, api: api2, mode, props, constants, inject }) => {
  const state = reactive({
    originScale: "",
    moveable: false,
    pageX: "",
    pageY: "",
    pageY2: "",
    pageX2: "",
    mfPreviewVisible: inject("mfPreviewVisible", null),
    scale: 1,
    time: null,
    index: mode === "pc" || mode === "mobile-first" ? 0 : props.startPosition,
    imageName: "",
    isShow: false,
    infinite: true,
    loading: false,
    transform: { scale: 1, deg: 0, offsetX: 0, offsetY: 0, objfit: "contain", enableTransition: false },
    urlList: props.urlList || Array.isArray(inject("urlList")) && inject("urlList").length !== 0 ? inject("urlList") : null,
    mode: constants.MODE.CONTAIN,
    previewVisible: props.previewVisible,
    fullScreen: props.imageFullCurrent,
    hammer: null,
    imageItemWidth: 0,
    imageAllWidth: 0,
    imageTransform: 0,
    imageTransformSize: 0,
    imageTransition: 300,
    imageList: null,
    arrowStyle: null,
    delete: false,
    isLast: computed(() => api2.computedIsLast()),
    isFirst: computed(() => api2.computedIsFirst()),
    isSingle: computed(() => api2.computedIsSingle()),
    imgStyle: computed(() => api2.computedImgStyle()),
    currentImg: computed(() => api2.computedCurrentImg()),
    zIndex: computed(() => api2.computeZIndex()),
    currentIndex: 0,
    mobileCurrentIndex: -1,
    isImagePreview: false,
    hiddenThumbnail: false,
    firstX: 0,
    endX: 0,
    showImageViewer: true,
    isThumbnail: props.isThumbnail,
    isMenuView: props.isMenuView,
    showFlag: 0,
    boxVisibility: false,
    fileName: "",
    scrollTop: 0,
    thumbnailTop: constants.THUMBNAILTOP,
    menuTop: constants.MENUTOP,
    centerIndex: -1
  });
  return state;
};
const initApi = ({ api: api2, state, props, parent, nextTick, emit, t, constants, vm, mode }) => {
  Object.assign(api2, {
    state,
    touchstart: index.touchstart({ state, mode, api: api2 }),
    touchmove: index.touchmove(state),
    touchend: index.touchend(state),
    reset: index.reset(state),
    prev: index.prev({ state, api: api2, vm }),
    next: index.next({ state, api: api2, vm }),
    getImageWidth: index.getImageWidth({ state, parent, props, vm, mode }),
    handleVisible: index.handleVisible({ state, emit, props }),
    handleActions: index.handleActions(state, props, emit),
    handleImgLoad: index.handleImgLoad(state),
    handleMouseDown: index.handleMouseDown(state),
    computedIsFirst: index.computedIsFirst(state),
    computedIsSingle: index.computedIsSingle(props),
    handleImgError: index.handleImgError({ state, t }),
    computedIsLast: index.computedIsLast({ state, props }),
    watchVisible: index.watchVisible(state),
    deviceSupportUninstall: index.deviceSupportUninstall({ state, mode }),
    computedCurrentImg: index.computedCurrentImg({ state, api: api2 }),
    computedImgStyle: index.computedImgStyle({ state, constants }),
    computeZIndex: index.computeZIndex({ constants, props }),
    hide: index.hide({ props, api: api2, state }),
    deviceSupportInstall: index.deviceSupportInstall({ state, api: api2, mode }),
    toggleMode: index.toggleMode({ state, constants, api: api2 }),
    swipeRight: index.swipeRight({ state, emit }),
    swipeLeft: index.swipeLeft({ state, emit }),
    handleDelete: index.handleDelete({ api: api2, state, emit }),
    activeItems: index.activeItems(state),
    imagePreview: index.imagePreview(state),
    initPage: index.initPage({ state, nextTick, api: api2 }),
    beforeDestroy: index.beforeDestroy({ api: api2, state }),
    itemClick: index.itemClick({ state, vm, nextTick }),
    selectOption: index.selectOption({ state, api: api2 }),
    langClick: index.langClick(state),
    getLastPrev: index.getLastPrev({ state, vm }),
    getDefaultPrev: index.getDefaultPrev({ state, vm }),
    getCenterPosition: index.getCenterPosition({ state, vm }),
    filterImageUrl: index.filterImageUrl()
  });
};
const initWatch = ({ watch, state, api: api2, props, nextTick, vm }) => {
  watch(
    () => state.index,
    (value) => {
      if (!props.keepStyle) {
        api2.reset();
      }
      props.onSwitch(value);
    },
    { immediate: true }
  );
  watch(
    () => props.previewVisible,
    (value) => {
      api2.watchVisible(value);
      if (props.previewVisible || state.mfPreviewVisible) {
        nextTick(() => {
          api2.getImageWidth();
        });
      }
    },
    { immediate: true }
  );
  watch(
    () => state.currentImg,
    (value) => {
      nextTick(() => {
        const index2 = state.urlList.indexOf(value);
        let imageIns = vm.$refs[`img_${index2}`];
        if (Array.isArray(imageIns)) {
          imageIns = imageIns[0];
        }
        if (imageIns) {
          state.loading = !imageIns.complete;
        }
      });
    },
    { immediate: true }
  );
  watch(
    () => state.isImagePreview,
    () => nextTick(() => {
      api2.getImageWidth();
    })
  );
  watch(
    () => props.urlList,
    () => {
      state.urlList = props.urlList;
    },
    { deep: true }
  );
};
const renderless = (props, { computed, onMounted, onBeforeUnmount, onUpdated, reactive, watch, inject, provide }, { t, parent, nextTick, emit, constants, vm, mode }) => {
  const api2 = {};
  const state = initState({ reactive, computed, api: api2, mode, props, constants, inject });
  initApi({ api: api2, state, props, parent, nextTick, emit, t, constants, vm, mode });
  initWatch({ watch, state, api: api2, props, nextTick, vm });
  onMounted(api2.deviceSupportInstall);
  onUpdated(() => {
    if (props.asyncClose) {
      setTimeout(() => {
        state.previewVisible = false;
        emit("update:preview-visible", false);
      }, 3e3);
    }
  });
  onBeforeUnmount(api2.beforeDestroy);
  provide("change-size", true);
  return api2;
};
exports.api = api;
exports.renderless = renderless;
