"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = [
  "state",
  "src",
  "zIndex",
  "previewSrcList",
  "loadImage",
  "clickHandler",
  "closeViewer",
  "handleLoad",
  "handleError",
  "deleteHander",
  "handleSwitch"
];
const initState = ({
  reactive,
  computed,
  api: api2,
  props,
  images
}) => {
  const state = reactive({
    mfPreviewVisible: props.previewVisible,
    /** mobile-first传入的一张image-error的图片 */
    images,
    error: false,
    loading: true,
    imageWidth: 0,
    imageHeight: 0,
    show: !props.lazy,
    showViewer: false,
    getPreview: computed(() => api2.computedGetPreview()),
    getImageStyle: computed(() => api2.computedGetImageStyle()),
    getAlignCenter: computed(() => api2.computedGetAlignCenter()),
    src: computed(() => api2.filterImageUrl())
  });
  return state;
};
const initApi = ({
  api: api2,
  state,
  emit,
  props,
  vm,
  constants,
  nextTick,
  attrs
}) => {
  Object.assign(api2, {
    state,
    closeViewer: index.closeViewer(state),
    clickHandler: index.clickHandler(state),
    handleLoad: index.handleLoad({ state, emit }),
    handleError: index.handleError({ state, emit }),
    computedGetPreview: index.computedGetPreview(props),
    removeLazyLoadListener: index.removeLazyLoadListener(state),
    getImageStyle: index.getImageStyle({ state, vm, constants }),
    computedGetAlignCenter: index.computedGetAlignCenter({ props, constants }),
    mounted: index.mounted({ api: api2, props }),
    handleLazyLoad: index.handleLazyLoad({ api: api2, state, vm, nextTick }),
    loadImage: index.loadImage({ api: api2, state, props, attrs }),
    computedGetImageStyle: index.computedGetImageStyle({ api: api2, props }),
    addLazyLoadListener: index.addLazyLoadListener({ api: api2, props, state, vm }),
    deleteHander: index.deleteHander(emit),
    filterImageUrl: index.filterImageUrl(props),
    handleSwitch: index.handleSwitch({ emit })
  });
};
const initWatch = ({ watch, state, api: api2, props }) => {
  watch(
    () => props.src,
    (value, oldValue) => value !== oldValue && state.show && api2.loadImage()
  );
  watch(
    () => state.show,
    (value) => value && api2.loadImage()
  );
};
const renderless = (props, { computed, onBeforeUnmount, onMounted, reactive, watch, provide }, { vm, emit, constants, nextTick, attrs }, { images }) => {
  const api2 = {};
  const state = initState({ reactive, computed, api: api2, props, images });
  initApi({ api: api2, state, emit, props, vm, constants, nextTick, attrs });
  initWatch({ watch, state, api: api2, props });
  onMounted(api2.mounted);
  onBeforeUnmount(() => props.lazy && api2.removeLazyLoadListener());
  provide("mfPreviewVisible", state.mfPreviewVisible);
  provide("urlList", props.previewSrcList);
  return api2;
};
exports.api = api;
exports.renderless = renderless;
