"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const dom = require("../common/deps/dom.js");
const useRect = require("../common/deps/useRect.js");
const useEventListener = require("../common/deps/useEventListener.js");
const debounce = require("../common/deps/debounce.js");
const api = ["state", "clickList", "check", "clickErrorText"];
const renderless = (props, {
  inject,
  reactive,
  computed,
  onMounted,
  unref,
  watch,
  onUpdated,
  isRef,
  onUnmounted,
  onActivated,
  onDeactivated,
  toRef
}, { emit, vm, nextTick }) => {
  const api2 = {};
  const state = reactive({
    loading: props.loading,
    tabStatus: inject("TabStatus", null),
    scrollParent: null,
    scroller: computed(() => props.scroller || state.scrollParent)
  });
  Object.assign(api2, {
    state,
    clickList: index.clickList({ emit, props }),
    useRect: useRect.useRect(unref),
    check: debounce.default(+props.delay, index.check({ api: api2, emit, props, state, vm })),
    clickErrorText: index.clickErrorText({ api: api2, emit }),
    useEventListener: useEventListener.useEventListener({
      unref,
      isRef,
      watch,
      nextTick,
      onMounted,
      onUnmounted,
      onActivated,
      onDeactivated
    })
  });
  watch(
    () => [props.loading, props.finished, props.error],
    () => api2.check()
  );
  watch(
    () => state.tabStatus,
    (tabActive) => tabActive && api2.check()
  );
  api2.useEventListener("scroll", api2.check, { target: toRef(state, "scroller"), passive: true });
  onUpdated(() => {
    state.loading = props.loading;
  });
  onMounted(() => {
    state.scrollParent = dom.getScrollParent(vm.$el);
    if (props.immediateCheck) {
      api2.check();
    }
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
