"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const debounce = require("../common/deps/debounce.js");
const api = ["state", "setIsCurrent", "filterNodes", "filterInput"];
const initState = ({ reactive, computed, props, api: api2 }) => {
  const state = reactive({
    isEmpty: computed(() => props.data.length !== 0 && state.filterValue && state.filterData.length === 0),
    activedNodeId: null,
    filterValue: "",
    currentPaths: [],
    filterData: [],
    data: computed(() => api2.computeData())
  });
  return state;
};
const renderless = (props, { reactive, watch, onMounted, computed, onUnmounted }, { vm, emit, nextTick }) => {
  const api2 = {};
  const state = initState({ reactive, computed, props, api: api2 });
  Object.assign(api2, {
    state,
    setIsCurrent: index.setIsCurrent({ props, vm, state }),
    filterNodes: debounce.default(100, index.filterNodes({ state, api: api2, nextTick })),
    setNodeHidden: index.setNodeHidden({ props, vm, state }),
    getActivedKeysPath: index.getActivedKeysPath({ state }),
    getExpandedKeysPath: index.getExpandedKeysPath({ state }),
    setAllNodeVisible: index.setAllNodeVisible({ vm, state }),
    filterInput: index.filterInput({ state }),
    computeData: index.computeData({ props, vm })
  });
  watch(
    () => props.expandAll,
    () => {
      props.expandAll && api2.setAllNodeVisible();
    },
    { immediate: true }
  );
  watch(
    () => state.filterValue,
    () => {
      api2.filterNodes(state.filterValue);
    }
  );
  watch(
    () => props.expandedKeys,
    () => {
      if (props.expandedKeys && props.expandedKeys.length) {
        api2.getExpandedKeysPath(props.expandedKeys);
      }
    },
    { deep: true, immediate: true }
  );
  watch(
    () => props.activedKeys,
    (val) => {
      if (val || val === 0) {
        state.activedKeys = props.activedKeys;
        api2.getActivedKeysPath(props.activedKeys);
        api2.setIsCurrent(props.data, { id: props.activedKeys });
      }
    },
    { immediate: true }
  );
  watch(
    () => props.data,
    (val) => {
      nextTick(() => {
        api2.setIsCurrent(val, { id: props.activedKeys });
      });
    }
  );
  onMounted(() => {
    vm.$on("node-clicked", (param) => {
      api2.setNodeHidden(param.id);
      vm.$emit("node-click", param);
    });
    vm.$on("node-changed", (param) => {
      vm.$emit("current-change", param);
      api2.setIsCurrent(state.data, param);
    });
  });
  onUnmounted(() => {
    vm.$off("node-clicked");
    vm.$off("node-changed");
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
