"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = [
  "state",
  "visibleHandle",
  "watchVisible",
  "change",
  "setValues",
  "getColumnValue",
  "confirm",
  "cancel",
  "getColumnIndex",
  "getValues",
  "getIndexes",
  "setIndexes",
  "setColumnIndex",
  "getColumnValues",
  "setColumnValues",
  "onChange",
  "onCascadeChange",
  "emitEvent",
  "getColumn",
  "setColumnValue"
];
const initState = ({ reactive, computed, props, api: api2 }) => {
  const state = reactive({
    columns: props.columns,
    toggle: false,
    itemHeight: Number(props.itemHeight),
    defaultIndex: props.defaultIndex,
    visibleItemCount: props.visibleItemCount,
    clumnsWrapHeight: null,
    formattedColumns: [],
    dataType: computed(() => api2.getDataType())
  });
  return state;
};
const initApi = ({ api: api2, props, state, emit, childrenPickerRefs }) => {
  Object.assign(api2, {
    state,
    getColumn: index.getColumn(childrenPickerRefs),
    getValues: index.getValues(childrenPickerRefs),
    getIndexes: index.getIndexes(childrenPickerRefs),
    getDataType: index.getDataType(state),
    visibleHandle: index.visibleHandle(emit),
    formatCascade: index.formatCascade({ state, props }),
    getColumnValues: index.getColumnValues(childrenPickerRefs),
    setColumnValues: index.setColumnValues(childrenPickerRefs),
    emitEvent: index.emitEvent({ api: api2, state, emit }),
    change: index.change(api2),
    onChange: index.onChange({ api: api2, state, emit }),
    cancel: index.cancel({ api: api2, emit }),
    confirm: index.confirm({ api: api2, childrenPickerRefs }),
    format: index.format({ state, api: api2 }),
    setValues: index.setValues(api2),
    setIndexes: index.setIndexes(api2),
    watchVisible: index.watchVisible(emit),
    setColumnIndex: index.setColumnIndex(api2),
    getColumnValue: index.getColumnValue(api2),
    getColumnIndex: index.getColumnIndex(api2),
    setColumnValue: index.setColumnValue(api2),
    onCascadeChange: index.onCascadeChange({ api: api2, state, props })
  });
};
const initWatch = ({ watch, props, state, api: api2 }) => {
  watch(
    () => props.visible,
    (value) => {
      api2.watchVisible({ state, value });
    }
  );
  watch(
    () => props.columns,
    (value) => {
      state.columns = value;
      api2.format();
    }
  );
};
const renderless = (props, { computed, onMounted, reactive, watch }, { emit, nextTick, vm, constants }) => {
  const api2 = {};
  const childrenPickerRefs = { childrenPicker: null };
  const state = initState({ reactive, computed, props, api: api2 });
  initApi({ api: api2, props, state, emit, childrenPickerRefs });
  initWatch({ watch, props, state, api: api2 });
  onMounted(() => {
    nextTick(() => {
      childrenPickerRefs.childrenPicker = index.getChildrenComponent({ state, vm, constants });
    });
    api2.format();
    state.clumnsWrapHeight = state.itemHeight * state.visibleItemCount;
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
