"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = [
  "state",
  "decrease",
  "increase",
  "handleBlur",
  "handleFocus",
  "handleInput",
  "handleInputChange",
  "mouseEvent",
  "focus",
  "select",
  "handleClear",
  "handleChange"
];
const initState = ({
  reactive,
  computed,
  props,
  api: api2,
  $service,
  constants,
  parent
}) => {
  const state = reactive({
    currentValue: props.modelValue,
    userInput: props.modelValue,
    lastInput: props.modelValue,
    inputStatus: false,
    decimal: null,
    strictInput: computed(() => props.strictInput),
    inputSize: computed(() => props.size || state.formSize),
    formSize: computed(() => (parent.tinyForm || {}).size),
    formDisabled: computed(() => (parent.tinyForm || {}).disabled),
    inputDisabled: computed(() => props.disabled || state.formDisabled),
    displayValue: computed(() => api2.displayValue()),
    numPrecision: computed(() => api2.getNumPecision()),
    minDisabled: computed(() => !props.circulate && state.currentValue <= props.min || state.formDisabled),
    maxDisabled: computed(() => !props.circulate && state.currentValue >= props.max || state.formDisabled),
    controlsAtRight: computed(() => props.controls && props.controlsPosition === "right"),
    format: computed(() => index.getUnitPrecision({ service: $service, props })),
    filterMenu: constants.FILTER_OPTION,
    filterValue: computed(() => api2.filterValue()),
    handleClear: computed(() => api2.handleClear()),
    handleChange: computed(() => api2.handleClear()),
    isDisplayOnly: computed(() => props.displayOnly || (parent.tinyForm || {}).displayOnly),
    displayOnlyText: computed(() => api2.getDisplayOnlyText()),
    controls: props.controls
  });
  return state;
};
const initApi = ({
  api: api2,
  props,
  state,
  parent,
  vm,
  emit,
  dispatch,
  constants,
  nextTick
}) => {
  Object.assign(api2, {
    state,
    focus: index.focus(vm),
    select: index.select(vm),
    getPrecision: index.getPrecision(),
    toPrecision: index.toPrecision(state),
    updated: index.updated({ constants, parent, state }),
    mounted: index.mounted({ constants, parent, props, state }),
    unmounted: index.unmounted({ parent, state }),
    getDecimal: index.getDecimal(props),
    handleFocus: index.handleFocus({ emit, state, props, api: api2, vm }),
    decrease: index.decrease({ api: api2, props, state }),
    increase: index.increase({ api: api2, props, state }),
    handleInput: index.handleInput({ state, api: api2, emit, props }),
    getNumPecision: index.getNumPecision({ api: api2, props }),
    displayValue: index.displayValue({ props, state, api: api2 }),
    internalDecrease: index.internalDecrease({ api: api2, state }),
    internalIncrease: index.internalIncrease({ api: api2, state }),
    handleInputChange: index.handleInputChange({ api: api2, state, props }),
    mouseEvent: index.mouseEvent({ api: api2, props, state }),
    handleBlur: index.handleBlur({ constants, dispatch, emit, props, state, api: api2 }),
    watchValue: index.watchValue({ api: api2, props, state }),
    setCurrentValue: index.setCurrentValue({ api: api2, constants, dispatch, emit, props, state }),
    getDisplayOnlyText: index.getDisplayOnlyText({ parent, props, state }),
    filterValue: index.filterValue({ state }),
    handleClear: index.handleClear({ state, emit }),
    handleChange: index.handleChange({ state, emit })
  });
  api2.getDecimal(0);
};
const initWatch = ({
  watch,
  props,
  api: api2
}) => {
  watch(
    () => [props.max, props.min],
    ([curMax, curMin]) => {
      if (curMax < curMin) {
        throw new Error("[Numeric]: The maximum value should not be less than to the minimum value");
      }
    },
    { immediate: true }
  );
  watch(() => props.modelValue, api2.watchValue, { immediate: true });
};
const renderless = (props, { computed, onMounted, onUpdated, onUnmounted, reactive, watch, inject }, { parent, emit, vm, constants, dispatch, service, nextTick }) => {
  const api2 = {};
  const $service = index.initService(service);
  const state = initState({ reactive, computed, props, api: api2, constants, $service, parent });
  parent.tinyForm = parent.tinyForm || inject("form", null);
  initApi({ api: api2, props, state, parent, vm, emit, dispatch, constants, nextTick });
  initWatch({ watch, props, api: api2 });
  onMounted(api2.mounted);
  onUpdated(api2.updated);
  onUnmounted(api2.unmounted);
  return api2;
};
exports.api = api;
exports.renderless = renderless;
