"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index$1 = require("./index.js");
const mb = require("./mb.js");
const index = require("../common/index.js");
const api = [
  "state",
  "btnClick",
  "handleEndChange",
  "handleEndInput",
  "focus",
  "handleFocus",
  "handleStartChange",
  "handleStartInput",
  "handleKeydown",
  "handleClickIcon",
  "handleMouseEnter",
  "handleInput",
  "handleChange",
  "handleClose",
  "handlePick",
  "handleSelectRange",
  "handleSelectChange",
  "popperElm",
  "handleEnterDisplayOnlyContent",
  "handleEnterPickerlabel",
  "timeMobileToggle",
  "timeMobileConfirm",
  "emitInput",
  "dateMobileToggle"
];
const initState = ({ api: api2, reactive, vm, computed, props, utils, parent, breakpoint }) => {
  const state = reactive({
    historyValue: [],
    historyInput: [],
    historyUserInput: [],
    historyUserValue: [],
    startStatus: false,
    endStatus: false,
    date: null,
    pickerVisible: false,
    showClose: false,
    userInput: null,
    valueOnOpen: null,
    popperElm: null,
    unwatchPickerOptions: null,
    ranged: computed(() => state.type.includes(index.DATEPICKER.Range)),
    reference: computed(() => vm.$refs.reference.$el || vm.$refs.reference),
    formDisabled: computed(() => (parent.tinyForm || {}).disabled),
    refInput: computed(() => state.reference ? [].slice.call(state.reference.querySelectorAll("input")) : []),
    valueIsEmpty: computed(() => api2.getValueEmpty()),
    triggerClass: computed(() => api2.computedTriggerClass()),
    selectionMode: computed(() => api2.getMode()),
    haveTrigger: computed(() => api2.computedHaveTrigger()),
    displayValue: computed(() => api2.displayValue()),
    parsedValue: computed(() => api2.parsedValue()),
    oldValue: props.modelValue,
    pickerSize: computed(() => props.size),
    pickerDisabled: computed(() => props.disabled || state.formDisabled || state.isDisplayOnly),
    firstInputId: computed(() => api2.firstInputId()),
    secondInputId: computed(() => api2.secondInputId()),
    type: computed(() => api2.getType()),
    timezone: computed(() => api2.getTimezone()),
    valueFormat: computed(() => index$1.getValueFormat({ props, utils })),
    format: computed(() => api2.computedFormat()),
    labelTooltip: "",
    displayOnlyTooltip: "",
    isDisplayOnly: computed(() => props.displayOnly || (parent.tinyForm || {}).displayOnly),
    isMobileScreen: computed(() => breakpoint.current.value === "default"),
    dateMobileOption: {
      visible: false,
      type: props.type,
      value: props.modelValue
    },
    timeMobileOption: {
      visible: false,
      type: props.type,
      value: [0, 0, 0],
      defaultValue: [0, 0, 0]
    },
    isDateMobileComponent: computed(
      () => [
        index.DATEPICKER.DateRange,
        index.DATEPICKER.DateTimeRange,
        index.DATEPICKER.DateTime,
        index.DATEPICKER.DatePicker,
        index.DATEPICKER.Date,
        index.DATEPICKER.Dates
      ].includes(props.type)
    ),
    isTimeMobileComponent: computed(
      () => [index.DATEPICKER.Time, index.DATEPICKER.TimeRange, index.DATEPICKER.TimeSelect, index.DATEPICKER.TimePicker].includes(props.type)
    ),
    showSeconds: computed(
      () => (state.format || props.pickerOptions && props.pickerOptions.format || "ss").includes("ss")
    )
  });
  return state;
};
const initApi = ({ api: api2, props, hooks, state, vnode, others, utils, parent }) => {
  const { t, emit, dispatch, nextTick, vm } = vnode;
  const { TimePanel, TimeRangePanel } = others;
  const { destroyPopper, popperElm, updatePopper } = index$1.initPopper({ props, hooks, vnode });
  state.popperElm = popperElm;
  state.picker = null;
  Object.assign(api2, {
    destroyPopper,
    emitDbTime: index$1.emitDbTime({ emit, state, t }),
    hidePicker: index$1.hidePicker({ destroyPopper, state }),
    handleSelectChange: ({ tz, date }) => !state.ranged && emit("select-change", { tz, date }),
    getPanel: index$1.getPanel(others),
    handleFocus: index$1.handleFocus({ emit, vm, state, api: api2 }),
    getTimezone: index$1.getTimezone({ props, utils }),
    emitChange: index$1.emitChange({ api: api2, dispatch, emit, props, state }),
    parsedValue: index$1.parsedValue({ api: api2, props, state, t }),
    parseAsFormatAndType: index$1.parseAsFormatAndType({ api: api2 }),
    typeValueResolveMap: index$1.typeValueResolveMap({ api: api2, props, t }),
    updateOptions: index$1.updateOptions({ api: api2, props, state }),
    focus: index$1.focus({ api: api2, props, vm }),
    handleInput: index$1.handleInput({ api: api2, state, props }),
    handleChange: index$1.handleChange({ api: api2, state }),
    isValidValue: index$1.isValidValue({ api: api2, state }),
    emitInput: index$1.emitInput({ api: api2, emit, props, state }),
    handleKeydown: index$1.handleKeydown({ api: api2, state }),
    handleEndChange: index$1.handleEndChange({ api: api2, state }),
    handleStartChange: index$1.handleStartChange({ api: api2, state }),
    handleClickIcon: index$1.handleClickIcon({ api: api2, props, state }),
    showPicker: index$1.showPicker({ api: api2, nextTick, updatePopper, state }),
    formatToValue: index$1.formatToValue({ api: api2, state }),
    formatAsFormatAndType: index$1.formatAsFormatAndType({ api: api2 }),
    parseString: index$1.parseString({ api: api2, state }),
    handleClose: index$1.handleClose({ api: api2, props, state }),
    displayValue: index$1.displayValue({ api: api2, props, state }),
    handlePick: index$1.handlePick({ api: api2, state }),
    watchPickerVisible: index$1.watchPickerVisible({ api: api2, vm, dispatch, emit, props, state, nextTick }),
    watchMobileVisible: index$1.watchMobileVisible({ api: api2, props, state, nextTick }),
    formatToString: index$1.formatToString({ api: api2, state }),
    watchIsRange: index$1.watchIsRange({ api: api2, state, TimePanel, TimeRangePanel }),
    mountPicker: index$1.mountPicker({ api: api2, vm, props, state, updatePopper }),
    watchModelValue: index$1.watchModelValue({ api: api2, props, state, dispatch }),
    computedFormat: index$1.computedFormat({ props, utils }),
    computedTriggerClass: index$1.computedTriggerClass({ props, state }),
    computedHaveTrigger: index$1.computedHaveTrigger({ props }),
    setInputPaddingLeft: index$1.setInputPaddingLeft({ props, state, vm, nextTick }),
    formatInputValue: index$1.formatInputValue({ props, state })
  });
  initApi2({ api: api2, props, state, t, parent });
  initMobileApi({ api: api2, props, state });
};
const initApi2 = ({ api: api2, props, state, t, parent }) => {
  Object.assign(api2, {
    t,
    state,
    blur: index$1.blur(state),
    getMode: index$1.getMode({ state }),
    getType: index$1.getType({ props, parent }),
    dateParser: index$1.dateParser({ t, props }),
    rangeParser: index$1.rangeParser(api2),
    rangeFormatter: index$1.rangeFormatter(api2),
    dateFormatter: index$1.dateFormatter({ t }),
    getValueEmpty: index$1.getValueEmpty(props),
    handleEndInput: index$1.handleEndInput({ state, props, api: api2 }),
    handleStartInput: index$1.handleStartInput({ state, props, api: api2 }),
    firstInputId: index$1.firstInputId({ props, state }),
    secondInputId: index$1.secondInputId({ props, state }),
    handleMouseEnter: index$1.handleMouseEnter({ props, state }),
    initGlobalTimezone: index$1.initGlobalTimezone({ api: api2, state, props }),
    parseValue: index$1.parseValue({ api: api2, props, state }),
    handleSelectRange: index$1.handleSelectRange(state),
    handleEnterPickerlabel: index$1.handleEnterPickerlabel({ state, props }),
    handleEnterDisplayOnlyContent: index$1.handleEnterDisplayOnlyContent({ state, t })
  });
};
const initMobileApi = ({ api: api2, props, state }) => {
  Object.assign(api2, {
    dateMobileToggle: mb.dateMobileToggle({ state, props }),
    timeMobileToggle: mb.timeMobileToggle({ state, props, api: api2 }),
    timeMobileConfirm: mb.timeMobileConfirm({ state, api: api2 }),
    dateToTimeArray: mb.dateToTimeArray,
    timeArrayToDate: mb.timeArrayToDate({ props })
  });
};
const initWatch = ({ api: api2, state, props, watch, markRaw }) => {
  watch(
    () => state.type,
    (type) => state.panel = markRaw(api2.getPanel(type)),
    { immediate: true }
  );
  watch(() => [state.dateMobileOption.visible, state.timeMobileOption.visible], api2.watchMobileVisible);
  watch(() => state.pickerVisible, api2.watchPickerVisible);
  watch(
    () => props.defaultValue,
    (value) => state.picker && (state.picker.state.defaultValue = value)
  );
  watch(
    () => state.parsedValue,
    (value) => state.picker && (state.picker.state.value = value),
    { immediate: true }
  );
  watch(() => props.type, api2.mountPicker);
  watch(() => props.isRange, api2.watchIsRange);
  watch(() => props.modelValue, api2.watchModelValue);
  watch(() => props.pickerOptions, api2.updateOptions, { deep: true });
  watch(() => props.label, api2.setInputPaddingLeft);
};
const renderless = (props, hooks, vnode, others) => {
  const api2 = {};
  const { reactive, computed, watch, onBeforeUnmount, inject, markRaw, onMounted } = hooks;
  const { vm, service, parent, useBreakpoint } = vnode;
  const { utils = {} } = service || {};
  const breakpoint = useBreakpoint();
  const state = initState({ api: api2, reactive, vm, computed, props, utils, parent, breakpoint });
  parent.tinyForm = parent.tinyForm || inject("form", null);
  initApi({ api: api2, props, hooks, state, vnode, others, utils, parent });
  initWatch({ api: api2, state, props, watch, markRaw });
  api2.initGlobalTimezone();
  onMounted(() => {
    api2.setInputPaddingLeft();
  });
  parent.$on("handle-clear", (event) => {
    state.showClose = true;
    api2.handleClickIcon(event);
  });
  onBeforeUnmount(() => {
    api2.destroyPopper("remove");
    state.popperElm = null;
    state.picker = null;
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
