"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const string = require("../common/string.js");
const api = [
  "state",
  "closeSuccessTips",
  "abort",
  "beforeAvatarUpload",
  "beforeRemove",
  "handleAvatarSuccess",
  "handleExceed",
  "errorEvent",
  "progressEvent",
  "closeErrorTips",
  "uploadSubmit",
  "showDialog",
  "closeDialog",
  "fileUploadChange",
  "formatFileSize",
  "deleteFile"
];
const initState = ({ reactive, props, computed, api: api2 }) => reactive({
  errorTips: [],
  uploadList: [],
  successTips: [],
  limit: props.limit,
  isShowDialog: false,
  accept: props.accept,
  action: props.action,
  headers: props.headers,
  disabled: props.disabled,
  multiple: props.multiple,
  uploadFileType: props.uploadFileType,
  withCredentials: props.withCredentials,
  maxUploadFileSize: props.maxUploadFileSize,
  tipsTitle: [void 0, void 0, void 0],
  fileSize: computed(() => api2.computedFileSize()),
  fileName: computed(() => api2.computedFileName()),
  fileStatus: computed(() => api2.computedFileStatus()),
  // tiny新增
  fileWords: computed(() => api2.computedFileWords()),
  cancelButtonText: computed(() => api2.computedCancel()),
  submitButtonText: computed(() => api2.computedconfirm()),
  errorNumTips: computed(() => api2.computedErrorNumTips()),
  dialogTitle: computed(() => api2.computedSetDialogTitle()),
  errorSizeTips: computed(() => api2.computedErrorSizeTips()),
  tipsTitleText: computed(() => api2.computedTipsTitleText()),
  limitTypeTips: computed(() => api2.computedLimitTypeTips()),
  uploadErrorTips: computed(() => api2.computedUploadError()),
  errorTypeTips: computed(() => api2.computedErrorTypeTips()),
  limitSizeTips: computed(() => api2.computedLimitSizeTips()),
  limitCountTips: computed(() => api2.computedLimitCountTips()),
  uploadSuccessTips: computed(() => api2.computedUploadSuccess()),
  uploadButtonText: computed(() => api2.computedUploadButtonText()),
  uploadsButtonText: computed(() => api2.computedUploadsButtonText())
});
const initApi = ({ api: api2, state, refs, emit, props, constants, t }) => {
  Object.assign(api2, {
    state,
    formatFileSize: string.formatFileSize,
    abort: index.abort(refs),
    showDialog: index.showDialog(state),
    watchLimit: index.watchLimit(state),
    watchAction: index.watchAction(state),
    uploadSubmit: index.uploadSubmit(refs),
    watchAccept: index.watchAccept(state),
    progressEvent: index.progressEvent(emit),
    watchHeaders: index.watchHeaders(state),
    watchMultiple: index.watchMultiple(state),
    watchDisabled: index.watchDisabled(state),
    closeErrorTips: index.closeErrorTips(state),
    deleteFile: index.deleteFile({ emit, state, props }),
    errorEvent: index.errorEvent({ emit, state }),
    closeSuccessTips: index.closeSuccessTips(state),
    fileUploadChange: index.fileUploadChange(state),
    closeDialog: index.closeDialog({ refs, state }),
    handleExceed: index.handleExceed({ emit, state }),
    watchUploaFileType: index.watchUploaFileType(state),
    beforeAvatarUpload: index.beforeAvatarUpload({ props, state }),
    watchWithCredentials: index.watchWithCredentials(state),
    computedFileName: index.computedFileName({ constants, t }),
    computedFileSize: index.computedFileSize({ constants, t }),
    computedCancel: index.computedCancel({ constants, props, t }),
    computedFileStatus: index.computedFileStatus({ constants, t }),
    computedFileWords: index.computedFileWords({ t }),
    computedconfirm: index.computedconfirm({ constants, props, t }),
    computedUploadError: index.computedUploadError({ constants, t }),
    computedErrorNumTips: index.computedErrorNumTips({ constants, t }),
    handleAvatarSuccess: index.handleAvatarSuccess({ api: api2, emit, state }),
    computedErrorTypeTips: index.computedErrorTypeTips({ constants, t }),
    computedErrorSizeTips: index.computedErrorSizeTips({ constants, t }),
    computedLimitSizeTips: index.computedLimitSizeTips({ constants, t }),
    computedLimitTypeTips: index.computedLimitTypeTips({ constants, t }),
    computedTipsTitleText: index.computedTipsTitleText({ constants, t }),
    computedUploadSuccess: index.computedUploadSuccess({ constants, t }),
    computedLimitCountTips: index.computedLimitCountTips({ constants, t }),
    watchMaxUploadFileSize: index.watchMaxUploadFileSize({ constants, state }),
    computedUploadsButtonText: index.computedUploadsButtonText({ constants, t }),
    computedSetDialogTitle: index.computedSetDialogTitle({ constants, props, t }),
    computedUploadButtonText: index.computedUploadButtonText({ constants, props, t })
  });
};
const initWatch = ({ watch, props, api: api2 }) => {
  watch(() => props.withCredentials, api2.watchWithCredentials, { immediate: true });
  watch(() => props.uploadFileType, api2.watchUploaFileType, { immediate: true });
  watch(() => props.maxUploadFileSize, api2.watchMaxUploadFileSize, { immediate: true });
  watch(() => props.accept, api2.watchAccept, { immediate: true });
  watch(() => props.disabled, api2.watchDisabled, { immediate: true });
  watch(() => props.multiple, api2.watchMultiple, { immediate: true });
  watch(() => props.action, api2.watchAction, { immediate: true });
  watch(() => props.headers, api2.watchHeaders, { immediate: true });
  watch(() => props.limit, api2.watchLimit, { immediate: true });
};
const renderless = (props, { computed, reactive, watch }, { t, constants, emit, refs }) => {
  const api2 = {};
  const state = initState({ reactive, props, computed, api: api2 });
  initApi({ api: api2, state, refs, emit, props, constants, t });
  initWatch({ watch, props, api: api2 });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
