"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const dom = require("../common/deps/dom.js");
const string = require("../common/string.js");
const index = require("../common/index.js");
const processTrigger = ({
  api,
  state,
  props,
  nextTick
}) => {
  const { referenceElm, popperElm } = state;
  if (props.trigger === "click") {
    dom.on(referenceElm, "click", api.doToggle);
    dom.on(document, "click", api.handleDocumentClick);
  } else if (props.trigger === "hover") {
    dom.on(referenceElm, "mouseenter", api.handleMouseEnter);
    dom.on(popperElm, "mouseenter", api.handleMouseEnter);
    dom.on(referenceElm, "mouseleave", api.handleMouseLeave);
    dom.on(popperElm, "mouseleave", api.handleMouseLeave);
  } else if (props.trigger === "focus") {
    if (referenceElm.querySelector("input, textarea")) {
      dom.on(referenceElm, "focusin", api.doShow);
      dom.on(referenceElm, "focusout", api.doClose);
    } else {
      dom.on(referenceElm, "mousedown", api.doShow);
      dom.on(referenceElm, "mouseup", api.doClose);
    }
  } else if (props.trigger === "manual") {
    if (props.modelValue) {
      nextTick(api.doShow);
    }
  }
};
const mounted = ({
  api,
  state,
  constants,
  props,
  nextTick,
  mode
}) => () => {
  state.mounted = true;
  const { referenceElm, popperElm, tooltipId } = state;
  if (referenceElm) {
    if (mode !== "mobile-first") {
      dom.addClass(referenceElm, `${constants.IDPREFIX}__reference`);
    }
    referenceElm.setAttribute("aria-describedby", tooltipId);
    referenceElm.setAttribute("tabindex", props.tabindex.toString());
    popperElm.setAttribute("tabindex", 0);
    if (props.trigger !== "click") {
      dom.on(referenceElm, "focusin", () => {
        api.handleFocus();
        const instance = referenceElm.__vue__;
        if (instance && typeof instance.focus === "function") {
          instance.focus();
        }
      });
      dom.on(popperElm, "focusin", api.handleFocus);
      dom.on(referenceElm, "focusout", api.handleBlur);
      dom.on(popperElm, "focusout", api.handleBlur);
    }
    dom.on(referenceElm, "keydown", api.handleKeydown);
    dom.on(referenceElm, "click", api.handleClick);
  }
  processTrigger({ api, state, props, nextTick });
};
const doToggle = (state) => () => {
  state.showPopper = !state.showPopper;
};
const doShow = (state) => () => {
  state.showPopper = true;
};
const doClose = (state) => () => {
  state.showPopper = false;
};
const handleFocus = ({ props, state }) => () => {
  dom.addClass(state.referenceElm, "focusing");
  if (props.trigger === "click" || props.trigger === "focus") {
    state.showPopper = true;
  }
};
const handleClick = (state) => (event) => {
  const popperElm = state.popperElm;
  if ((event == null ? void 0 : event.target) && popperElm) {
    state.webCompEventTarget = event.target;
  }
  dom.removeClass(state.referenceElm, "focusing");
};
const handleBlur = ({ props, state }) => () => {
  dom.removeClass(state.referenceElm, "focusing");
  if (props.trigger === "click" || props.trigger === "focus") {
    state.showPopper = false;
  }
};
const handleMouseEnter = ({ props, state }) => () => {
  clearTimeout(state.timer);
  if (props.openDelay) {
    state.timer = window.setTimeout(() => {
      state.showPopper = true;
    }, props.openDelay);
  } else {
    state.showPopper = true;
  }
};
const handleKeydown = ({ api, props }) => (event) => {
  if (event.keyCode === index.KEY_CODE.Escape && props.trigger !== "manual") {
    api.doClose();
  }
};
const handleMouseLeave = ({ props, state }) => () => {
  clearTimeout(state.timer);
  if (props.closeDelay) {
    state.timer = window.setTimeout(() => {
      state.showPopper = false;
    }, props.closeDelay);
  } else {
    state.showPopper = false;
  }
};
const handleDocumentClick = ({ vm, state }) => (event) => {
  const reference = state.referenceElm;
  const popperElm = state.popperElm;
  const $el = vm.$refs.root;
  let target = event.target;
  if ((target == null ? void 0 : target.shadowRoot) && popperElm) {
    target = state.webCompEventTarget;
  }
  if (!$el || !reference || $el.contains(target) || reference.contains(target) || !popperElm || popperElm.contains(target)) {
    return false;
  } else {
    state.showPopper = false;
    return true;
  }
};
const handleAfterEnter = (emit) => () => {
  emit("after-enter");
};
const handleAfterLeave = (emit) => () => {
  emit("after-leave");
};
const handleItemClick = ({ emit, state }) => (item) => {
  state.showPopper = false;
  emit("item-click", item);
};
const cleanup = ({ props, state }) => () => {
  if (props.openDelay) {
    clearTimeout(state.timer);
  }
};
const destroyed = ({ state, api }) => () => {
  const { referenceElm, popperElm } = state;
  dom.off(referenceElm, "click", api.doToggle);
  dom.off(referenceElm, "mouseup", api.doClose);
  dom.off(referenceElm, "mousedown", api.doShow);
  dom.off(referenceElm, "focusin", api.doShow);
  dom.off(referenceElm, "focusout", api.doClose);
  dom.off(referenceElm, "mouseleave", api.handleMouseLeave);
  dom.off(referenceElm, "mouseenter", api.handleMouseEnter);
  dom.off(document, "click", api.handleDocumentClick);
  dom.off(popperElm, "focusin", api.handleFocus);
  dom.off(popperElm, "focusout", api.handleBlur);
  dom.off(popperElm, "mouseenter", api.handleMouseEnter);
  dom.off(popperElm, "mouseleave", api.handleMouseLeave);
  dom.off(referenceElm, "click", api.handleClick);
  dom.off(referenceElm, "focusout", api.handleBlur);
  dom.off(referenceElm, "keydown", api.handleKeydown);
};
const computedTooltipId = (constants) => () => `${constants.IDPREFIX}-${string.guid("", 4)}`;
const wrapMounted = ({ api, props, vm, state }) => () => {
  const { reference, popper, wrapper } = vm.$refs;
  const referenceElm = state.referenceElm = props.reference || reference;
  state.popperElm = state.popperElm || popper;
  if (!referenceElm && wrapper.children) {
    state.referenceElm = wrapper.children[0] || wrapper;
  }
  state.referenceElm && api.mounted();
};
const observeCallback = ({ state, vm }) => (mutationsList) => {
  for (let mutation of mutationsList) {
    if (mutation.type === "attributes" && mutation.attributeName === "x-placement") {
      state.xPlacement = vm.$refs.popper.getAttribute("x-placement") || "bottom";
    }
  }
};
exports.cleanup = cleanup;
exports.computedTooltipId = computedTooltipId;
exports.destroyed = destroyed;
exports.doClose = doClose;
exports.doShow = doShow;
exports.doToggle = doToggle;
exports.handleAfterEnter = handleAfterEnter;
exports.handleAfterLeave = handleAfterLeave;
exports.handleBlur = handleBlur;
exports.handleClick = handleClick;
exports.handleDocumentClick = handleDocumentClick;
exports.handleFocus = handleFocus;
exports.handleItemClick = handleItemClick;
exports.handleKeydown = handleKeydown;
exports.handleMouseEnter = handleMouseEnter;
exports.handleMouseLeave = handleMouseLeave;
exports.mounted = mounted;
exports.observeCallback = observeCallback;
exports.wrapMounted = wrapMounted;
