"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = [
  "state",
  "getCurrentColor",
  "getLevelColor",
  "getColorArray",
  "customBeforeAppearHook",
  "customAppearHook",
  "customAfterAppearHook"
];
const renderless = (props, { computed, reactive }, { constants, mode }) => {
  const api2 = {};
  const state = reactive({
    percentTextSize: constants.TEXT_XS,
    rate: computed(() => api2.computedRate()),
    radius: computed(() => api2.computedRadius()),
    stroke: computed(() => api2.computedStroke()),
    content: computed(() => api2.computedContent()),
    barStyle: computed(() => api2.computedBarStyle()),
    trackPath: computed(() => api2.computedTrackPath()),
    perimeter: computed(() => api2.computedPerimeter()),
    iconClass: computed(() => api2.computedIconClass()),
    iconStyle: computed(() => api2.computedIconStyle()),
    circleStyle: computed(() => api2.computedCircleStyle()),
    trailPathStyle: computed(() => api2.computedTrailPathStyle()),
    circlePathStyle: computed(() => api2.computedCirclePathStyle()),
    progressTextSize: computed(() => api2.computedProgressTextSize()),
    strokeDashoffset: computed(() => api2.computedStrokeDashoffset()),
    strokeWidth: computed(
      () => mode === "mobile-first" ? props.strokeWidth : props.strokeWidth || constants.DEFAULT_STROKE_WIDTH
    ),
    width: computed(() => mode === "mobile-first" ? props.width : props.width || constants.DEFAULT_WIDTH),
    relativeStrokeWidth: computed(() => api2.computedRelativeStrokeWidth())
  });
  Object.assign(api2, {
    state,
    customAppearHook: index.customAppearHook,
    computedContent: index.computedContent({ props }),
    getColorArray: index.getColorArray({ props }),
    computedRate: index.computedRate({ constants, props }),
    computedPerimeter: index.computedPerimeter({ state }),
    computedRadius: index.computedRadius({ constants, props, state }),
    computedTrackPath: index.computedTrackPath({ constants, props, state }),
    computedIconClass: index.computedIconClass({ constants, props, mode }),
    computedIconStyle: index.computedIconStyle({ constants, props, state }),
    computedCircleStyle: index.computedCircleStyle({ state }),
    computedCirclePathStyle: index.computedCirclePathStyle({ props, state }),
    computedStrokeDashoffset: index.computedStrokeDashoffset({ state }),
    computedTrailPathStyle: index.computedTrailPathStyle({ state }),
    computedRelativeStrokeWidth: index.computedRelativeStrokeWidth({ state, constants }),
    computedProgressTextSize: index.computedProgressTextSize({ state, constants, props, mode }),
    customAfterAppearHook: index.customAfterAppearHook({ state, props }),
    customBeforeAppearHook: index.customBeforeAppearHook({ props, state }),
    getLevelColor: index.getLevelColor({ api: api2 }),
    computedBarStyle: index.computedBarStyle({ api: api2, props }),
    getCurrentColor: index.getCurrentColor({ api: api2, props }),
    computedStroke: index.computedStroke({ api: api2, constants, props })
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
