"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const dom = require("../common/deps/dom.js");
const useTouch = require("../common/deps/useTouch.js");
const useEventListener = require("../common/deps/useEventListener.js");
const touchEmulator = require("../common/deps/touch-emulator.js");
touchEmulator.default();
const api = ["state", "constants", "onTouchStart", "onTouchEnd", "getStatusText"];
const renderless = (props, { ref, unref, toRef, isRef, reactive, computed, watch, onMounted, onUnmounted, onActivated, onDeactivated }, { vm, emit, constants, t, nextTick, slots }, { Loading }) => {
  const api2 = {};
  const state = reactive({
    replaces: "",
    styleObj: {},
    translate3d: 0,
    draggposition: 0,
    value: props.modelValue,
    checkStatus: false,
    headHeight: props.headHeight,
    pullingText: props.pullingText,
    loosingText: props.loosingText,
    successText: props.successText,
    successDuration: props.successDuration,
    animationDuration: props.animationDuration,
    disabled: computed(() => props.disabled),
    /* mf state */
    reachTop: false,
    scrollParent: null,
    scrollTrack: null,
    status: constants.STATUS.NORMAL,
    distance: 0,
    duration: 0,
    headerHeight: ""
  });
  const touchState = {};
  Object.assign(api2, {
    state,
    constants,
    onTouchstart: index.onTouchstart({ state }),
    onTouchmove: index.onTouchmove({ vm, state }),
    onTouchend: index.onTouchend({ emit, props, state }),
    mountedHandler: index.mountedHandler({ api: api2, vm, state }),
    beforeUnmountHandler: index.beforeUnmountHandler({ api: api2, vm }),
    useTouch: useTouch.useTouch(ref),
    useEventListener: useEventListener.useEventListener({
      unref,
      isRef,
      watch,
      nextTick,
      onMounted,
      onUnmounted,
      onActivated,
      onDeactivated
    }),
    isTouchable: index.isTouchable({ props, state, constants }),
    ease: index.ease({ state, props }),
    setStatus: index.setStatus({ emit, props, state, constants }),
    getStatusText: index.getStatusText({ props, state, t, constants }),
    showSuccessTip: index.showSuccessTip({ api: api2, props, state, constants }),
    checkPosition: index.checkPosition({ state, touchState }),
    onTouchStart: index.onTouchStart(api2),
    onTouchMove: index.onTouchMove({ api: api2, state, touchState }),
    onTouchEnd: index.onTouchEnd({ api: api2, emit, nextTick, state, props, touchState, constants }),
    watchModelValueChange: index.watchModelValueChange({ api: api2, slots, state, props }),
    watchStatusChange: index.watchStatusChange({ api: api2, state, props, Loading, vm, constants, slots }),
    parseHeaderHeight: index.parseHeaderHeight({ state, props, constants }),
    checkSelfSimulate: index.checkSelfSimulate({ state, props })
  });
  watch(
    () => props.modelValue,
    (value) => api2.watchModelValueChange(value)
  );
  watch(
    () => state.status,
    (value, oldValue) => api2.watchStatusChange(value, oldValue)
  );
  watch(
    () => props.headHeight,
    () => api2.parseHeaderHeight(),
    { immediate: true }
  );
  Object.assign(touchState, api2.useTouch());
  state.stopHandle = api2.useEventListener("touchmove", api2.onTouchMove, { target: toRef(state, "scrollTrack") });
  onMounted(() => {
    state.scrollParent = dom.getScrollParent(vm.$refs.root);
    state.scrollTrack = vm.$refs.track;
  });
  onUnmounted(() => {
    typeof state.stopHandle === "function" && state.stopHandle();
    state.scrollParent = null;
    state.scrollTrack = null;
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
