"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const dateUtil = require("../common/deps/date-util.js");
const index = require("../common/index.js");
const getTarget = (event) => {
  let target = event.target;
  const tagName = target.tagName;
  if (tagName === "A") {
    target = target.parentNode.parentNode;
  }
  if (tagName === "DIV") {
    target = target.parentNode;
  }
  if (target.tagName !== "TD") {
    return;
  }
  return target;
};
const handleQuarterTableClick = ({ state, emit }) => (event) => {
  const target = getTarget(event);
  const currentDate = new Date(state.date.getFullYear(), index.DATEPICKER.QuarterMap[target.cellIndex], 1);
  state.value = currentDate;
  emit("pick", currentDate);
};
const showYearPicker = ({ state }) => () => state.currentView = index.DATEPICKER.Year;
const cusPrevYear = ({ state }) => () => {
  if (state.currentView === index.DATEPICKER.Year) {
    state.startYear = state.startYear - index.DATEPICKER.PanelYearNum;
  } else {
    state.date = dateUtil.prevYear(state.date);
  }
};
const cusNextYear = ({ state }) => () => {
  if (state.currentView === index.DATEPICKER.Year) {
    state.startYear = state.startYear + index.DATEPICKER.PanelYearNum;
  } else {
    state.date = dateUtil.nextYear(state.date);
  }
};
const handleYearPick = ({ state }) => (value) => {
  state.currentView = index.DATEPICKER.Quarter;
  state.date = dateUtil.modifyDate(state.date, value, state.date.getMonth(), state.date.getDate());
};
const getYearLabel = ({ state, t }) => () => {
  return state.date.getFullYear();
};
const getCellStyle = ({ api, props, state }) => (cell) => {
  const year = state.date.getFullYear();
  const quarter = cell.text.slice(1) - 1;
  const style = {};
  style.current = state.value && state.value.getFullYear() === year && state.value.getMonth() === index.DATEPICKER.QuarterMap[quarter];
  return style;
};
exports.cusNextYear = cusNextYear;
exports.cusPrevYear = cusPrevYear;
exports.getCellStyle = getCellStyle;
exports.getYearLabel = getYearLabel;
exports.handleQuarterTableClick = handleQuarterTableClick;
exports.handleYearPick = handleYearPick;
exports.showYearPicker = showYearPicker;
