"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const chunkG2ADBYYC = require("../chunk-G2ADBYYC.js");
const index = require("./index.js");
const dom = require("../common/deps/dom.js");
const api = [
  "state",
  "showDecimalIcon",
  "getIconStyle",
  "getTextStyle",
  "selectValue",
  "handelKey",
  "setCurrentValue",
  "resetCurrentValue"
];
const useChangeValue = ({
  constants,
  emit,
  props,
  reactive,
  toRefs,
  watch,
  onMounted,
  onUnmounted,
  parent,
  inject
}) => {
  parent.tinyForm = parent.tinyForm || inject("form", null);
  const state = reactive({
    hoverIndex: -1,
    currentValue: props.modelValue
  });
  const api2 = {
    selectValue: index.selectValue({ emit, props, state }),
    resetCurrentValue: index.resetCurrentValue({ props, state })
  };
  api2.setCurrentValue = index.setCurrentValue({
    constants,
    props,
    state,
    parent,
    api: api2
  });
  watch(
    () => props.modelValue,
    (value) => {
      state.currentValue = value;
    },
    { immediate: true }
  );
  const updateMousePostion = (event) => {
    state.mouseTarget = event.target;
  };
  onMounted(() => {
    dom.on(document, "mousemove", updateMousePostion);
    dom.on(document, "mouseleave", api2.resetCurrentValue);
  });
  onUnmounted(() => {
    dom.off(document, "mousemove", updateMousePostion);
    dom.off(document, "mouseleave", api2.resetCurrentValue);
    state.mouseTarget = null;
  });
  return {
    api: api2,
    state: toRefs(state)
  };
};
const renderless = (props, { computed, reactive, toRefs, watch, onMounted, onUnmounted, inject }, { constants, emit, parent }) => {
  const api2 = {};
  const changeValue = useChangeValue({
    constants,
    emit,
    props,
    reactive,
    toRefs,
    watch,
    onMounted,
    onUnmounted,
    parent,
    inject
  });
  const state = reactive(chunkG2ADBYYC.__spreadValues({
    pointerAtLeftHalf: true,
    colorMap: computed(() => api2.computedColorMap(props)),
    classMap: computed(() => api2.computedClassMap(props)),
    text: computed(() => api2.computedText({ props, state })),
    activeClass: computed(() => api2.computedActiveClass(state)),
    activeColor: computed(() => api2.computedActiveColor(state)),
    classes: computed(() => api2.computedClasses({ props, state })),
    decimalIconClass: computed(() => api2.computedActiveClass(state)),
    voidClass: computed(() => api2.computedVoidClass({ props, state })),
    decimalStyle: computed(() => api2.computedDecimalStyle({ props, state }))
  }, changeValue.state));
  Object.assign(api2, chunkG2ADBYYC.__spreadValues({
    state,
    computedText: index.computedText,
    computedClasses: index.computedClasses,
    computedClassMap: index.computedClassMap,
    computedColorMap: index.computedColorMap,
    computedVoidClass: index.computedVoidClass,
    computedDecimalStyle: index.computedDecimalStyle,
    getTextStyle: index.getTextStyle({ props, state }),
    handelKey: index.handelKey({ emit, props, state }),
    computedActiveColor: index.computedActiveColor(props),
    computedActiveClass: index.computedActiveClass(props),
    showDecimalIcon: index.showDecimalIcon({ props, state }),
    getIconStyle: index.getIconStyle({ api: api2, props, state })
  }, changeValue.api));
  return api2;
};
exports.api = api;
exports.renderless = renderless;
exports.useChangeValue = useChangeValue;
