"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const resizeEvent = require("../common/deps/resize-event.js");
const api = [
  "state",
  "handleVisibilityChange",
  "handleScroll",
  "scrollToItem",
  "computeViewStyle",
  "computeViewEvent",
  "updateVisibleItems",
  "resetTemporary"
];
const addWatchers = ({ watch, props, api: api2, state }) => {
  watch(
    () => props.items,
    () => api2.init()
  );
  watch(
    () => props.pageMode,
    () => {
      api2.applyPageMode();
      api2.updateVisibleItems(false);
    }
  );
  watch(
    () => state.sizes,
    () => api2.updateVisibleItems(false),
    { deep: true }
  );
  watch(
    () => props.gridItems,
    () => api2.updateVisibleItems(true)
  );
  watch(
    () => props.itemSecondarySize,
    () => api2.updateVisibleItems(true)
  );
};
const checkProps = (props) => {
  if (props.gridItems && !props.itemSize) {
    throw new Error("[TINY Error][RecycleScroller] You must provide an itemSize when using gridItems");
  }
};
const renderless = (props, { reactive, computed, onBeforeUnmount, onMounted, onActivated, watch, markRaw, shallowReactive }, { vm, nextTick, emit }) => {
  const state = reactive({
    pool: [],
    totalSize: 0,
    ready: false,
    hoverKey: null,
    simpleArray: computed(() => props.items.length && typeof props.items[0] !== "object"),
    sizes: computed(() => api2.computedSizes()),
    itemIndexByKey: computed(() => api2.computedItemIndexByKey())
  });
  const api2 = {
    state,
    computedSizes: index.computedSizes({ props, state }),
    computedItemIndexByKey: index.computedItemIndexByKey(props),
    getScroll: index.getScroll({ props, vm }),
    unuseView: index.unuseView(state),
    addView: index.addView({ markRaw, shallowReactive }),
    sortViews: index.sortViews(state),
    getListenerTarget: index.getListenerTarget({ props, vm }),
    scrollToPosition: index.scrollToPosition({ props, vm }),
    computeViewStyle: index.computeViewStyle({ props, state }),
    computeViewEvent: index.computeViewEvent({ props, state })
  };
  Object.assign(api2, {
    handleVisibilityChange: index.handleVisibilityChange({ api: api2, emit, state }),
    updateVisibleItems: index.updateVisibleItems({ api: api2, emit, props, state, vm }),
    handleScroll: index.handleScroll({ api: api2, props, state }),
    handleResize: index.handleResize({ api: api2, emit, state }),
    applyPageMode: index.applyPageMode({ api: api2, props }),
    addListeners: index.addListeners({ api: api2, state }),
    removeListeners: index.removeListeners({ api: api2, state }),
    scrollToItem: index.scrollToItem({ api: api2, props, state }),
    resetTemporary: index.resetTemporary({ state }),
    init: index.init({ api: api2 })
  });
  state.temporary = {
    startIndex: 0,
    endIndex: 0,
    views: /* @__PURE__ */ new Map(),
    unusedViews: /* @__PURE__ */ new Map(),
    scrollDirty: false,
    lastUpdateScrollPosition: 0
  };
  if (props.prerender) {
    state.temporary.prerender = true;
    api2.updateVisibleItems(false);
  }
  checkProps(props);
  onActivated(() => {
    const lastPosition = state.temporary.lastUpdateScrollPosition;
    if (typeof lastPosition === "number") {
      nextTick(() => {
        api2.scrollToPosition(lastPosition);
      });
    }
  });
  onMounted(() => {
    resizeEvent.addResizeListener(vm.$el, api2.handleResize);
    api2.applyPageMode();
    nextTick(() => {
      state.temporary.prerender = false;
      api2.updateVisibleItems(true);
      state.ready = true;
    });
  });
  onBeforeUnmount(() => {
    resizeEvent.removeResizeListener(vm.$el, api2.handleResize);
    api2.removeListeners();
  });
  addWatchers({ watch, props, api: api2, state });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
