"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const chunkG2ADBYYC = require("../chunk-G2ADBYYC.js");
const index = require("./index.js");
const debounce = require("../common/deps/debounce.js");
const type = require("../common/type.js");
const useUserAgent = require("../common/deps/useUserAgent.js");
const browser = require("../common/browser.js");
const api = [
  "state",
  "nodeCollapse",
  "nodeExpand",
  "toggleCheckAll",
  "handleCopyClick",
  "focus",
  "blur",
  "showTip",
  "doDestroy",
  "getOption",
  "emitChange",
  "handleBlur",
  "toggleMenu",
  "getValueKey",
  "handleFocus",
  "handleClose",
  "setSoftFocus",
  "getValueIndex",
  "scrollToOption",
  "resetHoverIndex",
  "onOptionDestroy",
  "resetInputWidth",
  "resetInputHeight",
  "managePlaceholder",
  "checkDefaultFirstOption",
  "setOptionHighlight",
  "toggleLastOptionHitState",
  "deleteTag",
  "setSelected",
  "selectOption",
  "handleResize",
  "deletePrevTag",
  "onInputChange",
  "deleteSelected",
  "handleMenuEnter",
  "resetInputState",
  "handleClearClick",
  "handleComposition",
  "handleQueryChange",
  "handleOptionSelect",
  "debouncedOnInputChange",
  "debouncedQueryChange",
  "navigateOptions",
  "selectChange",
  "radioChange",
  "treeNodeClick",
  "nodeCheckClick",
  "buildSelectConfig",
  "buildRadioConfig",
  "onMouseenterNative",
  "onMouseleaveNative",
  "onMouseenterSelf",
  "onCopying",
  "handleDropdownClick",
  "handleEnterTag",
  "getLabelSlotValue",
  "loadTreeData",
  "updateModelValue",
  "clearSearchText",
  "onClickCollapseTag",
  "computedShowTagText",
  "isTagClosable"
];
const initState = ({ reactive, computed, props, api: api2, emitter, parent, constants, useBreakpoint, vm, designConfig }) => {
  const stateAdd = initStateAdd({ computed, props, api: api2, parent });
  const state = reactive(chunkG2ADBYYC.__spreadProps(chunkG2ADBYYC.__spreadValues({}, stateAdd), {
    selectEmitter: emitter(),
    datas: [],
    initDatas: [],
    query: "",
    options: [],
    visible: false,
    showCopy: computed(() => api2.computedShowCopy()),
    showWarper: true,
    // 显示下拉外层控制
    selected: props.multiple ? [] : {},
    softFocus: false,
    hover: false,
    triggerSearch: false,
    firstAutoSearch: props.remoteConfig.autoSearch,
    tagsStyle: computed(() => api2.computedTagsStyle()),
    readonly: computed(() => api2.computedReadonly()),
    iconClass: computed(() => state.visible ? "" : constants.CLASS.IsReverse),
    showClose: computed(() => api2.computedShowClose()),
    optionsAllDisabled: computed(() => api2.computedOptionsAllDisabled()),
    collapseTagSize: computed(() => api2.computedCollapseTagSize()),
    showNewOption: computed(() => api2.computedShowNewOption()),
    selectSize: computed(() => browser.isBrowser ? props.size || state.formItemSize : 0),
    optimizeOpts: computed(() => api2.computeOptimizeOpts()),
    optimizeStore: { valueIndex: 0, recycleScrollerHeight: computed(() => api2.recycleScrollerHeight()) },
    collapseTags: computed(() => api2.computeCollapseTags()),
    multipleLimit: computed(() => api2.computeMultipleLimit()),
    disabledTooltipContent: computed(() => api2.computedDisabledTooltipContent()),
    isExpand: computed(() => api2.computedIsExpand()),
    collapseTagsLength: 0,
    initValue: [],
    key: 0,
    device: "",
    timer: null,
    modelValue: [],
    queryValue: "",
    selectedCopy: [],
    compareValue: null,
    selectedVal: computed(
      () => state.device === "mb" && props.multiple && state.visible ? state.selectedCopy : state.selected
    ),
    displayOnlyContent: computed(() => {
      if (props.multiple) {
        if (Array.isArray(state.selected)) {
          if (state.isDisplayOnly && props.options && props.options.length > 0) {
            return state.selected.map((item) => {
              const find = props.options.find((opt) => opt[props.valueField] === item.value);
              return find ? find[props.textField] : "";
            }).join("; ");
          } else {
            return state.selected.map((item) => item.state ? item.state.currentLabel : item.currentLabel).join("; ");
          }
        } else {
          return "";
        }
      } else {
        if (state.selected) {
          if (state.isDisplayOnly && props.options && props.options.length > 0) {
            const find = props.options.find((opt) => opt[props.valueField] === state.selected.value);
            return find ? find[props.textField] : "";
          } else {
            return (state.selected.state ? state.selected.state.currentLabel : state.selected.currentLabel || state.selected.label) || "";
          }
        } else {
          return "";
        }
      }
    }),
    breakpoint: useBreakpoint ? useBreakpoint().current : "",
    isSaaSTheme: vm.theme === "saas",
    disabledOptionHover: false,
    hasClearSelection: false,
    // tiny 新增
    getIcon: computed(() => api2.computedGetIcon()),
    getTagType: computed(() => api2.computedGetTagType()),
    isSelectAll: computed(() => state.selectCls === "checked-sur"),
    autoHideDownIcon: (() => {
      if ((designConfig == null ? void 0 : designConfig.state) && "autoHideDownIcon" in designConfig.state) {
        return designConfig.state.autoHideDownIcon;
      }
      return true;
    })(),
    designConfig,
    currentSizeMap: computed(() => api2.computedCurrentSizeMap())
  }));
  return state;
};
const initStateAdd = ({ computed, props, api: api2, parent }) => {
  const { isIOS } = useUserAgent.useUserAgent();
  return {
    selectedTags: [],
    tips: "",
    showTip: false,
    tipHover: false,
    selectHover: false,
    tipTimer: null,
    selectCls: "checked-sur",
    filteredSelectCls: "checked-sur",
    overflow: null,
    completed: false,
    inputWidth: 0,
    inputPaddingRight: 0,
    hoverIndex: -1,
    hoverValue: -1,
    optionsIndex: -1,
    inputLength: 20,
    optionsCount: 0,
    selectFiexd: {},
    createdLabel: null,
    isSilentBlur: false,
    cachedOptions: [],
    selectedLabel: "",
    previousQuery: null,
    inputHovering: false,
    createdSelected: false,
    isOnComposition: false,
    cachedPlaceHolder: props.placeholder,
    inputHeight: 0,
    initialInputHeight: 0,
    currentPlaceholder: props.placeholder,
    filteredOptionsCount: 0,
    gridData: [],
    treeData: [],
    remoteData: [],
    currentKey: props.modelValue,
    updateId: "",
    popperElm: null,
    debounce: computed(() => type.isNumber(props.queryDebounce) ? props.queryDebounce : props.remote ? 300 : 0),
    emptyText: computed(() => api2.emptyText()),
    emptyFlag: computed(() => api2.emptyFlag()),
    formItemSize: computed(() => (parent.formItem || { state: {} }).state.formItemSize),
    selectDisabled: computed(() => api2.computedSelectDisabled()),
    isDisplayOnly: computed(() => props.displayOnly || (parent.form || {}).displayOnly),
    isDisabled: computed(() => props.disabled || (parent.form || {}).disabled),
    isShowTagText: computed(() => api2.computedShowTagText()),
    gridCheckedData: computed(() => api2.getcheckedData()),
    isExpandAll: computed(() => api2.computedIsExpandAll()),
    searchSingleCopy: computed(() => props.allowCopy && !props.multiple && (props.filterable || props.searchable)),
    childrenName: computed(() => props.treeOp.props && props.treeOp.props.children || "children"),
    tooltipContent: {},
    isHidden: false,
    defaultCheckedKeys: [],
    optionIndexArr: [],
    isIOS,
    showCollapseTag: false,
    exceedMaxVisibleRow: false,
    // 是否超出默认最大显示行数
    toHideIndex: Infinity,
    // 第一个超出被隐藏的索引
    willFocusRun: false,
    // 进入focus时，延时等一下看是否触发blur,触发则不进入focus
    willFocusTimer: 0
  };
};
const initApi = ({
  api: api2,
  props,
  state,
  emit,
  maskState,
  constants,
  parent,
  nextTick,
  dispatch,
  t,
  vm,
  isMobileFirstMode,
  designConfig
}) => {
  Object.assign(api2, {
    state,
    maskState,
    doDestroy: index.doDestroy(vm),
    getTreeData: index.getTreeData(props, state),
    blur: index.blur({ vm, state }),
    focus: index.focus({ vm, state }),
    getValueKey: index.getValueKey(props),
    handleClose: index.handleClose(state),
    getValueIndex: index.getValueIndex(props),
    getChildValue: index.getChildValue(),
    getOption: index.getOption({ props, state, api: api2 }),
    getSelectedOption: index.getSelectedOption({ props, state }),
    emitChange: index.emitChange({ emit, props, state, constants }),
    directEmitChange: index.directEmitChange({ emit, props, state }),
    toggleMenu: index.toggleMenu({ vm, state, props, api: api2 }),
    showTip: index.showTip({ props, state, vm }),
    onOptionDestroy: index.onOptionDestroy(state),
    setSoftFocus: index.setSoftFocus({ vm, state }),
    getcheckedData: index.getcheckedData({ props, state }),
    resetInputWidth: index.resetInputWidth({ vm, state }),
    resetHoverIndex: index.resetHoverIndex({ props, state }),
    resetDatas: index.resetDatas({ props, state }),
    scrollToOption: index.scrollToOption({ vm, constants }),
    selectChange: index.selectChange({ props, vm, state, api: api2 }),
    radioChange: index.radioChange({ props, state, api: api2, vm }),
    handleCopyClick: index.handleCopyClick({ parent, props, state }),
    treeNodeClick: index.treeNodeClick({ props, state, api: api2, vm }),
    managePlaceholder: index.managePlaceholder({ vm, state }),
    nodeCheckClick: index.nodeCheckClick({ props, state, api: api2 }),
    checkDefaultFirstOption: index.checkDefaultFirstOption(state),
    setOptionHighlight: index.setOptionHighlight(state),
    nodeExpand: index.nodeExpand({ state, constants }),
    nodeCollapse: index.nodeCollapse({ state, constants }),
    handleBlur: index.handleBlur({ constants, dispatch, emit, state, designConfig }),
    toggleLastOptionHitState: index.toggleLastOptionHitState({ state }),
    emptyText: index.emptyText({ I18N: constants.I18N, props, state, t, isMobileFirstMode }),
    emptyFlag: index.emptyFlag({ props, state }),
    getOptionIndexArr: index.getOptionIndexArr({ props, state, api: api2 }),
    queryVisibleOptions: index.queryVisibleOptions({ props, vm, isMobileFirstMode }),
    recycleScrollerHeight: index.recycleScrollerHeight({ state, props, recycle: constants.RECYCLE }),
    watchPropsOption: index.watchPropsOption({ constants, parent, props, state }),
    buildSelectConfig: index.buildSelectConfig({ props, state }),
    buildRadioConfig: index.buildRadioConfig({ props, state }),
    onMouseenterNative: index.onMouseenterNative({ state }),
    onMouseleaveNative: index.onMouseleaveNative({ state }),
    onMouseenterSelf: index.onMouseenterSelf({ state }),
    onCopying: index.onCopying({ state, vm }),
    gridOnQueryChange: index.gridOnQueryChange({ props, vm, constants, state }),
    watchHoverIndex: index.watchHoverIndex({ state }),
    computeOptimizeOpts: index.computeOptimizeOpts({ props, designConfig }),
    computeCollapseTags: index.computeCollapseTags(props),
    computeMultipleLimit: index.computeMultipleLimit({ props, state }),
    watchInputHover: index.watchInputHover({ vm }),
    initQuery: index.initQuery({ props, state, constants, vm }),
    updateModelValue: index.updateModelValue({ props, emit, state }),
    computedTagsStyle: index.computedTagsStyle({ props, parent, state, vm }),
    computedReadonly: index.computedReadonly({ props, state }),
    computedShowClose: index.computedShowClose({ props, state }),
    computedCollapseTagSize: index.computedCollapseTagSize(state),
    computedShowNewOption: index.computedShowNewOption({ props, state }),
    computedShowCopy: index.computedShowCopy({ props, state }),
    computedOptionsAllDisabled: index.computedOptionsAllDisabled(state),
    computedDisabledTooltipContent: index.computedDisabledTooltipContent({ state }),
    computedSelectDisabled: index.computedSelectDisabled({ state }),
    computedIsExpand: index.computedIsExpand({ props, state }),
    computedIsExpandAll: index.computedIsExpandAll(props),
    watchInitValue: index.watchInitValue({ props, emit }),
    watchShowClose: index.watchShowClose({ nextTick, state, parent }),
    // tiny 新增
    computedGetIcon: index.computedGetIcon({ designConfig, props }),
    computedGetTagType: index.computedGetTagType({ designConfig, props }),
    clearSearchText: index.clearSearchText({ state, api: api2 }),
    clearNoMatchValue: index.clearNoMatchValue({ props, emit }),
    computedShowTagText: index.computedShowTagText({ state }),
    isTagClosable: index.isTagClosable(),
    computedCurrentSizeMap: index.computedCurrentSizeMap({ state, designConfig })
  });
  addApi({ api: api2, props, state, emit, constants, parent, nextTick, dispatch, vm, isMobileFirstMode, designConfig });
};
const addApi = ({
  api: api2,
  props,
  state,
  emit,
  constants,
  parent,
  nextTick,
  dispatch,
  vm,
  isMobileFirstMode,
  designConfig
}) => {
  Object.assign(api2, {
    resetInputHeight: index.resetInputHeight({ api: api2, constants, nextTick, props, vm, state, designConfig }),
    calcOverFlow: index.calcOverFlow({ vm, props, state }),
    handleFocus: index.handleFocus({ emit, props, state }),
    deleteTag: index.deleteTag({ api: api2, constants, emit, props, vm, nextTick, state }),
    watchValue: index.watchValue({ api: api2, constants, dispatch, props, vm, state }),
    toHide: index.toHide({ constants, state, props, vm, api: api2 }),
    toVisible: index.toVisible({ constants, state, props, vm, api: api2, nextTick }),
    setSelected: index.setSelected({ api: api2, constants, nextTick, props, vm, state }),
    selectOption: index.selectOption({ api: api2, state, props }),
    handleResize: index.handleResize({ api: api2, props, state }),
    watchOptions: index.watchOptions({ api: api2, constants, nextTick, parent, props, state }),
    watchVisible: index.watchVisible({ api: api2, constants, emit, state, vm, props }),
    deletePrevTag: index.deletePrevTag({ api: api2, constants, props, state, vm }),
    onInputChange: index.onInputChange({ api: api2, props, state, constants, nextTick }),
    deleteSelected: index.deleteSelected({ api: api2, constants, emit, props, vm, state }),
    handleMenuEnter: index.handleMenuEnter({ api: api2, nextTick, state, props }),
    resetInputState: index.resetInputState({ api: api2, vm, state }),
    navigateOptions: index.navigateOptions({ api: api2, state, props, nextTick }),
    handleClearClick: index.handleClearClick(api2),
    handleComposition: index.handleComposition({ api: api2, nextTick, state }),
    handleQueryChange: index.handleQueryChange({ api: api2, constants, nextTick, props, vm, state }),
    handleOptionSelect: index.handleOptionSelect({ api: api2, nextTick, props, vm, state }),
    getPluginOption: index.getPluginOption({ api: api2, props, state }),
    toggleCheckAll: index.toggleCheckAll({ api: api2, state, props }),
    handleDebouncedQueryChange: index.handleDebouncedQueryChange({ state, api: api2 }),
    debouncedQueryChange: (event) => {
      const value = props.shape ? event : event.target.value;
      api2.handleDebouncedQueryChange(value);
    },
    debouncedOnInputChange: debounce.default(state.debounce, () => {
      api2.onInputChange();
    }),
    debouncRquest: index.debouncRquest({ api: api2, state, props }),
    defaultOnQueryChange: index.defaultOnQueryChange({ props, state, constants, api: api2, nextTick }),
    queryChange: index.queryChange({ props, state, constants }),
    mounted: index.mounted({ api: api2, parent, state, props, vm, designConfig }),
    unMount: index.unMount({ api: api2, parent, vm, state }),
    watchOptimizeOpts: index.watchOptimizeOpts({ props, state }),
    handleDropdownClick: index.handleDropdownClick({ props, vm, state, emit }),
    handleEnterTag: index.handleEnterTag({ state }),
    calcCollapseTags: index.calcCollapseTags({ state, vm, props }),
    initValue: index.initValue({ state }),
    getLabelSlotValue: index.getLabelSlotValue({ props, state }),
    loadTreeData: index.loadTreeData({ state, vm, props, api: api2 }),
    onClickCollapseTag: index.onClickCollapseTag({ state, props, nextTick, api: api2 })
  });
};
const initWatch = ({ watch, props, api: api2, state, nextTick }) => {
  watch(
    () => props.treeOp.data,
    (data) => data && (state.treeData = data),
    { immediate: true, deep: true }
  );
  watch(
    () => props.gridOp.data,
    (data) => data && (state.gridData = data),
    { immediate: true, deep: true }
  );
  watch(
    () => state.selectDisabled,
    () => nextTick(api2.resetInputHeight)
  );
  watch(
    () => props.placeholder,
    (value) => {
      state.cachedPlaceHolder = state.currentPlaceholder = value;
    }
  );
  watch(
    () => props.modelValue,
    () => {
      if (props.multiple && Array.isArray(props.modelValue)) {
        state.modelValue = [...props.modelValue];
      } else {
        state.modelValue = props.modelValue;
      }
    },
    { immediate: true, deep: true }
  );
  watch(() => state.modelValue, api2.watchValue);
  watch(
    () => state.selectedLabel,
    () => {
      if (props.trim) {
        state.selectedLabel = state.selectedLabel.trim();
      }
    }
  );
  watch(
    () => props.extraQueryParams,
    () => api2.handleQueryChange(state.previousQuery, true),
    { deep: true }
  );
  watch(
    () => state.breakpoint,
    (val) => {
      if (val === "default") {
        state.device = "mb";
      } else {
        state.device = "pc";
      }
    },
    { immediate: true, deep: true }
  );
  watch(
    () => state.device,
    (newVal, oldVal) => {
      if (oldVal !== "" && state.visible) {
        api2.updateModelValue(state.modelValue, true);
      }
    }
  );
  watch(() => state.visible, api2.watchVisible);
  watch(() => state.initValue, api2.watchInitValue, { deep: true });
  addWatch({ watch, props, api: api2, state, nextTick });
};
const addWatch = ({ watch, props, api: api2, state, nextTick }) => {
  watch(() => [...state.options], api2.watchOptions);
  if (props.renderType === "grid" && !props.optimization) {
    watch(() => state.gridData, api2.setSelected, { immediate: true });
  }
  if (props.renderType === "tree" && !props.optimization) {
    watch(() => state.treeData, api2.setSelected, { immediate: true });
  }
  watch(() => state.hoverIndex, api2.watchHoverIndex);
  props.options && watch(() => props.options, api2.watchPropsOption, { immediate: true, deep: true });
  props.optimization && watch(() => state.optimizeOpts, api2.watchOptimizeOpts, { immediate: true });
  watch([() => state.inputHovering, () => state.visible], api2.watchInputHover);
  watch(() => state.showClose, api2.watchShowClose, { immediate: true });
  watch(
    () => state.selectHover,
    () => props.hoverExpand && !props.disabled && !state.isDisplayOnly && nextTick(api2.resetInputHeight)
  );
};
const renderless = (props, { computed, onBeforeUnmount, onMounted, reactive, watch, provide, inject }, { vm, parent, emit, constants, nextTick, dispatch, t, emitter, isMobileFirstMode, useBreakpoint, designConfig }) => {
  const api2 = {};
  const state = initState({
    reactive,
    computed,
    props,
    api: api2,
    emitter,
    parent,
    constants,
    useBreakpoint,
    vm,
    designConfig
  });
  const dialog = inject("dialog", null);
  provide("selectEmitter", state.selectEmitter);
  provide("selectVm", vm);
  const maskState = reactive({ width: "", height: "", top: "" });
  initApi({
    api: api2,
    props,
    state,
    emit,
    maskState,
    constants,
    parent,
    nextTick,
    dispatch,
    t,
    vm,
    isMobileFirstMode,
    designConfig
  });
  parent.$on("handle-clear", (event) => {
    api2.handleClearClick(event);
  });
  if (props.multiple && !Array.isArray(props.modelValue)) {
    emit("update:modelValue", []);
  }
  if (!props.multiple && Array.isArray(props.modelValue)) {
    emit("update:modelValue", "");
  }
  dialog && dialog.state.emitter.on("handleSelectClose", api2.handleClose);
  state.selectEmitter.on(constants.EVENT_NAME.handleOptionClick, api2.handleOptionSelect);
  state.selectEmitter.on(constants.EVENT_NAME.setSelected, api2.setSelected);
  state.selectEmitter.on(constants.EVENT_NAME.initValue, api2.initValue);
  initWatch({ watch, props, api: api2, state, nextTick });
  onMounted(api2.mounted);
  onBeforeUnmount(() => {
    api2.unMount();
    dialog && dialog.state.emitter.off("handleSelectClose", api2.handleClose);
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
