"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const api = [
  "state",
  "bindKeyDown",
  "bindMouseDown",
  "bindMouseMove",
  "bindMouseUp",
  "bindEvent",
  "unBindEvent",
  "bindResize",
  "setTipStyle",
  "getActiveButtonIndex",
  "setActiveButtonValue",
  "setBarStyle",
  "setButtonStyle",
  "initSlider",
  "calculateValue",
  "changeActiveValue",
  "formatTipValue",
  "getActiveButtonValue",
  "displayTip",
  "hideTip",
  "autoSlider",
  "customBeforeAppearHook",
  "customAppearHook",
  "customAfterAppearHook",
  "inputValueChange",
  "inputOnChange",
  "handleSlotInputFocus",
  "handleSlotInputBlur",
  "handleSlotInput"
];
const initState = ({ reactive, computed, props, api: api2, parent, inject }) => {
  const state = reactive({
    showAutoWidth: inject("showAutoWidth", null),
    tipStyle: {},
    barStyle: {},
    moveStyle: [],
    points: [],
    labels: [],
    markList: computed(() => api2.getMarkList()),
    inputValue: [0, 0],
    isDrag: false,
    sliderSize: 0,
    showTip: false,
    activeValue: 0,
    activeIndex: 0,
    isDouble: false,
    leftBtnValue: 0,
    sliderOffset: {},
    rightBtnValue: 0,
    leftBtnStyle: {},
    leftBtnPercent: 0,
    leftBtnShow: true,
    rightBtnStyle: "",
    rightBtnPercent: 0,
    rightBtnShow: false,
    innerTrigger: false,
    changeCompat: computed(() => props.changeCompat),
    rangeDiff: computed(() => props.max - props.min),
    tipValue: computed(() => api2.formatTipValue(state.activeValue)),
    formDisabled: computed(() => (parent.tinyForm || {}).disabled),
    disabled: computed(() => props.disabled || state.formDisabled),
    slotValue: "",
    isSlotTyping: false,
    mouseOuterBtn: false
  });
  return state;
};
const renderless = (props, { computed, onBeforeUnmount, onMounted, reactive, watch, inject }, { vm, parent, constants, nextTick, emit, mode }) => {
  const api2 = {};
  const state = initState({ reactive, computed, props, api: api2, parent, inject });
  parent.tinyForm = parent.tinyForm || inject("form", null);
  Object.assign(api2, {
    state,
    hideTip: index.hideTip(state),
    formatTipValue: index.formatTipValue(props),
    setBarStyle: index.setBarStyle({ props, state }),
    changeActiveValue: index.changeActiveValue(state),
    bindResize: index.bindResize({ vm, props, state }),
    setButtonStyle: index.setButtonStyle({ props, state }),
    calculateValue: index.calculateValue({ vm, props, state }),
    getActiveButtonValue: index.getActiveButtonValue(state),
    getActiveButtonIndex: index.getActiveButtonIndex({ constants, mode, state }),
    setTipStyle: index.setTipStyle({ vm, constants, mode, props, state }),
    customAfterAppearHook: index.customAfterAppearHook({ state, props }),
    customBeforeAppearHook: index.customBeforeAppearHook(props),
    customAppearHook: index.customAppearHook(),
    bindEvent: index.bindEvent(api2),
    autoSlider: index.autoSlider(api2),
    unBindEvent: index.unBindEvent(api2),
    displayTip: index.displayTip({ api: api2, nextTick, state }),
    bindKeyDown: index.bindKeyDown({ api: api2, props, state }),
    bindMouseUp: index.bindMouseUp({ api: api2, emit, state, props }),
    bindMouseMove: index.bindMouseMove({ api: api2, nextTick, state }),
    bindMouseDown: index.bindMouseDown({ api: api2, constants, mode, emit, state, props }),
    setActiveButtonValue: index.setActiveButtonValue({ api: api2, emit, props, state }),
    initSlider: index.initSlider({ api: api2, props, state }),
    watchModelValue: index.watchModelValue({ api: api2, state }),
    watchActiveValue: index.watchActiveValue({ api: api2, emit, props, state }),
    getPoints: index.getPoints({ props, state }),
    getLabels: index.getLabels({ props, state }),
    inputValueChange: index.inputValueChange({ props, api: api2, state, emit }),
    handleSlotInputFocus: index.handleSlotInputFocus(state),
    handleSlotInputBlur: index.handleSlotInputBlur({ state, api: api2 }),
    handleSlotInput: index.handleSlotInput({ state, api: api2 }),
    getMarkList: index.getMarkList({ props }),
    updateSlotValue: index.updateSlotValue({ state }),
    inputOnChange: index.inputOnChange({ api: api2, emit, props, state })
  });
  watch(
    () => props.modelValue,
    (value) => {
      if (props.max < props.min) {
        throw new Error("Slider min should not be greater than max.");
      }
      api2.watchModelValue(value);
    },
    { immediate: true }
  );
  props.changeCompat && watch(() => state.activeValue, api2.watchActiveValue, { immediate: true });
  watch(() => state.activeValue, api2.watchActiveValue, { immediate: true });
  watch(
    () => props.min,
    (min) => {
      const value = Math.max(props.modelValue, min);
      api2.initSlider(value);
      api2.setActiveButtonValue(value);
    }
  );
  watch(
    () => props.max,
    (max) => {
      const value = Math.min(props.modelValue, max);
      api2.initSlider(Math.min(props.modelValue, max));
      api2.setActiveButtonValue(value);
    }
  );
  watch(
    () => state.leftBtnValue,
    (newVal) => {
      state.inputValue[0] = newVal;
    },
    { immediate: true }
  );
  watch(
    () => state.rightBtnValue,
    (newVal) => {
      state.inputValue[1] = newVal;
    },
    { immediate: true }
  );
  onMounted(() => {
    api2.bindEvent();
    api2.getPoints();
    api2.getLabels();
  });
  onBeforeUnmount(api2.unBindEvent);
  return api2;
};
exports.api = api;
exports.renderless = renderless;
