"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const resizeEvent = require("../common/deps/resize-event.js");
const index = require("./index.js");
const wheel = require("./wheel.js");
const { addWheelListener, removeWheelListener } = wheel.getAddWheelListener(window, document);
const api = ["state", "wheelListener", "handleClickDropdownItem", "key", "emitAdd"];
const renderless = (props, { onMounted, onBeforeUnmount, reactive, watch, inject, computed }, { vm }) => {
  const tabs = inject("tabs", null);
  const state = reactive({
    moreList: [],
    moreLeft: false,
    moreRight: false,
    moreOptions: [],
    tabMoreWidth: 0,
    navPaddingRight: 0,
    withAdd: computed(() => tabs.withAdd),
    slotCustom: tabs.slots.custom,
    separator: inject("separator", null)
  });
  const api2 = {
    getBoundRect: index.getBoundRect(vm),
    handleClickDropdownItem: index.handleClickDropdownItem(tabs),
    key: index.key,
    emitAdd: index.emitAdd(tabs)
  };
  Object.assign(api2, {
    state,
    wheelListener: index.wheelListener({ vm, api: api2, tabs, state })
  });
  watch(
    () => state.moreList,
    () => {
      state.moreOptions = state.moreList.map((name) => tabs.state.items.find((item) => item.name === name));
    }
  );
  onMounted(() => {
    addWheelListener(vm.$refs.scroll, api2.wheelListener);
    resizeEvent.addResizeListener(vm.$el, api2.wheelListener);
    api2.wheelListener();
  });
  onBeforeUnmount(() => {
    removeWheelListener(vm.$refs.scroll, api2.wheelListener);
    resizeEvent.removeResizeListener(vm.$el, api2.wheelListener);
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
