"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const string = require("../common/string.js");
const useRelation = require("../common/deps/useRelation.js");
const index = require("./index.js");
const api = [
  "state",
  "setActive",
  "addItem",
  "addNav",
  "scrollTo",
  "clickMore",
  "removeItem",
  "beforeCarouselSwipe",
  "canLeave",
  "clearOtherTabSwipeScroll"
];
let uniqueId = 0;
const renderless = (props, hooks, { vm, emit, nextTick }) => {
  const { onMounted, onBeforeUnmount, provide, reactive, watch, computed, getCurrentInstance } = hooks;
  const instance = getCurrentInstance().proxy;
  const state = reactive({
    items: [],
    navs: [],
    currentItem: computed(() => state.items.find((item) => item.selected)),
    key: computed(() => state.currentItem ? state.currentItem.name : string.random()),
    separator: props.separator,
    swipeable: computed(() => api2.computedSwipeable()),
    maxTabSwipeHeight: 0,
    itemOrderKey: "",
    tabsId: ++uniqueId
  });
  const api2 = {};
  Object.assign(api2, {
    state,
    setActive: index.setActive({ state, api: api2 }),
    addItem: index.addItem(state),
    addNav: index.addNav(state),
    scrollTo: index.scrollTo({ vm, state }),
    removeItem: index.removeItem({ props, state, emit }),
    changeCurrentName: index.changeCurrentName({ state, emit }),
    clickMore: index.clickMore(api2),
    beforeCarouselSwipe: index.beforeCarouselSwipe({ api: api2, state, vm }),
    canLeave: index.canLeave(props),
    clearOtherTabSwipeScroll: index.clearOtherTabSwipeScroll({ state, vm }),
    computedSwipeable: index.computedSwipeable({ props, state }),
    observeTabSwipeSize: index.observeTabSwipeSize({ state, vm }),
    unobserveTabSwipeSize: index.unobserveTabSwipeSize({ state, vm }),
    sortItem: index.sortItem(state),
    useRelation: useRelation.useRelation(hooks),
    onRelationChange: index.onRelationChange({ api: api2, instance, nextTick, state })
  });
  provide("tabs", vm);
  provide("tabsId", state.tabsId);
  api2.useRelation({
    relationKey: `tabs-${state.tabsId}`,
    childrenKey: "childTabs",
    relationContainer: () => vm.$el.querySelector("[data-tag=tiny-tabs-hidden]"),
    onChange: () => api2.onRelationChange()
  });
  watch(
    () => props.activeName,
    (name) => name && api2.setActive(name)
  );
  watch(
    () => props.modelValue,
    (name) => name && api2.setActive(name)
  );
  onMounted(() => {
    nextTick(() => api2.observeTabSwipeSize());
    props.activeName && api2.scrollTo(props.activeName);
    props.modelValue && api2.scrollTo(props.modelValue);
  });
  onBeforeUnmount(() => {
    api2.unobserveTabSwipeSize();
    state.navs.forEach((n, i) => state.navs[i] = null);
    state.navs.length = 0;
  });
  provide("separator", state.separator);
  return api2;
};
exports.api = api;
exports.renderless = renderless;
