"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const dateUtil = require("../common/deps/date-util.js");
const api = [
  "state",
  "emitSelectRange",
  "adjustCurrentSpinner",
  "handleClick",
  "decrease",
  "increase",
  "amPm",
  "adjustSpinners"
];
const initState = ({ reactive, computed, props, api: api2 }) => {
  const state = reactive({
    selectableRange: [],
    currentScrollbar: null,
    hours: computed(() => props.date.getHours()),
    minutes: computed(() => props.date.getMinutes()),
    seconds: computed(() => props.date.getSeconds()),
    hoursList: computed(
      () => Array.from(dateUtil.getRangeHours(state.selectableRange)).map((disabled, hour) => ({ disabled, hour })).filter(({ hour }) => !(hour % state.step.hours))
    ),
    minutesList: computed(
      () => Array.from(dateUtil.getRangeMinutes(state.selectableRange, state.hours)).map((disabled, minute) => ({ disabled, minute })).filter(({ minute }) => !(minute % state.step.minutes))
    ),
    secondsList: computed(
      () => Array.from(new Array(60)).map((item, second) => ({ second })).filter(({ second }) => !(second % state.step.seconds))
    ),
    arrowHourList: computed(() => api2.getArrowHourList()),
    arrowMinuteList: computed(() => api2.getArrowMinuteList()),
    arrowSecondList: computed(() => api2.getArrowSecondList()),
    animationName: "",
    step: computed(() => ({
      hours: props.step.hour * 1 || 1,
      minutes: props.step.minute * 1 || 1,
      seconds: props.step.second * 1 || 1
    }))
  });
  return state;
};
const renderless = (props, { computed, onMounted, reactive, watch, nextTick }, { emit, vm, designConfig, constants }) => {
  const api2 = {};
  const state = initState({ reactive, computed, props, api: api2 });
  onMounted(() => {
    !props.arrowControl && api2.bindScrollEvent();
  });
  Object.assign(api2, {
    state,
    amPm: index.amPm(props),
    typeItemHeight: index.typeItemHeight({ vm, designConfig }),
    scrollBarHeight: index.scrollBarHeight(vm),
    getArrowHourList: index.getArrowHourList(state),
    getArrowSecondList: index.getArrowSecondList(state),
    getArrowMinuteList: index.getArrowMinuteList(state),
    emitSelectRange: index.emitSelectRange({ emit, state }),
    modifyDateField: index.modifyDateField({ emit, props, state }),
    bindScrollEvent: index.bindScrollEvent({ api: api2, vm }),
    adjustSpinners: index.adjustSpinners({ api: api2, state, vm }),
    adjustSpinner: index.adjustSpinner({ api: api2, props, vm, state }),
    increase: index.increase({ api: api2, state }),
    decrease: index.decrease({ api: api2, state }),
    handleClick: index.handleClick(api2),
    scrollDown: index.scrollDown({ api: api2, state }),
    handleScroll: index.handleScroll({ api: api2, vm, state }),
    adjustCurrentSpinner: index.adjustCurrentSpinner({ api: api2, state }),
    selectDateScroll: index.selectDateScroll({ state, props })
  });
  watch(
    () => props.date,
    () => {
      const timeType = ["hours", "minutes", "seconds"];
      timeType.forEach((type) => {
        api2.modifyDateField(type, Math.round(state[type] / state.step[type]) * state.step[type]);
      });
      nextTick(api2.adjustSpinners);
    },
    { immediate: true }
  );
  watch(
    () => state.hoursList,
    () => {
      state.animationName = constants.ANIMATIONNAME;
    },
    { immediate: true }
  );
  return api2;
};
exports.api = api;
exports.renderless = renderless;
