"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const index = require("./index.js");
const vuePopper = require("../common/deps/vue-popper.js");
const string = require("../common/string.js");
const api = [
  "state",
  "bindEvent",
  "hide",
  "show",
  "doDestroy",
  "handleFocus",
  "debounceClose",
  "handleShowPopper",
  "handleClosePopper",
  "setExpectedState",
  "updatePopper",
  "focusHandler"
];
const initState = ({ reactive, showPopper, popperElm, referenceElm, props, inject, popperJS, currentPlacement }) => reactive({
  popperJS,
  showPopper,
  popperElm,
  referenceElm,
  currentPlacement,
  timeout: null,
  focusing: false,
  expectedState: void 0,
  tooltipId: string.guid("tiny-tooltip-", 4),
  tabindex: props.tabindex,
  xPlacement: "bottom",
  showContent: inject("showContent", null),
  tipsMaxWidth: inject("tips-max-width", null)
});
const renderless = (props, {
  watch,
  toRefs,
  reactive,
  onBeforeUnmount,
  onDeactivated,
  onMounted,
  onUnmounted,
  inject
}, { vm, emit, slots, nextTick, parent }) => {
  const api2 = {};
  const popperVmRef = {};
  const popperParam = { emit, props, nextTick, toRefs, reactive, parent: parent.$parent, vm, popperVmRef };
  Object.assign(popperParam, { slots, onBeforeUnmount, onDeactivated, watch });
  const { showPopper, updatePopper, popperElm, referenceElm, doDestroy, popperJS, currentPlacement } = vuePopper.default(
    popperParam
  );
  const state = initState({
    reactive,
    showPopper,
    popperElm,
    referenceElm,
    props,
    inject,
    popperJS,
    currentPlacement
  });
  Object.assign(api2, {
    state,
    doDestroy,
    updatePopper,
    show: index.show({ api: api2, state, props }),
    hide: index.hide(api2),
    destroyed: index.destroyed({ state, api: api2, vm }),
    bindPopper: index.bindPopper({ vm, nextTick, popperVmRef }),
    watchFocusing: index.watchFocusing(state),
    removeFocusing: index.removeFocusing({ api: api2, state }),
    handleBlur: index.handleBlur({ api: api2, state }),
    handleFocus: index.handleFocus({ api: api2, state }),
    debounceClose: index.debounceClose({ api: api2, props }),
    setExpectedState: index.setExpectedState({ state }),
    handleShowPopper: index.handleShowPopper({ props, state }),
    handleClosePopper: index.handleClosePopper({ api: api2, props, state }),
    bindEvent: index.bindEvent({ api: api2, state, vm }),
    focusHandler: index.focusHandler({ slots, api: api2 }),
    handleDocumentClick: index.handleDocumentClick({ props, api: api2, state, popperVmRef }),
    observeCallback: index.observeCallback({ state, popperVmRef })
  });
  watch(() => state.focusing, api2.watchFocusing);
  watch(
    () => props.modelValue,
    (val) => nextTick(() => props.manual && (state.showPopper = val))
  );
  onMounted(() => {
    api2.bindPopper();
    if (props.genArrowByHtml) {
      const config = { attributes: true, childList: false, subtree: false };
      api2.observer = new MutationObserver(api2.observeCallback);
      api2.observer.observe(popperVmRef.popper, config);
    }
  });
  vm.$on("tooltip-update", api2.bindPopper);
  onUnmounted(() => {
    api2.destroyed();
    api2.observer && api2.observer.disconnect();
    vm.$off("tooltip-update");
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
