"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const dom = require("../common/deps/dom.js");
const popupManager = require("../common/deps/popup-manager.js");
const handleAfterLeave = (api) => () => api.destroy();
const destroy = (parent) => () => {
  parent.$el.parentNode.removeChild(parent.$el);
};
const bindEvent = ({ api }) => () => dom.on(document, "keydown", api.bindKeyDown);
const unBindEvent = ({ api }) => () => dom.off(document, "keydown", api.bindKeyDown);
const startTimer = ({ api, state }) => () => {
  if (state.duration > 0) {
    state.timer = setTimeout(() => {
      !state.closed && api.close();
    }, state.duration);
  }
};
const clearTimer = (state) => () => clearTimeout(state.timer);
const close = (state) => () => {
  state.closed = true;
  typeof state.onClose === "function" && state.onClose(state);
};
const watchClosed = ({ state }) => (value) => value && (state.visible = false);
const getZindex = () => popupManager.default.nextZIndex();
const getTypeClass = ({ constants, state }) => () => {
  const typeClass = constants[(state.type || "").toUpperCase()];
  return state.type && !state.iconClass && typeClass ? `${typeClass}` : "";
};
const getOffsetStyle = ({ state }) => () => ({ top: `${state.verticalOffset}px` });
exports.bindEvent = bindEvent;
exports.clearTimer = clearTimer;
exports.close = close;
exports.destroy = destroy;
exports.getOffsetStyle = getOffsetStyle;
exports.getTypeClass = getTypeClass;
exports.getZindex = getZindex;
exports.handleAfterLeave = handleAfterLeave;
exports.startTimer = startTimer;
exports.unBindEvent = unBindEvent;
exports.watchClosed = watchClosed;
