"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const debounce = require("../common/deps/debounce.js");
const index = require("./index.js");
const api = [
  "state",
  "getNodeKey",
  "handleClick",
  "handleContentClick",
  "closeMenu",
  "handleDragEnd",
  "handleDrop",
  "handleDragOver",
  "handleDragStart",
  "handleChildNodeExpand",
  "handleSelectChange",
  "handleContextMenu",
  "handleExpandIconClick",
  "openEdit",
  "addNode",
  "saveEdit",
  "deleteNode",
  "handleCheckChange"
];
const initState = ({ reactive, treeRoot, props, emitter, $parentEmitter, vm, api: api2, TreeAdapter, computed }) => {
  const state = reactive({
    tree: treeRoot,
    expanded: false,
    oldChecked: null,
    indeterminate: null,
    oldIndeterminate: null,
    expandIcon: props.expandIcon,
    childNodeRendered: false,
    showMenu: false,
    menuposition: { position: "absolute", top: 0, left: 0 },
    shrinkIcon: props.shrinkIcon,
    expandIconColor: props.expandIconColor,
    shrinkIconColor: props.shrinkIconColor,
    emitter: emitter(),
    parentEmitter: $parentEmitter,
    isSaaSTheme: (props.theme || vm.theme) === "saas",
    props: treeRoot.props,
    renderedChildNodes: computed(() => {
      return props.node.childNodes.filter((childNode) => TreeAdapter ? TreeAdapter.shouldRender(childNode) : true);
    }),
    computedExpandIcon: computed(() => api2.computedExpandIcon(treeRoot, state)),
    computedIndent: computed(() => api2.computedIndent(props, state))
  });
  return state;
};
const initApi = ({ api: api2, state, dispatch, broadcast, vm, props, treeRoot, nextTick, emit, designConfig }) => {
  Object.assign(api2, {
    state,
    dispatch,
    broadcast,
    watchExpanded: debounce.default(20, index.watchExpanded({ state })),
    created: index.created({ props, state }),
    getNodeKey: index.getNodeKey(state),
    closeMenu: index.closeMenu(state),
    handleSelectChange: index.handleSelectChange({ props, state }),
    handleContextMenu: index.handleContextMenu({ vm, treeRoot, props, state, nextTick }),
    handleExpandClick: index.handleExpandClick({ emit, vm, props, state }),
    handleCheckChange: index.handleCheckChange({ nextTick, props, state }),
    handleDragStart: index.handleDragStart({ state, vm }),
    handleDragOver: index.handleDragOver({ state, vm }),
    handleDrop: index.handleDrop(),
    handleDragEnd: index.handleDragEnd({ state, vm }),
    handleContentClick: index.handleContentClick({ nextTick, props, state, emit }),
    onSiblingToggleExpand: index.onSiblingToggleExpand({ state, props }),
    watchExpandedChange: index.watchExpandedChange({ state, props }),
    handleClick: index.handleClick({ api: api2, vm, props, state }),
    watchIndeterminate: index.watchIndeterminate({ api: api2, props }),
    watchChecked: index.watchChecked({ api: api2, props }),
    openEdit: index.openEdit({ state, vm }),
    addNode: debounce.default(500, true, index.addNode({ state, props, api: api2 })),
    saveEdit: index.saveEdit({ state }),
    deleteNode: index.deleteNode({ state }),
    handleChildNodeExpand: index.handleChildNodeExpand(state),
    handleExpandIconClick: index.handleExpandIconClick({ api: api2, state }),
    computedExpandIcon: index.computedExpandIcon({ designConfig }),
    computedIndent: index.computedIndent()
  });
};
const initWatcher = ({ watch, state, api: api2, props }) => {
  watch(() => state.expanded, api2.watchExpandedChange);
  watch(() => props.node.indeterminate, api2.watchIndeterminate, { deep: true });
  watch(() => props.node.checked, api2.watchChecked, { deep: true });
  watch(() => props.node.expanded, api2.watchExpanded, { deep: true });
};
const renderless = (props, { reactive, watch, inject, provide, computed }, { vm, nextTick, emit, broadcast, dispatch, emitter, designConfig }, { isVue2 }) => {
  const api2 = {};
  const treeRoot = inject("TreeRoot");
  const $parentEmitter = inject("parentEmitter");
  const TreeAdapter = inject("TreeAdapter", null);
  const state = initState({ reactive, treeRoot, props, emitter, $parentEmitter, vm, api: api2, TreeAdapter, computed });
  if (state.tree.accordion) {
    state.parentEmitter.on("sibling-node-toggle-expand", (event) => api2.onSiblingToggleExpand(event));
  }
  provide("parentEmitter", state.emitter);
  initApi({ api: api2, state, dispatch, broadcast, vm, props, treeRoot, nextTick, emit, designConfig });
  initWatcher({ watch, state, api: api2, props });
  api2.created((childrenKey) => {
    watch(
      () => props.node.data[childrenKey],
      () => {
        props.node.updateChildren();
      }
    );
    if (!isVue2) {
      props.node.updateMethod = (node, field) => {
        field === "expanded" && api2.watchExpanded(node[field]);
      };
    }
  });
  state.parentEmitter.on("closeMenu", () => {
    api2.closeMenu();
    state.emitter.emit("closeMenu");
  });
  return api2;
};
exports.api = api;
exports.renderless = renderless;
