"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const debounce = require("../common/deps/debounce.js");
const index = require("./index.js");
const string = require("../common/string.js");
const api = [
  "state",
  "closeMenu",
  "filter",
  "getNodeKey",
  "getNodePath",
  "getCheckedNodes",
  "getCheckedKeys",
  "getCurrentNode",
  "getCurrentKey",
  "setCheckedNodes",
  "setCheckedKeys",
  "setChecked",
  "getHalfCheckedNodes",
  "getHalfCheckedKeys",
  "setCurrentNode",
  "setCurrentKey",
  "getNode",
  "remove",
  "append",
  "insertBefore",
  "insertAfter",
  "handleNodeExpand",
  "updateKeyChildren",
  "initTabIndex",
  "handleKeydown",
  "setCurrentRadio",
  "expandAllNodes",
  "clearCurrentStore",
  "switchToggle",
  "initTreeStore",
  "getSameDataIndex",
  "loopGetTreeData",
  "cancelDelete",
  "deleteAction",
  "deleteConfirm",
  "openEdit",
  "saveNode",
  "addNode",
  "editNode",
  "closeEdit",
  "saveEdit",
  "setAddDisabledKeys",
  "setEditDisabledKeys",
  "setDeleteDisabledKeys",
  "handleCheckPlainNode",
  "handleClickPlainNode",
  "setCheckedByNodeKey"
];
const initState = ({ reactive, emitter, props, computed, api: api2, TreeAdapter }) => {
  const state = reactive({
    loaded: !props.lazy,
    checkEasily: false,
    root: null,
    store: null,
    treeItems: null,
    currentNode: null,
    checkboxItems: [],
    isEmpty: computed(() => api2.computedIsEmpty(props, state)),
    emitter: emitter(),
    expandIcon: props.expandIcon,
    shrinkIcon: props.shrinkIcon,
    expandIconColor: props.expandIconColor,
    shrinkIconColor: props.shrinkIconColor,
    dragState: { showDropIndicator: false, draggingNode: null, dropNode: null, allowDrop: true },
    currentRadio: { value: null },
    children: computed({
      get: () => api2.getChildren(props, state),
      set: (data) => api2.setChildren(data)
    }),
    treeItemArray: computed(() => api2.computedTreeItemArray(props, state)),
    showEmptyText: computed(() => api2.computedShowEmptyText(props, state)),
    currentStore: { flag: false, node: null },
    action: {
      deleteDisabled: [],
      editDisabled: [],
      addDisabled: [],
      show: false,
      node: null,
      data: [],
      label: "",
      type: "",
      isLeaf: false,
      isSaveChildNode: false,
      referenceElm: null,
      popoverVisible: false
    },
    treeEdit: {
      addData: [],
      deleteData: [],
      editData: []
    },
    newNodeId: Math.floor(string.random() * 1e4),
    plainNodeStore: {},
    allNodeKeys: [],
    renderedChildNodes: computed(() => {
      return state.root.childNodes.filter((childNode) => TreeAdapter ? TreeAdapter.shouldRender(childNode) : true);
    }),
    // tiny 新增
    filterText: ""
    // 记录当前过滤的内容
  });
  return state;
};
const initApi = ({ state, dispatch, broadcast, props, vm, constants, t, emit, api: api2, TreeAdapter }) => ({
  state,
  dispatch,
  broadcast,
  setChildren: index.setChildren(props),
  getChildren: index.getChildren(),
  computedTreeItemArray: index.computedTreeItemArray(),
  computedIsEmpty: index.computedIsEmpty(),
  watchDefaultCheckedKeys: index.watchDefaultCheckedKeys(state),
  watchData: index.watchData({ state }),
  watchCheckboxItems: index.watchCheckboxItems(),
  watchCheckStrictly: index.watchCheckStrictly(state),
  updated: index.updated({ vm, state }),
  filter: index.filter({ props, state, api: api2 }),
  getNodeKey: index.getNodeKey(props),
  getNodePath: index.getNodePath({ props, state }),
  getCheckedNodes: index.getCheckedNodes(state),
  getCheckedKeys: index.getCheckedKeys(state),
  getCurrentNode: index.getCurrentNode(state),
  setCheckedNodes: index.setCheckedNodes({ props, state }),
  setCheckedKeys: index.setCheckedKeys({ props, state, api: api2 }),
  setChecked: index.setChecked(state),
  getHalfCheckedNodes: index.getHalfCheckedNodes(state),
  getHalfCheckedKeys: index.getHalfCheckedKeys(state),
  setCurrentNode: index.setCurrentNode({ props, state }),
  setCurrentKey: index.setCurrentKey({ props, state }),
  getNode: index.getNode(state),
  remove: index.remove(state),
  append: index.append(state),
  insertBefore: index.insertBefore(state),
  insertAfter: index.insertAfter(state),
  updateKeyChildren: index.updateKeyChildren({ props, state }),
  initTabIndex: index.initTabIndex({ vm, state }),
  handleKeydown: index.handleKeydown({ vm, state, TreeAdapter }),
  computedShowEmptyText: index.computedShowEmptyText({ constants, t }),
  setCurrentRadio: index.setCurrentRadio({ props, state }),
  expandAllNodes: index.expandAllNodes({ state }),
  dragStart: index.dragStart({ props, state, emit }),
  dragOver: index.dragOver({ props, state, emit, vm }),
  dragEnd: index.dragEnd({ state, emit }),
  clearCurrentStore: index.clearCurrentStore(state),
  initIsCurrent: debounce.default(20, index.initIsCurrent({ props, state })),
  setCheckedByNodeKey: index.setCheckedByNodeKey({ props, state })
});
const initWatcher = ({ watch, props, api: api2, state, isVue2 }) => {
  watch(() => props.defaultCheckedKeys, api2.watchDefaultCheckedKeys);
  watch(() => props.defaultExpandedKeys, api2.watchDefaultExpandedKeys);
  watch(() => props.defaultExpandedKeysHighlight, api2.initIsCurrent);
  watch(() => props.data, api2.watchData, { deep: !isVue2 });
  watch(() => props.checkboxItems, api2.watchCheckboxItems);
  watch(() => props.checkStrictly, api2.watchCheckStrictly);
  watch(() => props.defaultCheckedKeys, api2.setCurrentRadio, { immediate: true });
  watch(
    () => props.deleteDisabledKeys,
    (value) => state.action.deleteDisabled = value || [],
    { immediate: true }
  );
  watch(
    () => props.editDisabledKeys,
    (value) => state.action.editDisabled = value || [],
    { immediate: true }
  );
  watch(
    () => props.addDisabledKeys,
    (value) => state.action.addDisabled = value || [],
    { immediate: true }
  );
};
const renderless = (props, { computed, onMounted, onUpdated, reactive, watch, provide, onBeforeUnmount, inject }, { vm, t, emit, constants, broadcast, dispatch, service, emitter, nextTick }, { isVue2 }) => {
  const api2 = {};
  const TreeAdapter = inject("TreeAdapter", null);
  const state = initState({ reactive, emitter, props, computed, api: api2, TreeAdapter });
  provide("parentEmitter", state.emitter);
  Object.assign(api2, initApi({ state, dispatch, broadcast, props, vm, constants, t, emit, api: api2, TreeAdapter }), {
    closeMenu: index.closeMenu(state),
    mounted: index.mounted({ api: api2, vm }),
    created: index.created({ api: api2, state }),
    getCurrentKey: index.getCurrentKey({ api: api2, props }),
    handleNodeExpand: index.handleNodeExpand(emit),
    beforeUnmount: index.beforeUnmount({ api: api2, vm, state }),
    wrapMounted: index.wrapMounted({ api: api2, props, service }),
    initTreeStore: index.initTreeStore({ api: api2, props, state, emit }),
    deleteAction: index.deleteAction({ state, api: api2, emit }),
    deleteConfirm: index.deleteConfirm({ state, props, api: api2 }),
    getSameDataIndex: index.getSameDataIndex,
    loopGetTreeData: index.loopGetTreeData,
    cancelDelete: index.cancelDelete({ state }),
    openEdit: index.openEdit({ props, state, api: api2, emit }),
    saveNode: index.saveNode({ state, emit, api: api2 }),
    addNode: index.addNode({ api: api2, props, state }),
    editNode: index.editNode({ state }),
    closeEdit: index.closeEdit({ props, state, api: api2, emit }),
    saveEdit: index.saveEdit({ props, state, emit }),
    setAddDisabledKeys: index.setAddDisabledKeys({ state }),
    setEditDisabledKeys: index.setEditDisabledKeys({ state }),
    setDeleteDisabledKeys: index.setDeleteDisabledKeys({ state }),
    watchDefaultExpandedKeys: index.watchDefaultExpandedKeys({ state, api: api2, nextTick }),
    switchToggle: index.switchToggle({ state }),
    initPlainNodeStore: index.initPlainNodeStore({ props, state }),
    handleCheckPlainNode: index.handleCheckPlainNode({ props, emit }),
    handleClickPlainNode: index.handleClickPlainNode(emit)
  });
  api2.created();
  initWatcher({ watch, props, api: api2, state, isVue2 });
  onMounted(api2.wrapMounted);
  onUpdated(api2.updated);
  onBeforeUnmount(api2.beforeUnmount);
  return api2;
};
exports.api = api;
exports.renderless = renderless;
