"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
require("../../index.js");
const BoundingRect = require("../../../../../zrender@5.4.1/node_modules/zrender/lib/core/BoundingRect.js");
const matrix = require("../../../../../zrender@5.4.1/node_modules/zrender/lib/core/matrix.js");
const util = require("../../../../../zrender@5.4.1/node_modules/zrender/lib/core/util.js");
function BMapCoordSys(bmap, api) {
  this._bmap = bmap;
  this.dimensions = ["lng", "lat"];
  this._mapOffset = [0, 0];
  this._api = api;
  this._projection = new BMap.MercatorProjection();
}
BMapCoordSys.prototype.type = "bmap";
BMapCoordSys.prototype.dimensions = ["lng", "lat"];
BMapCoordSys.prototype.setZoom = function(zoom) {
  this._zoom = zoom;
};
BMapCoordSys.prototype.setCenter = function(center) {
  this._center = this._projection.lngLatToPoint(new BMap.Point(center[0], center[1]));
};
BMapCoordSys.prototype.setMapOffset = function(mapOffset) {
  this._mapOffset = mapOffset;
};
BMapCoordSys.prototype.getBMap = function() {
  return this._bmap;
};
BMapCoordSys.prototype.dataToPoint = function(data) {
  var point = new BMap.Point(data[0], data[1]);
  var px = this._bmap.pointToOverlayPixel(point);
  var mapOffset = this._mapOffset;
  return [px.x - mapOffset[0], px.y - mapOffset[1]];
};
BMapCoordSys.prototype.pointToData = function(pt) {
  var mapOffset = this._mapOffset;
  pt = this._bmap.overlayPixelToPoint({
    x: pt[0] + mapOffset[0],
    y: pt[1] + mapOffset[1]
  });
  return [pt.lng, pt.lat];
};
BMapCoordSys.prototype.getViewRect = function() {
  var api = this._api;
  return new BoundingRect.default(0, 0, api.getWidth(), api.getHeight());
};
BMapCoordSys.prototype.getRoamTransform = function() {
  return matrix.create();
};
BMapCoordSys.prototype.prepareCustoms = function() {
  var rect = this.getViewRect();
  return {
    coordSys: {
      // The name exposed to user is always 'cartesian2d' but not 'grid'.
      type: "bmap",
      x: rect.x,
      y: rect.y,
      width: rect.width,
      height: rect.height
    },
    api: {
      coord: util.bind(this.dataToPoint, this),
      size: util.bind(dataToCoordSize, this)
    }
  };
};
BMapCoordSys.prototype.convertToPixel = function(ecModel, finder, value) {
  return this.dataToPoint(value);
};
BMapCoordSys.prototype.convertFromPixel = function(ecModel, finder, value) {
  return this.pointToData(value);
};
function dataToCoordSize(dataSize, dataItem) {
  dataItem = dataItem || [0, 0];
  return util.map([0, 1], function(dimIdx) {
    var val = dataItem[dimIdx];
    var halfSize = dataSize[dimIdx] / 2;
    var p1 = [];
    var p2 = [];
    p1[dimIdx] = val - halfSize;
    p2[dimIdx] = val + halfSize;
    p1[1 - dimIdx] = p2[1 - dimIdx] = dataItem[1 - dimIdx];
    return Math.abs(this.dataToPoint(p1)[dimIdx] - this.dataToPoint(p2)[dimIdx]);
  }, this);
}
var Overlay;
BMapCoordSys.dimensions = BMapCoordSys.prototype.dimensions;
function createOverlayCtor() {
  function Overlay2(root) {
    this._root = root;
  }
  Overlay2.prototype = new BMap.Overlay();
  Overlay2.prototype.initialize = function(map) {
    map.getPanes().labelPane.appendChild(this._root);
    return this._root;
  };
  Overlay2.prototype.draw = function() {
  };
  return Overlay2;
}
BMapCoordSys.create = function(ecModel, api) {
  var bmapCoordSys;
  var root = api.getDom();
  ecModel.eachComponent("bmap", function(bmapModel) {
    var painter = api.getZr().painter;
    var viewportRoot = painter.getViewportRoot();
    if (typeof BMap === "undefined") {
      throw new Error("BMap api is not loaded");
    }
    Overlay = Overlay || createOverlayCtor();
    if (bmapCoordSys) {
      throw new Error("Only one bmap component can exist");
    }
    var bmap;
    if (!bmapModel.__bmap) {
      var bmapRoot = root.querySelector(".ec-extension-bmap");
      if (bmapRoot) {
        viewportRoot.style.left = "0px";
        viewportRoot.style.top = "0px";
        root.removeChild(bmapRoot);
      }
      bmapRoot = document.createElement("div");
      bmapRoot.className = "ec-extension-bmap";
      bmapRoot.style.cssText = "position:absolute;width:100%;height:100%";
      root.appendChild(bmapRoot);
      var mapOptions = bmapModel.get("mapOptions");
      if (mapOptions) {
        mapOptions = util.clone(mapOptions);
        delete mapOptions.mapType;
      }
      bmap = bmapModel.__bmap = new BMap.Map(bmapRoot, mapOptions);
      var overlay = new Overlay(viewportRoot);
      bmap.addOverlay(overlay);
      painter.getViewportRootOffset = function() {
        return {
          offsetLeft: 0,
          offsetTop: 0
        };
      };
    }
    bmap = bmapModel.__bmap;
    var center = bmapModel.get("center");
    var zoom = bmapModel.get("zoom");
    if (center && zoom) {
      var bmapCenter = bmap.getCenter();
      var bmapZoom = bmap.getZoom();
      var centerOrZoomChanged = bmapModel.centerOrZoomChanged([bmapCenter.lng, bmapCenter.lat], bmapZoom);
      if (centerOrZoomChanged) {
        var pt = new BMap.Point(center[0], center[1]);
        bmap.centerAndZoom(pt, zoom);
      }
    }
    bmapCoordSys = new BMapCoordSys(bmap, api);
    bmapCoordSys.setMapOffset(bmapModel.__mapOffset || [0, 0]);
    bmapCoordSys.setZoom(zoom);
    bmapCoordSys.setCenter(center);
    bmapModel.coordinateSystem = bmapCoordSys;
  });
  ecModel.eachSeries(function(seriesModel) {
    if (seriesModel.get("coordinateSystem") === "bmap") {
      seriesModel.coordinateSystem = bmapCoordSys;
    }
  });
  return bmapCoordSys && [bmapCoordSys];
};
exports.default = BMapCoordSys;
