"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const _Stack = require("./_Stack.js");
const _arrayEach = require("./_arrayEach.js");
const _assignValue = require("./_assignValue.js");
const _cloneBuffer = require("./_cloneBuffer.js");
const _getAllKeys = require("./_getAllKeys.js");
const _getTag = require("./_getTag.js");
const _initCloneArray = require("./_initCloneArray.js");
const _initCloneByTag = require("./_initCloneByTag.js");
const _initCloneObject = require("./_initCloneObject.js");
const isArray = require("./isArray.js");
const isBuffer = require("./isBuffer.js");
const isMap = require("./isMap.js");
const isObject = require("./isObject.js");
const isSet = require("./isSet.js");
var CLONE_DEEP_FLAG = 1;
var argsTag = "[object Arguments]", arrayTag = "[object Array]", boolTag = "[object Boolean]", dateTag = "[object Date]", errorTag = "[object Error]", funcTag = "[object Function]", genTag = "[object GeneratorFunction]", mapTag = "[object Map]", numberTag = "[object Number]", objectTag = "[object Object]", regexpTag = "[object RegExp]", setTag = "[object Set]", stringTag = "[object String]", symbolTag = "[object Symbol]", weakMapTag = "[object WeakMap]";
var arrayBufferTag = "[object ArrayBuffer]", dataViewTag = "[object DataView]", float32Tag = "[object Float32Array]", float64Tag = "[object Float64Array]", int8Tag = "[object Int8Array]", int16Tag = "[object Int16Array]", int32Tag = "[object Int32Array]", uint8Tag = "[object Uint8Array]", uint8ClampedTag = "[object Uint8ClampedArray]", uint16Tag = "[object Uint16Array]", uint32Tag = "[object Uint32Array]";
var cloneableTags = {};
cloneableTags[argsTag] = cloneableTags[arrayTag] = cloneableTags[arrayBufferTag] = cloneableTags[dataViewTag] = cloneableTags[boolTag] = cloneableTags[dateTag] = cloneableTags[float32Tag] = cloneableTags[float64Tag] = cloneableTags[int8Tag] = cloneableTags[int16Tag] = cloneableTags[int32Tag] = cloneableTags[mapTag] = cloneableTags[numberTag] = cloneableTags[objectTag] = cloneableTags[regexpTag] = cloneableTags[setTag] = cloneableTags[stringTag] = cloneableTags[symbolTag] = cloneableTags[uint8Tag] = cloneableTags[uint8ClampedTag] = cloneableTags[uint16Tag] = cloneableTags[uint32Tag] = true;
cloneableTags[errorTag] = cloneableTags[funcTag] = cloneableTags[weakMapTag] = false;
function baseClone(value, bitmask, customizer, key, object, stack) {
  var result, isDeep = bitmask & CLONE_DEEP_FLAG;
  if (result !== void 0) {
    return result;
  }
  if (!isObject.default(value)) {
    return value;
  }
  var isArr = isArray.default(value);
  if (isArr) {
    result = _initCloneArray.default(value);
  } else {
    var tag = _getTag.default(value), isFunc = tag == funcTag || tag == genTag;
    if (isBuffer.default(value)) {
      return _cloneBuffer.default(value, isDeep);
    }
    if (tag == objectTag || tag == argsTag || isFunc && !object) {
      result = isFunc ? {} : _initCloneObject.default(value);
    } else {
      if (!cloneableTags[tag]) {
        return object ? value : {};
      }
      result = _initCloneByTag.default(value, tag, isDeep);
    }
  }
  stack || (stack = new _Stack.default());
  var stacked = stack.get(value);
  if (stacked) {
    return stacked;
  }
  stack.set(value, result);
  if (isSet.default(value)) {
    value.forEach(function(subValue) {
      result.add(baseClone(subValue, bitmask, customizer, subValue, value, stack));
    });
  } else if (isMap.default(value)) {
    value.forEach(function(subValue, key2) {
      result.set(key2, baseClone(subValue, bitmask, customizer, key2, value, stack));
    });
  }
  var keysFunc = _getAllKeys.default;
  var props = isArr ? void 0 : keysFunc(value);
  _arrayEach.default(props || value, function(subValue, key2) {
    if (props) {
      key2 = subValue;
      subValue = value[key2];
    }
    _assignValue.default(result, key2, baseClone(subValue, bitmask, customizer, key2, value, stack));
  });
  return result;
}
exports.default = baseClone;
