"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const hooks = require("vue");
const Cell = require("../cell/Cell.js");
const Collapse = require("../collapse/Collapse.js");
const index_esm = require("../../../../../@vant_use@1.6.0_vue@3.5.13_typescript@5.7.3_/node_modules/@vant/use/dist/index.esm.js");
const useExpose = require("../composables/use-expose.js");
const useLazyRender = require("../composables/use-lazy-render.js");
const index = require("../cell/index.js");
const basic = require("../utils/basic.js");
const create = require("../utils/create.js");
const props = require("../utils/props.js");
const [name, bem] = create.createNamespace("collapse-item");
const CELL_SLOTS = ["icon", "title", "value", "label", "right-icon"];
const collapseItemProps = basic.extend({}, Cell.cellSharedProps, {
  name: props.numericProp,
  isLink: props.truthProp,
  disabled: Boolean,
  readonly: Boolean,
  lazyRender: props.truthProp
});
var stdin_default = hooks.defineComponent({
  name,
  props: collapseItemProps,
  setup(props2, {
    slots
  }) {
    const wrapperRef = hooks.ref();
    const contentRef = hooks.ref();
    const {
      parent,
      index: index$1
    } = index_esm.useParent(Collapse.COLLAPSE_KEY);
    if (!parent) {
      if (process.env.NODE_ENV !== "production") {
        console.error("[Vant] <CollapseItem> must be a child component of <Collapse>.");
      }
      return;
    }
    const name2 = hooks.computed(() => {
      var _a;
      return (_a = props2.name) != null ? _a : index$1.value;
    });
    const expanded = hooks.computed(() => parent.isExpanded(name2.value));
    const show = hooks.ref(expanded.value);
    const lazyRender = useLazyRender.useLazyRender(() => show.value || !props2.lazyRender);
    const onTransitionEnd = () => {
      if (!expanded.value) {
        show.value = false;
      } else if (wrapperRef.value) {
        wrapperRef.value.style.height = "";
      }
    };
    hooks.watch(expanded, (value, oldValue) => {
      if (oldValue === null) {
        return;
      }
      if (value) {
        show.value = true;
      }
      const tick = value ? hooks.nextTick : index_esm.raf;
      tick(() => {
        if (!contentRef.value || !wrapperRef.value) {
          return;
        }
        const {
          offsetHeight
        } = contentRef.value;
        if (offsetHeight) {
          const contentHeight = `${offsetHeight}px`;
          wrapperRef.value.style.height = value ? "0" : contentHeight;
          index_esm.doubleRaf(() => {
            if (wrapperRef.value) {
              wrapperRef.value.style.height = value ? contentHeight : "0";
            }
          });
        } else {
          onTransitionEnd();
        }
      });
    });
    const toggle = (newValue = !expanded.value) => {
      parent.toggle(name2.value, newValue);
    };
    const onClickTitle = () => {
      if (!props2.disabled && !props2.readonly) {
        toggle();
      }
    };
    const renderTitle = () => {
      const {
        border,
        disabled,
        readonly
      } = props2;
      const attrs = basic.pick(props2, Object.keys(Cell.cellSharedProps));
      if (readonly) {
        attrs.isLink = false;
      }
      if (disabled || readonly) {
        attrs.clickable = false;
      }
      return hooks.createVNode(index.Cell, hooks.mergeProps({
        "role": "button",
        "class": bem("title", {
          disabled,
          expanded: expanded.value,
          borderless: !border
        }),
        "aria-expanded": String(expanded.value),
        "onClick": onClickTitle
      }, attrs), basic.pick(slots, CELL_SLOTS));
    };
    const renderContent = lazyRender(() => {
      var _a;
      return hooks.withDirectives(hooks.createVNode("div", {
        "ref": wrapperRef,
        "class": bem("wrapper"),
        "onTransitionend": onTransitionEnd
      }, [hooks.createVNode("div", {
        "ref": contentRef,
        "class": bem("content")
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)])]), [[hooks.vShow, show.value]]);
    });
    useExpose.useExpose({
      toggle,
      expanded,
      itemName: name2
    });
    return () => hooks.createVNode("div", {
      "class": [bem({
        border: index$1.value && props2.border
      })]
    }, [renderTitle(), renderContent()]);
  }
});
exports.collapseItemProps = collapseItemProps;
exports.default = stdin_default;
