"use strict";
Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: "Module" } });
const hooks = require("vue");
const index_esm = require("../../../../../@vant_use@1.6.0_vue@3.5.13_typescript@5.7.3_/node_modules/@vant/use/dist/index.esm.js");
const useTouch = require("../composables/use-touch.js");
const index = require("../icon/index.js");
const closest = require("../utils/closest.js");
const props = require("../utils/props.js");
const create = require("../utils/create.js");
const dom = require("../utils/dom.js");
const format = require("../utils/format.js");
const basic = require("../utils/basic.js");
const floatingBubbleProps = {
  gap: props.makeNumberProp(24),
  icon: String,
  axis: props.makeStringProp("y"),
  magnetic: String,
  offset: {
    type: Object,
    default: () => ({
      x: -1,
      y: -1
    })
  },
  teleport: {
    type: [String, Object],
    default: "body"
  }
};
const [name, bem] = create.createNamespace("floating-bubble");
var stdin_default = hooks.defineComponent({
  name,
  inheritAttrs: false,
  props: floatingBubbleProps,
  emits: ["click", "update:offset", "offsetChange"],
  setup(props2, {
    slots,
    emit,
    attrs
  }) {
    const rootRef = hooks.ref();
    const state = hooks.ref({
      x: 0,
      y: 0,
      width: 0,
      height: 0
    });
    const boundary = hooks.computed(() => ({
      top: props2.gap,
      right: dom.windowWidth.value - state.value.width - props2.gap,
      bottom: dom.windowHeight.value - state.value.height - props2.gap,
      left: props2.gap
    }));
    const dragging = hooks.ref(false);
    let initialized = false;
    const rootStyle = hooks.computed(() => {
      const style = {};
      const x = format.addUnit(state.value.x);
      const y = format.addUnit(state.value.y);
      style.transform = `translate3d(${x}, ${y}, 0)`;
      if (dragging.value || !initialized) {
        style.transition = "none";
      }
      return style;
    });
    const updateState = () => {
      if (!show.value) return;
      const {
        width,
        height
      } = index_esm.useRect(rootRef.value);
      const {
        offset
      } = props2;
      state.value = {
        x: offset.x > -1 ? offset.x : dom.windowWidth.value - width - props2.gap,
        y: offset.y > -1 ? offset.y : dom.windowHeight.value - height - props2.gap,
        width,
        height
      };
    };
    const touch = useTouch.useTouch();
    let prevX = 0;
    let prevY = 0;
    const onTouchStart = (e) => {
      touch.start(e);
      dragging.value = true;
      prevX = state.value.x;
      prevY = state.value.y;
    };
    const onTouchMove = (e) => {
      e.preventDefault();
      touch.move(e);
      if (props2.axis === "lock") return;
      if (!touch.isTap.value) {
        if (props2.axis === "x" || props2.axis === "xy") {
          let nextX = prevX + touch.deltaX.value;
          if (nextX < boundary.value.left) nextX = boundary.value.left;
          if (nextX > boundary.value.right) nextX = boundary.value.right;
          state.value.x = nextX;
        }
        if (props2.axis === "y" || props2.axis === "xy") {
          let nextY = prevY + touch.deltaY.value;
          if (nextY < boundary.value.top) nextY = boundary.value.top;
          if (nextY > boundary.value.bottom) nextY = boundary.value.bottom;
          state.value.y = nextY;
        }
        const offset = basic.pick(state.value, ["x", "y"]);
        emit("update:offset", offset);
      }
    };
    index_esm.useEventListener("touchmove", onTouchMove, {
      target: rootRef
    });
    const onTouchEnd = () => {
      dragging.value = false;
      hooks.nextTick(() => {
        if (props2.magnetic === "x") {
          const nextX = closest.closest([boundary.value.left, boundary.value.right], state.value.x);
          state.value.x = nextX;
        }
        if (props2.magnetic === "y") {
          const nextY = closest.closest([boundary.value.top, boundary.value.bottom], state.value.y);
          state.value.y = nextY;
        }
        if (!touch.isTap.value) {
          const offset = basic.pick(state.value, ["x", "y"]);
          emit("update:offset", offset);
          if (prevX !== offset.x || prevY !== offset.y) {
            emit("offsetChange", offset);
          }
        }
      });
    };
    const onClick = (e) => {
      if (touch.isTap.value) emit("click", e);
      else e.stopPropagation();
    };
    hooks.onMounted(() => {
      updateState();
      hooks.nextTick(() => {
        initialized = true;
      });
    });
    hooks.watch([dom.windowWidth, dom.windowHeight, () => props2.gap, () => props2.offset], updateState, {
      deep: true
    });
    const show = hooks.ref(true);
    hooks.onActivated(() => {
      show.value = true;
    });
    hooks.onDeactivated(() => {
      if (props2.teleport) {
        show.value = false;
      }
    });
    return () => {
      const Content = hooks.withDirectives(hooks.createVNode("div", hooks.mergeProps({
        "class": bem(),
        "ref": rootRef,
        "onTouchstartPassive": onTouchStart,
        "onTouchend": onTouchEnd,
        "onTouchcancel": onTouchEnd,
        "onClickCapture": onClick,
        "style": rootStyle.value
      }, attrs), [slots.default ? slots.default() : hooks.createVNode(index.default, {
        "name": props2.icon,
        "class": bem("icon")
      }, null)]), [[hooks.vShow, show.value]]);
      return props2.teleport ? hooks.createVNode(hooks.Teleport, {
        "to": props2.teleport
      }, {
        default: () => [Content]
      }) : Content;
    };
  }
});
exports.default = stdin_default;
exports.floatingBubbleProps = floatingBubbleProps;
