"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const hooks = require("vue");
const mountComponent = require("../utils/mount-component.js");
const Toast = require("./Toast.js");
const basic = require("../utils/basic.js");
const defaultOptions = {
  icon: "",
  type: "text",
  message: "",
  className: "",
  overlay: false,
  onClose: void 0,
  onOpened: void 0,
  duration: 2e3,
  teleport: "body",
  iconSize: void 0,
  iconPrefix: void 0,
  position: "middle",
  transition: "van-fade",
  forbidClick: false,
  loadingType: void 0,
  overlayClass: "",
  overlayStyle: void 0,
  closeOnClick: false,
  closeOnClickOverlay: false
};
let queue = [];
let allowMultiple = false;
let currentOptions = basic.extend({}, defaultOptions);
const defaultOptionsMap = /* @__PURE__ */ new Map();
function parseOptions(message) {
  if (basic.isObject(message)) {
    return message;
  }
  return {
    message
  };
}
function createInstance() {
  const {
    instance
  } = mountComponent.mountComponent({
    setup() {
      const message = hooks.ref("");
      const {
        open,
        state,
        close,
        toggle
      } = mountComponent.usePopupState();
      const onClosed = () => {
      };
      const render = () => {
        const attrs = {
          onClosed,
          "onUpdate:show": toggle
        };
        return hooks.createVNode(Toast.default, hooks.mergeProps(state, attrs), null);
      };
      hooks.watch(message, (val) => {
        state.message = val;
      });
      hooks.getCurrentInstance().render = render;
      return {
        open,
        close,
        message
      };
    }
  });
  return instance;
}
function getInstance() {
  if (!queue.length || allowMultiple) {
    const instance = createInstance();
    queue.push(instance);
  }
  return queue[queue.length - 1];
}
function showToast(options = {}) {
  if (!basic.inBrowser) {
    return {};
  }
  const toast = getInstance();
  const parsedOptions = parseOptions(options);
  toast.open(basic.extend({}, currentOptions, defaultOptionsMap.get(parsedOptions.type || currentOptions.type), parsedOptions));
  return toast;
}
exports.showToast = showToast;
