"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const hooks = require("vue");
const basic = require("./basic.js");
const useExpose = require("../composables/use-expose.js");
function usePopupState() {
  const state = hooks.reactive({
    show: false
  });
  const toggle = (show) => {
    state.show = show;
  };
  const open = (props) => {
    basic.extend(state, props, { transitionAppear: true });
    toggle(true);
  };
  const close = () => toggle(false);
  useExpose.useExpose({ open, close, toggle });
  return {
    open,
    close,
    state,
    toggle
  };
}
function mountComponent(RootComponent) {
  const app = hooks.createApp(RootComponent);
  const root = document.createElement("div");
  document.body.appendChild(root);
  return {
    instance: app.mount(root),
    unmount() {
      app.unmount();
      document.body.removeChild(root);
    }
  };
}
exports.mountComponent = mountComponent;
exports.usePopupState = usePopupState;
